
program est_mpec_market_mpi

    use test3kid
    use globaldef
    use mathfn
    use mpi

    implicit none

    integer, parameter :: nall = NPARAM + (NMARKET * NTYPEALL)

    real(dp) :: x(nall)
    real(dp) :: z(nall)
    real(dp) :: xold(nall)
    real(dp) :: momVec(NMOM)

    integer, parameter :: nmax = nall, nclmax = 1, ncnmax = NMARKET * NTYPEALL
    integer, parameter :: lda = nclmax, ldcj = ncnmax, ldr = nmax
    integer, parameter :: liwork = 3 * nall + nclmax + 2 * ncnmax
    integer, parameter :: lwork = 2 * nmax ** 2 + nmax * nclmax + 2 * nmax * ncnmax &
                          + 21 * nmax + 11 * nclmax + 22 * ncnmax + 1

    real(dp) :: objf
    integer  :: ifail, irevcm, iter, nclin, ncnln
    integer :: iX, jX, pX, pX1, pX2, mX

    real(dp) :: a(lda, nmax), bl(nmax + nclmax + ncnmax), bu(nmax + nclmax + ncnmax), c(ncnmax)
    real(dp) :: cjac(ldcj, nmax), clamda(nmax + nclmax + ncnmax), objgrd(nmax), r(ldr, nmax), work(lwork)

    integer :: istate(nmax + nclmax + ncnmax), iwork(liwork), needc(ncnmax)
    real(dp) :: xparam(NPARAM)
    real(dp) :: xlam(NMARKET * NTYPEALL)
    real(dp) :: xtax(NTAXPARAM)
    integer :: xtype(NPARAM)
    real(dp) :: rev

    integer :: ierror, id, nproc

    real(dp) :: f_temp(0:nall)
    real(dp) :: c_temp((NMARKET * NTYPEALL) * (nall + 1))
    real(dp) :: epsrel
    real(dp) :: f0, c0(NMARKET * NTYPEALL)
    real(dp), parameter :: eps = 1.0e-7_dp

    type(opt_t) :: opt
    type(ret_t) :: ret
    integer :: narg
    integer :: funit
    integer :: istat
    character(len = 32) :: filetag, thisarg
    character(len = 128) :: fname
    character(len = 128) :: fnamesol
    real(dp) :: eqcon(NMARKET * NTYPEALL)
    integer :: xmode
    integer :: estmode

    call mpi_init(ierror)

    !obtain id for each node
    call mpi_comm_rank(mpi_comm_world, id, ierror)

    !returns the number of nodes
    call mpi_comm_size(mpi_comm_world, nproc, ierror)

    if (nproc < nall + 1) then
        write (*,*) nall + 1, 'processors required'
        call mpi_finalize(ierror)
        stop
    end if

    narg = command_argument_count()
    if (narg == 0) then
        filetag = ''
        fname = 'output/estimation/x_market_mpec_mpi.log'
        fnamesol = 'output/estimation/xsol_market_mpec_mpi.asc'
        estmode = 0
    else if (narg == 1) then
        call get_command_argument(1, value = filetag, status = istat)
        fname = 'output/estimation/' // trim(adjustl(filetag)) // '/x_market_mpec_mpi.log'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        estmode = 0
    else if (narg == 2) then
        call get_command_argument(1, value = filetag, status = istat)
        fname = 'output/estimation/' // trim(adjustl(filetag)) // '/x_market_mpec_mpi.log'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        call get_command_argument(2, value = thisarg, status = istat)
        read(thisarg, '(i10)') estmode
    else
        call mpi_finalize(ierror)
        stop 'incorrect number of command arguments supplied'
    end if

    call setOptions(opt, OPT_ESTIMATION, estmode)

    x = 0.1_dp

#   ifdef _NTYPE3_
    select case(estmode)
    case(0)
        open(newunit = funit, file = 'output/estimation/600/xsol_market_mpec_mpi.asc', action = 'read', status = 'old')
    case(statemode)
        open(newunit = funit, file = 'output/estimation/600_state/xsol_market_mpec_mpi.asc', action = 'read', status = 'old')
    case(mcdmode)
        open(newunit = funit, file = 'output/estimation/600/xsol_market_mpec_mpi.asc', action = 'read', status = 'old')
    case default
        call mpi_finalize(ierror)
        stop 'unrecongised estmode'
    end select
#   endif

    do iX = 1, nall
        read(funit, *) x(iX)
    end do
    close(funit)

    xparam = x(1:NPARAM)
    xlam = x(NPARAM + 1 : nall)

    xold = -huge(0.0_dp)

    nclin = 0
    ncnln = NMARKET * NTYPEALL

    ! constraints on parameters
    bl(1:NPARAM) = -8.0_dp
    bu(1:NPARAM) =  7.0_dp
#   ifdef _NTYPE3_
    bu(15) = 1.0_dp !leisure curvature
    bl(17) = -log((1.0_dp - 0.05_dp) / 0.05_dp) ! ces share
    bl(18) = 1.0_dp ! match shock
    bu(18) = 3.0_dp ! match shock
    bu(18) = 2.2_dp ! match shock
    bu(21) = 1.0_dp ! single prod no kids, low educ
    bl(1)  = -3.0_dp ! state error
    bu(6)  = 1.25 ! male college mean
#   endif

#   ifdef _NTYPE4_
    bu(19) = 1.0_dp !leisure curvature
    bl(21) = -log((1.0_dp - 0.05_dp) / 0.05_dp) ! ces share
    bl(22) = 1.0_dp ! match shock
    bu(22) = 3.0_dp ! match shock
    bu(22) = 2.2_dp ! match shock
    bu(25) = 1.0_dp ! single prod no kids, low educ
    bu(25) = 2.0_dp ! single prod no kids, low educ
    bl(1) = -3.0_dp ! state error
    bu(8) = 1.25 ! male college mean
#   endif

    ! constraints on pareto weights
    bl(NPARAM + 1 : nall) = 0.05_dp
    bu(NPARAM + 1 : nall) = 0.95_dp

    ! equilibrium constraints
    bl(nall + 1 : nall + NMARKET * NTYPEALL) = 0.0_dp
    bu(nall + 1 : nall + NMARKET * NTYPEALL) = 0.0_dp

    ifail  = -1
    irevcm = 0

    !obtain id for each node
    call mpi_comm_rank(mpi_comm_world, id, ierror)

    if (id == 0) then

        select case(estmode)
        case(statemode)
            print *, 'estmode = statemode'
            print *
        case(mcdmode)
            print *, 'estmode = mcdmode'
            print *
        case default
        end select

        call E04UEF ('Derivative Level = 3')
        call E04UEF ('Step Size = 0.1')
        call E04UEF ('Optimality Tolerance = 1.0e-8')
        call E04UEF ('Verify Level = -1')
        call E04UEF ('Major Iteration Limit = 1000')
        call E04UEF ('Minor Iteration Limit = 2500')

        open(newunit = funit, file = fname, action = 'write', status = 'replace')

    end if

    do

        if (id == 0) then
            call E04UFF( irevcm, nall, nclin, ncnln, lda, ldcj, ldr, a, bl, bu, iter,  &
                         istate, c, cjac, clamda, objf, objgrd, r, x, needc, iwork,    &
                         liwork, work, lwork, ifail)
        end if

        call MPI_barrier(MPI_COMM_WORLD, ierror)

        call MPI_bcast(ifail, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(irevcm, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(x, nall, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(xold, nall, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

        if (ifail == -1 .and. irevcm > 0) then

            if ( .not. vecEqual(x, xold) ) then

                z = x
                if ( id > 0 ) then
                    epsrel = eps * max( 1.0_dp, abs( x(id) ) )
                    z(id) = x(id) + epsrel
                end if

                xparam = z(1:NPARAM)
                xlam = z(NPARAM + 1:nall)
                call objfun_mpec_market(xparam, xlam, xtax, opt, ret)
                objf = log(ret%dist)
                c = ret%eqconMarket

                call MPI_barrier(MPI_COMM_WORLD, ierror)

                call MPI_gather(objf, 1, MPI_DOUBLE_PRECISION,  &
                                f_temp, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

                call MPI_gather(c, NMARKET * NTYPEALL, MPI_DOUBLE_PRECISION,  &
                                c_temp, NMARKET*NTYPEALL, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

                if (id == 0) then

                    ! objective gradient
                    f0 = f_temp(0)
                    do iX = 1, nall
                        epsrel = eps * max( 1.0_dp, abs( x(iX) ) )
                        objgrd(iX) = ( f_temp(iX) - f0 ) / epsrel
                    end do
                    ! constraint jacobian
                    c0 = c_temp(1 : NTYPEALL * NMARKET)
                    do iX = 1, nall ! variables
                        epsrel = eps * max( 1.0_dp, abs( x(iX) ) )
                        do jX = 1, NMARKET * NTYPEALL ! constraints
                            cjac(jX, iX) = ( c_temp( iX * NMARKET * NTYPEALL + jX ) - c0(jX) ) / epsrel
                        end do
                    end do

                    do iX = 1, nall
                        write(funit, *) iX, x(iX), objf
                    end do

                    xold = x

                end if

            end if

        else

            exit

        end if

    end do

    if (id == 0) then

        call setOptions(opt, OPT_MODELFIT, estmode)

        close(funit)

        ! write solution file
        open(newunit = funit, file = fnamesol, action = 'write', status = 'replace')
        do iX = 1, nall
            write(funit, *) x(iX)
        end do
        close(funit)

        call objfun_mpec_market(xparam, xlam, xtax, opt, ret)
        print *, ret%dist
        pX = 0
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX + 1
                    print *, mX, pX1, pX2, xlam(pX)
                end do
            end do
        end do

        call objfun_market(xparam, xlam, xtax, opt, ret)
        print *, ret%dist
        pX = 0
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX + 1
                    print *, mX, pX1, pX2, xlam(pX)
                end do
            end do
        end do

    end if

    call mpi_finalize(ierror)

end program est_mpec_market_mpi
