module globaldef

    use ISO_FORTRAN_ENV

    implicit none

    public

    ! define precision type and standard output unit (defined in ISO_FORTRAN_ENV)
    integer, parameter :: dp = REAL64
    integer, parameter :: IOUT = output_unit

    ! maximum number of estimation parameters
#   ifdef _NTYPE3_
    integer, parameter :: NPARAM = 33
#   endif
#   ifdef _NTYPE4_
    integer, parameter :: NPARAM = 41
#   endif

    ! maximum kinks for empirical tax schedule
    integer, parameter :: MAXKINKS = 200

    integer, parameter :: TIME_LEISURE = 1
    integer, parameter :: TIME_PUBLIC = 2
    integer, parameter :: TIME_HOURS = 3
    integer, parameter :: TIME_EMPLOYMENT = 4
    integer, parameter :: GZDIM = 4

    integer, private, parameter :: time_alloc = 3 ! number of activities
    integer, private, parameter :: time_fine = 2  ! fineness of the grid
    integer, parameter :: N = time_alloc * (time_fine + 1) + 1
    integer, parameter :: MIN_N(3) = (/2, 1, 1/)  ! leisure, public, hours
    integer, parameter :: MAX_N(3) = (/8, 6, 6/)
    integer, private, parameter :: time_excess(3) = N - MAX_N + MIN_N - 1
    integer, parameter :: NDIM = 30
    integer, parameter :: NDIM_NOHOME = MAX_N(TIME_HOURS) - MIN_N(TIME_HOURS) + 1
    integer, parameter :: PRIMARY_MIN = 2
    real(dp), parameter :: MIN_TIME_LEISURE = 0.0_dp
    real(dp), parameter :: MIN_TIME_PUBLIC = 4.0_dp
    real(dp), parameter :: MIN_TIME_HOURS = 0.0_dp
    real(dp), parameter :: TIME_TOTAL_FULL = 112.0_dp
    real(dp), parameter :: TIME_TOTAL = TIME_TOTAL_FULL - MIN_TIME_LEISURE - MIN_TIME_PUBLIC

    ! utility function scale
    real(dp), parameter :: UCONS = 1.0_dp
    real(dp), parameter :: CSCALE = 1.0_dp! 1.0_dp / 100.0_dp
    real(dp), parameter :: CFLOOR = 1.0e-16_dp
    real(dp), parameter :: QCONS = 0.0_dp

    real(dp), parameter :: WEIGHTINC2 = 1.0_dp

    ! aggregated time categories for moments

    ! bands for non-employment / part-time / full-time
    integer, parameter :: BAND_WORKL(0:2) = (/1, 2, 4/)
    integer, parameter :: BAND_WORKU(0:2) = (/1, 3, N/)

    ! bands for home-production
    integer, parameter :: NHH = 3
    integer, parameter :: BAND_HOMEL(NHH) = (/1, 3, 5/)
    integer, parameter :: BAND_HOMEU(NHH) = (/2, 4, N/)

    ! integration nodes
    integer, parameter :: NR = 600
#   ifdef FINETAXGRID
    integer, parameter :: NODEW = 50
#   else
    integer, parameter :: NODEW = 30
#   endif
    integer, parameter :: NEARN = 100     ! earnings grid: only used for figures
    integer, parameter :: NODEWFINE = 200 ! fine wage grid: only used for figures

    ! number of markets for estimation
    integer, parameter :: NMARKET = 9

    ! smoothing kernel parameter (SMSC low means more smoothing)
    real(dp), parameter :: SMSC = 50.0_dp
    logical,  parameter :: SMOOTH = .true.
    logical,  parameter :: BOUND_PROB = .true.
    real(dp), parameter :: BOUND_PROB_VAL = 1.0e-8_dp

#   ifdef _NTYPE4_
    integer, parameter :: NTYPE1 = 4
    integer, parameter :: NTYPE2 = 4
    character(len=32), parameter :: TYPENAME1(NTYPE1) = (/ character(32) :: 'less then high school', 'high school', 'some college', 'college' /)
    character(len=32), parameter :: TYPENAME2(NTYPE2) = (/ character(32) :: 'less than high school', 'high school', 'some college', 'college' /)
#   endif
#   ifdef _NTYPE3_
    integer, parameter :: NTYPE1 = 3
    integer, parameter :: NTYPE2 = 3
    character(len=32), parameter :: TYPENAME1(NTYPE1) = (/ character(32) :: 'high school and below', 'some college', 'college' /)
    character(len=32), parameter :: TYPENAME2(NTYPE2) = (/ character(32) :: 'high school and below', 'some college', 'college' /)
#   endif

    integer, parameter :: NTYPEALL = NTYPE1 * NTYPE2
    integer, parameter :: NPOINT1 = 1
    integer, parameter :: NPOINT2 = 1
    integer, parameter :: NPOINTALL = NPOINT1 * NPOINT2

    ! tax specification
    integer, parameter :: TAXLEVEL = 1
    integer, parameter :: TAXRATE = 2
    integer, parameter :: TAXMODE = TAXLEVEL
    logical, parameter :: TAXNONLABOUR = .false.
    integer, parameter :: TAX_JOINT = 1        ! most general
    integer, parameter :: TAX_INDEPENDENT = 2  ! tax independent (same schedule for single and couples)
    integer, parameter :: TAX_INDEPENDENT3 = 5 ! income splitting (same schedule for singles and couples, avg earnings for couples)
    integer, parameter :: TAX_FULLJOINT2 = 6   ! joint taxation aggregation
    integer, parameter :: TAX_JOINTGBT = 8     ! most general with gender based taxes for married couples
    integer, parameter :: TAX_JOINTGBTALL = 9  ! most general with gender based taxes for married couples AND singles

    integer, parameter :: statemode = 2
    integer, parameter :: mcdmode = 3

    ! breaks

#   ifdef FINETAXGRID

    integer, parameter :: NTAXBR = 18
    real(dp), parameter :: TAXBR(NTAXBR) &
        = (/ 0.0_dp, 12500.0_dp, 18750.0_dp, 25000.0_dp, 31250.0_dp, 37500.0_dp, &
              48750.0_dp, 60000.0_dp, 72500.0_dp, 85000.0_dp, 98000.0_dp, 110000.0_dp, &
              130500.0_dp, 150000.0_dp, 170000.0_dp, 190000.0_dp, 220000.0_dp, 250000.0_dp /) / 52.0_dp
#   else

    integer,  parameter :: NTAXBR = 10
    real(dp), parameter :: TAXBR(NTAXBR) &
        = (/ 0.0_dp, 12500.0_dp, 25000.0_dp, 37500.0_dp, 60000.0_dp, &
             85000.0_dp, 110000.0_dp, 150000.0_dp, 190000.0_dp, 250000.0_dp /) / 52.0_dp

#   endif

    real(dp), parameter :: TAXDBR(NTAXBR - 1) = TAXBR(2:NTAXBR) - TAXBR(1:NTAXBR - 1)
    real(dp), parameter :: MINY = -200.0_dp

    ! number of tax parameters
    ! require NTAXPARAM >= maxval(NTAXRES)
    integer, parameter :: NTAXPARAM = NTAXBR * (NTAXBR + 2) !(NTAXBR + 1) * NTAXBR / 2 + NTAXBR

    integer, parameter :: NTAXRES(9) = (/ (NTAXBR + 1) * NTAXBR / 2 + NTAXBR, &
                                          NTAXBR, &
                                          2 * NTAXBR, &
                                          2 * NTAXBR, &
                                          NTAXBR, &
                                          NTAXBR, &
                                          NTAXBR, &
                                          NTAXBR * (NTAXBR + 1), &
                                          NTAXBR * (NTAXBR + 2) /)

    ! number of constraints
    integer, parameter :: NTAXCON(9) = (/ (NTAXBR - 1) + NTAXBR * (NTAXBR - 1), &
                                          (NTAXBR - 1), &
                                          2 * (NTAXBR - 1), &
                                          2 * (NTAXBR - 1), &
                                          (NTAXBR - 1), &
                                          (NTAXBR - 1), &
                                          (NTAXBR - 1), &
                                          (NTAXBR - 1) + 2 * (NTAXBR - 1) * (NTAXBR - 1), &
                                          2 * (NTAXBR - 1) + 2 * (NTAXBR - 1) * (NTAXBR - 1) /)

    integer, parameter :: MAXVALNTAXCON = NTAXCON(9)

    integer, parameter :: MODE_MOMENTS = 0
    integer, parameter :: MODE_WELFARE = 1

    integer, parameter :: TAXMODE_DATA = 0
    integer, parameter :: TAXMODE_SIMU = 1

    integer, parameter :: SEX_MALE = 1
    integer, parameter :: SEX_FEMALE = 2

    integer, parameter :: TRANSFORM_EXP = 1
    integer, parameter :: TRANSFORM_ONEEXP = 2
    integer, parameter :: TRANSFORM_LOGIT = 3
    integer, parameter :: TRANSFORM_LINEAR = 4
    integer, parameter :: TRANSFORM_ONE_EXP = 5
    integer, parameter :: TRANSFORM_INVEXP = 6
    integer, parameter :: TRANSFORM_CORR = 7

    integer, parameter :: X_TRANSFORM = 0
    integer, parameter :: X_NOTRANSFORM = 1

    integer, parameter :: OPT_MAIN = 1
    integer, parameter :: OPT_WAGESCALE = 2
    integer, parameter :: OPT_WAGESCALE1 = 3
    integer, parameter :: OPT_WAGESCALE2 = 4
    integer, parameter :: OPT_HOMESCALE = 5
    integer, parameter :: OPT_HOMESCALE1 = 6
    integer, parameter :: OPT_HOMESCALE2 = 7
    integer, parameter :: OPT_ESTIMATION = 8
    integer, parameter :: OPT_NLYSCALE = 9
    integer, parameter :: OPT_NLYSCALE1 = 10
    integer, parameter :: OPT_NLYSCALE2 = 11
    integer, parameter :: OPT_MODELFIT = 12
    integer, parameter :: OPT_MODELFITSAVE = 13
    integer, parameter :: OPT_SIMULATION = 14
    integer, parameter :: OPT_SIMULATIONFIT = 15

    real(dp), parameter :: ELAS_STEP = 0.01_dp

    integer, parameter :: XTYPE_RND = 1
    integer, parameter :: XTYPE_WAGEM = 2
    integer, parameter :: XTYPE_WAGEF = 3
    integer, parameter :: XTYPE_FC = 4
    integer, parameter :: XTYPE_LEISURE = 5
    integer, parameter :: XTYPE_HOMEPREF = 6
    integer, parameter :: XTYPE_HOMEPROD = 7
    integer, parameter :: XTYPE_MARRND = 8

    real(dp), parameter :: REMOVE_STATE = -9999999.9_dp
    real(dp), parameter :: WMINA =  4.5_dp
    real(dp), parameter :: WMAXA = 80.0_dp

    real(dp), parameter :: ATUSWT = 11.0_dp

    ! some options to pass

    type opt_t
        integer :: mode
        integer :: xmode
        integer :: tmode
        integer :: taxins
        logical :: wageElasticity
        logical :: wageElasticity1
        logical :: wageElasticity2
        real(dp) :: szWageElasticity
        real(dp) :: szWageElasticity1
        real(dp) :: szWageElasticity2
        logical :: homeElasticity
        real(dp) :: szHomeElasticity
        real(dp) :: szHomeElasticity1
        real(dp) :: szHomeElasticity2
        logical :: incomeElasticity
        real(dp) :: szIncomeElasticity
        real(dp) :: szIncomeElasticity1
        real(dp) :: szIncomeElasticity2
        logical :: verbose
        logical :: verboseExtra
        logical :: extraMom
        logical :: saveMom
        logical :: simMom
        logical :: fixMarriageMarket
        logical :: marriageMarketElasticity
        logical :: earnElasticity
        logical :: earnElasticityLower
        character(len = :), allocatable :: savePath
        integer :: vunit
        character(128) :: tag
        logical :: swfInc
        integer :: inc1
        integer :: inc2
        real(dp) :: forceAssortativeScale
        logical :: saveUpf
        integer :: nlam
        real(dp) :: scaleA
        real(dp) :: scaleAh
        real(dp) :: utilityShift
        logical :: medicaid
        real(dp) :: correlation
        real(dp) :: pMscale(NTYPE1)
        real(dp) :: pFscale(NTYPE2)
        character(len = :), allocatable :: locname
        real(dp) :: logwf_mu
        real(dp) :: logwf_sd
    end type opt_t

    type grid_t
        real(dp) :: gz(NDIM, GZDIM)
        real(dp) :: gzSq(NDIM, GZDIM)
        integer :: gzind(NDIM, GZDIM)
        real(dp) :: gridval(N)
        integer :: gridind(N)
    end type grid_t

    ! type for tax system

    type tax_t

        ! tax mode
        integer  :: tMode
        integer  :: taxins

        real(dp) :: tCou(NTAXBR, NTAXBR)
        real(dp) :: tCouL(3, NTAXBR - 1, NTAXBR - 1)
        real(dp) :: tCouU(3, NTAXBR - 1, NTAXBR - 1)

        real(dp) :: tSin(NTAXBR, 1:2)
        real(dp) :: tSinL(2, NTAXBR - 1, 1:2)

        real(dp) :: mtrViolation

        ! tax sim parameters
        real(dp) :: couBreaks(MAXKINKS)
        real(dp) :: couMtr(MAXKINKS)
        real(dp) :: couNet(MAXKINKS)
        real(dp) :: couVirtual(MAXKINKS)
        integer  :: couNumKinks

        real(dp) :: couKidsBreaks(MAXKINKS)
        real(dp) :: couKidsMtr(MAXKINKS)
        real(dp) :: couKidsNet(MAXKINKS)
        real(dp) :: couKidsVirtual(MAXKINKS)
        integer  :: couKidsNumKinks

        real(dp) :: sinBreaks(MAXKINKS)
        real(dp) :: sinMtr(MAXKINKS)
        real(dp) :: sinNet(MAXKINKS)
        real(dp) :: sinVirtual(MAXKINKS)
        integer  :: sinNumKinks

        real(dp) :: sinKidsBreaks(MAXKINKS)
        real(dp) :: sinKidsMtr(MAXKINKS)
        real(dp) :: sinKidsNet(MAXKINKS)
        real(dp) :: sinKidsVirtual(MAXKINKS)
        integer  :: sinKidsNumKinks

        ! food stamps
        real(dp) :: sinGrossElig
        real(dp) :: sinNetElig
        real(dp) :: sinStdDed
        real(dp) :: sinMaxFsp

        real(dp) :: sinKidsGrossElig
        real(dp) :: sinKidsNetElig
        real(dp) :: sinKidsStdDed
        real(dp) :: sinKidsMaxFsp

        real(dp) :: couGrossElig
        real(dp) :: couNetElig
        real(dp) :: couStdDed
        real(dp) :: couMaxFsp

        real(dp) :: couKidsGrossElig
        real(dp) :: couKidsNetElig
        real(dp) :: couKidsStdDed
        real(dp) :: couKidsMaxFsp

        integer :: state

        ! welfare
        logical  :: sinKidsGrossTest
        logical  :: sinKidsNetTest
        real(dp) :: sinKidsNeedStd
        real(dp) :: sinKidsGrossNeedScale
        real(dp) :: sinKidsNetNeedScale
        real(dp) :: sinKidsEligDisAmt
        real(dp) :: sinKidsEligDisPct

        logical  :: sinKidsMaxBen
        real(dp) :: sinKidsMaxBenVal
        real(dp) :: sinKidsPaymentStd
        real(dp) :: sinKidsBenDisAmt
        real(dp) :: sinKidsBenDisPct
        real(dp) :: sinKidsBenRatio

        logical  :: couKidsGrossTest
        logical  :: couKidsNetTest
        real(dp) :: couKidsNeedStd
        real(dp) :: couKidsGrossNeedScale
        real(dp) :: couKidsNetNeedScale
        real(dp) :: couKidsEligDisAmt
        real(dp) :: couKidsEligDisPct

        logical  :: couKidsMaxBen
        real(dp) :: couKidsMaxBenVal
        real(dp) :: couKidsPaymentStd
        real(dp) :: couKidsBenDisAmt
        real(dp) :: couKidsBenDisPct
        real(dp) :: couKidsBenRatio

        ! medicaid eligibility
        real(dp) :: singleMedicaidWork
        real(dp) :: singleMedicaidNotWork
        real(dp) :: singleKidsMedicaidChild
        real(dp) :: singleKidsMedicaidParentWork
        real(dp) :: singleKidsMedicaidParentNotWork
        real(dp) :: coupleMedicaidWork
        real(dp) :: coupleMedicaidNotWork
        real(dp) :: coupleKidsMedicaidChild
        real(dp) :: coupleKidsMedicaidParentWork
        real(dp) :: coupleKidsMedicaidParentNotWork

        ! medicaid value
        real(dp) :: adultMedicaidValue(0:2)
        real(dp) :: kidsMedicaidValue

        ! transfer value of medicaid enabled?
        logical :: medicaid

    end type tax_t

    ! market equilibrium
    type m_t

        real(dp) :: marketSize
        integer  :: marketId
        integer  :: taxId
        character(len=32) :: name

        real(dp) :: lam(NTYPE1, NTYPE2)
        real(dp) :: Pr1(NTYPE1, NTYPE2)
        real(dp) :: Pr2(NTYPE1, NTYPE2)
        real(dp) :: PrSingle1(NTYPE1)
        real(dp) :: PrSingle2(NTYPE2)
        real(dp) :: numSingle1(NTYPE1)
        real(dp) :: numSingle2(NTYPE2)
        real(dp) :: typeDemand(NTYPE1, NTYPE2)
        real(dp) :: typeSupply(NTYPE1, NTYPE2)

        real(dp) :: numSingleKid1(NTYPE1, 0:1)
        real(dp) :: numSingleKid2(NTYPE2, 0:1)
        real(dp) :: typeDemandKid(NTYPE1, NTYPE2, 0:1)
        real(dp) :: typeSupplyKid(NTYPE1, NTYPE2, 0:1)

        real(dp) :: excessDemand(NTYPE1, NTYPE2)

        real(dp) :: muTheta0 ! location parameter for unconditional distribution of marital shocks
        real(dp) :: sdTheta0 ! scale parameter for unconditional distribution of marital shocks

        real(dp) :: thetaAdjSingle1(NTYPE1)
        real(dp) :: thetaAdjSingle2(NTYPE2)
        real(dp) :: thetaAdj1(NTYPE1, NTYPE2)
        real(dp) :: thetaAdj2(NTYPE1, NTYPE2)

        real(dp) :: theta

        real(dp) :: Eu1(0:NTYPE1, 0:NTYPE2)
        real(dp) :: Eu2(0:NTYPE1, 0:NTYPE2)

        real(dp), allocatable :: EuLamGrid1(:, :, :)
        real(dp), allocatable :: EuLamGrid2(:, :, :)
        real(dp), allocatable :: lamgrid(:)

    end type m_t

    ! define the type for an adult

    type p_t

        real(dp) :: comp
        real(dp) :: fc
        real(dp) :: fckid
        real(dp) :: fcft
        real(dp) :: betl
        real(dp) :: betQ
        real(dp) :: betc
        real(dp) :: betcl
        real(dp) :: sigl    ! leisure curvature
        real(dp) :: sigQ    ! public good curvature
        real(dp) :: sigc    ! private good curvature
        real(dp) :: wage_mu ! mean log wages
        real(dp) :: wage_sd ! sd log wages
        real(dp) :: sd      ! scale parameter for state specific errors
        real(dp) :: nly     ! non-labour income
        real(dp) :: wlf     ! non-labour income
        real(dp) :: frac    ! measure of individuals of this type
        real(dp) :: logBetl
        real(dp) :: logBetQ
        real(dp) :: logBetc
        real(dp) :: psi

        real(dp) :: sigl_1  ! transformed leisure curvature
        real(dp) :: sigQ_1  ! transformed public good curvature
        real(dp) :: sigc_1  ! transformed private good curvature

        real(dp) :: wageNode(NODEW)
        real(dp) :: wageNodeStd(NODEW)
        real(dp) :: dwageNode(NODEW)
        real(dp) :: wageTrapzWeight(NODEW)
        real(dp) :: logWageNode(NODEW)
        real(dp) :: wageDensity(NODEW)
        real(dp) :: wageNodeFine(NODEWFINE)
        real(dp) :: wageNodeFineStd(NODEWFINE)
        real(dp) :: wageDensityFine(NODEWFINE)

        real(dp) :: probTypeK
        real(dp) :: probKidTax
        real(dp) :: probTypeQ

        integer :: taxmode

        real(dp) :: Qk
        real(dp) :: homeIntercept
        real(dp) :: homeSlope
        real(dp) :: homeBeta

    end type p_t

    type v_t
        real(dp) :: l(NDIM)
        real(dp) :: Q(NDIM)
        real(dp) :: Qw(NDIM)
        real(dp) :: c(NDIM, NODEW)
        real(dp) :: derc(NTAXPARAM, NDIM, NODEW)
        real(dp) :: earn(NDIM, NODEW)
        real(dp) :: taxable(NDIM, NODEW)
        real(dp) :: tottax(NDIM, NODEW)
        real(dp) :: netinc(NDIM, NODEW)
        real(dp) :: util(NDIM, NODEW)
        real(dp) :: utilr(NDIM, NR, NODEW)

        ! these are used for calculating the welfare weights
        real(dp) :: utilInc(NDIM, NODEW)
        real(dp) :: dInc(NDIM, NODEW)
        logical :: swfInc
        real(dp) :: inc(2)

    end type v_t

    type v2_t

        real(dp) :: lM(NDIM)
        real(dp) :: QM(NDIM, NDIM)
        real(dp) :: cM(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: lF(NDIM)
        real(dp) :: QF(NDIM, NDIM)
        real(dp) :: cF(NDIM, NDIM, NODEW, NODEW)

        real(dp) :: earnM(NDIM, NODEW)
        real(dp) :: earnF(NDIM, NODEW)
        real(dp) :: taxableM(NDIM, NODEW)
        real(dp) :: taxableF(NDIM, NODEW)
        real(dp) :: tottax(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: netinc(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: utilF(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: utilM(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: utilncF(NDIM, NDIM)
        real(dp) :: utilncM(NDIM, NDIM)

        real(dp) :: utilM_c(N, N, NODEW, NODEW)
        real(dp) :: utilF_c(N, N, NODEW, NODEW)

        ! these are used for calculating the welfare weights
        real(dp) :: utilIncF(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: utilIncM(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: dInc(NDIM, NDIM, NODEW, NODEW)
        logical :: swfInc

    end type v2_t


    ! define allocation

    type eall_t

        real(dp) :: Eu
        real(dp) :: Eswf
        real(dp) :: Eearn
        real(dp) :: Etaxable
        real(dp) :: Etottax
        real(dp) :: Enetinc
        real(dp) :: Eallocation(NDIM)
        ! real(dp) :: EallocationWage(NDIM, NODEW) ! allocation conditional on wage
        real(dp) :: EwageAllocation(NDIM)
        real(dp) :: EwageSqAllocation(NDIM)
        real(dp) :: EwageLevelAllocation(NDIM)
        real(dp) :: EwageLevelSqAllocation(NDIM)

        ! aggregated moments in allocation

        real(dp) :: hours(0:2)
        real(dp) :: wage(0:2)
        real(dp) :: wagesd(0:2)
        real(dp) :: work(0:1)

        real(dp) :: wageWork
        real(dp) :: wagesdWork

        real(dp) :: earnW
        real(dp) :: earn
        real(dp) :: earnWsd
        real(dp) :: earnsd

        real(dp) :: hourshW     ! conditional hours
        real(dp) :: hourshWsd

        real(dp) :: hoursh      ! unconditional hours
        real(dp) :: hourshsd

        real(dp) :: home(NHH)
        real(dp) :: homeh
        real(dp) :: homehsd

        ! home production hours by own employment status
        real(dp) :: homehW(0:1)
        real(dp) :: homehWsd(0:1)

        ! home production hours by own hours
        real(dp) :: homehH(0:2)
        real(dp) :: homehHsd(0:2)

    end type eall_t

    type eall2_t

        real(dp) :: Eu1
        real(dp) :: Eu2
        real(dp) :: Eswf1
        real(dp) :: Eswf2
        real(dp) :: Eearn1
        real(dp) :: Eearn2
        real(dp) :: Etaxable1
        real(dp) :: Etaxable2
        real(dp) :: Etottax
        real(dp) :: Enetinc
        real(dp) :: Eallocation(NDIM, NDIM)
        real(dp) :: Ewage1Allocation(NDIM, NDIM)
        real(dp) :: Ewage2Allocation(NDIM, NDIM)
        real(dp) :: Ewage12Allocation(NDIM, NDIM)
        real(dp) :: EwageSq1Allocation(NDIM, NDIM)
        real(dp) :: EwageSq2Allocation(NDIM, NDIM)
        real(dp) :: EwageSq12Allocation(NDIM, NDIM)

        real(dp) :: EwageLevel1Allocation(NDIM, NDIM)
        real(dp) :: EwageLevel2Allocation(NDIM, NDIM)
        real(dp) :: EwageLevelSq1Allocation(NDIM, NDIM)
        real(dp) :: EwageLevelSq2Allocation(NDIM, NDIM)

        real(dp) :: dEu1
        real(dp) :: dEu2

        ! this is used for calculation the marginal social welfare weights
        real(dp) :: EswfInc1
        real(dp) :: EswfInc2
        real(dp) :: dInc

        ! aggregated moments in allocation

        ! joint labour supply
        real(dp) :: jhours(0:2, 0:2)
        real(dp) :: mhours(0:2)
        real(dp) :: fhours(0:2)
        real(dp) :: mwork(0:1), fwork(0:1)
        real(dp) :: jwork

        real(dp) :: mhourshMw     ! conditional hours
        real(dp) :: mhourshMwsd
        real(dp) :: fhourshFw
        real(dp) :: fhourshFwsd

        real(dp) :: mhoursh       ! unconditional hours
        real(dp) :: mhourshsd
        real(dp) :: fhoursh
        real(dp) :: fhourshsd

        ! male wage distribution
        real(dp) :: mwage(0:2)
        real(dp) :: mwagesd(0:2)
        real(dp) :: mwageMw
        real(dp) :: mwagesdMw
        real(dp) :: mwageJw
        real(dp) :: mwagesdJw

        ! female wage distribution
        real(dp) :: fwage(0:2)
        real(dp) :: fwagesd(0:2)
        real(dp) :: fwageFw
        real(dp) :: fwagesdFw
        real(dp) :: fwageJw
        real(dp) :: fwagesdJw

        ! male wage distribution by spousal employment state
        real(dp) :: mwageFw(0:1)
        real(dp) :: mwageFwsd(0:1)

        ! female wage distribution by spousal employment state
        real(dp) :: fwageMw(0:1)
        real(dp) :: fwageMwsd(0:1)

        ! product of wages within a match
        real(dp) :: jwage
        real(dp) :: jwagesd
        real(dp) :: jwagecov

        ! **** ATUS ****

        ! marginal distribution of home production time
        real(dp) :: mhome(NHH)
        real(dp) :: fhome(NHH)

        real(dp) :: mhomeh, mhomehsd
        real(dp) :: fhomeh, fhomehsd

        ! male home production hours by own employment status
        real(dp) :: mhomehMw(0:1)
        real(dp) :: mhomehMwsd(0:1)

        ! male home production hours by own hours
        real(dp) :: mhomehMh(0:2)
        real(dp) :: mhomehMhsd(0:2)

        ! male home production by female employment status
        real(dp) :: mhomehFw(0:1)
        real(dp) :: mhomehFwsd(0:1)

        ! male home production by female hours
        real(dp) :: mhomehFh(0:2)
        real(dp) :: mhomehFhsd(0:2)

        ! female home production hours by own employment status
        real(dp) :: fhomehFw(0:1)
        real(dp) :: fhomehFwsd(0:1)

        ! female home production hours by own hours
        real(dp) :: fhomehFh(0:2)
        real(dp) :: fhomehFhsd(0:2)

        ! female home production by male employment status
        real(dp) :: fhomehMw(0:1)
        real(dp) :: fhomehMwsd(0:1)

        ! female home production by male hours
        real(dp) :: fhomehMh(0:2)
        real(dp) :: fhomehMhsd(0:2)

        ! earnings distribution
        real(dp) :: mearnMw, mearnMwsd
        real(dp) :: mearn, mearnsd
        real(dp) :: fearnFw, fearnFwsd
        real(dp) :: fearn, fearnsd

    end type eall2_t


    type rnd_t
        real(dp) :: ev(NDIM, NR)
        real(dp) :: evTr(NR, NDIM)
        real(dp) :: ev2(NDIM, NR)
        real(dp) :: ev2Tr(NR, NDIM)
    end type rnd_t

    ! household production technology

    type h_t

        real(dp) :: A     ! scale for public good
        real(dp) :: nu    ! economies of scale for public good
        real(dp) :: match ! additive match specific utility
        real(dp) :: rho   ! wage correlation
        real(dp) :: jointWageDensity(NODEW, NODEW) ! joint density function
        real(dp) :: wageTrapzWeightM(NODEW, NODEW)
        real(dp) :: probTypeK
        real(dp) :: a1, a2, a3
        integer  :: taxmode
        real(dp) :: coefM
        real(dp) :: coefF
        real(dp) :: fckid1
        real(dp) :: fckid2
    end type h_t

    type mom_t

        sequence

        ! **** IPUMS ****

        ! type distribution
        real(dp) :: Rjtype_M0(NTYPE1, NMARKET)
        real(dp) :: Rjtype_0F(NTYPE2, NMARKET)
        real(dp) :: Rjtype_MF(NTYPE1, NTYPE2, NMARKET)

        real(dp) :: Rmhours_M0(NTYPE1, NMARKET)
        real(dp) :: Rfhours_0F(NTYPE2, NMARKET)
        real(dp) :: Rmhours_Mf(NTYPE1, NMARKET)
        real(dp) :: Rfhours_mF(NTYPE2, NMARKET)

        real(dp) :: Rmwork_M0(NTYPE1, NMARKET)
        real(dp) :: Rfwork_0F(NTYPE2, NMARKET)
        real(dp) :: Rmwork_Mf(NTYPE1, NMARKET)
        real(dp) :: Rfwork_mF(NTYPE2, NMARKET)

        ! joint labour supply allocation (not-working/pt/ft)
        real(dp) :: mhours_M0(0:2, NTYPE1, 0:1)
        real(dp) :: fhours_0F(0:2, NTYPE2, 0:1)
        real(dp) :: mhours_MF(0:2, NTYPE1, NTYPE2, 0:1)
        real(dp) :: fhours_MF(0:2, NTYPE1, NTYPE2, 0:1)

        ! labour supply
        real(dp) :: mhoursh_M0(NTYPE1, 0:1)
        real(dp) :: mhourshsd_M0(NTYPE1, 0:1)
        real(dp) :: fhoursh_0F(NTYPE2, 0:1)
        real(dp) :: fhourshsd_0F(NTYPE2, 0:1)
        real(dp) :: mhoursh_Mf(NTYPE1, 0:1)
        real(dp) :: mhourshsd_Mf(NTYPE1, 0:1)
        real(dp) :: fhoursh_mF(NTYPE2, 0:1)
        real(dp) :: fhourshsd_mF(NTYPE2, 0:1)

        real(dp) :: mhoursh_M_F(NTYPE1, NTYPE2)
        real(dp) :: fhoursh_M_F(NTYPE1, NTYPE2)

        ! (wage distibutions not conditional on children)

        ! wage distribution (single men)
        real(dp) :: mwageMwAll_M0(NTYPE1)
        real(dp) :: mwagesdMwAll_M0(NTYPE1)

        ! wage distribution (single women)
        real(dp) :: fwageFwAll_0F(NTYPE2)
        real(dp) :: fwagesdFwAll_0F(NTYPE2)

        ! wage distribution (married men, integrates over spouse type/state)
        real(dp) :: mwageMwAll_Mf(NTYPE1)
        real(dp) :: mwagesdMwAll_Mf(NTYPE1)

        ! wage distribution (married women, integrates over spouse type/state)
        real(dp) :: fwageFwAll_mF(NTYPE2)
        real(dp) :: fwagesdFwAll_mF(NTYPE2)

        real(dp) :: mwork_M_F(NTYPE1, NTYPE2)
        real(dp) :: fwork_M_F(NTYPE1, NTYPE2)

        ! earnings
        real(dp) :: mearnMwAll_M0(NTYPE1)
        real(dp) :: mearnsdMwAll_M0(NTYPE1)

        real(dp) :: mearnMwAll_Mf(NTYPE1)
        real(dp) :: mearnsdMwAll_Mf(NTYPE1)

        real(dp) :: fearnFwAll_0F(NTYPE2)
        real(dp) :: fearnsdFwAll_0F(NTYPE2)

        real(dp) :: fearnFwAll_mF(NTYPE2)
        real(dp) :: fearnsdFwAll_mF(NTYPE2)


        ! **** ATUS ****

        ! marginal distribution of home production time (single men/women)
        real(dp) :: mhome_M0(NHH, NTYPE1, 0:1)
        real(dp) :: fhome_0F(NHH, NTYPE2, 0:1)

        ! marginal distribution of home production time (married men/women)
        real(dp) :: mhome_Mf(NHH, NTYPE1, 0:1)
        real(dp) :: fhome_mF(NHH, NTYPE2, 0:1)

        ! home production hours (single men/women)
        real(dp) :: mhomeh_M0(NTYPE1, 0:1)
        real(dp) :: mhomehsd_M0(NTYPE1, 0:1)
        real(dp) :: fhomeh_0F(NTYPE2, 0:1)
        real(dp) :: fhomehsd_0F(NTYPE2, 0:1)

        ! home production hours (married men/women)
        real(dp) :: mhomeh_Mf(NTYPE1, 0:1)
        real(dp) :: mhomehsd_Mf(NTYPE1, 0:1)
        real(dp) :: fhomeh_mF(NTYPE2, 0:1)
        real(dp) :: fhomehsd_mF(NTYPE2, 0:1)

        real(dp) :: mhomeh_M_F(NTYPE1, NTYPE2)
        real(dp) :: fhomeh_M_F(NTYPE1, NTYPE2)

    end type mom_t

    type momExtra_t

        sequence

        real(dp) :: jtype_M0(NTYPE1)
        real(dp) :: jtype_0F(NTYPE2)
        real(dp) :: jtype_MF(NTYPE1,NTYPE2)

        ! unconditional hours (single/married men)
        real(dp) :: mhoursuh_M0(NTYPE1, 0:1)
        real(dp) :: mhoursuhsd_M0(NTYPE1, 0:1)
        real(dp) :: mhoursuh_Mf(NTYPE1, 0:1)
        real(dp) :: mhoursuhsd_Mf(NTYPE1, 0:1)
        real(dp) :: mhoursuh_M_F(NTYPE1, NTYPE2, 0:1)
        real(dp) :: mhoursuhsd_M_F(NTYPE1, NTYPE2, 0:1)

        ! unconditional hours (single/married women)
        real(dp) :: fhoursuh_0F(NTYPE2, 0:1)
        real(dp) :: fhoursuhsd_0F(NTYPE2, 0:1)
        real(dp) :: fhoursuh_mF(NTYPE2, 0:1)
        real(dp) :: fhoursuhsd_mF(NTYPE2, 0:1)
        real(dp) :: fhoursuh_M_F(NTYPE1, NTYPE2, 0:1)
        real(dp) :: fhoursuhsd_M_F(NTYPE1, NTYPE2, 0:1)

        ! wage distribution (single men)
        real(dp) :: mwageMw_M0(NTYPE1, 0:1)
        real(dp) :: mwagesdMw_M0(NTYPE1, 0:1)

        ! wage distribution (single women)
        real(dp) :: fwageFw_0F(NTYPE2, 0:1)
        real(dp) :: fwagesdFw_0F(NTYPE2, 0:1)

        ! wage distribution (married men,  integrates over spouse type/state)
        real(dp) :: mwageMw_Mf(NTYPE1, 0:1)
        real(dp) :: mwagesdMw_Mf(NTYPE1, 0:1)

        ! wage distribution (married women,  integrates over spouse type/state)
        real(dp) :: fwageFw_mF(NTYPE2, 0:1)
        real(dp) :: fwagesdFw_mF(NTYPE2, 0:1)

        ! product of wages within a match
        real(dp) :: jwage_MF(NTYPE1, NTYPE2, 0:1)
        real(dp) :: jwagesd_MF(NTYPE1, NTYPE2, 0:1)

         ! home production hours (married men/women)
        real(dp) :: mhomeh_M_F(NTYPE1, NTYPE2, 0:1)
        real(dp) :: mhomehsd_M_F(NTYPE1, NTYPE2, 0:1)
        real(dp) :: fhomeh_M_F(NTYPE1, NTYPE2, 0:1)
        real(dp) :: fhomehsd_M_F(NTYPE1, NTYPE2, 0:1)

        ! joint labour supply allocation (not-working/pt/ft)
        real(dp) :: mhours_Mf(0:2, NTYPE1, 0:1)
        real(dp) :: fhours_mF(0:2, NTYPE2, 0:1)
        real(dp) :: jhours_MF(0:2, 0:2, NTYPE1, NTYPE2, 0:1)

        ! wage distribution (single men)
        real(dp) :: mwage_M0(1:2, NTYPE1, 0:1)
        real(dp) :: mwagesd_M0(1:2, NTYPE1, 0:1)

        ! wage distribution (single women)
        real(dp) :: fwage_0F(1:2, NTYPE2, 0:1)
        real(dp) :: fwagesd_0F(1:2, NTYPE2, 0:1)

        ! wage distribution (married men,  integrates over spouse type/state)
        real(dp) :: mwage_Mf(1:2, NTYPE1, 0:1)
        real(dp) :: mwagesd_Mf(1:2, NTYPE1, 0:1)

        ! wage distribution (married women,  integrates over spouse type/state)
        real(dp) :: fwage_mF(1:2, NTYPE2, 0:1)
        real(dp) :: fwagesd_mF(1:2, NTYPE2, 0:1)

        ! wage distribution (married men,  by spousal employment state)
        real(dp) :: mwageFw_Mf(0:1, NTYPE1, 0:1)
        real(dp) :: mwageFwsd_Mf(0:1, NTYPE1, 0:1)

        ! wage distribution (married women,  by spousal employment state)
        real(dp) :: fwageMw_mF(0:1, NTYPE2, 0:1)
        real(dp) :: fwageMwsd_mF(0:1, NTYPE2, 0:1)

        ! product of wages within a match
        real(dp) :: jwagecov_MF(NTYPE1, NTYPE2, 0:1)

        ! home production hours by own hours/employment (single men/women)
        real(dp) :: mhomehMw_m0(0:1, 0:1)
        real(dp) :: mhomehMwsd_m0(0:1, 0:1)
        real(dp) :: fhomehFh_0f(0:2, 0:1)
        real(dp) :: fhomehFhsd_0f(0:2, 0:1)

        ! home production hours by own hours/employment (married men/women)
        real(dp) :: mhomehMw_mf(0:1, 0:1)
        real(dp) :: mhomehMwsd_mf(0:1, 0:1)
        real(dp) :: fhomehFh_mf(0:2, 0:1)
        real(dp) :: fhomehFhsd_mf(0:2, 0:1)

        ! home production hours by spousal hours/employment (married men/women)
        real(dp) :: mhomehFw_mf(0:1, 0:1)
        real(dp) :: mhomehFwsd_mf(0:1, 0:1)
        real(dp) :: fhomehMw_mf(0:1, 0:1)
        real(dp) :: fhomehMwsd_mf(0:1, 0:1)

        ! aggregated from MOMFULL
        real(dp) :: mhoursh_M_0(NTYPE1)
        real(dp) :: fhoursh_0_F(NTYPE2)
        real(dp) :: mwork_M_0(NTYPE1)
        real(dp) :: fwork_0_F(NTYPE2)
        real(dp) :: mhomeh_M_0(NTYPE1)
        real(dp) :: fhomeh_0_F(NTYPE2)

        ! aggregated measures for elasticities
        real(dp) :: mhoursuh_sin(0:1)
        real(dp) :: fhoursuh_sin(0:1)
        real(dp) :: mhoursuh_cou(0:1)
        real(dp) :: fhoursuh_cou(0:1)

        real(dp) :: mhoursh_sin(0:1)
        real(dp) :: fhoursh_sin(0:1)
        real(dp) :: mhoursh_cou(0:1)
        real(dp) :: fhoursh_cou(0:1)

        real(dp) :: mwork_sin(0:1)
        real(dp) :: fwork_sin(0:1)
        real(dp) :: mwork_cou(0:1)
        real(dp) :: fwork_cou(0:1)

        real(dp) :: mhomeuh_sin(0:1)
        real(dp) :: fhomeuh_sin(0:1)
        real(dp) :: mhomeuh_cou(0:1)
        real(dp) :: fhomeuh_cou(0:1)

        ! aggregated labour supply integrating over own and spousal type
        real(dp) :: mhours_sin(0:2, 0:1)
        real(dp) :: fhours_sin(0:2, 0:1)
        real(dp) :: mhours_cou(0:2, 0:1)
        real(dp) :: fhours_cou(0:2, 0:1)

        ! aggregated home time integrating over own and spousal type
        real(dp) :: mhome_sin(NHH, 0:1)
        real(dp) :: fhome_sin(NHH, 0:1)
        real(dp) :: mhome_cou(NHH, 0:1)
        real(dp) :: fhome_cou(NHH, 0:1)

        real(dp) :: mwork_cou_knk
        real(dp) :: mhoursh_cou_knk
        real(dp) :: mhourshMw_cou_knk
        real(dp) :: mhomeh_cou_knk
        real(dp) :: mearnMw_cou_knk

        real(dp) :: fwork_cou_knk
        real(dp) :: fhoursh_cou_knk
        real(dp) :: fhourshFw_cou_knk
        real(dp) :: fhomeh_cou_knk
        real(dp) :: fearnFw_cou_knk

        real(dp) :: mwork_sin_knk
        real(dp) :: mhoursh_sin_knk
        real(dp) :: mhourshMw_sin_knk
        real(dp) :: mhomeh_sin_knk
        real(dp) :: mearnMw_sin_knk

        real(dp) :: fwork_sin_knk
        real(dp) :: fhoursh_sin_knk
        real(dp) :: fhourshFw_sin_knk
        real(dp) :: fhomeh_sin_knk
        real(dp) :: fearnFw_sin_knk

    end type momExtra_t

    type(mom_t), private :: momSizeTemp
    type(momExtra_t), private :: momExtraSizeTemp
    integer, parameter :: NMOM = storage_size( momSizeTemp ) / storage_size( 1.0_dp )
    integer, parameter :: NMOMEXTRA = storage_size( momExtraSizeTemp ) / storage_size( 1.0_dp )

    type ret_t
        real(dp) :: dist
        real(dp) :: swf
        real(dp) :: rev
        real(dp) :: eqcon(NTYPEALL)
        real(dp) :: eqconMarket(NTYPEALL * NMARKET)
        real(dp) :: momVec(NMOM)
        real(dp) :: momExtraVec(NMOMEXTRA)
        real(dp) :: timer
        real(dp) :: marriage(NTYPE1 + NTYPE2 + NTYPEALL)
        real(dp) :: marriageMatrix(0:NTYPE1, 0:NTYPE2)
        real(dp) :: marriageMarket((NTYPE1 + NTYPE2 + NTYPEALL) * NMARKET)
        real(dp) :: Eu1(0:NTYPE1, 0:NTYPE2)
        real(dp) :: Eu2(0:NTYPE1, 0:NTYPE2)
        real(dp) :: lam(NTYPE1, NTYPE2)
        real(dp), allocatable :: EuLamGrid1(:, :, :)
        real(dp), allocatable :: EuLamGrid2(:, :, :)
        real(dp), allocatable :: lamgrid(:)
        real(dp) :: demand(NTYPE1, NTYPE2)
        real(dp) :: supply(NTYPE1, NTYPE2)
    end type ret_t

contains

    subroutine p_derived(p)
        implicit none
        type(p_t), intent(inout) :: p
        p%logBetl = log(p%betl)
        p%logBetc = log(p%betc)
    end subroutine p_derived

    subroutine setOptions(opt, mode, estmode, perturbmode)
        implicit none
        type(opt_t), intent(inout) :: opt
        integer, intent(in) :: mode
        integer, intent(in), optional :: estmode
        integer, intent(in), optional :: perturbmode
        opt%logwf_mu = 0.0_dp
        opt%logwf_sd = 0.0_dp
        opt%pMscale = 0.0_dp
        opt%pFscale = 0.0_dp
        opt%correlation = 0.0_dp
        opt%scaleA = 1.0_dp
        opt%scaleAh = 1.0_dp
        opt%utilityShift = 0.0_dp
        opt%forceAssortativeScale = 0.0_dp
        opt%swfInc = .false.
        opt%verboseExtra = .false.
        opt%simMom = .false.
        opt%fixMarriageMarket = .false.
        opt%wageElasticity1 = .false.
        opt%wageElasticity2 = .false.
        opt%taxins = TAX_JOINT
        opt%vunit = IOUT
        opt%marriageMarketElasticity = .false.
        opt%tag = ''
        opt%saveUpf = .false.
        opt%nlam = 9
        opt%earnElasticity = .false.
        opt%earnElasticityLower = .false.
        opt%medicaid = .false.
#       ifdef _NTYPE4_
        opt%locname = 'ed4_age2545'
#       endif
#       ifdef _NTYPE3_
        opt%locname = 'ed3_age2545'
#       endif
        select case(mode)
        case(OPT_MAIN)
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%verbose = .false.
            opt%extraMom = .false.
            opt%saveMom = .false.
        case(OPT_WAGESCALE)
            opt%wageElasticity = .true.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%szWageElasticity = 1.00_dp + ELAS_STEP
            opt%szWageElasticity1 = 1.00_dp + ELAS_STEP
            opt%szWageElasticity2 = 1.00_dp + ELAS_STEP
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
        case(OPT_WAGESCALE1)
            opt%wageElasticity = .true.
            opt%wageElasticity1 = .true.
            opt%wageElasticity2 = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%szWageElasticity  = 1.00_dp + ELAS_STEP
            opt%szWageElasticity1 = 1.00_dp + ELAS_STEP
            opt%szWageElasticity2 = 1.00_dp
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
        case(OPT_WAGESCALE2)
            opt%wageElasticity = .true.
            opt%wageElasticity1 = .false.
            opt%wageElasticity2 = .true.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%szWageElasticity  = 1.00_dp + ELAS_STEP
            opt%szWageElasticity1 = 1.00_dp
            opt%szWageElasticity2 = 1.00_dp + ELAS_STEP
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
        case(OPT_HOMESCALE)
            opt%wageElasticity = .false.
            opt%homeElasticity = .true.
            opt%incomeElasticity = .false.
            opt%szHomeElasticity  = 1.00_dp + ELAS_STEP
            opt%szHomeElasticity1 = 1.00_dp + ELAS_STEP
            opt%szHomeElasticity2 = 1.00_dp + ELAS_STEP
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
        case(OPT_HOMESCALE1)
            opt%wageElasticity = .false.
            opt%homeElasticity = .true.
            opt%incomeElasticity = .false.
            opt%szHomeElasticity  = 1.00_dp + ELAS_STEP
            opt%szHomeElasticity1 = 1.00_dp + ELAS_STEP
            opt%szHomeElasticity2 = 1.00_dp
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
        case(OPT_HOMESCALE2)
            opt%wageElasticity = .false.
            opt%homeElasticity = .true.
            opt%incomeElasticity = .false.
            opt%szHomeElasticity  = 1.00_dp + ELAS_STEP
            opt%szHomeElasticity1 = 1.00_dp
            opt%szHomeElasticity2 = 1.00_dp + ELAS_STEP
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
        case(OPT_ESTIMATION)
            opt%mode = MODE_MOMENTS
            opt%xmode = X_TRANSFORM
            opt%tmode = TAXMODE_DATA
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%verbose = .false.
            opt%extraMom = .false.
            opt%saveMom = .false.
        case(OPT_NLYSCALE)
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .true.
            opt%szIncomeElasticity  = 1.00_dp + ELAS_STEP
            opt%szIncomeElasticity1 = 1.00_dp + ELAS_STEP
            opt%szIncomeElasticity2 = 1.00_dp + ELAS_STEP
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
         case(OPT_NLYSCALE1)
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .true.
            opt%szIncomeElasticity  = 1.00_dp + ELAS_STEP
            opt%szIncomeElasticity1 = 1.00_dp + ELAS_STEP
            opt%szIncomeElasticity2 = 1.00_dp
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
        case(OPT_NLYSCALE2)
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .true.
            opt%szIncomeElasticity  = 1.00_dp + ELAS_STEP
            opt%szIncomeElasticity1 = 1.00_dp
            opt%szIncomeElasticity2 = 1.00_dp + ELAS_STEP
            opt%verbose = .false.
            opt%extraMom = .true.
            opt%saveMom = .false.
        case(OPT_MODELFIT)
            opt%mode = MODE_MOMENTS
            opt%xmode = X_TRANSFORM
            opt%tmode = TAXMODE_DATA
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%verbose = .true.
            opt%extraMom = .false.
            opt%saveMom = .false.
        case(OPT_MODELFITSAVE)
            opt%mode = MODE_MOMENTS
            opt%xmode = X_TRANSFORM
            opt%tmode = TAXMODE_DATA
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%verbose = .true.
            opt%extraMom = .true.
            opt%saveMom = .true.
            opt%savePath = 'output/moments2/'
        case(OPT_SIMULATION)
            opt%mode = MODE_WELFARE
            opt%xmode = X_TRANSFORM
            opt%tmode = TAXMODE_SIMU
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%verbose = .false.
            opt%extraMom = .false.
            opt%saveMom = .false.
        case(OPT_SIMULATIONFIT)
            opt%mode = MODE_MOMENTS
            opt%xmode = X_TRANSFORM
            opt%tmode = TAXMODE_SIMU
            opt%wageElasticity = .false.
            opt%homeElasticity = .false.
            opt%incomeElasticity = .false.
            opt%verbose = .true.
            opt%extraMom = .false.
            opt%saveMom = .false.
            opt%simMom = .true.
        case default
            stop 'unknown options mode'
        end select

        if (present(estmode)) then
            select case(estmode)
            case(statemode)
                opt%locname = 'ed3_age2545_state'
            case(mcdmode)
                opt%medicaid = .true.
            case default
                ! ...
            end select
        end if

        if (present(perturbmode)) then
            select case(perturbmode)
            case(0)
                ! ...
            case(7)
                ! home production
                opt%scaleA = 0.0_dp
                opt%scaleAh = 0.0_dp
            case(21)
                ! sorting
                opt%utilityShift = -1.0_dp
            case(37)
                ! wage gap
                opt%logwf_mu = -0.50_dp
            case default
                stop 'unknown perturbmode'
            end select
        end if

    end subroutine setOptions

#   ifdef __INTEL_COMPILER
#       if __INTEL_COMPILER < 1500
            subroutine execute_command_line( cmd, wait )
                character(len = *), intent(in) :: cmd
                logical, intent(in) :: wait
                call system( cmd )
            end subroutine execute_command_line
#       endif
#   endif

end module globaldef
