program mainkid

    use globaldef
    use moments
    use environment
    use test3kid
    use taxes
!$  use omp_lib
    implicit none

    type(opt_t) :: opt
    type(ret_t) :: ret

    integer, parameter :: nall = NPARAM + NTYPEALL * NMARKET
    real(dp) :: x(NPARAM + NTYPEALL * NMARKET)
    real(dp) :: xparam(NPARAM)
    real(dp) :: xtax(NTAXPARAM)
    real(dp) :: xlamMarket(NTYPEALL * NMARKET)
    integer :: iX

    integer :: narg
    integer :: funit
    integer :: istat
    integer :: estmode
    character(len = 32) :: filetag, thisarg
    character(len = 128) :: fpath
    character(len = 128) :: fnamesol
    character(len = 128) :: fnamelist
    character(len = 128) :: label(NMOM)

    narg = command_argument_count()
    if (narg == 0) then
        filetag = ''
        fpath = 'output/post/'
        fnamesol = 'output/estimation/xsol_market_mpec_mpi.asc'
        fnamelist = 'output/estimation/variance/momlist.csv'
        estmode = 0
    else if (narg == 1) then
        call get_command_argument(1, value = filetag, status = istat)
        fpath = 'output/post/' // trim(adjustl(filetag)) // '/'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        fnamelist = 'output/estimation/' // trim(adjustl(filetag)) // '/variance/momlist.csv'
        estmode = 0
    else if (narg == 2) then
        call get_command_argument(1, value = filetag, status = istat)
        fpath = 'output/post/' // trim(adjustl(filetag)) // '/'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        fnamelist = 'output/estimation/' // trim(adjustl(filetag)) // '/variance/momlist.csv'
        call get_command_argument(2, value = thisarg, status = istat)
        read(thisarg, '(i10)') estmode
    else
        stop 'more than two command argument supplied'
    end if

    select case(estmode)
    case(statemode)
        print *, 'estmode = statemode'
        print *
    case(mcdmode)
        print *, 'estmode = mcdmode'
        print *
    case default
    end select

    ! read x input
    open(newunit = funit, file = fnamesol, status = 'old', action = 'read')
    do iX = 1, NPARAM + NTYPEALL * NMARKET
        read(funit, *) x(iX)
    end do
    close(funit)

    xparam = x(1:NPARAM)
    call printxlab(xparam)

    xlamMarket = x(NPARAM + 1:)

    call setOptions(opt, OPT_MODELFITSAVE, estmode)
    opt%savePath = trim(adjustl(fpath))
    opt%verbose = .true.
    opt%tag = trim(adjustl(filetag))

    call execute_command_line('mkdir -p dir "output/post/' // trim(adjustl(filetag)) //'"', wait = .true.)
    call execute_command_line('mkdir -p dir "output/estimation/' // trim(adjustl(filetag)) // '/variance/"', wait = .true.)

    call labelMoments(label)
    open(newunit = funit, file = fnamelist, status = 'replace', action = 'write')
    do iX = 1, NMOM
        write(funit, '(A)') trim(adjustl(label(iX)))
    end do
    close(funit)

    call printxlab(xparam)
    call printHomeProductivity(xparam)

    call objfun_mpec_market(xparam, xlamMarket, xtax, opt, ret)

    do iX = 1, NMARKET * NTYPEALL
        print *, xlamMarket(iX)
    end do

    print *, ret%dist, log(ret%dist), ret%rev
    call saveRevenue(ret%rev, trim(adjustl(fpath)))

    print *,'mm elas'
    call elasticityMarriage(xparam, xtax, estmode, verbose = .true., fpath = trim(adjustl(fpath)))

    print *,'ls elas'
    call elasticityMarket(xparam, xtax, estmode, texpath = trim(adjustl(fpath)), outpath = '', ftag = -1)

    call setOptions(opt, OPT_ESTIMATION, estmode)
    opt%saveUpf = .true.
    opt%savePath = trim(adjustl(fpath))
    opt%nlam = 19

    call objfun_market(xparam, xlamMarket, xtax, opt, ret)

end program mainkid
