
% figures_estimation.m: this creates all the figures and tables for the simulation stage

addpath('utility')

set(0, 'defaultTextInterpreter', 'none');

settings.ftag = '600';
settings.mode = 1;
settings.factor = 52 / 1000;
settings.theta = [0 10];
settings.phi = (1 + sqrt(5)) / 2;
settings.fontsize = 8;
settings.fontsize_sml = 7;
settings.width = 12;
settings.width_sml = 12 * 0.7;
settings.height = settings.width / settings.phi;
settings.height_sml = settings.width_sml / settings.phi;
settings.froot = '../output/simulation/';
settings.fpath = [settings.froot, settings.ftag, '/'];
settings.figurepath = ['./figures/', settings.ftag, '/'];
settings.tablepath = ['./tables/', settings.ftag, '/'];
settings.ed = 3;
settings.nsamples = 200;
settings.visible = 'off';
settings.singlemtr = false;

if (~exist(['./figures/', settings.ftag], 'dir'))
    mkdir('./figures/', settings.ftag);
end
if (~exist(['./tables/', settings.ftag], 'dir'))
    mkdir('./tables/', settings.ftag);
end

% Figure 5a, 6a: 3d optimal income schedule for alternative delta
figureMain(settings)

% Figure 5b, 6b: Optimal marginal rate structure with pointwise CI for alternative delta
figureMainCI(settings)

% Table 4: Marginal tax rates with restricted tax instruments
tableRestricted2(settings)

% Table 5: Marriage matching functions and weights with gendered taxation
tableRestrictedMmfGender(settings)

% Table 6: Welfare cost and marriage market importance
tableRestrictedWelfare(settings)

% Table 8 (Appendix): Social welfare weight for alternative delta
tableMainSwfWeight(settings)

% Table 9 (Appendix): Marriage matching functions with alternative tax instruments
tableRestrictedMmf(settings)

% Figure 7: Marginal tax rate with fixed marriage market
figureFixedNew(settings)

% Figure 8 (Appendix): Empirical marginal tax rates
figureEmpiricalTax(settings)

% Figure 9 (Appendix): Optimal marginal rate schedule with gendered taxation
figureMainGBT(settings)

% Figure 10 (Appendix): Optimal net-income schedule for single individuals
figureMainSingleGBT(settings)

% Figure 11 (Appendix): Optimal tax schedule with different number of nodes
figureFineGrid(settings)

% Figure 12 (Appendix): Optimal marginal rate schedule with perturbations
figurePerturbation(settings)
