function [hfig, ax1, hl] = couplesTaxConditionalNewAvg(tax, settings, ad, xlab)

xtickh = tax.xtickh2d;
ytickh = tax.ytickh2d;
NZ = 3;

cc = linspecer(NZ + 1);
cc = cc(2:4, :);

bgedge = [0.7 0.7 0.7];
gridstyle = '--';
gridstylen = '-';

mtrylim = [tax.ytickh2d(1) tax.ytickh2d(end)];

C = cell(NZ, 1);

hfig = figure('Visible', settings.visible);
hold all

for iX = 2:length(xtickh) - 1
    line( [xtickh(iX), xtickh(iX)], [ytickh(1) + eps, ytickh(end)], ...
        'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle )
end
line( [xtickh(end), xtickh(end)], [ytickh(1) + eps, ytickh(end)], ...
    'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstylen )


for iY = 2:length(ytickh) - 1
    line( [xtickh(1) + eps, xtickh(end)], [ytickh(iY), ytickh(iY)], ...
        'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle )
end
line( [xtickh(1) + eps, xtickh(end)], [ytickh(end), ytickh(end)], ...
    'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstylen )

% spouse earnings
mtr = zeros(tax.NTAXBR - 1, tax.NTAXBR);

for iZ = 1:tax.NTAXBR

    % brackets
    for ixBr = 1:tax.NTAXBR - 1
        if (ad == 1)
            y0 = evalTaxCouScalar( tax, tax.TAXBR(ixBr), tax.TAXBR(iZ) );
            y1 = evalTaxCouScalar( tax, 0.5 * (tax.TAXBR(ixBr + 1) + tax.TAXBR(ixBr)), tax.TAXBR(iZ) );
        else
            y0 = evalTaxCouScalar( tax, tax.TAXBR(iZ), tax.TAXBR(ixBr) );
            y1 = evalTaxCouScalar( tax, tax.TAXBR(iZ), 0.5 * (tax.TAXBR(ixBr + 1) + tax.TAXBR(ixBr)) );
        end
        thisMtr = (y1 - y0) / ( 0.5 * (tax.TAXBR(ixBr + 1) - tax.TAXBR(ixBr)) );
        % beautification: prevents visual change in schedule at the boundary
        if ( tax.TAXBRS(ixBr) > xtickh(end) - 1 )
            mtr(ixBr, iZ) = mtr(ixBr - 1, iZ);
        else
            mtr(ixBr, iZ) = thisMtr;
        end
    end

end


if (tax.NTAXBR == 10)
    mtrAvg(:,1) = ( mtr(:, 1) + mtr(:, 2) + mtr(:, 3) ) / 3;
    mtrAvg(:,2) = ( mtr(:, 4) + mtr(:, 5) + mtr(:, 6) ) / 3;
    mtrAvg(:,3) = ( mtr(:, 7) + mtr(:, 8) + mtr(:, 9) ) / 3;
elseif (tax.NTAXBR == 18)
    mtrAvg(:,1) = ( mtr(:, 1)  + mtr(:, 2)  + mtr(:, 3)  + mtr(:, 4)  + mtr(:, 5) ) / 5;
    mtrAvg(:,2) = ( mtr(:, 6)  + mtr(:, 7)  + mtr(:, 8)  + mtr(:, 9)  + mtr(:, 10) + mtr(:, 11) ) / 6;
    mtrAvg(:,3) = ( mtr(:, 12) + mtr(:, 13) + mtr(:, 14) + mtr(:, 15) + mtr(:, 16) + mtr(:, 17) ) / 6;
else
    % ...
end

xx = [tax.TAXBRS(1) repelem(tax.TAXBRS(2:tax.NTAXBR - 1), 1, 2) tax.TAXBRS(end)];

p = zeros(NZ, 1);
for ixZ = 1:NZ
    p(ixZ) = plot(xx, repelem(mtrAvg(:, ixZ), 2, 1), 'LineWidth', 1, 'color', cc(ixZ, :), 'LineStyle', '-');
end

C{1} = 'Low spousal earnings';
C{2} = 'Med spousal earnings';
C{3} = 'High spousal earnings';

ylim(mtrylim)
xlim([0, xtickh(end)]);

if (nargin < 4)
    xlabel('$z$')
else
    xlabel(xlab)
    if (ad == 1)
        C{1} = 'Low female earnings';
        C{2} = 'Med female earnings';
        C{3} = 'High female earnings';
    elseif (ad == 2)
        C{1} = 'Low male earnings';
        C{2} = 'Med male earnings';
        C{3} = 'High male earnings';
    else
        % ...
    end
end

[hl.leg, hl.obj, hl.hout, hl.mout] = legendflex(p, C, 'box', 'off', 'Anchor', [5 5], 'Buffer', [-10  10], ...
    'FontSize', 7);

set(gca, 'XTick', xtickh)
set(gca, 'YTick', ytickh)
set(gca, 'Layer', 'Top')
formatTicks(gca, 'y');
ax1 = gca;

hold off

end
