function [hfig, ax1] = couplesTaxConditionalNewAvgCI(tax, taxs, settings, nsamplesExist)

xtickh = tax.xtickh2d;
ytickh = tax.ytickh2d;

N = 10000;
NZ = 3;

earn = linspace(0, tax.TAXBR(end), N)';
dearn = [earn(1:end - 2); earn(end)];

cc = linspecer(NZ + 1);
ccSingle = cc(1, :);
cc = cc(2:4, :);

bgedge=[0.7 0.7 0.7];
gridstyle = '--';
gridstylen = '-';

mtrylim = [tax.ytickh2d(1) tax.ytickh2d(end)];

y = zeros(N, 1);
C = cell(NZ, 1);

hfig = figure('Visible', settings.visible);
hold all

for iX = 2:length(xtickh) - 1
    line( [xtickh(iX), xtickh(iX)], [ytickh(1) + eps, ytickh(end)], ...
        'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle )
end
line( [xtickh(end), xtickh(end)], [ytickh(1) + eps, ytickh(end)], ...
    'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstylen )

for iY = 2:length(ytickh) - 1
    line( [xtickh(1) + eps, xtickh(end)], [ytickh(iY), ytickh(iY)], ...
        'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle )
end
line( [xtickh(1) + eps, xtickh(end)], [ytickh(end), ytickh(end)], ...
    'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstylen )

z0 = [1; 4; 7];
z1 = [3; 6; 9];
mm = zeros(3, 1);

% spouse earnings
mtrL = zeros(tax.NTAXBR - 1, NZ);
mtrU = zeros(tax.NTAXBR - 1, NZ);

pd = 0.005;

for iZ = 1:NZ

    % brackets
    for ixBr = 1:tax.NTAXBR - 1
        thisMtr = zeros(nsamplesExist, 1);
        for ixS = 1:nsamplesExist
            for ixY = z0(iZ):z1(iZ)
                y0 = evalTaxCouScalar( taxs{ixS}, tax.TAXBR(ixBr), tax.TAXBR(ixY) );
                y1 = evalTaxCouScalar( taxs{ixS}, 0.5 * (tax.TAXBR(ixBr + 1) + tax.TAXBR(ixBr)), tax.TAXBR(ixY) );
                mm(ixY - z0(iZ) + 1) = (y1 - y0) / ( 0.5 * (tax.TAXBR(ixBr + 1) - tax.TAXBR(ixBr)) );
            end
            nm = z1(iZ) - z0(iZ) + 1;
            thisMtr(ixS) = mean(mm(1:nm));
        end

        % confidence bands
        if ( tax.TAXBRS(ixBr) > xtickh(end) - 1 )
            mtrL(ixBr, iZ) = mtrL(ixBr - 1, iZ);
            mtrU(ixBr, iZ) = mtrU(ixBr - 1, iZ);
        else
            pc = prctile(thisMtr, [0.5 99.5]);
            mtrL(ixBr, iZ) = pc(1) - pd;
            mtrU(ixBr, iZ) = pc(2) + pd;
        end

    end

    xx = [tax.TAXBRS(1) repelem(tax.TAXBRS(2:tax.NTAXBR - 1), 1, 2) tax.TAXBRS(end)];
    yL = repelem(mtrL(:, iZ), 2, 1);
    yU = repelem(mtrU(:, iZ), 2, 1);
    col = 0.3 * cc(iZ, :) + 0.7;
    plot( xx, yL, 'LineWidth', 1, 'color', col, 'LineStyle', '--');
    plot( xx, yU, 'LineWidth', 1, 'color', col, 'LineStyle', '--');

end

% this code only works if spouse earnings is fixed on one of the brackets

% spouse earnings
mtr = zeros(tax.NTAXBR - 1, NZ);
p = zeros(NZ, 1);

% brackets
for iZ = 1:NZ
    for ixBr = 1:tax.NTAXBR - 1
        for ixY = z0(iZ):z1(iZ)
            y0 = evalTaxCouScalar( tax, tax.TAXBR(ixBr), tax.TAXBR(ixY) );
            y1 = evalTaxCouScalar( tax, 0.5 * (tax.TAXBR(ixBr + 1) ...
                + tax.TAXBR(ixBr)), tax.TAXBR(ixY) );
            mm(ixY - z0(iZ) + 1) = (y1 - y0) / ( 0.5 * (tax.TAXBR(ixBr + 1) ...
                - tax.TAXBR(ixBr)) );
        end
        nm = z1(iZ) - z0(iZ) + 1;
        thisMtr = mean(mm(1:nm));
        mtr(ixBr, iZ) = thisMtr;
        if (tax.TAXBRS(ixBr) > xtickh(end) - 1)
            mtr(ixBr, iZ) = mtr(ixBr - 1, iZ);
        end
    end

    xx = [tax.TAXBRS(1) repelem(tax.TAXBRS(2:tax.NTAXBR - 1), 1, 2) tax.TAXBRS(end)];
    yL = repelem(mtr(:, iZ), 2, 1);
    p(iZ) = plot( xx, yL, 'LineWidth', 1, 'color', cc(iZ, :), 'LineStyle', '-');

end

C{1} = 'Low spousal earnings';
C{2} = 'Med spousal earnings';
C{3} = 'High spousal earnings';

ylim(mtrylim)
xlim([0, xtickh(end)]);
xlabel('$z_1$')
leg = legendflex(p, C, 'box', 'off', 'Anchor', [5 5], 'Buffer', [-10  10]);
set(gca,'XTick',xtickh)
set(gca, 'YTick', ytickh)
set(gca,'Layer', 'Top')
formatTicks(gca, 'y');
ax1 = gca;

hold off

end
