function [hfig, ax1, ax2] = couplesTaxSurfaceNew(tax, settings)

xtickh = tax.xtickh3d;
ytickh = tax.ytickh3d;
ztickh = tax.ztickh3d;

close all

screen = true;

if screen
    col1 = [0 0.5 0.4];
    col2 = [0.5 0.75 0.4];
    col2 = 0.5 * (col1 + col2);
    edge = 0.5 * col1;
    bg1 = [0.95 0.95 0.95];
    bg2 = [0.975 0.975 0.975];
    bgedge = [0.7 0.7 0.7];
    gridstyle = '-';
else
    col1 = [1 1 1];
    col2 = [1 1 1];
    edge = [0 0 0];
    bg1 = [1 1 1];
    bg2 = [1 1 1];
    bgedge = [0 0 0];
    gridstyle = ':';
end
hfig = figure('Visible', settings.visible);

az = -45;
el = 25;
view(az, el);

hold on;

x = [];
y = [];
z = [];

for ix = 1:length(xtickh) - 1
    for iy = 1:length(ytickh) - 1
        x = [x; xtickh(ix) xtickh(ix + 1) xtickh(ix + 1) xtickh(ix)];
        y = [y; ytickh(iy) ytickh(iy) ytickh(iy + 1) ytickh(iy + 1)];
        z = [z; 0 0 0 0];
    end
end
fill3(x', y', z', bg1, 'EdgeColor', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle);

x = [];
y = [];
z = [];

for ix = 1:length(xtickh) - 1
    for iz = 1:length(ztickh) - 1
        x = [x; xtickh(ix) xtickh(ix + 1) xtickh(ix + 1) xtickh(ix)];
        y = [y; ytickh(end) ytickh(end) ytickh(end) ytickh(end)];
        z = [z; ztickh(iz) ztickh(iz) ztickh(iz + 1) ztickh(iz + 1)];
    end
end

for iy = 1:length(ytickh) - 1
    for iz = 1:length(ztickh) - 1
        x = [x; xtickh(end) xtickh(end) xtickh(end) xtickh(end)];
        y = [y; ytickh(iy) ytickh(iy + 1) ytickh(iy + 1) ytickh(iy)];
        z = [z; ztickh(iz) ztickh(iz) ztickh(iz + 1) ztickh(iz + 1)];
    end
end

fill3(x', y', z', bg2, 'EdgeColor', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle);

er = 1 * tax.factor;

% new axis
ax1 = gca;
set(ax1, 'box', 'off');
ax2 = axes('Position', get(ax1, 'Position'), 'Color', 'none');
set(ax2, 'XTick', [], 'YTick', [], 'ZTick', [], 'XColor', 'k', 'YColor', 'k', 'box', 'off', 'layer', 'top')
set(ax2, 'view', get(ax1, 'view'));
set(ax2, 'XTick', xtickh, 'YTick', ytickh, 'ZTick', ztickh)
set(ax1, 'XTick', [], 'YTick', [], 'ZTick',[])

pause(1);

xlim(ax1, [xtickh(1) xtickh(end)])
zlim(ax1, [ztickh(1) ztickh(end)])
ylim(ax1, [ytickh(1) ytickh(end)])

xlim(ax2, [xtickh(1) xtickh(end)])
zlim(ax2, [ztickh(1) ztickh(end)])
ylim(ax2, [ytickh(1) ytickh(end)])


for iX2 = 1:tax.NTAXBR - 2
    for iX1 = 1:tax.NTAXBR - 2

        if (settings.mode == 1)

            %earn11 = tax.TAXBRS(iX1) + tax.TAXBRS(iX2);
            earn12 = tax.TAXBRS(iX1) + tax.TAXBRS(iX2 + 1);
            earn21 = tax.TAXBRS(iX1 + 1) + tax.TAXBRS(iX2);
            earn22 = tax.TAXBRS(iX1 + 1) + tax.TAXBRS(iX2 + 1);

            q = patch( [tax.TAXBRS(iX1 + 1) tax.TAXBRS(iX1 + 1) tax.TAXBRS(iX1)], ...
                [tax.TAXBRS(iX2) tax.TAXBRS(iX2 + 1) tax.TAXBRS(iX2 + 1)], ...
                [earn21 - tax.tCouLev(iX1 + 1, iX2) - er earn22 - tax.tCouLev(iX1 + 1, iX2 + 1) - er ...
                earn12 - tax.tCouLev(iX1, iX2 + 1) - er], col2, 'LineStyle', 'none');

        else
            %...
        end

    end
end

for iX2 = 1:tax.NTAXBR - 2
    for iX1 = 1:tax.NTAXBR - 2

        if (settings.mode == 1)

            earn11 = tax.TAXBRS(iX1) + tax.TAXBRS(iX2);
            earn12 = tax.TAXBRS(iX1) + tax.TAXBRS(iX2 + 1);
            earn21 = tax.TAXBRS(iX1 + 1) + tax.TAXBRS(iX2);
            %earn22 = tax.TAXBRS(iX1+1) + tax.TAXBRS(iX2+1);

            q = patch( [tax.TAXBRS(iX1) tax.TAXBRS(iX1) tax.TAXBRS(iX1 + 1)], ...
                [tax.TAXBRS(iX2) tax.TAXBRS(iX2 + 1) tax.TAXBRS(iX2)], ...
                [earn11 - tax.tCouLev(iX1, iX2) earn12 - tax.tCouLev(iX1, iX2 + 1) ...
                earn21 - tax.tCouLev(iX1 + 1, iX2)], col1, 'LineWidth', .4, 'EdgeColor', edge );

        else
            %...
        end

    end
end

%boundary line (X)
iX2 = tax.NTAXBR - 1;
for iX1 = 1:tax.NTAXBR - 2

    earn11 = tax.TAXBRS(iX1) + tax.TAXBRS(iX2);
    earn21 = tax.TAXBRS(iX1 + 1) + tax.TAXBRS(iX2);

    line( [tax.TAXBRS(iX1) tax.TAXBRS(iX1) tax.TAXBRS(iX1 + 1)], ...
        [tax.TAXBRS(iX2) tax.TAXBRS(iX2) tax.TAXBRS(iX2)], ...
        [earn11 - tax.tCouLev(iX1, iX2) earn11 - tax.tCouLev(iX1, iX2) ...
        earn21 - tax.tCouLev(iX1 + 1, iX2)], 'LineWidth', .4, 'Color', edge, 'LineStyle', '--' )

end

%boundary line (Y)
iX1 = tax.NTAXBR - 1;
for iX2 = 1:tax.NTAXBR - 2

    earn11 = tax.TAXBRS(iX1) + tax.TAXBRS(iX2);
    earn12 = tax.TAXBRS(iX1) + tax.TAXBRS(iX2 + 1);

    line( [tax.TAXBRS(iX1) tax.TAXBRS(iX1) tax.TAXBRS(iX1)], ...
        [tax.TAXBRS(iX2) tax.TAXBRS(iX2) tax.TAXBRS(iX2 + 1)], ...
        [earn11 - tax.tCouLev(iX1, iX2) earn11 - tax.tCouLev(iX1, iX2) ...
        earn12 - tax.tCouLev(iX1, iX2 + 1)], 'LineWidth', .4, 'Color', edge, 'LineStyle', '--' )

end

% singles

cc = linspecer(1);

for iX1 = 1:tax.NTAXBR - 1

    if (settings.mode == 1)

        earn1 = tax.TAXBRS(iX1);
        earn2 = tax.TAXBRS(iX1 + 1);

        line( [tax.TAXBRS(iX1) tax.TAXBRS(iX1) tax.TAXBRS(iX1 + 1)], ...
            [tax.TAXBRS(1) tax.TAXBRS(1) tax.TAXBRS(1)], ...
            [earn1 - tax.tSinLev(iX1, 1) earn1 - tax.tSinLev(iX1, 1) ...
            earn2 - tax.tSinLev(iX1 + 1, 1)], 'LineWidth', 1, 'color', cc(1, :) )

        line( [tax.TAXBRS(1) tax.TAXBRS(1) tax.TAXBRS(1)], ...
            [tax.TAXBRS(iX1) tax.TAXBRS(iX1 + 1) tax.TAXBRS(iX1)], ...
            [earn1 - tax.tSinLev(iX1, 2) earn2 - tax.tSinLev(iX1 + 1, 2) ...
            earn1 - tax.tSinLev(iX1, 2)], 'LineWidth', 1, 'color', cc(1, :) )

    else
        %...
    end

end

% labels on ax2
xlabel(ax2, '$z_1$');
ylabel(ax2, '$z_2$');
if (settings.mode == 1)
    zlabel(ax2, '$z_1 + z_2 - T(z_1, z_2)$');
else
    % ...
end

vecE1 = zeros(tax.NTAXBR * tax.NTAXBR, 1);
vecE2 = zeros(tax.NTAXBR * tax.NTAXBR, 1);
vecT = zeros(tax.NTAXBR * tax.NTAXBR, 1);
vecN = zeros(tax.NTAXBR * tax.NTAXBR, 1);

iX = 0;
for iX2 = 1:tax.NTAXBR
    for iX1 = 1:tax.NTAXBR
        iX = iX + 1;
        vecE1(iX) = tax.TAXBRS(iX1);
        vecE2(iX) = tax.TAXBRS(iX2);
        vecT(iX) = tax.tCouLev(iX1, iX2);
        vecN(iX) = vecE1(iX) + vecE2(iX) - vecT(iX);
    end
end

hold off

end