function stat = figureMarketLabourSupply(fpath, figurepath, mode)

% hours (market level)
mhours_M0 = load([fpath, 'Rmhours_M0.csv']);
fhours_0F = load([fpath, 'Rfhours_0F.csv']);
mhours_Mf = load([fpath, 'Rmhours_Mf.csv']);
fhours_mF = load([fpath, 'Rfhours_mF.csv']);

% employment (market level)
mwork_M0 = load([fpath, 'Rmwork_M0.csv']);
fwork_0F = load([fpath, 'Rfwork_0F.csv']);
mwork_Mf = load([fpath, 'Rmwork_Mf.csv']);
fwork_mF = load([fpath, 'Rfwork_mF.csv']);

% DERIVED unconditional hours (market level)
munhours_M0 = mhours_M0;
funhours_0F = fhours_0F;
munhours_Mf = mhours_Mf;
funhours_mF = fhours_mF;

for col = 3:4
    munhours_M0(:, col) = mhours_M0(:, col) .* mwork_M0(:, col);
    funhours_0F(:, col) = fhours_0F(:, col) .* fwork_0F(:, col);
    munhours_Mf(:, col) = mhours_Mf(:, col) .* mwork_Mf(:, col);
    funhours_mF(:, col) = fhours_mF(:, col) .* fwork_mF(:, col);
end

pauselen = 0;
legendmode = 0;

nmf = max(mhours_M0(:, 1));
cc = linspecer(2 * nmf);


if (legendmode == 0)
    legendtag = '_nolegend';
else
    legendtag = '_legend';
end

for sex = 1:2

    close all
    hfig = figure('Visible', 'off');

    box on;
    hold on;


    if (sex == 1)
        hrSin = munhours_M0;
        hrCou = munhours_Mf;
        tag = 'munhours';
        xtickh = [35 40 45 50];
    else
        hrSin = funhours_0F;
        hrCou = funhours_mF;
        tag = 'funhours';
        xtickh = [15 25 35 45];
    end

    ytickh = xtickh;

    xlim([xtickh(1) xtickh(end)]);
    ylim([ytickh(1) ytickh(end)]);

    bgedge = [0.7 0.7 0.7];
    gridstyle = '--';
    gridstylen = '-';

    for iX = 2:length(xtickh) - 1
        line([xtickh(iX), xtickh(iX)], [ytickh(1) + eps, ytickh(end)], 'Color', bgedge, ...
            'LineWidth', 0.5, 'LineStyle', gridstyle)
    end

    for iY = 2:length(ytickh) - 1
        line([xtickh(1) + eps, xtickh(end)], [ytickh(iY), ytickh(iY)], 'Color', bgedge, ...
            'LineWidth', 0.5, 'LineStyle', gridstyle)
    end

    s = zeros(6, 1);

    for pX = 1:nmf

        simSingle = hrSin(hrSin(:, 1) == pX, 3);
        datSingle = hrSin(hrSin(:, 1) == pX, 4);
        s(pX) = scatter(simSingle, datSingle, 45, cc(pX, :), 'filled', ...
            'MarkerEdgeColor', cc(pX, :) * 0.5);

    end

    for pX = 1:nmf

        simCouple = hrCou(hrCou(:, 1) == pX, 3);
        datCouple = hrCou(hrCou(:, 1) == pX, 4);
        s(pX + 3) = scatter(simCouple, datCouple, 45, cc(pX + 3, :), 'filled', ...
            'MarkerEdgeColor', cc(pX + 3, :) * 0.5);

    end

    xlabel('Model');
    ylabel('Data');
    set(gca, 'YTick', ytickh);
    set(gca, 'XTick', xtickh);

    if (legendmode > 0)
        if (nmf == 3)
            C{1} = 'Single: high school and below';
            C{2} = 'Single: some college';
            C{3} = 'Single: college and above';
            C{4} = 'Couple: high school and below';
            C{5} = 'Couple: some college';
            C{6} = 'Couple: college and above';
        else
            % ...
        end
        leg = legendflex(s, C, 'box', 'off', 'Anchor', [3 8], 'Buffer', [10  0]);
    end

    set(gca, 'Layer',  'Top')

    phi = (1 + sqrt(5)) / 2;
    fontsize = 8;
    width = 13;
    set(hfig, 'units', 'centimeters', ...
        'NumberTitle', 'off');
    pos = get(hfig, 'position');
    formatTicks(gca, 'y');
    formatTicks(gca, 'x');

    set(gca, 'PlotBoxAspectRatio', [1 1 1], 'FontSize', fontsize);
    set(hfig, 'position', [pos(1:2), width, width / phi]);

    fname = [figurepath, '/market', tag, legendtag];
    pause(pauselen)
    matlabfrag(fname);

end

close all
hfig = figure('Visible', 'off');
hold on;

s = zeros(6, 1);

if (nmf == 3)
    C{1} = 'Single: high school and below';
    C{2} = 'Single: some college';
    C{3} = 'Single: college and above';
    C{4} = 'Couple: high school and below';
    C{5} = 'Couple: some college';
    C{6} = 'Couple: college and above';
else
    % ...
end

for pX = 1:nmf
    s(pX) = scatter([0 1], [0 1], 45, cc(pX, :), 'filled', ...
        'MarkerEdgeColor', cc(pX, :) * 0.5, 'visible', 'off');
end

for pX = 1:nmf
    s(pX + 3) = scatter([0 1], [0 1], 45, cc(pX + 3, :), 'filled', ...
        'MarkerEdgeColor', cc(pX + 3, :) * 0.5, 'visible', 'off');
end

set(gca, 'Visible', 'off')
fontsize = 10;
width = 13;
set(hfig, 'units', 'centimeters', 'NumberTitle', 'off');
pos = get(hfig, 'position');

set(gca, 'PlotBoxAspectRatio', [2 1 1], 'FontSize', fontsize);
set(hfig, 'position', [pos(1:2), width, 2]);
leg = legendflex(s, C,  'ncol',  3, 'ref', gca, 'Anchor', [8 8],  'Interpreter',  'none', 'box', 'off');
set(gca,  'LooseInset',  get(gca, 'TightInset'))

fname = [figurepath, '/market_legend'];
pause(pauselen)
matlabfrag(fname);

stat = 0;

end
