function stat = figureMarketMarriage(fpath, figurepath, mode)

% match distribution (market level)
Rjtype_M0 = load([fpath, 'Rjtype_M0.csv']);
Rjtype_0F = load([fpath, 'Rjtype_0F.csv']);
Rjtype_MF = load([fpath, 'Rjtype_MF.csv']);

pauselen = 0;
nmf = max(Rjtype_M0(:, 1));

cc = linspecer(2 * nmf + nmf ^ 2);
xtickh = [0 0.1 0.2 0.3];

close all
hfig = figure('Visible', 'off');

box on;
hold on;

ytickh = xtickh;

xlim([xtickh(1) xtickh(end)]);
ylim([ytickh(1) ytickh(end)]);

bgedge = [0.7 0.7 0.7];
gridstyle = '--';
gridstylen = '-';

for iX = 2:length(xtickh) - 1
    line([xtickh(iX), xtickh(iX)], [ytickh(1) + eps, ytickh(end)], 'Color', bgedge, ...
        'LineWidth', 0.5, 'LineStyle', gridstyle)
end

for iY = 2:length(ytickh) - 1
    line([xtickh(1) + eps, xtickh(end)], [ytickh(iY), ytickh(iY)], 'Color', bgedge, ...
        'LineWidth', 0.5, 'LineStyle', gridstyle)
end

s = zeros(2 * nmf + nmf ^ 2, 1);
for pX = 1:nmf
    sim = Rjtype_0F(Rjtype_0F(:, 1) == pX, 3);
    dat = Rjtype_0F(Rjtype_0F(:, 1) == pX, 4);
    s(pX) = scatter(sim, dat, 45, cc(pX, :), 'filled', 'MarkerEdgeColor', cc(pX, :) * 0.5);
end

if (nmf == 3)
    C{1} = 'Single women: h/s and below';
    C{2} = 'Single women: some college';
    C{3} = 'Single women: college+';
    C{4} = 'Single men: h/s and below';
    C{5} = 'Single men: some college';
    C{6} = 'Single men: college+';    
    CL{1} = '$\mu_{L0}$';
    CL{2} = '$\mu_{M0}$';
    CL{3} = '$\mu_{H0}$';
    CL{4} = '$\mu_{0L}$';
    CL{5} = '$\mu_{0M}$';
    CL{6} = '$\mu_{0H}$';
    lab{1} = 'h/s and below';
    lab{2} = 'some college';
    lab{3} = 'college+';
    labL{1} = 'L';
    labL{2} = 'M';
    labL{3} = 'H';
elseif (nmf == 4)
    C{1} = 'Single women: below h/school';
    C{2} = 'Single women: high school';
    C{3} = 'Single women: some college';
    C{4} = 'Single women: college+';
    C{5} = 'Single women: below h/school';
    C{6} = 'Single women: high school';
    C{7} = 'Single women: some college';
    C{8} = 'Single women: college+';
    CL{1} = '$\mu_{10}$';
    CL{2} = '$\mu_{20}$';
    CL{3} = '$\mu_{30}$';
    CL{4} = '$\mu_{40}$';
    CL{5} = '$\mu_{01}$';
    CL{6} = '$\mu_{02}$';
    CL{7} = '$\mu_{03}$';
    CL{8} = '$\mu_{04}$';
    lab{1} = 'below h/s';
    lab{2} = 'high school';
    lab{3} = 'some college';
    lab{4} = 'college+';
    labL{1} = '1';
    labL{2} = '2';
    labL{3} = '3';
    labL{4} = '4';
else
    % ...
end


for pX = 1:nmf
    sim = Rjtype_M0(Rjtype_M0(:, 1) == pX, 3);
    dat = Rjtype_M0(Rjtype_M0(:, 1) == pX, 4);
    s(pX + nmf) = scatter(sim, dat, 45, cc(pX + nmf, :), 'filled', 'MarkerEdgeColor', cc(pX + nmf, :) * 0.5);
end

pX = 2 * nmf;
for pX2 = 1:nmf
    for pX1 = 1:nmf
        pX = pX + 1;
        sim = Rjtype_MF(Rjtype_MF(:, 1) == pX1 & Rjtype_MF(:, 2) == pX2, 4);
        dat = Rjtype_MF(Rjtype_MF(:, 1) == pX1 & Rjtype_MF(:, 2) == pX2, 5);
        s(pX) = scatter(sim, dat, 45, cc(pX, :), 'filled', 'MarkerEdgeColor', cc(pX, :) * 0.5);
        C{pX} = ['Married: ', lab{pX1}, ',', lab{pX2}];
        CL{pX} = ['$\mu_{', labL{pX1}, labL{pX2}, '}$'];
    end
end

xlabel('Model');
ylabel('Data');
set(gca, 'YTick', ytickh);
set(gca, 'XTick', xtickh);

set(gca, 'Layer', 'Top')

phi = (1 + sqrt(5)) / 2;
fontsize = 10;
width = 13;
set(hfig, 'units', 'centimeters', 'NumberTitle', 'off');
pos = get(hfig, 'position');

set(gca, 'PlotBoxAspectRatio', [1 1 1], 'FontSize', fontsize);

set(hfig, 'position', [pos(1:2), 1.5 * width, width / phi]);

legendflex(s, CL, 'nrow', 6, 'ref', gca, 'Anchor', [4 8], 'Buffer', [-0.2125  0], ...
    'padding', [0 -10 -40], 'Interpreter', 'none', 'box', 'off', 'bufferunit', 'normalized');
fname = [figurepath, '/marketmarriage'];
pause(pauselen)
matlabfrag(fname);

close all

stat = 0;

end
