function stat = figureTimeAllocation2(fpath, figurepath, mode)

% marginal hours distribution (aggregated)
mhours_sin = load([fpath, 'mhours_sin.csv']);
fhours_sin = load([fpath, 'fhours_sin.csv']);
mhours_cou = load([fpath, 'mhours_cou.csv']);
fhours_cou = load([fpath, 'fhours_cou.csv']);

% marginal home time distribution (aggregated)
mhome_sin = load([fpath, 'mhome_sin.csv']);
fhome_sin = load([fpath, 'fhome_sin.csv']);
mhome_cou = load([fpath, 'mhome_cou.csv']);
fhome_cou = load([fpath, 'fhome_cou.csv']);

pauselen = 0.0;

% label
nhh = max(mhome_sin(:, 1));
if (nhh == 3)
    homelab = {'$L$', '$M$', '$H$'};
else
    homelab = [];
end

cc(1, :) = [213, 62,  79]  / 255;
cc(2, :) = [244, 109, 67]  / 255;
cc(3, :) = [253, 174, 97]  / 255;
cc(4, :) = [254, 224, 139] / 255;
cc(5, :) = [230, 245, 152] / 255;
cc(6, :) = [171, 221, 164] / 255;
cc(7, :) = [102, 194, 165] / 255;
cc(8, :) = [50,  136, 189] / 255;

colFn_a = cc(4, :);
colFn_b = cc(1, :);
colFk_a = cc(3, :);
colFk_b = cc(2, :);

colMn_a = cc(5, :);
colMn_b = cc(8, :);
colMk_a = cc(6, :);
colMk_b = cc(7, :);

close all
fontsize = 8;
widthSml = 4.9;

phi = (1 + sqrt(5)) / 2;

C = cell(2,1);
C{1} = 'Model';
C{2} = 'Data';

% female: aggregated marginal hours distribution

thiswidth = widthSml;
thistag = '_sml';

for kid = 0:1

    sim0 = fhours_sin(fhours_sin(:, 2) == kid, 3);
    dat0 = fhours_sin(fhours_sin(:, 2) == kid, 4);

    sim = fhours_cou(fhours_cou(:, 2) == kid, 3);
    dat = fhours_cou(fhours_cou(:, 2) == kid, 4);

    sim0 = roundFp(sim0);
    dat0 = roundFp(dat0);
    sim = roundFp(sim);
    dat = roundFp(dat);

    for mtype = 0:1

        close all
        hfig = figure('Visible', 'off');

        if (mtype == 0)
            b = bar([sim0 dat0], 'BarWidth', 1);
        else
            b = bar([sim dat], 'BarWidth', 1);
        end

        if (kid == 0)
            set(b(1), 'facecolor', colFn_a);
            set(b(2), 'facecolor', colFn_b);
        else
            set(b(1), 'facecolor', colFk_a);
            set(b(2), 'facecolor', colFk_b);
        end

        set(gca, 'XTickLabel', {'$UN$', '$PT$', '$FT$'})
        ylim([0 1])
        set(hfig, 'units', 'centimeters', 'NumberTitle', 'off');
        pos = get(hfig, 'position');
        leg = legendflex(b, C, 'box', 'off', 'Anchor', [1 1], 'Buffer', [10 -10], 'FontSize', fontsize);

        set(gca, 'PlotBoxAspectRatio', [phi 1 1], 'FontSize', fontsize);

        set(findobj(leg,'Type','text'),'fontsize', fontsize)
        set(hfig, 'position', [pos(1:2), thiswidth, thiswidth / phi]);

        formatTicks(gca, 'y');

        pause(pauselen);

        if (mtype == 0)
            fname = [figurepath, '/fhoursM0Ff_kid', int2str(kid), thistag];
        else
            fname = [figurepath, '/fhoursMmFf_kid', int2str(kid), thistag];
        end

        matlabfrag(fname);

    end

end


% male: aggregated marginal hours distribution

thiswidth = widthSml;
thistag = '_sml';

for kid = 0:1

    sim0 = mhours_sin(mhours_sin(:, 2) == kid, 3);
    dat0 = mhours_sin(mhours_sin(:, 2) == kid, 4);

    sim = mhours_cou(mhours_cou(:, 2) == kid, 3);
    dat = mhours_cou(mhours_cou(:, 2) == kid, 4);

    sim0 = roundFp(sim0);
    dat0 = roundFp(dat0);
    sim = roundFp(sim);
    dat = roundFp(dat);

    for ftype = 0:1

        close all
        hfig = figure('Visible', 'off');

        if (ftype == 0)
            b = bar([sim0 dat0], 'BarWidth', 1);
        else
            b = bar([sim dat], 'BarWidth', 1);
        end

        if (kid == 0)
            set(b(1), 'facecolor', colMn_a);
            set(b(2), 'facecolor', colMn_b);
        else
            set(b(1), 'facecolor', colMk_a);
            set(b(2), 'facecolor', colMk_b);
        end

        leg = legendflex(b, C, 'box', 'off', 'Anchor', [1 1], 'Buffer', [10 -10]);

        set(gca, 'XTickLabel', {'$UN$', '$PT$', '$FT$'})
        ylim([0 1])
        set(hfig, 'units', 'centimeters', 'NumberTitle', 'off');
        pos = get(hfig, 'position');
        set(gca, 'PlotBoxAspectRatio', [phi 1 1], 'FontSize', fontsize);
        set(findobj(leg, 'Type', 'text'), 'fontsize', fontsize);

        set(hfig, 'position', [pos(1:2), thiswidth, thiswidth / phi]);
        formatTicks(gca, 'y');

        pause(pauselen);

        if (ftype == 0)
            fname = [figurepath, '/mhoursMmF0_kid', int2str(kid), thistag];
        else
            fname = [figurepath, '/mhoursMmFf_kid', int2str(kid), thistag];
        end

        matlabfrag(fname);

    end

end

% female: aggregated marginal home time distribution

thiswidth = widthSml;
thistag = '_sml';

for kid = 0:1

    sim0 = fhome_sin( fhome_sin(:, 2) == kid, 3);
    dat0 = fhome_sin( fhome_sin(:, 2) == kid, 4);

    sim = fhome_cou( fhome_cou(:, 2) == kid, 3);
    dat = fhome_cou( fhome_cou(:, 2) == kid, 4);

    sim0 = roundFp(sim0);
    dat0 = roundFp(dat0);
    sim = roundFp(sim);
    dat = roundFp(dat);

    for mtype = 0:1

        close all
        hfig = figure('Visible', 'off');

        if (mtype == 0)
            b = bar([sim0 dat0], 'BarWidth', 1);
        else
            b = bar([sim dat], 'BarWidth', 1);
        end

        if (kid == 0)
            set(b(1), 'facecolor', colFn_a);
            set(b(2), 'facecolor', colFn_b);
        else
            set(b(1), 'facecolor', colFk_a);
            set(b(2), 'facecolor', colFk_b);
        end

        set(gca, 'XTickLabel', homelab)

        ylim([0 1])
        set(hfig, 'units', 'centimeters', 'NumberTitle', 'off');
        pos = get(hfig, 'position');
        set(gca, 'PlotBoxAspectRatio', [phi 1 1], 'FontSize', fontsize);
        set(hfig, 'position', [pos(1:2), thiswidth, thiswidth / phi]);
        formatTicks(gca, 'y');

        pause(pauselen);

        if (mtype == 0)
            fname = [figurepath, '/fhomeM0Ff_kid', int2str(kid), thistag];
        else
            fname = [figurepath, '/fhomeMmFf_kid', int2str(kid), thistag];
        end

        matlabfrag(fname);

    end

end

% male: aggregated marginal home time distribution

thiswidth = widthSml;
thistag = '_sml';

for kid = 0:1

    sim0 = mhome_sin(mhome_sin(:, 2) == kid, 3);
    dat0 = mhome_sin(mhome_sin(:, 2) == kid, 4);

    sim = mhome_cou(mhome_cou(:, 2) == kid, 3);
    dat = mhome_cou(mhome_cou(:, 2) == kid, 4);

    sim0 = roundFp(sim0);
    dat0 = roundFp(dat0);
    sim = roundFp(sim);
    dat = roundFp(dat);

    for mtype = 0:1

        close all
        hfig = figure('Visible', 'off');

        if (mtype == 0)
            b = bar([sim0 dat0], 'BarWidth', 1);
        else
            b = bar([sim dat], 'BarWidth', 1);
        end

        if (kid == 0)
            set(b(1), 'facecolor', colMn_a);
            set(b(2), 'facecolor', colMn_b);
        else
            set(b(1), 'facecolor', colMk_a);
            set(b(2), 'facecolor', colMk_b);
        end

        set(gca, 'XTickLabel', homelab)

        ylim([0 1])
        set(hfig, 'units', 'centimeters', 'NumberTitle', 'off');
        pos = get(hfig, 'position');
        set(gca, 'PlotBoxAspectRatio', [phi 1 1], 'FontSize', fontsize);
        set(findobj(leg, 'Type', 'text'), 'fontsize', fontsize);
        set(hfig, 'position', [pos(1:2), thiswidth, thiswidth / phi]);
        formatTicks(gca, 'y');

        pause(pauselen);

        if (mtype == 0)
            fname = [figurepath, '/mhomeMmF0_kid', int2str(kid), thistag];
        else
            fname = [figurepath, '/mhomeMmFf_kid', int2str(kid), thistag];
        end

        matlabfrag(fname);

    end

end

stat = 0;

end
