function stat = figureUpf(fpath, figurepath, mode)

set(0, 'defaultTextInterpreter', 'none');

phi = (1 + sqrt(5)) / 2;
fontsize = 8;
width = 9;
c = 0.9;
market = 1;
pX1 = 3;

if (exist([fpath, '/upf_market', int2str(market), '.csv'], 'file'))
    upfvar = load([fpath, '/upf_market', int2str(market), '.csv']);
else
    warning([fpath, '/upf_market', int2str(market), '.csv'])
    stat = 1;
    return
end

for pX2 = 1:3

    close all

    hfig = figure('Visible', 'off');
    box on;
    hold on;

    xtickh = [5 6 7 8];
    ytickh = [5 6 7 8];

    cc = linspecer(4);

    xlim([xtickh(1) xtickh(end)]);
    ylim([ytickh(1) ytickh(end)]);

    bgedge = [0.7 0.7 0.7];
    gridstyle = '--';
    gridstylen = '-';

    for iX = 2:length(xtickh) - 1
        line( [xtickh(iX), xtickh(iX)], [ytickh(1) + eps, ytickh(end)], ...
            'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle )
    end

    for iY = 2:length(ytickh) - 1
        line( [xtickh(1) + eps, xtickh(end)], [ytickh(iY), ytickh(iY)], ...
            'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle )
    end

    eupf = upfvar(upfvar(:, 1) > 0 & upfvar(:, 2) == pX1 & upfvar(:, 3) == pX2, 4:5);
    line(eupf(:, 1), eupf(:, 2), 'Color', cc(1, :), 'LineWidth', 1.5)

    eopt = upfvar(upfvar(:, 1) == -9 & upfvar(:, 2) == pX1 & upfvar(:, 3) == pX2, 4:5);
    lam = upfvar(upfvar(:, 1) == -8 & upfvar(:, 2) == pX1 & upfvar(:, 3) == pX2, 4:5);

    eout = zeros(2, 1);
    eout(1) = upfvar(upfvar(:, 1) == 0 & upfvar(:, 2) == pX1 & upfvar(:, 3) == 0, 4);
    eout(2) = upfvar(upfvar(:, 1) == 0 & upfvar(:, 2) == 0 & upfvar(:, 3) == pX2, 5);

    hold on

    k = sqrt(c ^ 2 / (1 + (lam(1) / lam(2)) ^ 2));

    eline = zeros(3, 2);
    eline(2, 1) = eopt(1);
    eline(2, 2) = eopt(2);
    eline(1, 1) = eopt(1) - k;
    eline(1, 2) = eopt(2) + k * lam(1) / lam(2);
    eline(3, 1) = eopt(1) + k;
    eline(3, 2) = eopt(2) - k * lam(1) / lam(2);

    line(eline(:, 1), eline(:, 2), 'Color', cc(2, :), 'LineWidth', 1.5)

    scatter(eopt(1), eopt(2), 45, cc(3, :), 'filled', 'MarkerEdgeColor', cc(3, :) * 0.5);
    scatter(eout(1), eout(2), 45, cc(4, :), 'filled', 'MarkerEdgeColor', cc(4, :) * 0.5);

    set(gca, 'XTick', xtickh);
    set(gca, 'YTick', ytickh);

    xlabel('Male utility');

    if (pX2 == 1)
        ylabel('Female utility');
    end

    set(hfig, 'units', 'centimeters', 'NumberTitle', 'off');
    pos = get(hfig, 'position');
    formatTicks(gca, 'y');
    set(gca, 'PlotBoxAspectRatio', [1 1 1], 'FontSize', fontsize);
    set(hfig, 'position', [pos(1:2), width, width / phi]);
    xlabel('Male utility', 'FontSize', fontsize);
    if (pX2 == 1 || mode == 1)
        ylabel('Female utility', 'FontSize', fontsize);
    end
    tag = [int2str(market), '_', int2str(pX1), '_', int2str(pX2)];
    if (mode == 1)
        tag = ['ax', tag];
    end
    fname = [figurepath, '/upf_market', tag];
    pause(0)
    matlabfrag(fname);

    end

end
