function ses = sensitivityGroups(fpath)

% load required components

% empirical variance matrix for moments
V = load([fpath, 'V.csv']);

% "sensitivity" matrix
sensitivity = load([fpath, 'sensitivity.csv']);

% load labels for parameters
fid = fopen([fpath, '/label.csv'], 'r');
lab = textscan(fid,  '%*d %*f %q %q', 'Delimiter', ',');
n = length(lab{1});
label = lab{1};
for iX = 1:n
    label{iX} = [label{iX}, ': ', lab{2}{iX}];
end
fclose(fid);
clear lab

if (n == 33)
    nmf = 3;
else
    % ...
end

nmom = size(V, 1);
momgroup = zeros(nmom, 1);

% easiest to do this with contiguous moment blocks, and then
% aggregate them into those used in the paper

momlab = {'MarriageMatchingProb', ...
          'MarketLabourSupply', ...
          'LabourSupply', ...
          'Wages', ...
          'LabourSupplyC', ...
          'Earnings', ...
          'HomeHours'};

if (nmf == 3)
    momgroupL = [1   136 352 562 586 604 628];
    momgroupU = [135 351 561 585 603 627 765];
else
    % ...
end

nm = length(momgroupL);
for mX = 1:nm
    momgroup(momgroupL(mX):momgroupU(mX)) = mX;
end

% Creating standardized sensitivity by multipling by multipling by
% sqrt(V(j,j)).

sensitivity_nd = zeros(size(sensitivity));

S = sqrt(diag(V));
for jX = 1:nmom
    sensitivity_nd(:, jX) = sensitivity(:, jX) * S(jX);
end

sensitivity_scale = sensitivity_nd;
for iX = 1:n
    maxs = max(abs(sensitivity_nd(iX, :)));
    sensitivity_scale(iX, :) = sensitivity_nd(iX, :) / maxs;
end
sensitivity_scale = abs(sensitivity_scale);

clip = 0.2;
sensitivity_groups = zeros(n, nm);

% which moments are most important?
for iX = 1:n
    for mX = 1:nm
        if (max(sensitivity_scale(iX, momgroupL(mX):momgroupU(mX))) >= clip)
            sensitivity_groups(iX, mX) = 1;
        end
    end
end

% create a file with sensitivity groups (aggregate more)
momlabG = {'MarriageMatchingProb', ...
          'LabourSupply', ...
          'Wages', ...
          'Earnings', ...
          'HomeHours'};

nm2 = 5;
sensitivity_groupsG = zeros(n, nm2);
sensitivity_groupsG(:, 1) = sensitivity_groups(:, 1);
sensitivity_groupsG(:, 2) = max(max(sensitivity_groups(:, 2), sensitivity_groups(:, 3)), sensitivity_groups(:, 5));
sensitivity_groupsG(:, 3) = sensitivity_groups(:, 4);
sensitivity_groupsG(:, 4) = sensitivity_groups(:, 6);
sensitivity_groupsG(:, 5) = sensitivity_groups(:, 7);

G = cell(n, 1);
for iX = 1:n
    thisSens = '';
    for mX = 1:nm2
        if (sensitivity_groupsG(iX, mX) > 0)
            thisSens = [thisSens, ', g', int2str(mX)];
        end
    end
    G{iX} = thisSens(3:end);
end

ses = G;

end
