function stat = tableMainSwfWeight(settings)

% initialise the tax structure (number of breaks)
tax = initTax(10, settings.factor);
tax.taxins = tax.JOINT;

tag = [0, 10];


for ixTag = 1:length(tag)

    swf = load([settings.fpath, 'primal1_t', sprintf('%02d', tag(ixTag)), '/swfweight.csv']);

    tab = NaN(tax.NTAXBR - 1, tax.NTAXBR - 1);
    pmf = NaN(tax.NTAXBR - 1, tax.NTAXBR - 1);

    for ixT2 = 1:tax.NTAXBR - 1 % highest income bin
        for ixT1 = 1:ixT2
            tab(ixT1, ixT2) = swf( swf(:, 1) == ixT1 & swf(:, 2) == ixT2, 8);
            pmf(ixT1, ixT2) = swf( swf(:, 1) == ixT1 & swf(:, 2) == ixT2, 7);
        end
    end

    bands = cell(tax.NTAXBR - 1, 1);
    bandsL = cell(tax.NTAXBR - 1, 1);
    bandsU = cell(tax.NTAXBR - 1, 1);

    for ixT2 = 1:tax.NTAXBR - 2
        bands{ixT2} = [sprintf('%1g', tax.TAXBRS(ixT2)), '--', sprintf('%1g', tax.TAXBRS(ixT2 + 1))];
        bandsL{ixT2} = [sprintf('%1g', tax.TAXBRS(ixT2)), '--'];
        bandsU{ixT2} = sprintf('%1g', tax.TAXBRS(ixT2 + 1));
    end
    bands{tax.NTAXBR - 1} = [sprintf('%1g', tax.TAXBRS(tax.NTAXBR - 1)), '+'];
    bandsL{tax.NTAXBR - 1} = [sprintf('%1g', tax.TAXBRS(tax.NTAXBR - 1)), '+'];
    bandsU{tax.NTAXBR - 1} = '';

    label = '';
    labelL = '';
    labelU = '';
    for ixT2 = 1:tax.NTAXBR-1
        label = [label, ' & ', bands{ixT2}];
        labelL = [labelL, ' & ', bandsL{ixT2}];
        labelU = [labelU, ' & ', bandsU{ixT2}];
    end
    format = '%5.3f';

    strx = ['\multicolumn{2}{c}{$\delta$ = ', int2str(-tag(ixTag) / 10), ':}'];
    sfxx = '';

    str0 = ['\begin{tabular}{lll', repmat('c', 1, tax.NTAXBR - 1), '}\n', ...
        '\toprule\n', ...
        '& & & \multicolumn{', int2str(tax.NTAXBR - 1), '}{c}{Lowest earnings range (\$000s)}\\\n', ...
        '\cmidrule(lr){4-', int2str(3 + tax.NTAXBR - 1), '}', ...
        '& &', labelL, '\\\n', ...
        '& &', labelU, '\\\n', ...
        '\midrule\n', ...
        '\addlinespace[1.5ex]\n', strx];

    str2 = ['\addlinespace[1.5ex]\n', ...
        '\bottomrule\n', ...
        '\end{tabular}'];

    if (ixTag == 1)
        str = str0;
    else
        str = [str, '\addlinespace[1.5ex]\n', ...
            '\midrule\n', ...
            '\addlinespace[1.5ex]\n', strx];
    end

    str1 = '';
    for ixT2 = 1:tax.NTAXBR - 1
        thisRow = sprintf([' & ', format], tab(:, ixT2));
        thisRow2 = strrep(thisRow, 'NaN', '--');
        if (ixT2 == 1)
            str1 = [str1, '& ', bands{ixT2}, thisRow2, '\\\n'];
        else
            str1 = [str1, '& & ', bands{ixT2}, thisRow2, '\\\n'];
        end
        thisRow = sprintf([' & \\color{blue}[',format,']'], 100 * pmf(:, ixT2));
        thisRow(thisRow == ' ') = '';
        thisRow2 = strrep(thisRow, '[NaN]', '');
        if (ixT2 == 1)
            str1 = [str1, '\parbox[t]{1mm}{\multirow{', ...
            int2str(2 * (tax.NTAXBR - 1) - 2), '}{*}{\rotatebox[origin=c]{90}{Highest earnings range (\$000s)}}}', ...
                '& & ', '', thisRow2, '\\\n'];
        else
            str1 = [str1, '& & ', '', thisRow2, '\\\n'];
        end
    end

    % full table
    str = [str, str1];


end

str = [str, str2];

strout = regexprep(str, '(\\n)', '\n');

fid = fopen([settings.tablepath, '/swfweightall.tex'], 'w');
count = fprintf(fid, '%s', strout);
stat = fclose(fid);

end
