function stat = tableParametersSens(fpath, tablepath, mode)

if (~exist([fpath, '/var.csv'], 'file'))
    warning([fpath, '/var.csv does not exist'])
    novar = true;
else
    var = load([fpath, '/var.csv']);
    novar = false;
end

x = load([fpath, '/x.csv']);
n = length(x);

if (n == 33)
    nmf = 3;
elseif (n == 41)
    nmf = 4;
else
    % ...
end

ntable = 1;
datasplit = [1 3];

ngrp = 3;
grp{1}.name = 'Log-wage offers:';
if (nmf == 3)
    grp{1}.ind = 2:13;
elseif (nmf == 4)
    grp{1}.ind = 2:17;
else
    % ...
end
a = [grp{1}.ind];

grp{2}.name = 'Preference parameters:';
if (nmf == 3)
    grp{2}.ind = [19 20 15 16 14 18 1];
elseif (nmf == 4)
    grp{2}.ind = [23 24 19 20 18 22 1];
else
    % ...
end
a = [a grp{2}.ind];

grp{3}.name = 'Home production technology:';
if (nmf == 3)
    grp{3}.ind = [17 21:33];
elseif (nmf == 4)
    grp{3}.ind = [21 25:41];
else
   % ...
end
a = [a grp{3}.ind];

b = unique(a);

if (length(a) ~= length(b))
    error('parameter vector not unique!')
end


fid = fopen([fpath, '/label.csv'], 'r');
lab = textscan(fid,  '%*d %*f %*q %q', 'Delimiter', ',');
lab = lab{1};
lab = strrep(lab, 'educational', 'educ.');
lab = strrep(lab, 'productivity', 'prod.');
fclose(fid);

if (novar)
    se = zeros(n, 1);
    ses = cell(n, 1);
    for iX = 1:n
        ses{iX} = '';
    end
else
    ses = sensitivityGroups(fpath);
    se = diag(var) .^ 0.5;
end

format = '%6.3f';
stat = 0;

for ixTable = 1:ntable

    str = ['\begin{tabular}{lccc}\n'];
    str = [str, '\toprule\n'];
    str = [str, '& Estimate & Standard & Sensitivity\\\n'];
    str = [str, '&  & error & moments \\\n'];
    str = [str, '\midrule\n'];
    str = [str, '\\[-6pt]\n'];

    for ixGrp = datasplit(ixTable, 1):datasplit(ixTable, 2)
        str = [str, '\multicolumn{3}{l}{\emph{', grp{ixGrp}.name, '}}\\\n\\[-6pt]\n'];
        n = length(grp{ixGrp}.ind);
        for iX = 1:n
            jX = grp{ixGrp}.ind(iX);
            thisRow = sprintf([' & ', format], [x(jX) se(jX)]);
            thisRow = [thisRow, '& \textsc{', ses{jX}, '}'];
            if (iX==n)
                str = [str, '\quad ', lab{jX}, thisRow, '\\[5pt]\n'];
            else
                str = [str, '\quad ', lab{jX}, thisRow, '\\\n'];
            end
        end
    end

    str = [str, '\bottomrule\n'];
    str = [str, '\end{tabular}\n'];

    strout = regexprep(str, '(\\n)', '\n');

    fname = [tablepath, '/param', int2str(ixTable), '.tex'];
    fid = fopen(fname,  'w');
    count = fprintf(fid, '%s', strout);
    stat = stat + fclose(fid);

end

end
