function stat = tableRestricted2(settings)

res = [1 2 5 6];
nres = length(res);
taxv = cell(nres, 1);
earnPoint = 1:9;
nearnPoint = length(earnPoint);
ixGender = 1;

mtrSingle = zeros(nearnPoint, nres);
mtrCoupleDualWorker1 = zeros(nearnPoint, nearnPoint, nres);
mtrCoupleDualWorker2 = zeros(nearnPoint, nearnPoint, nres);

mtrCoupleDualWorkerAvg1 = zeros(nearnPoint, 3, nres);

for ixTheta = 1:1
    for ixRes = 1:nres

        close all

        % initialise the tax structure (number of breaks)
        tax = initTax(10, settings.factor);
        tax.taxins = res(ixRes);

        %set up taxes
        respath = [settings.fpath, 'primal', int2str(res(ixRes)), '_t', sprintf('%02d', settings.theta(ixTheta)), '/'];
        fname = [respath, '/xtax_sol.asc'];
        y = loadxtax2(fname, settings.ed);
        x = unitToLevel(tax, y, tax.ymin);
        tax = setTax(x, tax, settings.factor);
        taxv{ixRes} = tax;

        % solve for mtr on all nodes on the joint grid

        % single
        for ixZ = 1:nearnPoint
            ixEarn = earnPoint(ixZ);
            earn0 = taxv{ixRes}.TAXBR(ixEarn);
            earn1 = 0.5 * ( taxv{ixRes}.TAXBR(ixEarn + 1) + taxv{ixRes}.TAXBR(ixEarn) );
            y0 = evalTaxSinScalar(taxv{ixRes}, earn0, ixGender);
            y1 = evalTaxSinScalar(taxv{ixRes}, earn1, ixGender);
            thisMtr = 100 * (y1 - y0) / ( 0.5 * (tax.TAXBR(ixEarn + 1) - tax.TAXBR(ixEarn)) );
            mtrSingle(ixZ, ixRes) = thisMtr;
        end

        % couple, dual earner (need mtr for both workers)
        for ixZ2 = 1:nearnPoint
            for ixZ1 = 1:nearnPoint

                ixEarn1 = earnPoint(ixZ1);
                ixEarn2 = earnPoint(ixZ2);

                % mtr1
                earn0 = taxv{ixRes}.TAXBR(ixEarn1);
                earn1 = 0.5 * ( taxv{ixRes}.TAXBR(ixEarn1 + 1) + taxv{ixRes}.TAXBR(ixEarn1) );
                y0 = evalTaxCouScalar( tax, earn0, tax.TAXBR(ixEarn2) );
                y1 = evalTaxCouScalar( tax, earn1, tax.TAXBR(ixEarn2) );
                thisMtr = 100 * (y1 - y0) / ( earn1 - earn0 );
                mtrCoupleDualWorker1(ixZ1, ixZ2, ixRes) = thisMtr;

                % mtr2
                earn0 = taxv{ixRes}.TAXBR(ixEarn2);
                earn1 = 0.5 * ( taxv{ixRes}.TAXBR(ixEarn2 + 1) + taxv{ixRes}.TAXBR(ixEarn2) );
                y0 = evalTaxCouScalar( tax, tax.TAXBR(ixEarn1), earn0 );
                y1 = evalTaxCouScalar( tax, tax.TAXBR(ixEarn1), earn1 );
                thisMtr = 100 * (y1 - y0) / ( earn1 - earn0 );
                mtrCoupleDualWorker2(ixZ1, ixZ2, ixRes) = thisMtr;

            end
        end

        mtrCoupleDualWorkerAvg1(:, 1, ixRes) = ( mtrCoupleDualWorker1(:, 1, ixRes) ...
            + mtrCoupleDualWorker1(:, 2, ixRes) + mtrCoupleDualWorker1(:, 3, ixRes) ) / 3;

        mtrCoupleDualWorkerAvg1(:, 2, ixRes) = ( mtrCoupleDualWorker1(:, 4, ixRes) ...
            + mtrCoupleDualWorker1(:, 5, ixRes) + mtrCoupleDualWorker1(:, 6, ixRes) ) / 3;

        mtrCoupleDualWorkerAvg1(:, 3, ixRes) = ( mtrCoupleDualWorker1(:, 7, ixRes) ...
            + mtrCoupleDualWorker1(:, 8, ixRes) + mtrCoupleDualWorker1(:, 9, ixRes) ) / 3;

    end % ixRes
end % ixTheta

format = '%3.0f';

str = ['\begin{tabular}{ccccccccccccc}\n',  ...
       '\toprule\n', ...
       '& \multicolumn{3}{c}{Unrestricted} & \multicolumn{3}{c}{Independent} ', ...
       '& \multicolumn{3}{c}{Income splitting} & \multicolumn{3}{c}{Income aggregation} \\\n', ...
       '\cmidrule(lr){2-4}\cmidrule(lr){5-7}\cmidrule(lr){8-10}\cmidrule(lr){11-13}\n', ...
       '$z_1$ & \emph{Low} & \emph{Med.} & \emph{High} & \emph{Low} & \emph{Med.} & \emph{High} ', ...
       '& \emph{Low} & \emph{Med.} & \emph{High} & \emph{Low} & \emph{Med.} & \emph{High}\\\n', ...
       '\midrule\n'];

% couple, dual earner
for ixZ1 = 1:nearnPoint - 1
    ixEarn1 = earnPoint(ixZ1);
    str = [str, sprintf('%.1f', tax.TAXBRS(ixEarn1))];
    for ixRes = 1:nres
        str = [str, sprintf([' & ', format, ' & ', format, ' & ', format], ...
            [mtrCoupleDualWorkerAvg1(ixZ1, 1, ixRes), ...
             mtrCoupleDualWorkerAvg1(ixZ1, 2, ixRes), ...
             mtrCoupleDualWorkerAvg1(ixZ1, 3, ixRes)])];
    end
    str = [str, '\\\n'];
end

str = [str, '\bottomrule\n', ...
            '\end{tabular}\n'];

strout = regexprep(str, '(\\n)', '\n');

fid = fopen([settings.tablepath, 'restricted_mtr2.tex'], 'w');
count = fprintf(fid, '%s', strout);
stat = fclose(fid);
