function stat = tableRestrictedWelfare(settings)

res = [1 2 5 6];
nres = length(res);

marriageMarketMatters = zeros(nres, 1);
marriageMarketMatters2 = zeros(nres, 1);
welfareCost = zeros(nres, 1);

for ixTheta = 1:1
    for ixRes = 1:nres

        close all

        % initialise the tax structure (number of breaks)
        tax = initTax(10, settings.factor);
        tax.taxins = res(ixRes);

        %set up taxes
        respath = [settings.fpath, 'primal', int2str(res(ixRes)), '_t', sprintf('%02d', settings.theta(ixTheta)), '/'];
        fname = [respath, '/xtax_sol.asc'];
        [~, ~, ~, rev_primal, ~] = loadxtax2(fname, settings.ed);

        % determine the importance of the marriage market
        fname = [settings.fpath, 'fixed', int2str(res(ixRes)), '_t', sprintf('%02d', settings.theta(ixTheta)), '/xtax_sol.asc'];
        [~, ~, ~, rev_fixed, ~] = loadxtax2(fname, settings.ed);
        fname = [settings.fpath, 'fixed', int2str(res(ixRes)), '_t', sprintf('%02d', settings.theta(ixTheta)), '/dual/xtax_sol.asc'];
        [~, ~, ~, rev_fixed_dual, ~] = loadxtax2(fname, settings.ed);
        marriageMarketMatters(ixRes) = 100 * (rev_fixed_dual - rev_fixed) / rev_primal;
        marriageMarketMatters2(ixRes) = 100 * (rev_fixed_dual - rev_primal) / rev_primal;

        % determine the welfare cost relative to unrestricted schedule
        if (ixRes ~= 1)
            fname = [settings.fpath, 'dual', int2str(res(ixRes)), '_t', sprintf('%02d', settings.theta(ixTheta)), '/xtax_sol.asc'];
            [~, ~, ~, rev_dual, ~] = loadxtax2(fname, settings.ed);
            welfareCost(ixRes) = 100 * (rev_primal - rev_dual) / rev_primal;
        end

    end % ixRes
end % ixTheta

format2 = '%3.1f';

str = ['\begin{tabular}{lcccc}\n',  ...
       '\toprule\n', ...
       '& Unrestricted & Independent & Income splitting & Income aggregation \\\n', ...
       '\midrule\n'];

str = [str, '$\Delta_\text{unrestricted}$'];
str = [str, ' & -- '];
for ixRes = 2:nres
    str = [str, sprintf([' & ', format2], welfareCost(ixRes))];
end
str = [str, '\\\n'];

str = [str, '$\Delta_\text{eq-welfare}$'];
for ixRes = 1:nres
    str = [str, sprintf([' & ', format2], marriageMarketMatters(ixRes))];
end
str = [str, '\\\n'];

str = [str, '$\Delta_\text{eq-revenue}$'];
for ixRes = 1:nres
    str = [str, sprintf([' & ', format2], marriageMarketMatters2(ixRes))];
end
str = [str, '\\\n'];

str = [str, '\bottomrule\n', ...
            '\end{tabular}\n'];

strout = regexprep(str, '(\\n)', '\n');

fid = fopen([settings.tablepath, 'restricted_mtr_welfare.tex'], 'w');
count = fprintf(fid, '%s', strout);
stat = fclose(fid);
