function [hfig, ax1, hl] = taxEmpirical(settings)

factor = 52 / 1000;
xtickh = factor * (0:30000:150000) / 52;
ytickh = [-0.2, 0.0, 0.2, 0.4, 0.6, 0.8];

NZ = 3;

cc = linspecer(NZ + 1);
ccSingle = cc(1, :);
cc = cc(2:4, :);

bgedge = [0.7 0.7 0.7];
gridstyle = '--';
gridstylen = '-';

mtrylim = [ytickh(1) ytickh(end)];

C = cell(NZ, 1);

hfig = figure('Visible', settings.visible);
hold all

for iX = 2:length(xtickh) - 1
    line( [xtickh(iX), xtickh(iX)], [ytickh(1) + eps, ytickh(end)], ...
        'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle )
end
line( [xtickh(end), xtickh(end)], [ytickh(1) + eps, ytickh(end)], ...
    'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstylen )


for iY = 2:length(ytickh) - 1
    line( [xtickh(1) + eps, xtickh(end)], [ytickh(iY), ytickh(iY)], ...
        'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstyle )
end
line( [xtickh(1) + eps, xtickh(end)], [ytickh(end), ytickh(end)], ...
    'Color', bgedge, 'LineWidth', 0.5, 'LineStyle', gridstylen )

% spouse earnings

mtrEmp = csvread('../taxes/output/empirical_mtr.csv',1, 0);
taxbrs = ([0 12500 25000 37500 60000 85000 110000 150000 190000 250000]/1000);
ntaxbr = length(taxbrs);

mtrAvg = zeros(ntaxbr - 1, 3);
for ixZ = 1:NZ
    mtrAvg(:, ixZ) = mtrEmp(mtrEmp(:, 1) == ixZ, 3);
    % ensure we don't display mtr changes at boundary
    mtrAvg(8:9, ixZ) = mtrAvg(7, ixZ);
end

xx = [taxbrs(1) repelem(taxbrs(2:ntaxbr - 1), 1, 2) taxbrs(end)];

p = zeros(NZ, 1);
for ixZ = 1:NZ
    p(ixZ) = plot(xx, repelem(mtrAvg(:, ixZ), 2, 1), 'LineWidth', 1, 'color', cc(ixZ, :), 'LineStyle', '-');
end

C{1} = 'Low spousal earnings';
C{2} = 'Med spousal earnings';
C{3} = 'High spousal earnings';

ylim(mtrylim)
xlim([0, xtickh(end)]);

xlabel('$z$')

[hl.leg, hl.obj, hl.hout, hl.mout] = legendflex(p, C, 'box', 'off', 'Anchor', [5 5], 'Buffer', [-10  10], ...
    'FontSize', 7);
set(gca, 'XTick', xtickh)
set(gca, 'YTick', ytickh)
set(gca, 'Layer', 'Top')
formatTicks(gca, 'y');
ax1 = gca;

hold off

end
