module momentIntegrate

    use globaldef

    interface momentIntegrateGroup
        module procedure momentIntegrateGroupMean
        module procedure momentIntegrateGroupMeanSd
    end interface momentIntegrateGroup

    private :: momentIntegrateGroup

contains


    ! integrates over female type for couples allocations

    subroutine integrateCouple_F( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(NTYPE2)
        real(dp) :: group_prob(NTYPE2)

        integer :: pX1, pX2, kX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX1 = 1, NTYPE1

                do pX2 = 1, NTYPE2
                    group_mean(pX2) = condMean(pX1, pX2, kX)
                    group_prob(pX2) = thisProb(pX1, pX2, kX)
                end do

                call momentIntegrateGroup( NTYPE2, group_prob, group_mean, momMean(pX1, kX) )

            end do
        end do

    end subroutine integrateCouple_F

    ! integrates over female type for couples allocations (mean and s.d.)

    subroutine integrateCoupleMeanSd_F( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1, 0:1)
        real(dp), intent(out) :: momSd(NTYPE1, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)
        real(dp) :: group_mean(NTYPE2)
        real(dp) :: group_sd(NTYPE2)
        real(dp) :: group_prob(NTYPE2)

        integer :: pX1, pX2, kX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX1 = 1, NTYPE1

                do pX2 = 1, NTYPE2
                    group_mean(pX2) = condMean(pX1, pX2, kX)
                    group_sd(pX2) = condSd(pX1, pX2, kX)
                    group_prob(pX2) = thisProb(pX1, pX2, kX)
                end do

                call momentIntegrateGroup( NTYPE2, group_prob, group_mean, group_sd, &
                                           momMean(pX1, kX), momSd(pX1, kX) )

            end do
        end do

    end subroutine integrateCoupleMeanSd_F

    subroutine integrateCouple_KidsF( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)
        real(dp) :: group_mean(2 * NTYPE2)
        real(dp) :: group_prob(2 * NTYPE2)

        integer :: pX, pX1, pX2, kX

        thisProb = condProb * marketProb

        do pX1 = 1, NTYPE1

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    pX = pX2 + kX * NTYPE2
                    group_mean(pX) = condMean(pX1, pX2, kX)
                    group_prob(pX) = thisProb(pX1, pX2, kX)
                end do
            end do

            call momentIntegrateGroup( 2 * NTYPE2, group_prob, group_mean, momMean(pX1) )

        end do

    end subroutine integrateCouple_KidsF


    subroutine integrateCoupleMeanSd_KidsF( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1)
        real(dp), intent(out) :: momSd(NTYPE1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)
        real(dp) :: group_mean(2 * NTYPE2)
        real(dp) :: group_sd(2 * NTYPE2)
        real(dp) :: group_prob(2 * NTYPE2)

        integer :: pX, pX1, pX2, kX

        thisProb = condProb * marketProb

        do pX1 = 1, NTYPE1

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    pX = pX2 + kX * NTYPE2
                    group_mean(pX) = condMean(pX1, pX2, kX)
                    group_sd(pX) = condSd(pX1, pX2, kX)
                    group_prob(pX) = thisProb(pX1, pX2, kX)
                end do
            end do

            call momentIntegrateGroup( 2 * NTYPE2, group_prob, group_mean, group_sd, &
                                       momMean(pX1), momSd(pX1) )

        end do

    end subroutine integrateCoupleMeanSd_KidsF

    ! integrates over male type for couples allocations

    subroutine integrateCouple_M( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE2, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)
        real(dp) :: group_mean(NTYPE1)
        real(dp) :: group_prob(NTYPE1)

        integer :: pX1, pX2, kX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2

                do pX1 = 1, NTYPE1
                    group_mean(pX1) = condMean(pX1, pX2, kX)
                    group_prob(pX1) = thisProb(pX1, pX2, kX)
                end do

                call momentIntegrateGroup( NTYPE1, group_prob, group_mean, momMean(pX2, kX) )

            end do
        end do

    end subroutine integrateCouple_M

    ! integrates over male type for couples allocations (mean and s.d.)

    subroutine integrateCoupleMeanSd_M( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE2, 0:1)
        real(dp), intent(out) :: momSd(NTYPE2, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(NTYPE1)
        real(dp) :: group_sd(NTYPE1)
        real(dp) :: group_prob(NTYPE1)

        integer :: pX1, pX2, kX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2

                do pX1 = 1, NTYPE1
                    group_mean(pX1) = condMean(pX1, pX2, kX)
                    group_sd(pX1) = condSd(pX1, pX2, kX)
                    group_prob(pX1) = thisProb(pX1, pX2, kX)
                end do

                call momentIntegrateGroup( NTYPE1, group_prob, group_mean, group_sd, &
                                           momMean(pX2, kX), momSd(pX2, kX) )

            end do
        end do

    end subroutine integrateCoupleMeanSd_M

    subroutine integrateCouple_KidsM( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE2)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(2 * NTYPE1)
        real(dp) :: group_prob(2 * NTYPE1)

        integer :: pX, pX1, pX2, kX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2

            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    pX = pX1 + kX * NTYPE1
                    group_mean(pX) = condMean(pX1, pX2, kX)
                    group_prob(pX) = thisProb(pX1, pX2, kX)
                end do
            end do

            call momentIntegrateGroup( 2 * NTYPE1, group_prob, group_mean, momMean(pX2) )

        end do

    end subroutine integrateCouple_KidsM


    subroutine integrateCoupleMeanSd_KidsM( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE2)
        real(dp), intent(out) :: momSd(NTYPE2)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(2 * NTYPE1)
        real(dp) :: group_sd(2 * NTYPE1)
        real(dp) :: group_prob(2 * NTYPE1)

        integer :: pX, pX1, pX2, kX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2

            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    pX = pX1 + kX * NTYPE1
                    group_mean(pX) = condMean(pX1, pX2, kX)
                    group_sd(pX) = condSd(pX1, pX2, kX)
                    group_prob(pX) = thisProb(pX1, pX2, kX)
                end do
            end do

            call momentIntegrateGroup( 2 * NTYPE1, group_prob, group_mean, group_sd, &
                                       momMean(pX2), momSd(pX2) )

        end do

    end subroutine integrateCoupleMeanSd_KidsM

    subroutine integrateCoupleMeanSd_Kids( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1, NTYPE2)
        real(dp), intent(out) :: momSd(NTYPE1, NTYPE2)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(0:1)
        real(dp) :: group_sd(0:1)
        real(dp) :: group_prob(0:1)

        integer :: pX1, pX2, kX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1

                do kX = 0, 1
                    group_mean(kX) = condMean(pX1, pX2, kX)
                    group_sd(kX) = condSd(pX1, pX2, kX)
                    group_prob(kX) = thisProb(pX1, pX2, kX)
                end do

                call momentIntegrateGroup( 2, group_prob, group_mean, group_sd, &
                                           momMean(pX1, pX2), momSd(pX1, pX2) )

            end do
        end do

    end subroutine integrateCoupleMeanSd_Kids


    subroutine integrateCouple_Kids( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1, NTYPE2)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(0:1)
        real(dp) :: group_prob(0:1)

        integer :: pX1, pX2, kX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1

                do kX = 0, 1
                    group_mean(kX) = condMean(pX1, pX2, kX)
                    group_prob(kX) = thisProb(pX1, pX2, kX)
                end do

                call momentIntegrateGroup( 2, group_prob, group_mean, momMean(pX1, pX2) )

            end do
        end do

    end subroutine integrateCouple_Kids


    ! integrates over male type AND demale type for couples allocations (mean and s.d.)

    subroutine integrateCoupleMeanSd_MF( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(0:1)
        real(dp), intent(out) :: momSd(0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(NTYPE1 * NTYPE2)
        real(dp) :: group_sd(NTYPE1 * NTYPE2)
        real(dp) :: group_prob(NTYPE1 * NTYPE2)

        integer :: pX, pX1, pX2, kX

        thisProb = condProb * marketProb

        do kX = 0, 1

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX1 + (pX2 - 1) * NTYPE1
                    group_mean(pX) = condMean(pX1, pX2, kX)
                    group_sd(pX) = condSd(pX1, pX2, kX)
                    group_prob(pX) = thisProb(pX1, pX2, kX)
                end do
            end do

            call momentIntegrateGroup( NTYPE1 * NTYPE2, group_prob, group_mean, group_sd, &
                                       momMean(kX), momSd(kX) )

        end do

    end subroutine integrateCoupleMeanSd_MF

    ! integrates over male type AND demale type for couples allocations (mean and s.d.)

    subroutine integrateCouple_MF( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(NTYPE1 * NTYPE2)
        real(dp) :: group_prob(NTYPE1 * NTYPE2)

        integer :: pX, pX1, pX2, kX

        thisProb = condProb * marketProb

        do kX = 0, 1

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX1 + (pX2 - 1) * NTYPE1
                    group_mean(pX) = condMean(pX1, pX2, kX)
                    group_prob(pX) = thisProb(pX1, pX2, kX)
                end do
            end do

            call momentIntegrateGroup( NTYPE1 * NTYPE2, group_prob, group_mean, momMean(kX) )

        end do

    end subroutine integrateCouple_MF


    subroutine integrateSingleMeanSdM_M( marketProb, condProb, condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE1, 0:1)
        real(dp), intent(out) :: momMean(0:1)
        real(dp), intent(out) :: momSd(0:1)

        real(dp) :: thisProb(NTYPE1, 0:1)

        real(dp) :: group_mean(NTYPE1)
        real(dp) :: group_sd(NTYPE1)
        real(dp) :: group_prob(NTYPE1)

        integer :: kX, pX1

        thisProb = marketProb * condProb

        do kX = 0, 1

            do pX1 = 1, NTYPE1
                group_mean(pX1) = condMean(pX1, kX)
                group_sd(pX1) = condSd(pX1, kX)
                group_prob(pX1) = thisProb(pX1, kX)
            end do

            call momentIntegrateGroup( NTYPE1, group_prob, group_mean, group_sd, &
                                       momMean(kX), momSd(kX) )

        end do

    end subroutine integrateSingleMeanSdM_M

    subroutine integrateSingleM_Kids( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1)

        real(dp) :: thisProb(NTYPE1, 0:1)

        real(dp) :: group_mean(0:1)
        real(dp) :: group_prob(0:1)

        integer :: kX, pX1

        thisProb = marketProb * condProb

        do pX1 = 1, NTYPE1

            do kX = 0, 1
                group_mean(kX) = condMean(pX1, kX)
                group_prob(kX) = thisProb(pX1, kX)
            end do

            call momentIntegrateGroup( 2, group_prob, group_mean, momMean(pX1) )

        end do

    end subroutine integrateSingleM_Kids


    subroutine integrateSingleMeanSdM_Kids( marketProb, condProb, condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE1, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1)
        real(dp), intent(out) :: momSd(NTYPE1)

        real(dp) :: thisProb(NTYPE1, 0:1)

        real(dp) :: group_mean(0:1)
        real(dp) :: group_sd(0:1)
        real(dp) :: group_prob(0:1)

        integer :: kX, pX1

        thisProb = marketProb * condProb

        do pX1 = 1, NTYPE1

            do kX = 0, 1
                group_mean(kX) = condMean(pX1, kX)
                group_sd(kX) = condSd(pX1, kX)
                group_prob(kX) = thisProb(pX1, kX)
            end do

            call momentIntegrateGroup( 2, group_prob, group_mean, group_sd, &
                                       momMean(pX1), momSd(pX1) )

        end do

    end subroutine integrateSingleMeanSdM_Kids

    subroutine integrateSingleM_M( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1)
        real(dp), intent(out) :: momMean(0:1)

        real(dp) :: thisProb(NTYPE1, 0:1)

        real(dp) :: group_mean(NTYPE1)
        real(dp) :: group_prob(NTYPE1)

        integer :: kX, pX1

        thisProb = marketProb * condProb

        do kX = 0, 1

            do pX1 = 1, NTYPE1
                group_mean(pX1) = condMean(pX1, kX)
                group_prob(pX1) = thisProb(pX1, kX)
            end do

            call momentIntegrateGroup( NTYPE1, group_prob, group_mean, momMean(kX) )

        end do

    end subroutine integrateSingleM_M



    subroutine integrateSingleMeanSdF_F( marketProb, condProb, condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(0:1)
        real(dp), intent(out) :: momSd(0:1)

        real(dp) :: thisProb(NTYPE2, 0:1)

        real(dp) :: group_mean(NTYPE2)
        real(dp) :: group_sd(NTYPE2)
        real(dp) :: group_prob(NTYPE2)

        integer :: kX, pX2

        thisProb = marketProb * condProb

        do kX = 0, 1

            do pX2 = 1, NTYPE2
                group_mean(pX2) = condMean(pX2, kX)
                group_sd(pX2) = condSd(pX2, kX)
                group_prob(pX2) = thisProb(pX2, kX)
            end do

            call momentIntegrateGroup( NTYPE2, group_prob, group_mean, group_sd, &
                                       momMean(kX), momSd(kX) )

        end do

    end subroutine integrateSingleMeanSdF_F

    subroutine integrateSingleF_Kids( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE2)

        real(dp) :: thisProb(NTYPE2, 0:1)

        real(dp) :: group_mean(0:1)
        real(dp) :: group_prob(0:1)

        integer :: kX, pX2

        thisProb = marketProb * condProb

        do pX2 = 1, NTYPE2

            do kX = 0, 1
                group_mean(kX) = condMean(pX2, kX)
                group_prob(kX) = thisProb(pX2, kX)
            end do

            call momentIntegrateGroup( 2, group_prob, group_mean, momMean(pX2) )

        end do

    end subroutine integrateSingleF_Kids


    subroutine integrateSingleMeanSdF_Kids( marketProb, condProb, condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1)
        real(dp), intent(in)  :: condSd(NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE2)
        real(dp), intent(out) :: momSd(NTYPE2)

        real(dp) :: thisProb(NTYPE2, 0:1)

        real(dp) :: group_mean(0:1)
        real(dp) :: group_sd(0:1)
        real(dp) :: group_prob(0:1)

        integer :: kX, pX2

        thisProb = marketProb * condProb

        do pX2 = 1, NTYPE2

            do kX = 0, 1
                group_mean(kX) = condMean(pX2, kX)
                group_sd(kX) = condSd(pX2, kX)
                group_prob(kX) = thisProb(pX2, kX)
            end do

            call momentIntegrateGroup( 2, group_prob, group_mean, group_sd, &
                                       momMean(pX2), momSd(pX2) )

        end do

    end subroutine integrateSingleMeanSdF_Kids


    subroutine integrateSingleF_F( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(0:1)

        real(dp) :: thisProb(NTYPE2, 0:1)

        real(dp) :: group_mean(NTYPE2)
        real(dp) :: group_prob(NTYPE2)

        integer :: kX, pX2

        thisProb = marketProb * condProb

        do kX = 0, 1

            do pX2 = 1, NTYPE2
                group_mean(pX2) = condMean(pX2, kX)
                group_prob(pX2) = thisProb(pX2, kX)
            end do

            call momentIntegrateGroup( NTYPE2, group_prob, group_mean, momMean(kX) )

        end do

    end subroutine integrateSingleF_F

    subroutine integrateSingleF_Null( condMean, momMean )

        implicit none

        real(dp), intent(in)  :: condMean(NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE2, 0:1)

        momMean = condMean

    end subroutine integrateSingleF_Null

    subroutine integrateSingleMeanSdF_Null( condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: condMean(NTYPE2, 0:1)
        real(dp), intent(in)  :: condsd(NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE2, 0:1)
        real(dp), intent(out) :: momSd(NTYPE2, 0:1)

        momMean = condMean
        momSd = condSd

    end subroutine integrateSingleMeanSdF_Null

    subroutine integrateSingleM_Null( condMean, momMean )

        implicit none

        real(dp), intent(in)  :: condMean(NTYPE1, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1, 0:1)

        momMean = condMean

    end subroutine integrateSingleM_Null

    subroutine integrateSingleMeanSdM_Null( condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: condMean(NTYPE1, 0:1)
        real(dp), intent(in)  :: condsd(NTYPE1, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1, 0:1)
        real(dp), intent(out) :: momSd(NTYPE1, 0:1)

        momMean = condMean
        momSd = condSd

    end subroutine integrateSingleMeanSdM_Null

    subroutine integrateCouple_Null( condMean, momMean )

        implicit none

        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1, NTYPE2, 0:1)

        momMean = condMean

    end subroutine integrateCouple_Null

    subroutine integrateCoupleMeanSd_Null( condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condsd(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momSd(NTYPE1, NTYPE2, 0:1)

        momMean = condMean
        momSd = condSd

    end subroutine integrateCoupleMeanSd_Null


    subroutine momentIntegrateGroupMeanSd( n, frac, group_mean, group_sd, total_mean, total_sd )
        implicit none
        integer,  intent(in)  :: n
        real(dp), intent(in)  :: frac(n)
        real(dp), intent(in)  :: group_mean(n)
        real(dp), intent(in)  :: group_sd(n)
        real(dp), intent(out) :: total_mean
        real(dp), intent(out) :: total_sd

        real(dp) :: group_var(n), total_var
        real(dp) :: sumfrac

        sumfrac = sum(frac)

        if (sumfrac > 0.0_dp) then

            group_var = group_sd ** 2
            total_var = sum(frac * group_var) / sumfrac
            total_mean = sum(group_mean * frac) / sumfrac
            total_var = total_var + sum(frac * (group_mean - total_mean) ** 2) / sumfrac
            total_sd = sqrt(total_var)

        else

            total_mean = 0.0_dp
            total_sd = 0.0_dp

        end if

    end subroutine momentIntegrateGroupMeanSd

    subroutine momentIntegrateGroupMean( n, frac, group_mean, total_mean )
        implicit none
        integer,  intent(in)  :: n
        real(dp), intent(in)  :: frac(n)
        real(dp), intent(in)  :: group_mean(n)
        real(dp), intent(out) :: total_mean
        real(dp) :: sumfrac
        sumfrac = sum(frac)
        if (sumfrac > 0.0_dp) then
            total_mean = sum(group_mean * frac) / sumfrac
        else
            total_mean = 0.0_dp
        end if
    end subroutine momentIntegrateGroupMean

    subroutine integrateSingleMeanSdM_Market( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1, 0:1)
        real(dp), intent(out) :: momSd(NTYPE1, 0:1)

        real(dp) :: thisProb(NTYPE1, 0:1, NMARKET)
        real(dp) :: group_mean(NMARKET), group_sd(NMARKET), group_prob(NMARKET)

        integer :: kX, pX1, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX1 = 1, NTYPE1

                do mX = 1, NMARKET
                    group_mean(mX) = condMean(pX1, kX, mX)
                    group_sd(mX) = condSd(pX1, kX, mX)
                    group_prob(mX) = thisProb(pX1, kX, mX)
                end do

                call momentIntegrateGroup( NMARKET, group_prob, group_mean, group_sd, momMean(pX1, kX), momSd(pX1, kX) )

            end do
        end do

    end subroutine integrateSingleMeanSdM_Market

    subroutine integrateSingleMeanSdM_KidsMarket( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1)
        real(dp), intent(out) :: momSd(NTYPE1)

        real(dp) :: thisProb(NTYPE1, 0:1, NMARKET)
        real(dp) :: group_mean(2 * NMARKET), group_sd(2 * NMARKET), group_prob(2 * NMARKET)

        integer :: kX, pX1, mX, pX

        thisProb = condProb * marketProb

        do pX1 = 1, NTYPE1

            do kX = 0, 1
                do mX = 1, NMARKET
                    pX = mX + kX * NMARKET
                    group_mean(pX) = condMean(pX1, kX, mX)
                    group_sd(pX) = condSd(pX1, kX, mX)
                    group_prob(pX) = thisProb(pX1, kX, mX)
                end do
            end do

            call momentIntegrateGroup( 2 * NMARKET, group_prob, group_mean, group_sd, momMean(pX1), momSd(pX1) )

        end do

    end subroutine integrateSingleMeanSdM_KidsMarket

    subroutine integrateSingleM_KidsMarket( marketProb, condProb, condMean, momMean )
        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1)

        real(dp) :: thisProb(NTYPE1, 0:1, NMARKET)
        real(dp) :: group_mean(2 * NMARKET), group_prob(2 * NMARKET)

        integer :: kX, pX1, mX, pX

        thisProb = condProb * marketProb

        do pX1 = 1, NTYPE1

            do kX = 0, 1
                do mX = 1, NMARKET
                    pX = mX + kX * NMARKET
                    group_mean(pX) = condMean(pX1, kX, mX)
                    group_prob(pX) = thisProb(pX1, kX, mX)
                end do
            end do

            call momentIntegrateGroup( 2 * NMARKET, group_prob, group_mean, momMean(pX1) )

        end do

    end subroutine integrateSingleM_KidsMarket


    subroutine integrateSingleM_Market( marketProb, condProb, condMean, momMean )
        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1, 0:1)

        real(dp) :: thisProb(NTYPE1, 0:1, NMARKET)
        real(dp) :: group_mean(NMARKET), group_prob(NMARKET)

        integer :: kX, pX1, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX1 = 1, NTYPE1

                do mX = 1, NMARKET
                    group_mean(mX) = condMean(pX1, kX, mX)
                    group_prob(mX) = thisProb(pX1, kX, mX)
                end do

                call momentIntegrateGroup( NMARKET, group_prob, group_mean, momMean(pX1, kX) )

            end do
        end do

    end subroutine integrateSingleM_Market

    subroutine integrateSingleMeanSdF_Market( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE2, 0:1)
        real(dp), intent(out) :: momSd(NTYPE2, 0:1)

        real(dp) :: thisProb(NTYPE2, 0:1, NMARKET)
        real(dp) :: group_mean(NMARKET), group_sd(NMARKET), group_prob(NMARKET)

        integer :: kX, pX2, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2

                do mX = 1, NMARKET
                    group_mean(mX) = condMean(pX2, kX, mX)
                    group_sd(mX) = condSd(pX2, kX, mX)
                    group_prob(mX) = thisProb(pX2, kX, mX)
                end do

                call momentIntegrateGroup( NMARKET, group_prob, group_mean, group_sd, momMean(pX2, kX), momSd(pX2, kX) )

            end do
        end do

    end subroutine integrateSingleMeanSdF_Market

    subroutine integrateSingleMeanSdF_KidsMarket( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE2)
        real(dp), intent(out) :: momSd(NTYPE2)

        real(dp) :: thisProb(NTYPE2, 0:1, NMARKET)
        real(dp) :: group_mean(2 * NMARKET), group_sd(2 * NMARKET), group_prob(2 * NMARKET)

        integer :: kX, pX2, mX, pX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2

            do kX = 0, 1
                do mX = 1, NMARKET
                    pX = mX + kX * NMARKET
                    group_mean(pX) = condMean(pX2, kX, mX)
                    group_sd(pX) = condSd(pX2, kX, mX)
                    group_prob(pX) = thisProb(pX2, kX, mX)
                end do

            end do

            call momentIntegrateGroup( 2 * NMARKET, group_prob, group_mean, group_sd, momMean(pX2), momSd(pX2) )

        end do

    end subroutine integrateSingleMeanSdF_KidsMarket

    subroutine integrateSingleF_KidsMarket( marketProb, condProb, condMean, momMean )
        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE2)

        real(dp) :: thisProb(NTYPE2, 0:1, NMARKET)
        real(dp) :: group_mean(2 * NMARKET), group_prob(2 * NMARKET)

        integer :: kX, pX2, mX, pX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2

            do kX = 0, 1
                do mX = 1, NMARKET
                    pX = mX + kX * NMARKET
                    group_mean(pX) = condMean(pX2, kX, mX)
                    group_prob(pX) = thisProb(pX2, kX, mX)
                end do
            end do

            call momentIntegrateGroup( 2 * NMARKET, group_prob, group_mean, momMean(pX2) )

        end do

    end subroutine integrateSingleF_KidsMarket

    subroutine integrateSingleF_Market( marketProb, condProb, condMean, momMean )
        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE2, 0:1)

        real(dp) :: thisProb(NTYPE2, 0:1, NMARKET)
        real(dp) :: group_mean(NMARKET), group_prob(NMARKET)

        integer :: kX, pX2, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2

                do mX = 1, NMARKET
                    group_mean(mX) = condMean(pX2, kX, mX)
                    group_prob(mX) = thisProb(pX2, kX, mX)
                end do

                call momentIntegrateGroup( NMARKET, group_prob, group_mean, momMean(pX2, kX) )

            end do
        end do

    end subroutine integrateSingleF_Market

    subroutine integrateCoupleMeanSd_Market( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momSd(NTYPE1, NTYPE2, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NMARKET)
        real(dp) :: group_sd(NMARKET)
        real(dp) :: group_prob(NMARKET)

        integer :: kX, pX1, pX2, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1

                    do mX = 1, NMARKET
                        group_mean(mX) = condMean(pX1, pX2, kX, mX)
                        group_sd(mX) = condSd(pX1, pX2, kX, mX)
                        group_prob(mX) = thisProb(pX1, pX2, kX, mX)
                    end do

                    call momentIntegrateGroup( NMARKET, group_prob, group_mean, group_sd, &
                                               momMean(pX1, pX2, kX), momSd(pX1, pX2, kX) )

                end do
            end do
        end do

    end subroutine integrateCoupleMeanSd_Market

    subroutine integrateCoupleMeanSd_KidsMarket( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1, NTYPE2)
        real(dp), intent(out) :: momSd(NTYPE1, NTYPE2)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(2 * NMARKET)
        real(dp) :: group_sd(2 * NMARKET)
        real(dp) :: group_prob(2 * NMARKET)

        integer :: kX, pX1, pX2, mX, pX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1

                do kX = 0, 1
                    do mX = 1, NMARKET
                        pX = mX + kX * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_sd(pX) = condSd(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do

                call momentIntegrateGroup( 2 * NMARKET, group_prob, group_mean, group_sd, &
                                           momMean(pX1, pX2), momSd(pX1, pX2) )

            end do
        end do

    end subroutine integrateCoupleMeanSd_KidsMarket

    subroutine integrateCouple_KidsMarket( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1, NTYPE2)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(2 * NMARKET)
        real(dp) :: group_prob(2 * NMARKET)

        integer :: kX, pX1, pX2, mX, pX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1

                do kX = 0, 1
                    do mX = 1, NMARKET
                        pX = mX + kX * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do

                call momentIntegrateGroup( 2 * NMARKET, group_prob, group_mean, momMean(pX1, pX2) )

            end do
        end do

    end subroutine integrateCouple_KidsMarket

    subroutine integrateCouple_Market( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1, NTYPE2, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NMARKET)
        real(dp) :: group_prob(NMARKET)

        integer :: kX, pX1, pX2, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1

                    do mX = 1, NMARKET
                        group_mean(mX) = condMean(pX1, pX2, kX, mX)
                        group_prob(mX) = thisProb(pX1, pX2, kX, mX)
                    end do

                    call momentIntegrateGroup( NMARKET, group_prob, group_mean, momMean(pX1, pX2, kX) )

                end do
            end do
        end do

    end subroutine integrateCouple_Market


    ! integrates over market AND female type for couples allocations (mean and s.d.)

    subroutine integrateCoupleMeanSd_MarketF( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1, 0:1)
        real(dp), intent(out) :: momSd(NTYPE1, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NMARKET * NTYPE2)
        real(dp) :: group_sd(NMARKET * NTYPE2)
        real(dp) :: group_prob(NMARKET * NTYPE2)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX1 = 1, NTYPE1

                do pX2 = 1, NTYPE2
                    do mX = 1, NMARKET
                        pX = mX + (pX2 - 1) * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_sd(pX) = condSd(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do

                call momentIntegrateGroup( NMARKET * NTYPE2, group_prob, group_mean, group_sd, &
                                           momMean(pX1, kX), momSd(pX1, kX) )

            end do
        end do

    end subroutine integrateCoupleMeanSd_MarketF

    subroutine integrateCoupleMeanSd_KidsMarketF( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1)
        real(dp), intent(out) :: momSd(NTYPE1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(2 * NMARKET * NTYPE2)
        real(dp) :: group_sd(2 * NMARKET * NTYPE2)
        real(dp) :: group_prob(2 * NMARKET * NTYPE2)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do pX1 = 1, NTYPE1

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do mX = 1, NMARKET
                        pX = mX + (pX2 - 1) * NMARKET + kX * NMARKET * NTYPE2
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_sd(pX) = condSd(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do
            end do

            call momentIntegrateGroup( 2 * NMARKET * NTYPE2, group_prob, group_mean, group_sd, &
                                       momMean(pX1), momSd(pX1) )

        end do

    end subroutine integrateCoupleMeanSd_KidsMarketF


    subroutine integrateCouple_MarketF( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE1, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NMARKET * NTYPE2)
        real(dp) :: group_prob(NMARKET * NTYPE2)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX1 = 1, NTYPE1

                do pX2 = 1, NTYPE2
                    do mX = 1, NMARKET
                        pX = mX + (pX2 - 1) * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do

                call momentIntegrateGroup( NMARKET * NTYPE2, group_prob, group_mean, momMean(pX1, kX) )

            end do
        end do

    end subroutine integrateCouple_MarketF



    subroutine integrateCoupleMeanSd_MarketM( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE2, 0:1)
        real(dp), intent(out) :: momSd(NTYPE2, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NMARKET * NTYPE1)
        real(dp) :: group_sd(NMARKET * NTYPE1)
        real(dp) :: group_prob(NMARKET * NTYPE1)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2

                do pX1 = 1, NTYPE1
                    do mX = 1, NMARKET
                        pX = mX + (pX1 - 1) * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_sd(pX) = condSd(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do

                call momentIntegrateGroup( NMARKET * NTYPE1, group_prob, group_mean, group_sd, &
                                           momMean(pX2, kX), momSd(pX2, kX) )

            end do
        end do

    end subroutine integrateCoupleMeanSd_MarketM

    subroutine integrateCoupleMeanSd_KidsMarketM( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE2)
        real(dp), intent(out) :: momSd(NTYPE2)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(2 * NMARKET * NTYPE1)
        real(dp) :: group_sd(2 * NMARKET * NTYPE1)
        real(dp) :: group_prob(2 * NMARKET * NTYPE1)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do pX2 = 1, NTYPE2

            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    do mX = 1, NMARKET
                        pX = mX + (pX1 - 1) * NMARKET + kX * NMARKET * NTYPE1
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_sd(pX) = condSd(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do
            end do

            call momentIntegrateGroup( 2 * NMARKET * NTYPE1, group_prob, group_mean, group_sd, &
                                       momMean(pX2), momSd(pX2) )

        end do

    end subroutine integrateCoupleMeanSd_KidsMarketM


    subroutine integrateCoupleMeanSd_KidsMarketMF( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean
        real(dp), intent(out) :: momSd

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(2 * NMARKET * NTYPE1 * NTYPE2)
        real(dp) :: group_sd(2 * NMARKET * NTYPE1 * NTYPE2)
        real(dp) :: group_prob(2 * NMARKET * NTYPE1 * NTYPE2)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        pX = 0
        do mX = 1, NMARKET
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        pX = pX + 1
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_sd(pX) = condSd(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do
            end do
        end do

        call momentIntegrateGroup( 2 * NMARKET * NTYPE1 * NTYPE2, group_prob, group_mean, group_sd, &
                                   momMean, momSd )

    end subroutine integrateCoupleMeanSd_KidsMarketMF



    subroutine integrateCouple_MarketM( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(NTYPE2, 0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NMARKET * NTYPE1)
        real(dp) :: group_prob(NMARKET * NTYPE1)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2

                do pX1 = 1, NTYPE1
                    do mX = 1, NMARKET
                        pX = mX + (pX1 - 1) * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do

                call momentIntegrateGroup( NMARKET * NTYPE1, group_prob, group_mean, momMean(pX2, kX) )

            end do
        end do

    end subroutine integrateCouple_MarketM

    subroutine integrateSingleMeanSdM_MarketM( marketProb, condProb, condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(0:1)
        real(dp), intent(out) :: momSd(0:1)

        real(dp) :: thisProb(NTYPE1, 0:1, NMARKET)

        real(dp) :: group_mean(NTYPE1 * NMARKET)
        real(dp) :: group_sd(NTYPE1 * NMARKET)
        real(dp) :: group_prob(NTYPE1 * NMARKET)

        integer :: kX, pX, pX1, mX

        thisProb = marketProb * condProb

        do kX = 0, 1

            do pX1 = 1, NTYPE1
                do mX = 1, NMARKET
                    pX = mX + (pX1 - 1) * NMARKET
                    group_mean(pX) = condMean(pX1, kX, mX)
                    group_sd(pX) = condSd(pX1, kX, mX)
                    group_prob(pX) = thisProb(pX1, kX, mX)
                end do
            end do

            call momentIntegrateGroup( NTYPE1 * NMARKET, group_prob, group_mean, group_sd, &
                                       momMean(kX), momSd(kX) )

        end do

    end subroutine integrateSingleMeanSdM_MarketM

    subroutine integrateSingleM_MarketM( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(0:1)

        real(dp) :: thisProb(NTYPE1, 0:1, NMARKET)

        real(dp) :: group_mean(NTYPE1 * NMARKET)
        real(dp) :: group_prob(NTYPE1 * NMARKET)

        integer :: kX, pX, pX1, mX

        thisProb = marketProb * condProb

        do kX = 0, 1

            do pX1 = 1, NTYPE1
                do mX = 1, NMARKET
                    pX = mX + (pX1 - 1) * NMARKET
                    group_mean(pX) = condMean(pX1, kX, mX)
                    group_prob(pX) = thisProb(pX1, kX, mX)
                end do
            end do

            call momentIntegrateGroup( NTYPE1 * NMARKET, group_prob, group_mean, momMean(kX) )

        end do

    end subroutine integrateSingleM_MarketM


    subroutine integrateSingleMeanSdF_MarketF( marketProb, condProb, condMean, condSd, momMean, momSd )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(0:1)
        real(dp), intent(out) :: momSd(0:1)

        real(dp) :: thisProb(NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NTYPE2 * NMARKET)
        real(dp) :: group_sd(NTYPE2 * NMARKET)
        real(dp) :: group_prob(NTYPE2 * NMARKET)

        integer :: kX, pX, pX2, mX

        thisProb = marketProb * condProb

        do kX = 0, 1

            do pX2 = 1, NTYPE2
                do mX = 1, NMARKET
                    pX = mX + (pX2 - 1) * NMARKET
                    group_mean(pX) = condMean(pX2, kX, mX)
                    group_sd(pX) = condSd(pX2, kX, mX)
                    group_prob(pX) = thisProb(pX2, kX, mX)
                end do
            end do

            call momentIntegrateGroup( NTYPE2 * NMARKET, group_prob, group_mean, group_sd, &
                                       momMean(kX), momSd(kX) )

        end do

    end subroutine integrateSingleMeanSdF_MarketF

    subroutine integrateSingleF_MarketF( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(0:1)

        real(dp) :: thisProb(NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NTYPE2 * NMARKET)
        real(dp) :: group_prob(NTYPE2 * NMARKET)

        integer :: kX, pX, pX2, mX

        thisProb = marketProb * condProb

        do kX = 0, 1

            do pX2 = 1, NTYPE2
                do mX = 1, NMARKET
                    pX = mX + (pX2 - 1) * NMARKET
                    group_mean(pX) = condMean(pX2, kX, mX)
                    group_prob(pX) = thisProb(pX2, kX, mX)
                end do
            end do

            call momentIntegrateGroup( NTYPE2 * NMARKET, group_prob, group_mean, momMean(kX) )

        end do

    end subroutine integrateSingleF_MarketF

    subroutine integrateCoupleMeanSd_MarketMF( marketProb, condProb, condMean, condSd, momMean, momSd )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condSd(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(0:1)
        real(dp), intent(out) :: momSd(0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NTYPE1 * NTYPE2 * NMARKET)
        real(dp) :: group_sd(NTYPE1 * NTYPE2 * NMARKET)
        real(dp) :: group_prob(NTYPE1 * NTYPE2 * NMARKET)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do kX = 0, 1

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do mX = 1, NMARKET
                        pX = mX + (pX1 - 1) * NMARKET + (pX2 - 1) * NTYPE1 * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_sd(pX) = condSd(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do
            end do

            call momentIntegrateGroup( NTYPE1 * NTYPE2 * NMARKET, group_prob, group_mean, group_sd, &
                                       momMean(kX), momSd(kX) )

        end do

    end subroutine integrateCoupleMeanSd_MarketMF


    subroutine integrateCouple_MarketMF( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean(0:1)

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(NTYPE1 * NTYPE2 * NMARKET)
        real(dp) :: group_prob(NTYPE1 * NTYPE2 * NMARKET)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do kX = 0, 1

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do mX = 1, NMARKET
                        pX = mX + (pX1 - 1) * NMARKET + (pX2 - 1) * NTYPE1 * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do
            end do

            call momentIntegrateGroup( NTYPE1 * NTYPE2 * NMARKET, group_prob, group_mean, momMean(kX) )

        end do

    end subroutine integrateCouple_MarketMF

    subroutine integrateCouple_KidsMarketMF( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(2 * NTYPE1 * NTYPE2 * NMARKET)
        real(dp) :: group_prob(2 * NTYPE1 * NTYPE2 * NMARKET)

        integer :: pX, pX1, pX2, kX, mX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do mX = 1, NMARKET
                        pX = mX + (pX1 - 1) * NMARKET + (pX2 - 1) * NTYPE1 * NMARKET + kX * NTYPE1 * NTYPE2 * NMARKET
                        group_mean(pX) = condMean(pX1, pX2, kX, mX)
                        group_prob(pX) = thisProb(pX1, pX2, kX, mX)
                    end do
                end do
            end do
        end do

        call momentIntegrateGroup( 2 * NTYPE1 * NTYPE2 * NMARKET, group_prob, group_mean, momMean )

    end subroutine integrateCouple_KidsMarketMF

    subroutine integrateCouple_KidsMF( marketProb, condProb, condMean, momMean )
        implicit none
        real(dp), intent(in)  :: marketProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, NTYPE2, 0:1)
        real(dp), intent(out) :: momMean

        real(dp) :: thisProb(NTYPE1, NTYPE2, 0:1)

        real(dp) :: group_mean(2 * NTYPE1 * NTYPE2)
        real(dp) :: group_prob(2 * NTYPE1 * NTYPE2)

        integer :: pX, pX1, pX2, kX

        thisProb = condProb * marketProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX1 + (pX2 - 1) * NTYPE1 + kX * NTYPE1 * NTYPE2
                    group_mean(pX) = condMean(pX1, pX2, kX)
                    group_prob(pX) = thisProb(pX1, pX2, kX)
                end do
            end do
        end do

        call momentIntegrateGroup( 2 * NTYPE1 * NTYPE2, group_prob, group_mean, momMean )

    end subroutine integrateCouple_KidsMF


    subroutine integrateSingleF_KidsMarketF( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1, NMARKET)
        real(dp), intent(out) :: momMean

        real(dp) :: thisProb(NTYPE2, 0:1, NMARKET)

        real(dp) :: group_mean(2 * NTYPE2 * NMARKET)
        real(dp) :: group_prob(2 * NTYPE2 * NMARKET)

        integer :: kX, pX, pX2, mX

        thisProb = marketProb * condProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do mX = 1, NMARKET
                    pX = mX + (pX2 - 1) * NMARKET + kX * NTYPE2 * NMARKET
                    group_mean(pX) = condMean(pX2, kX, mX)
                    group_prob(pX) = thisProb(pX2, kX, mX)
                end do
            end do
        end do

        call momentIntegrateGroup( 2 * NTYPE2 * NMARKET, group_prob, group_mean, momMean )

    end subroutine integrateSingleF_KidsMarketF

    subroutine integrateSingleF_KidsF( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE2, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE2, 0:1)
        real(dp), intent(out) :: momMean

        real(dp) :: thisProb(NTYPE2, 0:1)

        real(dp) :: group_mean(2 * NTYPE2)
        real(dp) :: group_prob(2 * NTYPE2)

        integer :: kX, pX, pX2

        thisProb = marketProb * condProb

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                pX = pX2 + kX * NTYPE2
                group_mean(pX) = condMean(pX2, kX)
                group_prob(pX) = thisProb(pX2, kX)
            end do
        end do

        call momentIntegrateGroup( 2 * NTYPE2, group_prob, group_mean, momMean )

    end subroutine integrateSingleF_KidsF


    subroutine integrateSingleM_KidsMarketM( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1, NMARKET)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1, NMARKET)
        real(dp), intent(out) :: momMean

        real(dp) :: thisProb(NTYPE1, 0:1, NMARKET)

        real(dp) :: group_mean(2 * NTYPE1 * NMARKET)
        real(dp) :: group_prob(2 * NTYPE1 * NMARKET)

        integer :: kX, pX, pX1, mX

        thisProb = marketProb * condProb

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do mX = 1, NMARKET
                    pX = mX + (pX1 - 1) * NMARKET + kX * NTYPE1 * NMARKET
                    group_mean(pX) = condMean(pX1, kX, mX)
                    group_prob(pX) = thisProb(pX1, kX, mX)
                end do
            end do
        end do

        call momentIntegrateGroup( 2 * NTYPE1 * NMARKET, group_prob, group_mean, momMean )

    end subroutine integrateSingleM_KidsMarketM

    subroutine integrateSingleM_KidsM( marketProb, condProb, condMean, momMean )

        implicit none

        real(dp), intent(in)  :: marketProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condProb(NTYPE1, 0:1)
        real(dp), intent(in)  :: condMean(NTYPE1, 0:1)
        real(dp), intent(out) :: momMean

        real(dp) :: thisProb(NTYPE1, 0:1)

        real(dp) :: group_mean(2 * NTYPE1)
        real(dp) :: group_prob(2 * NTYPE1)

        integer :: kX, pX, pX1

        thisProb = marketProb * condProb

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                pX = pX1 + kX * NTYPE1
                group_mean(pX) = condMean(pX1, kX)
                group_prob(pX) = thisProb(pX1, kX)
            end do
        end do

        call momentIntegrateGroup( 2 * NTYPE1, group_prob, group_mean, momMean )

    end subroutine integrateSingleM_KidsM

end module momentIntegrate
