program optimal_level_fixed_mpi

    use globaldef
    use mathfn
    use test3kid
    use taxes
#   ifdef _MPI_
    use mpi
#   endif

    implicit none

    logical :: empiricalMarriageDist

    ! parameters for optimisation routine
    integer, parameter :: nmax = NTAXPARAM + NTYPEALL
    integer, parameter :: nclmax = MAXVALNTAXCON
    integer, parameter :: ncnmax = NTYPEALL + 1

    integer, parameter :: lda = max(1, nclmax), ldcj = max(1, ncnmax), ldr = nmax
    integer, parameter :: liwork = 3 * nmax + nclmax + 2 * ncnmax
    integer, parameter :: lwork = 2 * nmax ** 2 + nmax * nclmax + 2 * nmax * ncnmax + 21 * nmax + 11 * nclmax + 22 * ncnmax + 1

    real(dp) :: x(nmax)                           ! tax parameters
    real(dp) :: xs(nmax)                          ! tax parameters (saved from primal)
    real(dp) :: xest(NPARAM + NMARKET * NTYPEALL) ! estimation parameters
    real(dp) :: xparam(NPARAM)                    ! preference parameters
    integer  :: xtype(NPARAM)
    real(dp) :: xtax(NTAXPARAM)                   ! tax parameters
    real(dp) :: xlam(NTYPEALL)                    ! parteo weights
    real(dp) :: xlamMarket(NMARKET * NTYPEALL)    ! parteo weights for each market
    real(dp) :: eqcon(NTYPEALL)                   ! constraint violation

    real(dp) :: objf
    real(dp) :: xold(nmax) ! previous parameter vector
    real(dp) :: z(nmax)    ! perturbed parameter vector
    integer :: ifail, irevcm, iter, nclin, ncnln
    integer :: iX, jX
    integer :: pX, pX1, pX2

    real(dp) :: a(lda, nmax), bl(nmax + nclmax + ncnmax), bu(nmax + nclmax + ncnmax), c(max(1, ncnmax)),     &
                cjac(ldcj, nmax), clamda(nmax + nclmax + ncnmax), objgrd(nmax), r(ldr, nmax), work(lwork)

    integer :: istate(nmax + nclmax + ncnmax), iwork(liwork), needc(ncnmax)

    real(dp) :: f_temp(0:nmax)
    real(dp) :: c_temp(0:nmax)
    real(dp) :: epsrel
    real(dp) :: f0, c0(1)
    integer  :: ixL0, ixL1
    integer  :: ixParam
    real(dp) :: normscale

    real(dp), parameter :: eps = 1.0e-8_dp

    real(dp) :: f, swf, rev, targetRev, targetSwf
    logical  :: verbose
    real(dp) :: ascaled(lda, nmax)

    integer :: narg, istat, funit, funit2
    character(len = 32) :: filetag, thisarg, thetastr
    character(len = 128) :: fname
    character(len = 128) :: fnamesol
    character(len = 128) :: fnamerev
    character(len = 128) :: fnamelam
    character(len = 128) :: fnamemmf
    character(len = 128) :: fnamenoneq
    character(len = 128) :: fnametaxsol
    character(len = 128) :: fnametaxsol_f
    character(len = 128) :: fnametaxinp
    character(len = 128) :: fnameweight
    character(len = 128) :: fsavepath
    character(len = :), allocatable :: fixedpath

    type(opt_t) :: opt
    type(ret_t) :: ret

    ! for mpi
    integer :: ierror, id, nproc

    ! control aversion to inequality
    real(dp) :: theta
    integer :: thetaint

    integer :: thistax,  ntax, nall

    ! fixed parameters
    integer :: thistax_fixed, thistax_input
    real(dp) :: theta_fixed, theta_input
    real(dp) :: swf_fixed, swf_input
    real(dp) :: rev_fixed, rev_input

    real(dp) :: xmarriage(NTYPE1 + NTYPE2 + NTYPEALL)

    ! temporary
    integer :: tempint
    integer :: tempreal
    integer :: mX_d, pX1_d, pX2_d
    integer :: estmode
    integer :: mmfmode

    ! specify the restriction mode
    thistax = 1
    theta = 0.0_dp
    id = 0

    narg = command_argument_count()

    if (narg == 4) then


        call get_command_argument(1, value = filetag, status = istat)
        call get_command_argument(2, value = thisarg, status = istat)
        read(thisarg, '(i10)') thistax
        call get_command_argument(3, value = thetaStr, status = istat)
        read(thetaStr, '(i10)') thetaint
        call get_command_argument(4, value = thisarg, status = istat)
        read(thisarg, '(i10)') estmode

        theta = -real(thetaint, dp) / 10.0_dp

        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        fnamerev = 'output/post/' // trim(adjustl(filetag)) // '/revenue.asc'
        fnamelam = 'output/post/' // trim(adjustl(filetag)) // '/weight.csv'
        fnamemmf = 'output/post/' // trim(adjustl(filetag)) // '/marriage.csv'
        fnametaxinp = 'output/simulation/' // trim(adjustl(filetag)) // '/primal' // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_sol.asc'

        fname = 'output/simulation/' // trim(adjustl(filetag)) // '/fixed' // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_level.log'
        fnametaxsol = 'output/simulation/' // trim(adjustl(filetag)) // '/fixed' // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_sol.asc'
        fnametaxsol_f = 'output/simulation/' // trim(adjustl(filetag)) // '/fixed' // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_sol_f.asc'
        fnamenoneq = 'output/simulation/' // trim(adjustl(filetag)) // '/fixed' // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_noneq.asc'
        fsavepath = 'output/simulation/' // trim(adjustl(filetag)) // '/fixed' // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/'

        ! INPUT

        ! fnamesol: x vector from estimation
        ! fnamerev: revenue target
        ! fnamelam: empirical weights
        ! fnamemmf: empirical marriage matching function
        ! fnametaxinp: this provides the initial values (from primal problem)

        ! OUTPUT

        ! fname: log file from simulation (keeps track of all function evaluations)
        ! fnametaxsol: this saves the final solution (equilibrium)
        ! fnametaxsol_f: this saves the final solution (out-of-equilibrium)
        ! fnamenoneq: marriage market imbalance
        ! fsavepath: this saves output pertaining to the allocation at the solution

    else
        stop 'incorrect number of command arguments supplied'
    end if



    select case(estmode)
    case(statemode)
        print *, 'estmode = statemode'
        print *
    case(mcdmode)
        print *, 'estmode = mcdmode'
        print *
    case default
    end select

    ! size of problem
    ntax = NTAXRES(thistax)
    nall = ntax

    ! START MPI COMMUNICATION
#   ifdef _MPI_
    call mpi_init(ierror)

    !obtain id for each node
    call mpi_comm_rank(mpi_comm_world, id, ierror)

    !returns the number of nodes
    call mpi_comm_size(mpi_comm_world, nproc, ierror)

    if (nproc /= nall + 1) then
        write (*,*) nall + 1, 'processors required'
        call mpi_finalize(ierror)
        stop
    end if
#   endif

    ! INPUT

    ! load estimation parameter vector
    open(newunit = funit, file = fnamesol, status = 'old', action = 'read')
    do iX = 1, NPARAM + NTYPEALL * NMARKET
        read(funit, *) xest(iX)
    end do
    close(funit)

    ! parameter vector for model
    xparam = xest(1:NPARAM)

    ! weights from estimation
    xlamMarket = xest(NPARAM + 1 : NPARAM + NTYPEALL * NMARKET)

    ! load revenue target
    open(newunit = funit, file = fnamerev, status = 'old', action = 'read')
    read(funit, *) targetRev
    close(funit)

    ! load the pareto vector from estimated model
    open(newunit = funit, file = fnamelam, status = 'old', action = 'read')
    pX = 0
    do pX2 = 1, NTYPE2
        do pX1 = 1, NTYPE1
            pX = pX + 1
            read(funit, *) mX_d, pX1_d, pX2_d, xlam(pX)
        end do
    end do
    close(funit)

    ! load the mmf from estimated model
    open(newunit = funit, file = fnamemmf, status = 'old', action = 'read')
    pX = 0
    do pX1 = 1, NTYPE1
        pX = pX + 1
        read(funit, *) mX_d, pX1_d, pX2_d, xmarriage(pX)
    end do

    do pX2 = 1, NTYPE2
        pX = pX + 1
        read(funit, *) mX_d, pX1_d, pX2_d, xmarriage(pX)
    end do

    do pX2 = 1, NTYPE2
        do pX1 = 1, NTYPE1
            pX = pX + 1
            read(funit, *) mX_d, pX1_d, pX2_d, xmarriage(pX)
        end do
    end do

    close(funit)


    ! use the solution from the equilibrium primal problem for initial values
    open(newunit = funit, file = fnametaxinp, status = 'old', action = 'read')
    read(funit, *) thistax_input
    read(funit, *) theta_input
    read(funit, *) swf_input
    read(funit, *) rev_input

    do iX = 1, ntax
        read(funit, *) x(iX)
    end do

    close(funit)

    xs = x

    call getTaxLinearRestrictionsLevelScaled(TAXMODE, MINY, thistax, a)
    call rescaleUnitIntervalToTaxLevel(x(1:NTAXPARAM), xtax, thistax, MINY)

    call setOptions(opt, OPT_SIMULATION, estmode)
    opt%fixMarriageMarket = .true.
    opt%taxins = thistax

    call swffun_mpec_fixed(xparam, xlam, xtax, xmarriage, opt, ret, theta)
    normscale = abs(ret%swf)

    if (id == 0) then

        print *
        print *, 'SWF/REV AT INITIAL AND PRIMAL SOL'
        print *, '---------------------------------'
        print *
        print *, ret%swf, swf_input
        print *, ret%rev, rev_input
        print *

    end if

    ! initialise previous x vector
    xold = -huge(0.0_dp)

    ! linear constraints to ensure MTR is bounded everywhere
    nclin = NTAXCON(thistax)

    ! non-linear constraints for revenue constraint
    ncnln = 1

    ! parameter and constraint bounds

    ! trap silly bound errors
    bl =  9999.9_dp
    bu = -9999.9_dp

    bl(1:ntax) = 0.0_dp
    bu(1:ntax) = 1.0_dp

    ! next set of bounds for the linear parameter constraints
    ixParam = ntax
    call getTaxParameterBounds(bl, bu, ixParam, thistax)

    if (opt%taxins == TAX_JOINT) then
        bl(1:NTAXBR) = x(1:NTAXBR)
        bu(1:NTAXBR) = x(1:NTAXBR)
    end if
    
    ! bound for revenue constraint
    bl(nall + nclin + 1) = 0.0_dp
    bu(nall + nclin + 1) = 10.0_dp ** 20

    ! initial settings
    ifail = -1
    irevcm =  0

    if ( id == 0 ) then

        call E04UEF ('Derivative Level = 3')
        call E04UEF ('Step Size = 0.1')
        call E04UEF ('Verify Level = -1')
        call E04UEF ('Major Iteration Limit = 1000')
        call E04UEF ('Minor Iteration Limit = 2500')
        call E04UEF ('Optimality Tolerance = 1e-5')

        ! open file with history of search
        open(newunit = funit, file = fname, action = 'write', status = 'replace')

    end if

    do

        if (id == 0) then
            call E04UFF( irevcm, nall, nclin, ncnln, lda, ldcj, ldr, a, bl, bu, iter,  &
                         istate, c, cjac, clamda, objf, objgrd, r, x, needc, iwork,    &
                         liwork, work, lwork, ifail )
        end if

        call MPI_barrier(MPI_COMM_WORLD, ierror)

        call MPI_bcast(ifail, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(irevcm, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(x(1:nall), nall, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(xold(1:nall), nall, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

        if (ifail == -1 .and. irevcm > 0) then

            if ( .not. vecEqual(x(1:nall), xold(1:nall)) ) then

                z(1:nall) = x(1:nall)
                if (id > 0) then
                    epsrel = eps * max( 1.0_dp, abs( x(id) ) )
                    z(id) = x(id) + epsrel
                end if

                call rescaleUnitIntervalToTaxLevel(z(1:NTAXPARAM), xtax, thistax, MINY)

                call swffun_mpec_fixed(xparam, xlam, xtax, xmarriage, opt, ret, theta)

                ! minimising
                objf = -ret%swf / normscale
                c(1) = ret%rev - targetRev

                call MPI_GATHER(objf, 1, MPI_DOUBLE_PRECISION, &
                    f_temp(0:nall), 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

                call MPI_GATHER(c, 1, MPI_DOUBLE_PRECISION, &
                    c_temp(0:nall), 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

                if ( id == 0 ) then

                    ! objective gradient
                    f0 = f_temp(0)
                    do iX = 1, nall
                        epsrel = eps * max(1.0_dp, abs(x(iX)))
                        objgrd(iX) = ( f_temp(iX) - f0 ) / epsrel
                    end do

                    c0 = c_temp(0)
                    do iX = 1, nall ! variables
                        epsrel = eps * max(1.0_dp, abs(x(iX)))
                        cjac(1, iX) = ( c_temp(iX) - c0(1) ) / epsrel
                    end do

                    do iX = 1, nall
                        write(funit, *) iX, x(iX), ret%swf
                    end do

                    xold = x

                end if

            end if

        else

            exit

        end if

    end do


    if (id == 0) then

        close(funit)

        ! solution file (out-of-equilibrium)
        ! ----------------------------------

        open(newunit = funit, file = fnametaxsol_f, action = 'write', status = 'replace')
        write(funit, *) thistax
        write(funit, *) theta
        write(funit, *) ret%swf
        write(funit, *) ret%rev
        do iX = 1, nall
            write(funit, *) x(iX)
        end do
        do iX = 1, NTYPEALL
            write(funit, *) xlam(iX)
        end do
        close(funit)


        call rescaleUnitIntervalToTaxLevel(x(1:NTAXPARAM), xtax, thistax, MINY)
        call setOptions(opt, OPT_SIMULATION, estmode)
        opt%taxins = thistax
        ! opt%fixMarriageMarket = .true.

        call swffun_mpec(xparam, xlam, xtax, opt, ret, theta)

        write(IOUT, *) 'marriage market imbalance (fixed)'

        ! imbalance in marriage market
        open(newunit = funit, file = fnamenoneq, action = 'write', status = 'replace')
        do pX = 1, NTYPEALL
            write(funit, *) pX, ret%eqcon(pX)
            write(IOUT, *) pX, ret%eqcon(pX)
        end do
        close(funit)

        ! now calculate equilibrium (do this very accurately for the dual problem)
        opt%nlam = 49
        call swffun(xparam, xlam, xtax, opt, ret, theta)

        write(IOUT, *) 'swf', ret%swf
        write(IOUT, *) 'rev', ret%rev

        call swffun_mpec(xparam, xlam, xtax, opt, ret, theta)

        write(IOUT, *) 'swf_mpec', ret%swf
        write(IOUT, *) 'rev_mpec', ret%rev

        ! solution file (equilibrium welfare and revenue)
        ! -----------------------------------------------

        open(newunit = funit, file = fnametaxsol, action = 'write', status = 'replace')
        write(funit, *) thistax
        write(funit, *) theta
        write(funit, *) ret%swf
        write(funit, *) ret%rev
        do iX = 1, nall
            write(funit, *) x(iX)
        end do
        do iX = 1, NTYPEALL
            write(funit, *) xlam(iX)
        end do
        close(funit)

    end if

#   ifdef _MPI_
    call mpi_finalize(ierror)
#   endif

end program optimal_level_fixed_mpi
