program optimal_level_restricted_mpi

    use globaldef
    use mathfn
    use test3kid
    use taxes
    use mpi

    implicit none

    ! parameters for optimisation routine
    integer, parameter :: nmax = NTAXPARAM + NTYPEALL
    integer, parameter :: nclmax = MAXVALNTAXCON
    integer, parameter :: ncnmax = NTYPEALL + 1

    integer, parameter :: lda = max(1, nclmax), ldcj = max(1, ncnmax), ldr = nmax
    integer, parameter :: liwork = 3 * nmax + nclmax + 2 * ncnmax
    integer, parameter :: lwork = 2 * nmax ** 2 + nmax * nclmax + 2 * nmax * ncnmax + 21 * nmax + 11 * nclmax + 22 * ncnmax + 1

    real(dp) :: x(nmax)                           ! tax parameters + pareto weights
    real(dp) :: xest(NPARAM + NMARKET * NTYPEALL) ! estimation parameters
    integer  :: xtype(NPARAM)
    real(dp) :: xparam(NPARAM)                    ! preference parameters
    real(dp) :: xtax(NTAXPARAM)                   ! tax parameters
    real(dp) :: xlam(NTYPEALL)                    ! parteo weights
    real(dp) :: xlamMarket(NMARKET * NTYPEALL)    ! parteo weights for each market
    real(dp) :: eqcon(NTYPEALL)                   ! constraint violation

    real(dp) :: objf
    real(dp) :: xold(nmax) ! previous parameter vector
    real(dp) :: z(nmax) ! perturbed parameter vector
    integer :: ifail, irevcm, iter, nclin, ncnln
    integer :: iX, jX

    real(dp) :: a(lda, nmax), bl(nmax + nclmax + ncnmax), bu(nmax + nclmax + ncnmax), c(max(1, ncnmax)),     &
                cjac(ldcj, nmax), clamda(nmax + nclmax + ncnmax), objgrd(nmax), r(ldr, nmax), work(lwork)

    integer :: istate(nmax + nclmax + ncnmax), iwork(liwork), needc(ncnmax)

    real(dp) :: f_temp(0:nmax)
    real(dp) :: c_temp( (NTYPEALL + 1) * (nmax + 1) )
    real(dp) :: epsrel
    real(dp) :: f0, c0(NTYPEALL+1)
    integer  :: ixL0, ixL1
    integer  :: ixParam
    integer  :: estmode
    real(dp) :: normscale

    ! for fd
    real(dp), parameter :: eps = 1.0e-8_dp

    real(dp) :: f, swf, rev, targetRev, targetSwf
    logical  :: verbose
    real(dp) :: ascaled(lda, nmax)

    integer :: narg, istat, funit, funit2
    character(len = 32) :: filetag, thisarg, thetastr, primalstr
    character(len = 128) :: fname
    character(len = 128) :: fnamesol
    character(len = 128) :: fnamerev
    character(len = 128) :: fnametaxsol
    character(len = 128) :: fnameweight
    character(len = 128) :: fsavepath

    type(opt_t) :: opt
    type(ret_t) :: ret
    real(dp) :: junk
    ! for mpi
    integer :: ierror, id, nproc

    ! control aversion to inequality
    real(dp) :: theta
    integer :: thetaint
    integer :: thistax,  ntax, nall

    ! specify the restriction mode
    thistax = 1
    theta = 0.0_dp

#   ifdef FINETAXGRID
    primalstr = '/primal_fine'
#   else
    primalstr = '/primal'
#   endif

    narg = command_argument_count()
    if (narg == 0) then
        filetag = ''
        estmode = 0
        fnamesol = 'output/estimation/xsol_market_mpec_mpi.asc'
        fnamerev = 'output/post/revenue.asc'
        fname = 'output/simulation/primal/xtax_level.log'
        fnametaxsol = 'output/simulation/primal/xtax_sol.asc'
        fsavepath = 'output/simulation/primal/'
        fnameweight = 'output/simulation/primal/swfweight.csv'

    else if (narg == 1) then
        call get_command_argument(1, value = filetag, status = istat)
        estmode = 0
        fname = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '/xtax_level.log'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        fnamerev = 'output/post/' // trim(adjustl(filetag)) // '/revenue.asc'
        fnametaxsol = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '/xtax_sol.asc'
        fnameweight = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '/swfweight.csv'
        fsavepath = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '/'

    else if (narg == 2) then
        call get_command_argument(1, value = filetag, status = istat)
        call get_command_argument(2, value = thisarg, status = istat)
        read(thisarg, '(i10)') thistax
        estmode = 0
        fname = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '/xtax_level.log'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        fnamerev = 'output/post/' // trim(adjustl(filetag)) // '/revenue.asc'
        fnametaxsol = 'output/simulation/'//trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '/xtax_sol.asc'
        fnameweight = 'output/simulation/'//trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '/swfweight.csv'
        fsavepath = 'output/simulation/'//trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '/'

    else if (narg == 3) then
        call get_command_argument(1, value = filetag, status = istat)
        call get_command_argument(2, value = thisarg, status = istat)
        read(thisarg, '(i10)') thistax
        call get_command_argument(3, value = thetaStr, status = istat)
        read(thetaStr, '(i10)') thetaint
        estmode = 0
        theta = -real(thetaint, dp) / 10.0_dp
        fname = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_level.log'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        fnamerev = 'output/post/' // trim(adjustl(filetag)) // '/revenue.asc'
        fnametaxsol = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_sol.asc'
        fnameweight = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/swfweight.csv'
        fsavepath = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/'

    else if (narg == 4) then
        call get_command_argument(1, value = filetag, status = istat)
        call get_command_argument(2, value = thisarg, status = istat)
        read(thisarg, '(i10)') thistax
        call get_command_argument(3, value = thetaStr, status = istat)
        read(thetaStr, '(i10)') thetaint
        call get_command_argument(4, value = thisarg, status = istat)
        read(thisarg, '(i10)') estmode
        theta = -real(thetaint, dp) / 10.0_dp
        fname = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_level.log'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        fnamerev = 'output/post/' // trim(adjustl(filetag)) // '/revenue.asc'
        fnametaxsol = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/xtax_sol.asc'
        fnameweight = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/swfweight.csv'
        fsavepath = 'output/simulation/' // trim(adjustl(filetag)) // trim(adjustl(primalstr)) // intToStr(thistax) // '_t' // trim(adjustl(thetaStr)) // '/'

    else
        stop 'incorrect number of command line arguments supplied'
    end if

    select case(estmode)
    case(statemode)
        print *, 'estmode = statemode'
        print *
    case(mcdmode)
        print *, 'estmode = mcdmode'
        print *
    case default
    end select

    ! load estimation parameter vector
    open(newunit = funit, file = fnamesol, status = 'old', action = 'read')
    do iX = 1, NPARAM + NTYPEALL * NMARKET
        read(funit, *) xest(iX)
    end do
    close(funit)

    ! load revenue target
    open(newunit = funit, file = fnamerev, status = 'old', action = 'read')
    read(funit, *) targetRev
    close(funit)

    ntax = NTAXRES(thistax)
    nall = ntax + NTYPEALL

    call mpi_init(ierror)

    !obtain id for each node
    call mpi_comm_rank(mpi_comm_world, id, ierror)

    !returns the number of nodes
    call mpi_comm_size(mpi_comm_world, nproc, ierror)

    if (nproc /= nall + 1) then
        write (*, *) nall + 1, 'processors required'
        call mpi_finalize(ierror)
        stop
    end if

    ! parameter vector for model
    xparam = xest(1:NPARAM)

    ! weights from estimation
    xlamMarket = xest(NPARAM + 1 : NPARAM + NTYPEALL * NMARKET)

    call getTaxLinearRestrictionsLevelScaled(TAXMODE, MINY, thistax, a)

    x = 0.15_dp

    call rescaleUnitIntervalToTaxLevel(x(1:NTAXPARAM), xtax, thistax, MINY)

    call setOptions(opt, OPT_SIMULATION, estmode)
    opt%taxins = thistax

    ! solve for the equilibrium associated with the initial guess
    ! and use the implied Pareto weights as the initial guesses
    ! in the mpec

    call swffun(xparam, xlam, xtax, opt, ret, theta)
    normscale = abs(ret%swf)

    x(ntax + 1 : nall) = xlam

    xlam = x(ntax + 1 : nall)

    ! initialise previous x vector
    xold = x
    xold(1 : nall) = -huge(0.0_dp)

    ! linear constraints to ensure MTR is bounded everywhere
    nclin = NTAXCON(thistax)

    ! non-linear constraints for marriage market equilibrium and revenue constraint
    ncnln = NTYPEALL + 1

    ! parameter and constraint bounds

    ! trap silly bound errors
    bl =  9999.9_dp
    bu = -9999.9_dp

    bl(1:ntax) = 0.0_dp
    bu(1:ntax) = 1.0_dp

    if (thistax == TAX_JOINTGBTALL) then
        bu(1) = 0.5_dp
        bu(NTAXBR + 1) = 0.5_dp
    end if

    ! pareto weights
    bl(ntax + 1 : ntax + NTYPEALL) = 0.01_dp
    bu(ntax + 1 : ntax + NTYPEALL) = 0.99_dp

    ! next set of bounds for the linear parameter constraints

    ! singles
    ixParam = ntax + NTYPEALL
    call getTaxParameterBounds(bl, bu, ixParam, thistax)

    ! bounds for equilibrium constraints
    bl(nall + nclin + 1 : nall + nclin + NTYPEALL) = 0.0_dp
    bu(nall + nclin + 1 : nall + nclin + NTYPEALL) = 0.0_dp

    ! bound for revenue constraint
    bl(nall + nclin + NTYPEALL + 1) = 0.0_dp
    bu(nall + nclin + NTYPEALL + 1) = 10.0_dp ** 20

    ! initial settings
    ifail = -1
    irevcm =  0

    !obtain id for each node
    call mpi_comm_rank(mpi_comm_world, id, ierror)

    if (id == 0) then

        call E04UEF ('Derivative Level = 3')
        call E04UEF ('Step Size = 0.1')
        call E04UEF ('Verify Level = -1')
        call E04UEF ('Major Iteration Limit = 500')
        call E04UEF ('Minor Iteration Limit = 2500')
        call E04UEF ('Optimality Tolerance = 1.0e-5')

        ! open file with history of search
        open(newunit = funit, file = fname, action = 'write', status = 'replace')

    end if

    do

        if ( id == 0 ) then

            call E04UFF( irevcm, nall, nclin, ncnln, lda, ldcj, ldr, a, bl, bu, iter,  &
                istate, c, cjac, clamda, objf, objgrd, r, x, needc, iwork,             &
                liwork, work, lwork, ifail )

        end if

        call MPI_barrier(MPI_COMM_WORLD, ierror)

        call MPI_bcast(ifail, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(irevcm, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(x(1:nall), nall, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)
        call MPI_bcast(xold(1:nall), nall, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

        if (ifail == -1 .and. irevcm > 0) then

            if ( .not. vecEqual(x(1:nall), xold(1:nall)) ) then

                z(1:nall) = x(1:nall)
                if (id > 0) then
                    epsrel = eps * max( 1.0_dp, abs( x(id) ) )
                    z(id) = x(id) + epsrel
                end if

                call rescaleUnitIntervalToTaxLevel(z(1:NTAXPARAM), xtax, thistax, MINY)

                xlam = z(ntax + 1 : nall)

                call swffun_mpec(xparam, xlam, xtax, opt, ret, theta)

                ! minimising
                objf = -ret%swf / normscale
                c(1 : NTYPEALL) = ret%eqcon
                c(NTYPEALL + 1) = ret%rev - targetRev

                call MPI_GATHER(objf, 1, MPI_DOUBLE_PRECISION, &
                    f_temp(0:nall), 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

                call MPI_GATHER(c, NTYPEALL + 1, MPI_DOUBLE_PRECISION, &
                    c_temp(1:(NTYPEALL + 1) * (nall + 1)), NTYPEALL + 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)

                if (id == 0) then

                    ! objective gradient
                    f0 = f_temp(0)
                    do iX = 1, nall
                        epsrel = eps * max( 1.0_dp, abs( x(iX) ) )
                        objgrd(iX) = ( f_temp(iX) - f0 ) / epsrel
                    end do

                    c0 = c_temp(1:NTYPEALL + 1)
                    do iX = 1, nall ! variables
                        epsrel = eps * max( 1.0_dp, abs( x(iX) ) )
                        do jX = 1, NTYPEALL + 1 ! constraints
                            cjac(jX, iX) = ( c_temp( iX * (NTYPEALL + 1) + jX ) - c0(jX) ) / epsrel
                        end do
                    end do

                    do iX = 1, nall
                        write(funit, *) iX, x(iX), ret%swf!objf
                    end do

                    xold = x

                end if

            end if

        else

            exit

        end if

    end do

    if (id == 0) then

        close(funit)
        call rescaleUnitIntervalToTaxLevel(x(1:NTAXPARAM), xtax, thistax, MINY)

        call setOptions(opt, OPT_SIMULATION, estmode)
        opt%taxins = thistax

        call swffun_mpec(xparam, xlam, xtax, opt, ret, theta)

        ! solution file
        ! -------------

        open(newunit = funit, file = fnametaxsol, action = 'write', status = 'replace')
        write(funit, *) thistax
        write(funit, *) theta
        write(funit, *) ret%swf
        write(funit, *) ret%rev
        do iX = 1, nall
            write(funit, *) x(iX)
        end do
        close(funit)

        call setOptions(opt, OPT_SIMULATIONFIT, estmode)
        opt%taxins = thistax
        opt%saveMom = .true.
        opt%extraMom = .true.
        opt%savePath = trim(adjustl(fsavepath))

        call swffun_mpec(xparam, xlam, xtax, opt, ret, theta)

        ! calculate welfare weights
        ! -------------------------

        if ( thistax == TAX_JOINT ) then
            call setOptions(opt, OPT_SIMULATION, estmode)
            opt%taxins = TAX_JOINT
            call swffun_weights(xparam, xlam, xtax, opt, ret, theta, fnameweight)
        end if

    end if

    call mpi_finalize(ierror)

end program optimal_level_restricted_mpi
