
set more off
set seed 182679671

/* *********************************************************************** */
/* *********************************************************************** */


use "$atus_data/02 Original Data/atusrost_0313", clear
merge 1:1 tucaseid tulineno using "$atus_data/02 Original Data/atuscps_0313.dta"
tab _merge
keep if (_merge == 3)
sort tucaseid tulineno
tempfile atusraw_ind
save "`atusraw_ind'", replace

use "$atus_data/02 Original Data/atusact_0313", clear
merge m:1 tucaseid using "$atus_data/02 Original Data/atusresp_0313"
tab _merge
keep if (_merge == 3)
drop _merge
sort tucaseid
tempfile atusraw_time
save "`atusraw_time'", replace


/* *********************************************************************** */
/* *********************************************************************** */

use "`atusraw_ind'", clear
run "label_atus.do"

format tucaseid %14.0f

// recode self
recode terrp  (19 = 18)

// restrict sample to spouse (20), unmarried partner (21), and self (18)
keep hryear4 tucaseid tulineno terrp teage tesex gereg gestfips hefaminc ///
     hrnumhou hufaminc peeduca pehspnon pemaritl prempnot prernhly prernwa ///
     pemlr prhrusl prnmchld ptdtrace pulineno pecohab pespouse tubwgt pehruslt ///
     pehrusl1 prnlfsch prfamrel pudis*


// child age
gen byte _kid05   = terrp == 22 & teage >= 0  & teage <= 5
gen byte _kid611  = terrp == 22 & teage >= 6  & teage <= 11
gen byte _kid1218 = terrp == 22 & teage >= 12 & teage <  18

egen byte nkids05   = total(_kid05),   by(tucaseid)
egen byte nkids611  = total(_kid611),  by(tucaseid)
egen byte nkids1218 = total(_kid1218), by(tucaseid)

drop _kid05 _kid611 _kid1218

gen byte nkids = nkids05 + nkids611 + nkids1218
gen byte kids = nkids > 0

/* *********************************************************************** */
/* *********************************************************************** */

// only keep information for the main respondent and spouse/partner
// as we only have time use information for them

keep if (terrp == 18 | terrp == 20 | terrp == 21)


// rename variables
rename tucaseid caseid
rename tulineno perlineno
rename terrp relate
rename teage age
rename tesex sex
rename gereg region
rename gestfips statefip
rename peeduca educ
rename pehspnon hisp
rename pemaritl marr
rename prempnot empl
rename prernhly earnhr
rename prernwa earnwk
rename pehruslt uhours
rename pehrusl1 uhours1
rename pemlr empld
rename prnmchld numchild
rename ptdtrace race

// recode state
matrix define recodeFips ///
    = (1, 2, ., 3, 4, 5, ., 6, 7, 8, 9, 10, 11, ., 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,  ///
       24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, ., 40, 41, 42, 43, 44, 45, ///
       46, 47, ., 48, 49, 50, 51)
gen byte state = recodeFips[1, statefip]
run label_state51.do
label values state state_lbl

tab state, m
if $stateonly == 1 {
    if $nummarket == 9 {
        keep if ( state == 22 | ///
                  state == 33 | ///
                  state == 14 | ///
                  state == 26 | ///
                  state == 10 | ///
                  state == 43 | ///
                  state == 44 | ///
                  state == 3  | ///
                  state == 5 )
    }
}

drop if empl < 0

gen byte head = relate == 18

// still in school
gen byte school = prnlfsch

// kids
gen byte kids2 = numchild
replace kids2 = 0 if kids2 < 0

// prernwa/earnwk contains two implied decimal places
gen double earn = earnwk
replace earn = earn / 100.0 if earnwk > 0
assert earn ~=.

// categorical education variable... a lot of BLANKS (these are kids)
if $numed == 3 {
    gen byte edulev = 1
    replace edulev = 2 if (educ >= 40 & educ <= 42)
    replace edulev = 3 if (educ >= 43 & educ <= 46)
    label variable edulev "Highest education level (new def)"
    label define edulev_lbl 1 "High school degree or below" 2 "Some college/associate's degree" 3 "Bachelor's degree and above"
}
else if $numed == 4 {
    gen byte edulev = 1
    replace edulev = 2 if (educ == 39)
    replace edulev = 3 if (educ >= 40 & educ <= 42)
    replace edulev = 4 if (educ >= 43 & educ <= 46)
    label variable edulev "Highest education level (new def)"
    label define edulev_lbl 1 "Less than high school" 2 "High school degree" 3 "Some college/associate's degree" 4 "Bachelor's degree and above"
}

// Attention: The racefour could follow "Race Bridge" program on IPUMS.
// https://usa.ipums.org/usa/resources/volii/race_bridge_stata_program.txt

gen byte racefour = 4
replace racefour = 1 if (race == 1)
replace racefour = 2 if (race == 2)
replace racefour = 3 if (hisp == 1)
label variable racefour "Four race segment"
label values racefour racefour_lbl

// state labels
label values statefip statefip_lbl

// number of adults
egen byte numads = count(perlineno !=0), by(caseid)
gen byte _married = relate == 20
gen byte _cohabit = relate == 21

egen byte married = max(_married), by(caseid)
egen byte cohabit = max(_cohabit), by(caseid)
gen  byte single  = 1 - married-cohabit

// drop if there is a relationship type inconsistency or a missing
drop if (numads  == 1 & (marr == -1 | marr == 1 | marr == 2))
drop if (married == 1 & (marr == -1 | marr == 3 | marr == 4 | marr == 5 | marr == 6))
drop if (cohabit == 1 & (marr == -1 | marr == 1 | marr == 2))
drop if (single == 0 & numads == 1)

// drop if the selection criteria excludes a spouse
egen byte numads2 = count(perlineno !=0), by(caseid)
drop if numads2 != numads
drop numads2

// age group
gen byte agegrp = 0
replace agegrp = 1 if (age < $minage | age > $maxage )
replace agegrp = 2 if (age >= $minage & age <= $maxage)
assert agegrp != 0
label variable agegrp "Age group"
label values agegrp agegrp_lbl
egen byte agegrp_sp = total(agegrp), by(caseid)
replace agegrp_sp = agegrp_sp - agegrp
egen long age_sp = total(age), by(caseid)
replace age_sp = age_sp - age
qui compress age_sp

drop if agegrp == 1 & married == 0
drop if agegrp == 1 & married == 1 & head == 1
drop if agegrp_sp == 1 & married == 1 & head == 0

if $bothage == 1 {
    drop if ( agegrp == 1 | agegrp_sp == 1 ) & married == 1
    }

drop if ( age < $minage_hard | age > $maxage_hard | age_sp < $minage_hard | age_sp > $maxage_hard ) & married == 1
drop if ( age < $minage_hard | age > $maxage_hard ) & married == 0

// drop same-sex partnerships
egen byte sex_sp = total(sex), by(caseid)
replace sex_sp = sex_sp - sex

// drop same-sex partnerships
keep if (sex != sex_sp)

// work indicator
gen byte work = empl == 1
gen byte disab = pudis == 1
gen byte disabres = ( pudis1 == 1 | pudis2 == 1 )

gen byte pt = work == 1 & uhours >= 0  & uhours <= 31
gen byte ft = work == 1 & uhours >  31 & uhours <= 999

ren hryear4 year
keep if year >= $minyear_atus & year <= $maxyear_atus

// variables to keep
local keepvar "year caseid perlineno age age_sp sex relate married single cohabit statefip region"

// generate more spousal variables
//local type : type `var'
local spvar "race earnhr earn uhours uhours1 nkids05 nkids611 nkids1218 nkids kids edulev racefour work empl empld disab disabres ft pt"
foreach var in `spvar' {
    local keepvar "`keepvar' `var' `var'_sp"
    egen double `var'_sp = total(`var'), by(caseid)
    qui replace `var'_sp = `var'_sp - `var'
    qui compress `var'_sp
    assert `var'_sp == 0 if single == 1
    qui replace `var'_sp= . if single == 1
    }

tab single numads
keep `keepvar'

local spvar "nkids05 nkids611 nkids1218 nkids kids"
foreach var in `spvar' {
    assert `var' == `var'_sp if married == 1
    drop `var'_sp
    }

if $whiteonly == 1 {
    keep if ( married == 0 & race == 1 | (married == 1 & race == 1 & race_sp == 1) )
    }

sort caseid perlineno
tempfile atus_group1
save "`atus_group1'", replace

/* *********************************************************************** */
/* *********************************************************************** */

// time use data is for a specific person
use "`atusraw_time'", clear
keep teernhro tehrftpt tudiaryday tufnwgtp tehruslt tuspusft tucaseid tuactivity_n ///
     tuactdur24 trcodep trtier1p trtier2p tuactdur tulineno ///
     tespempnot tespuhrs telfs turetot tulk tulay tuabsot teret1 tudis tufwk

format tucaseid %14.0f

gen byte spouse_emp = tespempnot == 1 if tespempnot > 0 & tespempnot <.
gen double spouse_workhours = tespuhrs if tespuhrs >= 0 & tespuhrs <= 99

gen byte working = telfs == 1 | telfs == 2
gen byte unemp = telfs == 3 | telfs == 4
gen byte retired = telfs == 5 & (turetot == 1 | tulk == 3 | tufwk == 3 | tulay == 3 | tuabsot == 3) & teret1 == 2
gen byte disabled = telfs == 5 & (tuabsot == 4 | tufwk == 4 | tulay == 4 | tulk == 4 | tudis == 1)

rename tudiaryday day
rename tufnwgtp weight
rename tucaseid caseid
rename tulineno perlineno
rename tuactivity_n actlineno
rename tuactdur24 actdur24
rename tuactdur actdur
rename trcodep actcode
rename trtier1p actcode2
rename trtier2p actcode4

gen double sg01_mkt_work = actdur24 if ///
    ( actcode4 == 0501   ) | ///
    ( actcode4 == 0502   ) | ///
    ( actcode4 == 0599   ) | ///
    ( actcode  == 180501 ) | ///
    ( actcode  == 180502 ) | ///
    ( actcode  == 180589 )
label variable sg01_mkt_work "Subgroup 01: Market Work"

gen double sg02_oth_inc = actdur24 if ///
    ( actcode4 == 0503 )
label variable sg02_oth_inc "Subgroup 02: Other Income-Generating Activities"

gen double sg03_job_search = actdur24 if ///
    ( actcode4 == 0504 )
label variable sg03_job_search "Subgroup 03: Job Search"

gen double sg04_child_care = actdur24 if ///
    ( actcode4 == 0301   ) | ///
    ( actcode4 == 0302   ) | ///
    ( actcode4 == 0303   ) | ///
    ( actcode4 == 0401   ) | ///
    ( actcode4 == 0402   ) | ///
    ( actcode4 == 0403   ) | ///
    ( actcode  == 180381 ) | ///
    ( actcode  == 180382 ) | ///
    ( actcode  == 180399 ) | ///
    ( actcode  == 180481 ) | ///
    ( actcode  == 180482 ) | ///
    ( actcode  == 180499 )

label variable sg04_child_care "Subgroup 04: Child Care"

gen double sg05_home_prod = actdur24 if ///
    ( actcode4 == 0201 ) | ///
    ( actcode4 == 0202 ) | ///
    ( actcode4 == 0203 & actcode != 020301) | ///
    ( actcode4 == 0207 ) | ///
    ( actcode4 == 0208 ) | ///
    ( actcode4 == 0209 & actcode != 020903 & actcode != 020904 ) | ///
    ( actcode4 == 0299 ) | ///
    ( actcode  == 180280 )
label variable sg05_home_prod "Subgroup 05: Core Home Production"

gen double sg06_home_own = actdur24 if ///
    ( actcode4 == 0204) | ///
    ( actcode4 == 0205) | ///
    ( actcode  == 020301 )
label variable sg06_home_own "Subgroup 06: Home Ownership Activities"

gen double sg07_obtain_gs = actdur24 if ///
    ( actcode2 == 07 ) | ///
    ( actcode2 == 08 & actcode4 != 0804) | ///
    ( actcode2 == 09 ) | ///
    ( actcode2 == 10 ) | ///
    ( actcode4 == 1807 ) | ///
    ( actcode4 == 1808 & actcode != 180804) | ///
    ( actcode4 == 1809 ) | ///
    ( actcode4 == 1810 )
label variable sg07_obtain_gs "Subgroup 07: Obtaining Goods and Services"

gen double sg08_oth_care = actdur24 if ///
    ( actcode4 == 0304 ) | ///
    ( actcode4 == 0305 ) | ///
    ( actcode4 == 0399 ) | ///
    ( actcode4 == 0404 ) | ///
    ( actcode4 == 0405 ) | ///
    ( actcode4 == 0499 )
label variable sg08_oth_care "Subgroup 08: Others Care"

gen double sg09_tv_watch = actdur24 if ///
    ( actcode == 120303 ) | ///
    ( actcode == 120304 )
label variable sg09_tv_watch "Subgroup 09: TV Watching"

gen double sg10_social = actdur24 if ///
    ( actcode4 == 1201 )   | ///
    ( actcode4 == 1202 )   | ///
    ( actcode  == 120307 ) | ///
    ( actcode  == 120501 ) | ///
    ( actcode  == 120502 ) | ///
    ( actcode2 == 16 )     | ///
    ( actcode  == 180201 ) | ///
    ( actcode  == 180202 ) | ///
    ( actcode4 == 1816 )
label variable sg10_social "Subgroup 10: Socializing"

gen double sg11_sleep = actdur24 if ///
    ( actcode4 == 0101 )
label variable sg11_sleep "Subgroup 11: Sleeping"

gen double sg12_eat_percare = actdur24 if ///
    ( actcode4 == 0102 ) | ///
    ( actcode4 == 0104 ) | ///
    ( actcode4 == 0105 ) | ///
    ( actcode4 == 0199 ) | ///
    ( actcode2 == 11 )   | ///
    ( actcode4 == 1801 ) | ///
    ( actcode4 == 1811 )
label variable sg12_eat_percare "Subgroup 12: Eating and Personal Care"

gen double sg13_oth_leis = actdur24 if ///
    ( actcode4 == 0206 )   | ///
    ( actcode  == 020903 ) | ///
    ( actcode  == 020904 ) | ///
    ( actcode4 == 1203 & actcode != 120303 & actcode != 120304 & actcode != 120307) | ///
    ( actcode4 == 1204 ) | ///
    ( actcode4 == 1205 & actcode != 120501 & actcode != 120502) | ///
    ( actcode4 == 1299 ) | ///
    ( actcode2 == 13 ) | ///
    ( actcode  == 180206 ) | ///
    ( actcode4 == 1812 & actcode != 180201 & actcode != 180202 ) | ///
    ( actcode4 == 1813 )
label variable sg13_oth_leis "Subgroup 13: Other Leisure"

gen double sg14_educa = actdur24 if ///
    ( actcode2 == 06 ) | ///
    ( actcode4 == 1806 )
label variable sg14_educa "Subgroup 14: Education"

gen double sg15_civic_relig = actdur24 if ///
    ( actcode2 == 14 )   | ///
    ( actcode2 == 15 )   | ///
    ( actcode4 == 1814 ) | ///
    ( actcode4 == 1815 )
label variable sg15_civic_relig "Subgroup 15: Civic and Religious Activities"

gen double sg16_own_medcare = actdur24 if ///
    ( actcode4 == 0103 ) | ///
    ( actcode4 == 0804 ) | ///
    ( actcode  == 180804 )
label variable sg16_own_medcare "Subgroup 16: Own Medical Care"

// added 1899
gen double sg17_unclass = actdur24 if ///
    ( actcode2 == 50 ) | ///
    ( actcode4 == 1899 ) | ///
    ( actcode4 == 1818 )
label variable sg17_unclass "Subgroup 17: Unclassified"

gen double notdiscretionary = actdur24 if ///
    ( actcode4 == 0101 ) | ///
    ( actcode4 == 0102 ) | ///
    ( actcode4 == 0104 ) | ///
    ( actcode4 == 0105 ) | ///
    ( actcode4 == 0199 )

sort caseid
tempfile atus_timegrp
save "`atus_timegrp'", replace


/* *********************************************************************** */
/* *********************************************************************** */

local activity sg01_mkt_work sg02_oth_inc sg03_job_search sg04_child_care sg05_home_prod sg06_home_own sg07_obtain_gs ///
    sg08_oth_care sg09_tv_watch sg10_social sg11_sleep sg12_eat_percare sg13_oth_leis sg14_educa sg15_civic_relig     ///
    sg16_own_medcare sg17_unclass

collapse (mean) teernhro tehrftpt tehruslt  tuspusft day weight spouse_emp spouse_workhours working unemp retired disabled ///
    (sum) `activity' actdur24 (sum) notdiscretionary, by(caseid perlineno) fast
egen sgsum = rsum(sg*)
assert sgsum == actdur24

gen double group_labor_market = ///
    sg01_mkt_work + ///
    sg02_oth_inc + ///
    sg03_job_search

gen double group_home_production = ///
    sg04_child_care + ///
    sg05_home_prod + ///
    sg06_home_own + ///
    sg07_obtain_gs + ///
    sg08_oth_care

gen double group_other_activity = ///
    sg09_tv_watch + ///
    sg10_social + ///
    sg11_sleep + ///
    sg12_eat_percare + ///
    sg13_oth_leis + ///
    sg14_educa + ///
    sg15_civic_relig + ///
    sg16_own_medcare + ///
    sg17_unclass

gen double all_day_activity = ///
    group_labor_market + ///
    group_home_production + ///
    group_other_activity

drop if (disabled == 1)

label variable sg01_mkt_work "Subgroup 01: Market Work"
label variable sg02_oth_inc "Subgroup 02: Other Income-Generating Activities"
label variable sg03_job_search "Subgroup 03: Job Search"
label variable sg04_child_care "Subgroup 04: Child Care"
label variable sg05_home_prod "Subgroup 05: Core Home Production"
label variable sg06_home_own "Subgroup 06: Home Ownership Activities"
label variable sg07_obtain_gs "Subgroup 07: Obtaining Goods and Services"
label variable sg08_oth_care "Subgroup 08: Others Care"
label variable sg09_tv_watch "Subgroup 09: TV Watching"
label variable sg10_social "Subgroup 10: Socializing"
label variable sg11_sleep "Subgroup 11: Sleeping"
label variable sg12_eat_percare "Subgroup 12: Eating and Personal Care"
label variable sg13_oth_leis "Subgroup 13: Other Leisure"
label variable sg14_educa "Subgroup 14: Education"
label variable sg15_civic_relig "Subgroup 15: Civic and Religious Activities"
label variable sg16_own_medcare "Subgroup 16: Own Medical Care"
label variable sg17_unclass "Subgroup 17: Unclassified"
label variable group_labor_market "Group: Labor Market Hour"
label variable group_home_production "Group: Home Production Hour"
label variable group_other_activity "Group: Other Activity Hour"
label variable all_day_activity "Sum: All Day Activity Hour"

tab all_day_activity
keep if (all_day_activity == 1440)
replace group_labor_market = group_labor_market / 60 * 7
replace group_home_production = group_home_production / 60 * 7
replace group_other_activity = group_other_activity / 60 * 7
replace all_day_activity = all_day_activity / 60 * 7
sort caseid
tempfile atus_timegrpsum
save "`atus_timegrpsum'", replace

use "`atus_group1'", clear
merge 1:1 caseid perlineno using "`atus_timegrpsum'"
tab _merge
drop if (_merge == 2)
drop _merge

qui gen double hours_home_discrete = .
qui gen byte hours_home_group = .

local minh = "-0.1 6 18 31 43 55 67 79 92 98"
local nh : word count `minh'
local time 108
local timeint = `time' / `= `nh' - 1'
local mintime = 1

forval h = 1 / `nh' {

    local lower : word `h' of `minh'
    local upper : word `= `h' + 1' of `minh'
    if `h'==`nh' {
        local upper 9999
        }

    qui replace hours_home_discrete = 4 + `= `h' - 1' * `timeint' if group_home_production > `lower' & group_home_production <= `upper'
    qui replace hours_home_group = `h' if group_home_production > `lower' & group_home_production <= `upper'

    }

replace hours_home_discrete = 64 if hours_home_discrete > 64 & hours_home_discrete < .

gen byte hours_home = .

if $numhome == 5 {
    replace hours_home = 1 if hours_home_group == 1
    replace hours_home = 2 if hours_home_group == 2
    replace hours_home = 3 if hours_home_group == 3
    replace hours_home = 4 if hours_home_group == 4
    replace hours_home = 5 if hours_home_group >= 5 & hours_home_group~=.
    }

if $numhome == 4 {
    replace hours_home = 1 if hours_home_group == 1 | hours_home_group == 2
    replace hours_home = 2 if hours_home_group == 3
    replace hours_home = 3 if hours_home_group == 4
    replace hours_home = 4 if hours_home_group >= 5 & hours_home_group ~=.
    }

if $numhome == 3 {
    replace hours_home = 1 if hours_home_group == 1 | hours_home_group == 2
    replace hours_home = 2 if hours_home_group == 3 | hours_home_group == 4
    replace hours_home = 3 if hours_home_group >= 5 & hours_home_group ~=.
    }


tab hours_home, gen(home)

qui tab day [aw = weight], matcell(day_freq)
qui count if day~=.
matrix define day_norm_freq = day_freq / `r(N)'
qui gen double dayweight = weight

forval day = 1 / 7 {
    qui replace dayweight = dayweight * 1/7 * 1/day_norm_freq[`day', 1] if day == `day'
    }


gen byte group = .
local group = 0
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval sex = 1 / 2 {
            forval married = 0 / 1 {
                local ++group
                replace group = `group' if married == `married' & sex == `sex' & edulev == `e' & kids == `k'
            }
        }
    }
}

qui gen double dayweight_group = weight
forval group = 1 / `group' {
    qui tab day [aw = weight] if group == `group', matcell(day_freq)
    qui count if day ~=. & group == `group'
    matrix define day_norm_freq = day_freq / `r(N)'
    forval day = 1 / 7 {
        qui replace dayweight_group = dayweight_group * 1/7 * 1/day_norm_freq[`day', 1] if day == `day' & group == `group'
        }
    }

global weight dayweight

save "$save_data/atus_intermediate_ed$numed`'_age$minage$maxage$statetag.dta", replace

/* marginal distribution of home production, singles */

forval k = 0 / 1 {

    forval h = 1 / $numhome {
        local postlistf "`postlistf' mhome`h'_k`k'_sin"
        }

    forval h = 1 / $numhome {
        local postlistf "`postlistf' fhome`h'_k`k'_sin"
        }

    forval e = 1 / $numed {

        forval h = 1 / $numhome {
            local postlistf "`postlistf' mhome`h'_k`k'_M`e'F0"
            }

        local postlistf "`postlistf' mhomeh_k`k'_M`e'F0__mean mhomeh_k`k'_M`e'F0__sd"

        forval h = 1 / $numhome {
            local postlistf "`postlistf' fhome`h'_k`k'_M0F`e'"
            }

        local postlistf "`postlistf' fhomeh_k`k'_M0F`e'__mean fhomeh_k`k'_M0F`e'__sd"

        }
    }

/* marginal distribution of home production, couples */

forval k = 0 / 1 {

    forval h = 1 / $numhome {
        local postlistf "`postlistf' mhome`h'_k`k'_cou"
        }

    forval h = 1 / $numhome {
        local postlistf "`postlistf' fhome`h'_k`k'_cou"
        }

    forval e = 1 / $numed {

        forval h = 1 / $numhome {
            local postlistf "`postlistf' mhome`h'_k`k'_M`e'Ff"
            }

        local postlistf "`postlistf' mhomeh_k`k'_M`e'Ff__mean mhomeh_k`k'_M`e'Ff__sd"

        forval h = 1 / $numhome {
            local postlistf "`postlistf' fhome`h'_k`k'_MmF`e'"
            }

        local postlistf "`postlistf' fhomeh_k`k'_MmF`e'__mean fhomeh_k`k'_MmF`e'__sd"

        }
    }

/* distribution of home production by own/spouse employment status */
/* for sample size reasons, I am not conditioning these on education */

// single men home production (self hours)
forval k = 0 / 1 {
    local postlistf "`postlistf' mhomehMw1_k`k'_MmF0__mean mhomehMw1_k`k'_MmF0__sd"
    local postlistf "`postlistf' mhomehMw0_k`k'_MmF0__mean mhomehMw0_k`k'_MmF0__sd"
    }

// married men home production (self hours)
forval k = 0 / 1 {
    local postlistf "`postlistf' mhomehMw1_k`k'_MmFf__mean mhomehMw1_k`k'_MmFf__sd"
    local postlistf "`postlistf' mhomehMw0_k`k'_MmFf__mean mhomehMw0_k`k'_MmFf__sd"
    }

// married men home production (spouse hours)
forval k = 0 / 1 {
    local postlistf "`postlistf' mhomehFw1_k`k'_MmFf__mean mhomehFw1_k`k'_MmFf__sd"
    local postlistf "`postlistf' mhomehFw0_k`k'_MmFf__mean mhomehFw0_k`k'_MmFf__sd"
    }


// single women home production
forval k = 0 / 1{
    local postlistf "`postlistf' fhomehFh2_k`k'_M0Ff__mean fhomehFh2_k`k'_M0Ff__sd"
    local postlistf "`postlistf' fhomehFh1_k`k'_M0Ff__mean fhomehFh1_k`k'_M0Ff__sd"
    local postlistf "`postlistf' fhomehFh0_k`k'_M0Ff__mean fhomehFh0_k`k'_M0Ff__sd"
    }

// married women home production (self hours)
forval k = 0 / 1 {
    local postlistf "`postlistf' fhomehFh2_k`k'_MmFf__mean fhomehFh2_k`k'_MmFf__sd"
    local postlistf "`postlistf' fhomehFh1_k`k'_MmFf__mean fhomehFh1_k`k'_MmFf__sd"
    local postlistf "`postlistf' fhomehFh0_k`k'_MmFf__mean fhomehFh0_k`k'_MmFf__sd"
    }

// married women home production (spouse hours)
forval k = 0 / 1 {
    local postlistf "`postlistf' fhomehMw1_k`k'_MmFf__mean fhomehMw1_k`k'_MmFf__sd"
    local postlistf "`postlistf' fhomehMw0_k`k'_MmFf__mean fhomehMw0_k`k'_MmFf__sd"
    }

// single men/women home production (by match)
forval e = 1 / $numed {
    local postlistf "`postlistf' mhomeh_M`e'F0__mean  mhomeh_M`e'F0__sd fhomeh_M0F`e'__mean fhomeh_M0F`e'__sd"
    }

// married men/women home production (by match)
forval esp = 1 / $numed {
    forval e = 1 / $numed {
        local postlistf "`postlistf' mhomeh_M`e'F`esp'__mean  mhomeh_M`e'F`esp'__sd fhomeh_M`e'F`esp'__mean fhomeh_M`e'F`esp'__sd"
        }
    }

// home time by kids and marital status integrating type
forval k = 0 / 1 {
    local postlistf "`postlistf' mhomeh_k`k'_MmF0__mean mhomeh_k`k'_MmF0__sd"
    local postlistf "`postlistf' fhomeh_k`k'_M0Ff__mean fhomeh_k`k'_M0Ff__sd"
    local postlistf "`postlistf' mhomeh_k`k'_MmFf__mean mhomeh_k`k'_MmFf__sd"
    local postlistf "`postlistf' fhomeh_k`k'_MmFf__mean fhomeh_k`k'_MmFf__sd"
    }

// agregate home time
local postlistf "`postlistf' mhomeh_MmF0__mean mhomeh_MmF0__sd"
local postlistf "`postlistf' fhomeh_M0Ff__mean fhomeh_M0Ff__sd"
local postlistf "`postlistf' mhomeh_MmFf__mean mhomeh_MmFf__sd"
local postlistf "`postlistf' fhomeh_MmFf__mean fhomeh_MmFf__sd"

/* post file for bootstrap repetitions */

local postlist
foreach pval in `postlistf' {
    local postlist "`postlist' (`pval')"
    }
cap postclose momATUS

postfile momATUS int r double(`postlistf') using "$save_data/mom_atus_ed$numed`'_age$minage$maxage$statetag.dta", replace

forval r = 0 / $bsrep {

    di "`r'"

    preserve

    if `r' > 0 {
        qui bsample
        }

    /* sample number */
    gen int r = `r'

    /* marginal distribution of home production, singles */
    forval k = 0 / 1 {

        forval h = 1 / $numhome {
            su home`h' [aw = $weight] if married == 0 & sex == 1 & kid == `k', meanonly
            scalar mhome`h'_k`k'_sin = `r(mean)'
            su home`h' [aw = $weight] if married == 0 & sex == 2 & kid == `k', meanonly
            scalar fhome`h'_k`k'_sin = `r(mean)'
            }

        forval e = 1 / $numed {

            forval h = 1 / $numhome {
                su home`h' [aw = $weight] if edulev == `e' & married == 0 & sex == 1 & kid == `k', meanonly
                scalar mhome`h'_k`k'_M`e'F0 = `r(mean)'
                su home`h' [aw = $weight] if edulev == `e' & married == 0 & sex == 2 & kid == `k', meanonly
                scalar fhome`h'_k`k'_M0F`e' = `r(mean)'
                }

            qui su hours_home_discrete [aw = $weight] if edulev == `e' & married == 0 & sex == 1 & kid == `k'
            scalar mhomeh_k`k'_M`e'F0__mean = r(mean)
            scalar mhomeh_k`k'_M`e'F0__sd = r(sd)

            qui su hours_home_discrete [aw = $weight] if edulev == `e' & married == 0 & sex == 2 & kid == `k'
            scalar fhomeh_k`k'_M0F`e'__mean = r(mean)
            scalar fhomeh_k`k'_M0F`e'__sd = r(sd)

            }
        }

    /* marginal distribution of home production, couples (men) */

    forval k = 0 / 1 {

        forval h = 1 / $numhome {
            su home`h' [aw = $weight] if married == 1 & sex == 1 & kid == `k', meanonly
            scalar mhome`h'_k`k'_cou = `r(mean)'
            }

        forval e = 1 / $numed {

            forval h = 1 / $numhome {
                su home`h' [aw = $weight] if edulev == `e' & married == 1 & sex == 1 & kid == `k', meanonly
                scalar mhome`h'_k`k'_M`e'Ff = `r(mean)'
                }

            qui su hours_home_discrete [aw = $weight] if edulev == `e' & married == 1 & sex == 1 & kid == `k'
            scalar mhomeh_k`k'_M`e'Ff__mean = r(mean)
            scalar mhomeh_k`k'_M`e'Ff__sd = r(sd)

            }
        }

    /* marginal distribution of home production, couples (women) */

    forval k = 0 / 1 {

        forval h = 1 / $numhome {
            su home`h' [aw = $weight] if  married == 1 & sex == 2 & kid == `k', meanonly
            scalar fhome`h'_k`k'_cou = `r(mean)'
            }

        forval e = 1 / $numed {

            forval h = 1 / $numhome {
                su home`h' [aw = $weight] if edulev == `e' & married == 1 & sex == 2 & kid == `k', meanonly
                scalar fhome`h'_k`k'_MmF`e' = `r(mean)'
                }

            qui su hours_home_discrete [aw = $weight] if edulev == `e' & married == 1 & sex == 2 & kid == `k'
            scalar fhomeh_k`k'_MmF`e'__mean = r(mean)
            scalar fhomeh_k`k'_MmF`e'__sd = r(sd)

            }
        }


    /* distribution of home production by own/spouse employment status */

    // single men home production (self hours)
    forval k = 0 / 1 {

        qui su hours_home_discrete [aw = $weight] if married == 0 & work==1 & sex == 1 & kid == `k'
        scalar mhomehMw1_k`k'_MmF0__mean = `r(mean)'
        scalar mhomehMw1_k`k'_MmF0__sd =`r(sd)'

        qui su hours_home_discrete [aw = $weight] if married == 0 & work==0 & sex == 1 & kid == `k'
        scalar mhomehMw0_k`k'_MmF0__mean = `r(mean)'
        scalar mhomehMw0_k`k'_MmF0__sd =`r(sd)'

        }

    // married men home production (self hours)
    forval k = 0 / 1 {

        qui su hours_home_discrete [aw = $weight] if married == 1 & work==1 & sex == 1 & kid == `k'
        scalar mhomehMw1_k`k'_MmFf__mean = `r(mean)'
        scalar mhomehMw1_k`k'_MmFf__sd =`r(sd)'

        qui su hours_home_discrete [aw = $weight] if married == 1 & work==0 & sex == 1 & kid == `k'
        scalar mhomehMw0_k`k'_MmFf__mean = `r(mean)'
        scalar mhomehMw0_k`k'_MmFf__sd =`r(sd)'

        }


    // married men home production (spouse work)
    forval k = 0 / 1 {

        qui su hours_home_discrete [aw = $weight] if married == 1 & work_sp==1 & sex == 1 & kid == `k'
        scalar mhomehFw1_k`k'_MmFf__mean = `r(mean)'
        scalar mhomehFw1_k`k'_MmFf__sd = `r(sd)'

        qui su hours_home_discrete [aw = $weight] if married == 1 & work_sp==0 & sex == 1 & kid == `k'
        scalar mhomehFw0_k`k'_MmFf__mean = `r(mean)'
        scalar mhomehFw0_k`k'_MmFf__sd =`r(sd)'

        }

    // single women home production (self hours)
    forval k = 0 / 1 {

        qui su hours_home_discrete [aw = $weight] if married == 0 & ft==1 & sex == 2 & kid == `k'
        scalar fhomehFh2_k`k'_M0Ff__mean = `r(mean)'
        scalar fhomehFh2_k`k'_M0Ff__sd =`r(sd)'

        qui su hours_home_discrete [aw = $weight] if married == 0 & pt==1 & sex == 2 & kid == `k'
        scalar fhomehFh1_k`k'_M0Ff__mean = `r(mean)'
        scalar fhomehFh1_k`k'_M0Ff__sd =`r(sd)'

        qui su hours_home_discrete [aw = $weight] if married == 0 & work==0 & sex == 2 & kid == `k'
        scalar fhomehFh0_k`k'_M0Ff__mean = `r(mean)'
        scalar fhomehFh0_k`k'_M0Ff__sd =`r(sd)'

        }

    // married women home production (self hours)
    forval k = 0 / 1 {

        qui su hours_home_discrete [aw = $weight] if married == 1 & ft==1 & sex == 2 & kid == `k'
        scalar fhomehFh2_k`k'_MmFf__mean = `r(mean)'
        scalar fhomehFh2_k`k'_MmFf__sd =`r(sd)'

        qui su hours_home_discrete [aw = $weight] if married == 1 & pt==1 & sex == 2 & kid == `k'
        scalar fhomehFh1_k`k'_MmFf__mean = `r(mean)'
        scalar fhomehFh1_k`k'_MmFf__sd =`r(sd)'

        qui su hours_home_discrete [aw = $weight] if married == 1 & work==0 & sex == 2 & kid == `k'
        scalar fhomehFh0_k`k'_MmFf__mean = `r(mean)'
        scalar fhomehFh0_k`k'_MmFf__sd =`r(sd)'

        }


    // married women home production (spouse work)
    forval k = 0 / 1 {

        qui su hours_home_discrete [aw = $weight] if married == 1 & work_sp==1 & sex == 2 & kid == `k'
        scalar fhomehMw1_k`k'_MmFf__mean = `r(mean)'
        scalar fhomehMw1_k`k'_MmFf__sd =`r(sd)'

        qui su hours_home_discrete [aw = $weight] if married == 1 & work_sp==0 & sex == 2 & kid == `k'
        scalar fhomehMw0_k`k'_MmFf__mean = `r(mean)'
        scalar fhomehMw0_k`k'_MmFf__sd =`r(sd)'

        }

    // single men and women hours by match
    forval e = 1 / $numed {
        qui su hours_home_discrete [aw = $weight] if married == 0 & sex == 1 & edulev == `e'
        scalar mhomeh_M`e'F0__mean = `r(mean)'
        scalar mhomeh_M`e'F0__sd = `r(sd)'
        qui su hours_home_discrete [aw = $weight] if married == 0 & sex == 2 & edulev == `e'
        scalar fhomeh_M0F`e'__mean = `r(mean)'
        scalar fhomeh_M0F`e'__sd = `r(sd)'
        }

    // married men and women hours by match
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su hours_home_discrete [aw = $weight] if married == 1 & sex == 1 & edulev == `e' & edulev_sp == `esp'
            scalar mhomeh_M`e'F`esp'__mean = `r(mean)'
            scalar mhomeh_M`e'F`esp'__sd = `r(sd)'
            qui su hours_home_discrete [aw = $weight] if married == 1 & sex == 2 & edulev == `e' & edulev_sp == `esp'
            scalar fhomeh_M`e'F`esp'__mean = `r(mean)'
            scalar fhomeh_M`e'F`esp'__sd = `r(sd)'
            }
        }

    // home time by kids and marital status integrating type
    forval k = 0 / 1 {
        qui su hours_home_discrete [aw = $weight] if married == 0 & sex == 1 & kid == `k'
        scalar mhomeh_k`k'_MmF0__mean = `r(mean)'
        scalar mhomeh_k`k'_MmF0__sd = `r(sd)'
        qui su hours_home_discrete [aw = $weight] if married == 0 & sex == 2 & kid == `k'
        scalar fhomeh_k`k'_M0Ff__mean = `r(mean)'
        scalar fhomeh_k`k'_M0Ff__sd = `r(sd)'
        qui su hours_home_discrete [aw = $weight] if married == 1 & sex == 1 & kid == `k'
        scalar mhomeh_k`k'_MmFf__mean = `r(mean)'
        scalar mhomeh_k`k'_MmFf__sd = `r(sd)'
        qui su hours_home_discrete [aw = $weight] if married == 1 & sex == 2 & kid == `k'
        scalar fhomeh_k`k'_MmFf__mean = `r(mean)'
        scalar fhomeh_k`k'_MmFf__sd = `r(sd)'
        }

    // home time by marital status integrating type and kids
    qui su hours_home_discrete [aw = $weight] if married == 0 & sex == 1
    scalar mhomeh_MmF0__mean = `r(mean)'
    scalar mhomeh_MmF0__sd = `r(sd)'
    qui su hours_home_discrete [aw = $weight] if married == 0 & sex == 2
    scalar fhomeh_M0Ff__mean = `r(mean)'
    scalar fhomeh_M0Ff__sd = `r(sd)'
    qui su hours_home_discrete [aw = $weight] if married == 1 & sex == 1
    scalar mhomeh_MmFf__mean = `r(mean)'
    scalar mhomeh_MmFf__sd = `r(sd)'
    qui su hours_home_discrete [aw = $weight] if married == 1 & sex == 2
    scalar fhomeh_MmFf__mean = `r(mean)'
    scalar fhomeh_MmFf__sd = `r(sd)'

    post momATUS (`r') `postlist'

    restore

    }

postclose momATUS

use "$save_data/mom_atus_ed$numed`'_age$minage$maxage$statetag.dta", clear
