set more off

cap program drop dirconfirm
program define dirconfirm, rclass
    version 11.2
    local current_dir "`c(pwd)'"
    qui capture cd "`1'"
    local retcode = _rc

    if _rc != 0 {
        di as error "directory " "`1'" " not found"
        exit `retcode'
        }

    qui cd "`current_dir'"
    return scalar rc = `retcode'

end

cap program drop createMomentFiles
program define createMomentFiles

    version 11.2
    syntax [, bs(integer 0) all data sd]

    cap file close momfile

    local mode 1

    local fpath "$mom_file/ed$numed`'_age$minage$maxage$statetag"
    cap dirconfirm "`fpath'"
    if _rc != 0 {
        mkdir "`fpath'"
        }

    local statout "r(mean)"
    if `bs' > 0 {
        local fpath "$mom_file/ed$numed`'_age$minage$maxage$statetag/sample"
        cap dirconfirm "`fpath'"
        if _rc != 0 {
            mkdir "`fpath'"
            }
        local fpath "$mom_file/ed$numed`'_age$minage$maxage$statetag/sample/`bs'"
        cap dirconfirm "`fpath'"
        if _rc != 0 {
            mkdir "`fpath'"
            }
        local stat "r == `bs', meanonly"
        local statout "r(mean)"
        }
    else {
        local fpath "$mom_file/ed$numed`'_age$minage$maxage$statetag/point"
        cap dirconfirm "`fpath'"
        if _rc != 0 {
            mkdir "`fpath'"
            }
        local stat "r == 0, meanonly"
        local statout "r(mean)"
        }

    if "`sd'" == "sd" {
        local bs 0
        local fpath "$mom_file/ed$numed`'_age$minage$maxage$statetag/stddev"
        cap dirconfirm "`fpath'"
        if _rc != 0 {
            mkdir "`fpath'"
            }
        local stat "r > 0"
        local statout "r(sd)"
        }

    if "`data'" == "data" {
        local bs 0
        local fpath "$mom_file/ed$numed`'_age$minage$maxage$statetag/mode"
        cap dirconfirm "`fpath'"
        if _rc != 0 {
            mkdir "`fpath'"
            }
        local stat "r < 0, meanonly"
        local statout "mode"
        }

    if "`all'" == "all" {
        local bs 0
        local fpath "$mom_file/ed$numed`'_age$minage$maxage$statetag"
        cap dirconfirm "`fpath'"
        if _rc != 0 {
            mkdir "`fpath'"
            }
        local stat "r == 0, meanonly"
        local statout "r(mean)"
        }

    di "Save path: `fpath'"

    // save number of bootstrap samples and markets
    if "`all'" == "all" {

        qui file open momfile using "`fpath'/nsamples.asc", text write replace
        file write momfile "$bsrep"
        file write momfile _n
        file close momfile

        qui file open momfile using "`fpath'/nmarket.asc", text write replace
        file write momfile "$nummarket"
        file write momfile _n
        file close momfile

        }

    // type distribution

    // jtype_M0(NTYPE1)
    qui file open momfile using "`fpath'/jtype_M0.asc", text write replace
    forval e = 1 / $numed {
        qui su jtype_M`e'_F0 if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su jtype_M`e'_F0 if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // jtype_0F(NTYPE2)
    qui file open momfile using "`fpath'/jtype_0F.asc", text write replace
    forval e = 1 / $numed {
        qui su jtype_M0_F`e' if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su jtype_M0_F`e' if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // jtype_MF(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/jtype_MF.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su jtype_M`e'_F`esp' if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su jtype_M`e'_F`esp' if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile



    // Rjtype_M0(NTYPE1, NMARKET)
    qui file open momfile using "`fpath'/Rjtype_M0.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'jtype_M`e'_F0 if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'jtype_M`e'_F0 if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // Rjtype_0F(NTYPE2, NMARKET)
    qui file open momfile using "`fpath'/Rjtype_0F.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'jtype_M0_F`e' if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'jtype_M0_F`e' if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // Rjtype_MF(NTYPE1, NTYPE2, NMARKET)
    qui file open momfile using "`fpath'/Rjtype_MF.asc", text write replace
    forval m = 1 / $nummarket {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                qui su R`m'jtype_M`e'_F`esp' if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su R`m'jtype_M`e'_F`esp' if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // joint labour supply allocation

    // mhours_M0(NHH, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhours_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 0 / 2 {
                qui su mhours`h'_k`k'_M`e'_F0 if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mhours`h'_k`k'_M`e'_F0 if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // fhours_0F(NHH, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhours_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 0 / 2 {
                qui su fhours`h'_k`k'_M0_F`e' if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su fhours`h'_k`k'_M0_F`e' if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // mhours_Mf(NHH, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhours_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 0 / 2 {
                qui su mhours`h'_k`k'_M`e'_Ff if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mhours`h'_k`k'_M`e'_Ff if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // fhours_mF(NHH, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhours_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 0 / 2 {
                qui su fhours`h'_k`k'_Mm_F`e' if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su fhours`h'_k`k'_Mm_F`e' if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile



    // mhours_sin(NHH, 0:1)
    qui file open momfile using "`fpath'/mhours_sin.asc", text write replace
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui su mhours`h'_k`k'_sin if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhours`h'_k`k'_sin if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhours_sin(NHH, 0:1)
    qui file open momfile using "`fpath'/fhours_sin.asc", text write replace
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui su fhours`h'_k`k'_sin if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhours`h'_k`k'_sin if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhours_cou(NHH, 0:1)
    qui file open momfile using "`fpath'/mhours_cou.asc", text write replace
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui su mhours`h'_k`k'_cou if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhours`h'_k`k'_cou if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhours_cou(NHH, 0:1)
    qui file open momfile using "`fpath'/fhours_cou.asc", text write replace
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui su fhours`h'_k`k'_cou if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhours`h'_k`k'_cou if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // jhours_MF(NHH, NHH, NTYPE1, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/jhours_MF.asc", text write replace
    forval k = 0 / 1 {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                forval hsp = 0 / 2 {
                    forval h = 0 / 2 {
                        qui su jhoursMh`h'Fh`hsp'_k`k'_M`e'_F`esp' if `stat'
                        file write momfile "``statout''"
                        if "`all'" == "all" {
                            qui su jhoursMh`h'Fh`hsp'_k`k'_M`e'_F`esp' if r > 0
                            file write momfile ", `r(sd)', `mode'"
                            }
                        file write momfile _n
                        }
                    }
                }
            }
        }
    file close momfile

    // mhoursmean_M0(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhoursmean_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mhours_k`k'_M`e'_F0__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhours_k`k'_M`e'_F0__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhourssd_M0(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhourssd_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mhours_k`k'_M`e'_F0__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhours_k`k'_M`e'_F0__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // fhoursmean_0F(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhoursmean_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su fhours_k`k'_M0_F`e'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhours_k`k'_M0_F`e'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhourssd_0F(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhourssd_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su fhours_k`k'_M0_F`e'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhours_k`k'_M0_F`e'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhoursmean_Mf(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhoursmean_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mhours_k`k'_M`e'_Ff__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhours_k`k'_M`e'_Ff__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhourssd_Mf(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhourssd_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mhours_k`k'_M`e'_Ff__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhours_k`k'_M`e'_Ff__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // fhoursmean_mF(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhoursmean_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su fhours_k`k'_Mm_F`e'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhours_k`k'_Mm_F`e'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhourssd_mF(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhourssd_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su fhours_k`k'_Mm_F`e'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhours_k`k'_Mm_F`e'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // Rmhoursmean_M0(NTYPE1, NMARKET)
    qui file open momfile using "`fpath'/Rmhoursmean_M0.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'mhoursh_M`e'_F0 if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'mhoursh_M`e'_F0 if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // Rfhoursmean_0F(NTYPE2, NMARKET)
    qui file open momfile using "`fpath'/Rfhoursmean_0F.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'fhoursh_M0_F`e' if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'fhoursh_M0_F`e' if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // Rmhoursmean_Mf(NTYPE1, NMARKET)
    qui file open momfile using "`fpath'/Rmhoursmean_Mf.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'mhoursh_M`e'_Ff if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'mhoursh_M`e'_Ff if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // Rfhoursmean_mF(NTYPE2, NMARKET)
    qui file open momfile using "`fpath'/Rfhoursmean_mF.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'fhoursh_Mm_F`e' if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'fhoursh_Mm_F`e' if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // Rmwork_M0(NTYPE1, NMARKET)
    qui file open momfile using "`fpath'/Rmwork_M0.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'mhoursW_M`e'_F0 if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'mhoursW_M`e'_F0 if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // Rfwork_0F(NTYPE2, NMARKET)
    qui file open momfile using "`fpath'/Rfwork_0F.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'fhoursW_M0_F`e' if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'fhoursW_M0_F`e' if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // Rmwork_Mf(NTYPE1, NMARKET)
    qui file open momfile using "`fpath'/Rmwork_Mf.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'mhoursW_M`e'_Ff if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'mhoursW_M`e'_Ff if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // Rfwork_mF(NTYPE2, NMARKET)
    qui file open momfile using "`fpath'/Rfwork_mF.asc", text write replace
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            qui su R`m'fhoursW_Mm_F`e' if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su R`m'fhoursW_Mm_F`e' if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // wage distribution (single men)

    // mwage_M0(NHH, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwage_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su mlogwage_M`e'F0_k`k'_h`h'__mean if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mlogwage_M`e'F0_k`k'_h`h'__mean if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // mwagesd_M0(NHH, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwagesd_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su mlogwage_M`e'F0_k`k'_h`h'__sd if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mlogwage_M`e'F0_k`k'_h`h'__sd if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // wage distribution all workers (single men)

    // mwageMw_M0(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwageMw_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mlogwageMw_M`e'F0_k`k'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mlogwageMw_M`e'F0_k`k'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mwageMwAll_M0(NTYPE1)
    qui file open momfile using "`fpath'/mwageMwAll_M0.asc", text write replace
    forval e = 1 / $numed {
        qui su mlogwageMw_M`e'F0__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mlogwageMw_M`e'F0__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // mwagesdMw_M0(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwagesdMw_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mlogwageMw_M`e'F0_k`k'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mlogwageMw_M`e'F0_k`k'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mwagesdMwAll_M0(NTYPE1)
    qui file open momfile using "`fpath'/mwagesdMwAll_M0.asc", text write replace
    forval e = 1 / $numed {
        qui su mlogwageMw_M`e'F0__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mlogwageMw_M`e'F0__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile


    // wage distribution (single women)

    // fwage_0F(NHH, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwage_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su flogwage_M0F`e'_k`k'_h`h'__mean if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su flogwage_M0F`e'_k`k'_h`h'__mean if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // fwagesd_0F(NHH, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwagesd_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su flogwage_M0F`e'_k`k'_h`h'__sd if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su flogwage_M0F`e'_k`k'_h`h'__sd if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // wage distribution all workers (single women)

    // fwageFw_0F(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwageFw_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su flogwageFw_M0F`e'_k`k'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su flogwageFw_M0F`e'_k`k'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fwageFwAll_0F(NTYPE2)
    qui file open momfile using "`fpath'/fwageFwAll_0F.asc", text write replace
    forval e = 1 / $numed {
        qui su flogwageFw_M0F`e'__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su flogwageFw_M0F`e'__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fwagesdFw_0F(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwagesdFw_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su flogwageFw_M0F`e'_k`k'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su flogwageFw_M0F`e'_k`k'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fwagesdFwAll_0F(NTYPE2)
    qui file open momfile using "`fpath'/fwagesdFwAll_0F.asc", text write replace
    forval e = 1 / $numed {
        qui su flogwageFw_M0F`e'__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su flogwageFw_M0F`e'__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // wage distribution (married men, integrates over spouse type/state)

    // mwage_Mf(NHH, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwage_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su mlogwage_M`e'Ff_k`k'_h`h'__mean if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mlogwage_M`e'Ff_k`k'_h`h'__mean if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // mwagesd_Mf(NHH, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwagesd_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su mlogwage_M`e'Ff_k`k'_h`h'__sd if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mlogwage_M`e'Ff_k`k'_h`h'__sd if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // wage distribution all workers (married men, integrates over spouse type/state)

    // mwageMw_Mf(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwageMw_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mlogwageMw_M`e'Ff_k`k'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mlogwageMw_M`e'Ff_k`k'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mwageMwAll_Mf(NTYPE1)
    qui file open momfile using "`fpath'/mwageMwAll_Mf.asc", text write replace
    forval e = 1 / $numed {
        qui su mlogwageMw_M`e'Ff__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mlogwageMw_M`e'Ff__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile


    // mwagesdMw_Mf(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwagesdMw_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mlogwageMw_M`e'Ff_k`k'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mlogwageMw_M`e'Ff_k`k'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mwagesdMwAll_Mf(NTYPE1)
    qui file open momfile using "`fpath'/mwagesdMwAll_Mf.asc", text write replace
    forval e = 1 / $numed {
        qui su mlogwageMw_M`e'Ff__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mlogwageMw_M`e'Ff__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // wage distribution (married women, integrates over spouse type/state)

    // fwage_mF(NHH, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwage_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su flogwage_MmF`e'_k`k'_h`h'__mean if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su flogwage_MmF`e'_k`k'_h`h'__mean if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // fwagesd_mF(NHH, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwagesd_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su flogwage_MmF`e'_k`k'_h`h'__sd if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su flogwage_MmF`e'_k`k'_h`h'__sd if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // wage distribution all workers (married women, integrates over spouse type/state)

    // fwageFw_mF(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwageFw_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su flogwageFw_MmF`e'_k`k'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su flogwageFw_MmF`e'_k`k'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fwageFwAll_mF(NTYPE2)
    qui file open momfile using "`fpath'/fwageFwAll_mF.asc", text write replace
    forval e = 1 / $numed {
        qui su flogwageFw_MmF`e'__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su flogwageFw_MmF`e'__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fwagesdFw_mF(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwagesdFw_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su flogwageFw_MmF`e'_k`k'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su flogwageFw_MmF`e'_k`k'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fwagesdFwAll_mF(NTYPE2)
    qui file open momfile using "`fpath'/fwagesdFwAll_mF.asc", text write replace
    forval e = 1 / $numed {
        qui su flogwageFw_MmF`e'__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su flogwageFw_MmF`e'__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile


    // wage distribution (married men, by spousal employment state)

    // mwageFw_Mf(0:1, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwageFw_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval w = 0 / 1 {
                qui su mlogwageFw`w'_M`e'Ff_k`k'__mean if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mlogwageFw`w'_M`e'Ff_k`k'__mean if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // mwageFwsd_Mf(0:1, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mwageFwsd_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval w = 0 / 1 {
                qui su mlogwageFw`w'_M`e'Ff_k`k'__sd if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mlogwageFw`w'_M`e'Ff_k`k'__sd if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // wage distribution (married women, by spousal employment state)

    // fwageMw_mF(0:1, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwageMw_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval w = 0 / 1 {
                qui su flogwageMw`w'_MmF`e'_k`k'__mean if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su flogwageMw`w'_MmF`e'_k`k'__mean if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // fwageMwAll_mF(0:1, NTYPE2)
    qui file open momfile using "`fpath'/fwageMwAll_mF.asc", text write replace
    forval e = 1 / $numed {
        forval w = 0 / 1 {
            qui su flogwageMw`w'_MmF`e'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su flogwageMw`w'_MmF`e'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fwageMwsd_mF(0:1, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fwageMwsd_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval w = 0 / 1 {
                qui su flogwageMw`w'_MmF`e'_k`k'__sd if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su flogwageMw`w'_MmF`e'_k`k'__sd if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // fwageMwsdAll_mF(0:1, NTYPE2)
    qui file open momfile using "`fpath'/fwageMwsdAll_mF.asc", text write replace
    forval e = 1 / $numed {
        forval w = 0 / 1 {
            qui su flogwageMw`w'_MmF`e'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su flogwageMw`w'_MmF`e'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // product of wages within a match

    // jwage_MF(NTYPE1, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/jwage_MF.asc", text write replace
    forval k = 0 / 1 {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                qui su jlogwage_M`e'F`esp'_k`k'__mean if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su jlogwage_M`e'F`esp'_k`k'__mean if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // jwageAll_MF(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/jwageAll_MF.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su jlogwage_M`e'F`esp'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su jlogwage_M`e'F`esp'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // jwagesd_MF(NTYPE1, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/jwagesd_MF.asc", text write replace
    forval k = 0 / 1 {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                qui su jlogwage_M`e'F`esp'_k`k'__sd if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su jlogwage_M`e'F`esp'_k`k'__sd if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // jwagesdAll_MF(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/jwagesdAll_MF.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su jlogwage_M`e'F`esp'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su jlogwage_M`e'F`esp'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // jwagecov_MF(NTYPE1, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/jwagecov_MF.asc", text write replace
    forval k = 0 / 1 {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                qui su jlogwage_M`e'F`esp'_k`k'__cov if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su jlogwage_M`e'F`esp'_k`k'__cov if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // jwagecovAll_MF(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/jwagecovAll_MF.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su jlogwage_M`e'F`esp'__cov if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su jlogwage_M`e'F`esp'__cov if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // marginal distribution of male hours
    qui file open momfile using "`fpath'/mhours_MF.asc", text write replace
    forval k = 0 / 1 {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                forval h = 0 / 2 {
                    qui su mhoursMh`h'_k`k'_M`e'_F`esp' if `stat'
                    file write momfile "``statout''"
                    if "`all'" == "all" {
                        qui su mhoursMh`h'_k`k'_M`e'_F`esp' if r > 0
                        file write momfile ", `r(sd)', `mode'"
                        }
                    file write momfile _n
                    }
                }
            }
        }
    file close momfile


    // marginal distribution of female hours
    qui file open momfile using "`fpath'/fhours_MF.asc", text write replace
    forval k = 0 / 1 {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                forval h = 0 / 2 {
                    qui su fhoursFh`h'_k`k'_M`e'_F`esp' if `stat'
                    file write momfile "``statout''"
                    if "`all'" == "all" {
                        qui su fhoursFh`h'_k`k'_M`e'_F`esp' if r > 0
                        file write momfile ", `r(sd)', `mode'"
                        }
                    file write momfile _n
                    }
                }
            }
        }
    file close momfile

    // married men and women work hours by match

    // mhoursh_M_F(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/mhoursh_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su mhoursh_M`e'_F`esp'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhoursh_M`e'_F`esp'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhourshsd_M_F(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/mhourshsd_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su mhoursh_M`e'_F`esp'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhoursh_M`e'_F`esp'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // fhoursh_M_F(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/fhoursh_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su fhoursh_M`e'_F`esp'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhoursh_M`e'_F`esp'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhourshsd_M_F(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/fhourshsd_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su fhoursh_M`e'_F`esp'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhoursh_M`e'_F`esp'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // single men and women work hours by match

    // mhoursh_M_0(NTYPE1)
    qui file open momfile using "`fpath'/mhoursh_M_0.asc", text write replace
    forval e = 1 / $numed {
        qui su mhoursh_M`e'_F0__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mhoursh_M`e'_F0__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // mhourshsd_M_0(NTYPE1)
    qui file open momfile using "`fpath'/mhourshsd_M_0.asc", text write replace
    forval e = 1 / $numed {
        qui su mhoursh_M`e'_F0__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mhoursh_M`e'_F0__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fhoursh_0_F(NTYPE2)
    qui file open momfile using "`fpath'/fhoursh_0_F.asc", text write replace
    forval e = 1 / $numed {
        qui su fhoursh_M0_F`e'__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fhoursh_M0_F`e'__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fhourshsd_0_F(NTYPE2)
    qui file open momfile using "`fpath'/fhourshsd_0_F.asc", text write replace
    forval e = 1 / $numed {
        qui su fhoursh_M0_F`e'__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fhoursh_M0_F`e'__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile



    // married men and women employment by match

    // mwork_M_F(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/mwork_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su mwork_M`e'_F`esp' if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mwork_M`e'_F`esp' if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fwork_M_F(NTYPE1, NTYPE2)
    qui file open momfile using "`fpath'/fwork_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su fwork_M`e'_F`esp' if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fwork_M`e'_F`esp' if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // single men and women work hours by match

    // mwork_M_0(NTYPE1)
    qui file open momfile using "`fpath'/mwork_M_0.asc", text write replace
    forval e = 1 / $numed {
        qui su mwork_M`e'_F0 if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mwork_M`e'_F0 if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fwork_0_F(NTYPE2)
    qui file open momfile using "`fpath'/fwork_0_F.asc", text write replace
    forval e = 1 / $numed {
        qui su fwork_M0_F`e' if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fwork_M0_F`e' if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile


    // earnings distribution all workers (single men)

    // mearnMwAll_M0(NTYPE1)
    qui file open momfile using "`fpath'/mearnMwAll_M0.asc", text write replace
    forval e = 1 / $numed {
        qui su mearnMw_M`e'F0__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mearnMw_M`e'F0__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // mearnsdMwAll_M0(NTYPE1)
    qui file open momfile using "`fpath'/mearnsdMwAll_M0.asc", text write replace
    forval e = 1 / $numed {
        qui su mearnMw_M`e'F0__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mearnMw_M`e'F0__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // earnings distribution all workers (single women)

    // fearnFwAll_0F(NTYPE2)
    qui file open momfile using "`fpath'/fearnFwAll_0F.asc", text write replace
    forval e = 1 / $numed {
        qui su fearnFw_M0F`e'__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fearnFw_M0F`e'__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fearnsdFwAll_0F(NTYPE2)
    qui file open momfile using "`fpath'/fearnsdFwAll_0F.asc", text write replace
    forval e = 1 / $numed {
        qui su fearnFw_M0F`e'__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fearnFw_M0F`e'__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // earnings distribution all workers (married men)

    // mearnMwAll_Mf(NTYPE1)
    qui file open momfile using "`fpath'/mearnMwAll_Mf.asc", text write replace
    forval e = 1 / $numed {
        qui su mearnMw_M`e'Ff__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mearnMw_M`e'Ff__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // mearnsdMwAll_Mf(NTYPE1)
    qui file open momfile using "`fpath'/mearnsdMwAll_Mf.asc", text write replace
    forval e = 1 / $numed {
        qui su mearnMw_M`e'Ff__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mearnMw_M`e'Ff__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // earnings distribution all workers (married women)

    // fearnFwAll_mF(NTYPE2)
    qui file open momfile using "`fpath'/fearnFwAll_mF.asc", text write replace
    forval e = 1 / $numed {
        qui su fearnFw_MmF`e'__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fearnFw_MmF`e'__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fearnsdFwAll_mF(NTYPE2)
    qui file open momfile using "`fpath'/fearnsdFwAll_mF.asc", text write replace
    forval e = 1 / $numed {
        qui su fearnFw_MmF`e'__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fearnFw_MmF`e'__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // aggregated work

    // mwork_m0
    qui file open momfile using "`fpath'/mwork_m0.asc", text write replace
    qui su mwork_MmF0 if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mwork_MmF0 if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mwork_mf
    qui file open momfile using "`fpath'/mwork_mf.asc", text write replace
    qui su mwork_MmFf if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mwork_MmFf if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fwork_0f
    qui file open momfile using "`fpath'/fwork_0f.asc", text write replace
    qui su fwork_M0Ff if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fwork_M0Ff if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fwork_mf
    qui file open momfile using "`fpath'/fwork_mf.asc", text write replace
    qui su fwork_MmFf if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fwork_MmFf if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // aggregated conditional hours (mean)

    // mhourshMw_m0
    qui file open momfile using "`fpath'/mhourshMw_m0.asc", text write replace
    qui su mhourshMw_MmF0__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhourshMw_MmF0__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mhourshMw_mf
    qui file open momfile using "`fpath'/mhourshMw_mf.asc", text write replace
    qui su mhourshMw_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhourshMw_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhourshFw_0f
    qui file open momfile using "`fpath'/fhourshFw_0f.asc", text write replace
    qui su fhourshFw_M0Ff__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhourshFw_M0Ff__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhourshFw_mf
    qui file open momfile using "`fpath'/fhourshFw_mf.asc", text write replace
    qui su fhourshFw_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhourshFw_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // aggregated conditional hours (sd)

    // mhourshsdMw_m0
    qui file open momfile using "`fpath'/mhourshsdMw_m0.asc", text write replace
    qui su mhourshMw_MmF0__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhourshMw_MmF0__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mhourshsdMw_mf
    qui file open momfile using "`fpath'/mhourshsdMw_mf.asc", text write replace
    qui su mhourshMw_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhourshMw_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhourshsdFw_0f
    qui file open momfile using "`fpath'/fhourshsdFw_0f.asc", text write replace
    qui su fhourshFw_M0Ff__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhourshFw_M0Ff__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhourshsdFw_mf
    qui file open momfile using "`fpath'/fhourshsdFw_mf.asc", text write replace
    qui su fhourshFw_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhourshFw_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // aggregated unconditional hours (mean)

    // mhoursh_m0
    qui file open momfile using "`fpath'/mhoursh_m0.asc", text write replace
    qui su mhoursh_MmF0__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhoursh_MmF0__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mhoursh_mf
    qui file open momfile using "`fpath'/mhoursh_mf.asc", text write replace
    qui su mhoursh_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhoursh_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhoursh_0f
    qui file open momfile using "`fpath'/fhoursh_0f.asc", text write replace
    qui su fhoursh_M0Ff__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhoursh_M0Ff__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhoursh_mf
    qui file open momfile using "`fpath'/fhoursh_mf.asc", text write replace
    qui su fhoursh_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhoursh_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // aggregated unconditional hours (sd)

    // mhourshsd_m0
    qui file open momfile using "`fpath'/mhourshsd_m0.asc", text write replace
    qui su mhoursh_MmF0__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhoursh_MmF0__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mhourshsd_mf
    qui file open momfile using "`fpath'/mhourshsd_mf.asc", text write replace
    qui su mhoursh_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhoursh_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhourshsd_0f
    qui file open momfile using "`fpath'/fhourshsd_0f.asc", text write replace
    qui su fhoursh_M0Ff__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhoursh_M0Ff__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhourshsd_mf
    qui file open momfile using "`fpath'/fhourshsd_mf.asc", text write replace
    qui su fhoursh_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhoursh_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile


    // aggregated conditional earnings (mean)

    // mearnMw_m0
    qui file open momfile using "`fpath'/mearnMw_m0.asc", text write replace
    qui su mearnMw_MmF0__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mearnMw_MmF0__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mearnMw_mf
    qui file open momfile using "`fpath'/mearnMw_mf.asc", text write replace
    qui su mearnMw_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mearnMw_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fearnFw_0f
    qui file open momfile using "`fpath'/fearnFw_0f.asc", text write replace
    qui su fearnFw_M0Ff__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fearnFw_M0Ff__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fearnFw_mf
    qui file open momfile using "`fpath'/fearnFw_mf.asc", text write replace
    qui su fearnFw_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fearnFw_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // aggregated conditional earnings (sd)

    // mearnsdMw_m0
    qui file open momfile using "`fpath'/mearnsdMw_m0.asc", text write replace
    qui su mearnMw_MmF0__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mearnMw_MmF0__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mearnsdMw_mf
    qui file open momfile using "`fpath'/mearnsdMw_mf.asc", text write replace
    qui su mearnMw_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mearnMw_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fearnsdFw_0f
    qui file open momfile using "`fpath'/fearnsdFw_0f.asc", text write replace
    qui su fearnFw_M0Ff__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fearnFw_M0Ff__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fearnsdFw_mf
    qui file open momfile using "`fpath'/fearnsdFw_mf.asc", text write replace
    qui su fearnFw_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fearnFw_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile


    // aggregated unconditional earnings (mean)

    // mearn_m0
    qui file open momfile using "`fpath'/mearn_m0.asc", text write replace
    qui su mearn_MmF0__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mearn_MmF0__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mearn_mf
    qui file open momfile using "`fpath'/mearn_mf.asc", text write replace
    qui su mearn_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mearn_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fearn_0f
    qui file open momfile using "`fpath'/fearn_0f.asc", text write replace
    qui su fearn_M0Ff__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fearn_M0Ff__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fearn_mf
    qui file open momfile using "`fpath'/fearn_mf.asc", text write replace
    qui su fearn_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fearn_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // aggregated unconditional earnings (sd)

    // mearnsd_m0
    qui file open momfile using "`fpath'/mearnsd_m0.asc", text write replace
    qui su mearn_MmF0__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mearn_MmF0__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mearnsd_mf
    qui file open momfile using "`fpath'/mearnsd_mf.asc", text write replace
    qui su mearn_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mearn_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fearnsd_0f
    qui file open momfile using "`fpath'/fearnsd_0f.asc", text write replace
    qui su fearn_M0Ff__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fearn_M0Ff__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fearnsd_mf
    qui file open momfile using "`fpath'/fearnsd_mf.asc", text write replace
    qui su fearn_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fearn_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    /* **** ATUS **** */


    cap file close momfile
    local mode 2

    // marginal distribution of home production time (single men/women)
    // mhome_M0(NHH, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhome_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / $numhome {
                qui su mhome`h'_k`k'_M`e'F0 if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mhome`h'_k`k'_M`e'F0 if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // fhome_0F(NHH, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhome_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / $numhome {
                qui su fhome`h'_k`k'_M0F`e' if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su fhome`h'_k`k'_M0F`e' if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile


    // marginal distribution of home production time (married men/women)

    // mhome_Mf(NHH, NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhome_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / $numhome {
                qui su mhome`h'_k`k'_M`e'Ff if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su mhome`h'_k`k'_M`e'Ff if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile

    // fhome_mF(NHH, NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhome_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / $numhome {
                qui su fhome`h'_k`k'_MmF`e' if `stat'
                file write momfile "``statout''"
                if "`all'" == "all" {
                    qui su fhome`h'_k`k'_MmF`e' if r > 0
                    file write momfile ", `r(sd)', `mode'"
                    }
                file write momfile _n
                }
            }
        }
    file close momfile



    // mhome_sin(NHH, 0:1)
    qui file open momfile using "`fpath'/mhome_sin.asc", text write replace
    forval k = 0 / 1 {
        forval h = 1 / $numhome {
            qui su mhome`h'_k`k'_sin if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhome`h'_k`k'_sin if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhome_sin(NHH, 0:1)
    qui file open momfile using "`fpath'/fhome_sin.asc", text write replace
    forval k = 0 / 1 {
        forval h = 1 / $numhome {
            qui su fhome`h'_k`k'_sin if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhome`h'_k`k'_sin if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhome_cou(NHH, 0:1)
    qui file open momfile using "`fpath'/mhome_cou.asc", text write replace
    forval k = 0 / 1 {
        forval h = 1 / $numhome {
            qui su mhome`h'_k`k'_cou if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhome`h'_k`k'_cou if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhome_cou(NHH, 0:1)
    qui file open momfile using "`fpath'/fhome_cou.asc", text write replace
    forval k = 0 / 1 {
        forval h = 1 / $numhome {
            qui su fhome`h'_k`k'_cou if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhome`h'_k`k'_cou if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // home production hours (single men)
    // mhomeh_M0(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhomeh_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mhomeh_k`k'_M`e'F0__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomeh_k`k'_M`e'F0__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhomehsd_M0(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhomehsd_M0.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mhomeh_k`k'_M`e'F0__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomeh_k`k'_M`e'F0__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // home production hours (single men, just by kids)
    // mhomehk_m0(0:1)
    qui file open momfile using "`fpath'/mhomehk_m0.asc", text write replace
    forval k = 0 / 1 {
        qui su mhomeh_k`k'_MmF0__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mhomeh_k`k'_MmF0__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // mhomehksd_m0(0:1)
    qui file open momfile using "`fpath'/mhomehksd_m0.asc", text write replace
    forval k = 0 / 1 {
        qui su mhomeh_k`k'_MmF0__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mhomeh_k`k'_MmF0__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // home production hours (single women)
    // fhomeh_0F(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhomeh_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su fhomeh_k`k'_M0F`e'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomeh_k`k'_M0F`e'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhomehsd_0F(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhomehsd_0F.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su fhomeh_k`k'_M0F`e'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomeh_k`k'_M0F`e'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // home production hours (single women, just by kids)
    // fhomehk_0f(0:1)
    qui file open momfile using "`fpath'/fhomehk_0f.asc", text write replace
    forval k = 0 / 1 {
        qui su fhomeh_k`k'_M0Ff__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fhomeh_k`k'_M0Ff__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fhomehksd_0F(0:1)
    qui file open momfile using "`fpath'/fhomehksd_0f.asc", text write replace
    forval k = 0 / 1 {
        qui su fhomeh_k`k'_M0Ff__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fhomeh_k`k'_M0Ff__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile


    // home production hours (married men)
    // mhomeh_Mf(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhomeh_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mhomeh_k`k'_M`e'Ff__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomeh_k`k'_M`e'Ff__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhomehsd_Mf(NTYPE1, 0:1)
    qui file open momfile using "`fpath'/mhomehsd_Mf.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su mhomeh_k`k'_M`e'Ff__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomeh_k`k'_M`e'Ff__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // home production hours (married men, just by kids)
    // mhomehk_mf(0:1)
    qui file open momfile using "`fpath'/mhomehk_mf.asc", text write replace
    forval k = 0 / 1 {
        qui su mhomeh_k`k'_MmFf__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mhomeh_k`k'_MmFf__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // mhomehksd_mf(0:1)
    qui file open momfile using "`fpath'/mhomehksd_mf.asc", text write replace
    forval k = 0 / 1 {
        qui su mhomeh_k`k'_MmFf__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mhomeh_k`k'_MmFf__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // home production hours (married women)
    // fhomeh_mF(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhomeh_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su fhomeh_k`k'_MmF`e'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomeh_k`k'_MmF`e'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhomehsd_mF(NTYPE2, 0:1)
    qui file open momfile using "`fpath'/fhomehsd_mF.asc", text write replace
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su fhomeh_k`k'_MmF`e'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomeh_k`k'_MmF`e'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // home production hours (married women, just by kids)
    // fhomehk_mf(0:1)
    qui file open momfile using "`fpath'/fhomehk_mf.asc", text write replace
    forval k = 0 / 1 {
        qui su fhomeh_k`k'_MmFf__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fhomeh_k`k'_MmFf__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fhomehsd_mf(0:1)
    qui file open momfile using "`fpath'/fhomehksd_mf.asc", text write replace
    forval k = 0 / 1 {
        qui su fhomeh_k`k'_MmFf__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fhomeh_k`k'_MmFf__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile


    // married men and women home hours by match
    qui file open momfile using "`fpath'/mhomeh_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su mhomeh_M`e'F`esp'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomeh_M`e'F`esp'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    qui file open momfile using "`fpath'/mhomehsd_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su mhomeh_M`e'F`esp'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomeh_M`e'F`esp'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    qui file open momfile using "`fpath'/fhomeh_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su fhomeh_M`e'F`esp'__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomeh_M`e'F`esp'__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    qui file open momfile using "`fpath'/fhomehsd_M_F.asc", text write replace
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            qui su fhomeh_M`e'F`esp'__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomeh_M`e'F`esp'__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // single men and women home hours by match

    // mhomeh_M_0(NTYPE1)
    qui file open momfile using "`fpath'/mhomeh_M_0.asc", text write replace
    forval e = 1 / $numed {
        qui su mhomeh_M`e'F0__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mhomeh_M`e'F0__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // mhomehsd_M_0(NTYPE1)
    qui file open momfile using "`fpath'/mhomehsd_M_0.asc", text write replace
    forval e = 1 / $numed {
        qui su mhomeh_M`e'F0__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su mhomeh_M`e'F0__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fhomeh_0_F(NTYPE2)
    qui file open momfile using "`fpath'/fhomeh_0_F.asc", text write replace
    forval e = 1 / $numed {
        qui su fhomeh_M0F`e'__mean if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fhomeh_M0F`e'__mean if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    // fhomehsd_0_F(NTYPE2)
    qui file open momfile using "`fpath'/fhomehsd_0_F.asc", text write replace
    forval e = 1 / $numed {
        qui su fhomeh_M0F`e'__sd if `stat'
        file write momfile "``statout''"
        if "`all'" == "all" {
            qui su fhomeh_M0F`e'__sd if r > 0
            file write momfile ", `r(sd)', `mode'"
            }
        file write momfile _n
        }
    file close momfile

    /* distribution of home production by own/spouse employment status */

    // single men home production (self hours)
    // mhomehMw_m0(0:1, 0:1)
    qui file open momfile using "`fpath'/mhomehMw_m0.asc", text write replace
    forval k = 0 / 1 {
        forval w = 0 / 1 {
            qui su mhomehMw`w'_k`k'_MmF0__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomehMw`w'_k`k'_MmF0__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhomehMwsd_m0(0:1, 0:1)
    qui file open momfile using "`fpath'/mhomehMwsd_m0.asc", text write replace
    forval k = 0 / 1 {
        forval w = 0 / 1 {
            qui su mhomehMw`w'_k`k'_MmF0__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomehMw`w'_k`k'_MmF0__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile



    // single women home production (self hours)
    // fhomehFh_0f(0:2, 0:1)
    qui file open momfile using "`fpath'/fhomehFh_0f.asc", text write replace
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui su fhomehFh`h'_k`k'_M0Ff__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomehFh`h'_k`k'_M0Ff__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhomehFhsd_0f(0:2, 0:1)
    qui file open momfile using "`fpath'/fhomehFhsd_0f.asc", text write replace
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui su fhomehFh`h'_k`k'_M0Ff__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomehFh`h'_k`k'_M0Ff__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile



    // married men home production (self hours)
    // mhomehMw_mf(0:1, 0:1)
    qui file open momfile using "`fpath'/mhomehMw_mf.asc", text write replace
    forval k = 0 / 1 {
        forval w = 0 / 1 {
            qui su mhomehMw`w'_k`k'_MmFf__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomehMw`w'_k`k'_MmFf__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhomehMwsd_mf(0:1, 0:1)
    qui file open momfile using "`fpath'/mhomehMwsd_mf.asc", text write replace
    forval k = 0 / 1 {
        forval w = 0 / 1 {
            qui su mhomehMw`w'_k`k'_MmFf__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomehMw`w'_k`k'_MmFf__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile



    // married men home production (spouse hours)
    // mhomehFh_mf(0:1, 0:1)
    qui file open momfile using "`fpath'/mhomehFw_mf.asc", text write replace
    forval k = 0 / 1 {
        forval w = 0 / 1 {
            qui su mhomehFw`w'_k`k'_MmFf__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomehFw`w'_k`k'_MmFf__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhomehFhsd_mf(0:1, 0:1)
    qui file open momfile using "`fpath'/mhomehFwsd_mf.asc", text write replace
    forval k = 0 / 1 {
        forval w = 0 / 1 {
            qui su mhomehFw`w'_k`k'_MmFf__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su mhomehFw`w'_k`k'_MmFf__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile



    // married women home production (self hours)
    // fhomehFh_mf(0:2, 0:1)
    qui file open momfile using "`fpath'/fhomehFh_mf.asc", text write replace
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui su fhomehFh`h'_k`k'_MmFf__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomehFh`h'_k`k'_MmFf__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhomehFhsd_mf(0:2, 0:1)
    qui file open momfile using "`fpath'/fhomehFhsd_mf.asc", text write replace
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui su fhomehFh`h'_k`k'_MmFf__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomehFh`h'_k`k'_MmFf__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile


    // married women home production (spouse hours)
    // fhomehMw_mf(0:1, 0:1)
    qui file open momfile using "`fpath'/fhomehMw_mf.asc", text write replace
    forval k = 0 / 1 {
        forval w = 0 / 1 {
            qui su fhomehMw`w'_k`k'_MmFf__mean if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomehMw`w'_k`k'_MmFf__mean if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // fhomehMwsd_mf(0:1, 0:1)
    qui file open momfile using "`fpath'/fhomehMwsd_mf.asc", text write replace
    forval k = 0 / 1 {
        forval w = 0 / 1 {
            qui su fhomehMw`w'_k`k'_MmFf__sd if `stat'
            file write momfile "``statout''"
            if "`all'" == "all" {
                qui su fhomehMw`w'_k`k'_MmFf__sd if r > 0
                file write momfile ", `r(sd)', `mode'"
                }
            file write momfile _n
            }
        }
    file close momfile

    // mhomeh_m0
    qui file open momfile using "`fpath'/mhomeh_m0.asc", text write replace
    qui su mhomeh_MmF0__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhomeh_MmF0__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mhomehsd_m0
    qui file open momfile using "`fpath'/mhomehsd_m0.asc", text write replace
    qui su mhomeh_MmF0__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhomeh_MmF0__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mhomeh_mf
    qui file open momfile using "`fpath'/mhomeh_mf.asc", text write replace
    qui su mhomeh_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhomeh_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // mhomehsd_mf
    qui file open momfile using "`fpath'/mhomehsd_mf.asc", text write replace
    qui su mhomeh_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su mhomeh_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhomeh_0f
    qui file open momfile using "`fpath'/fhomeh_0f.asc", text write replace
    qui su fhomeh_M0Ff__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhomeh_M0Ff__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhomehsd_0f
    qui file open momfile using "`fpath'/fhomehsd_0f.asc", text write replace
    qui su fhomeh_M0Ff__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhomeh_M0Ff__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhomeh_mf
    qui file open momfile using "`fpath'/fhomeh_mf.asc", text write replace
    qui su fhomeh_MmFf__mean if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhomeh_MmFf__mean if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

    // fhomehsd_mf
    qui file open momfile using "`fpath'/fhomehsd_mf.asc", text write replace
    qui su fhomeh_MmFf__sd if `stat'
    file write momfile "``statout''"
    if "`all'" == "all" {
        qui su fhomeh_MmFf__sd if r > 0
        file write momfile ", `r(sd)', `mode'"
        }
    file write momfile _n
    file close momfile

end

createMomentFiles
createMomentFiles, data
createMomentFiles, sd
createMomentFiles, all

forval r = 1 / $bsrep {
    createMomentFiles, bs(`r')
    }
