#define pure
module taxes

    use globaldef

    interface evalTax
        module procedure evalTaxSinScalar
        module procedure evalTaxSinVector
        module procedure evalTaxCouScalar
        module procedure evalTaxCouVector12
        module procedure evalTaxCouVector1
        module procedure evalTaxCouVector2
    end interface ! evalTax

contains

    pure function evalWelfareFoodStampsCouScalar(tax, kid, earn) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: kid
        real(dp), intent(in) :: earn
        real(dp) :: wlf, fsp, y
        logical, parameter :: test = .false.
        if (kid > 0) then
            wlf = evalCouKidsWelfare(tax, test, earn)
        else
            wlf = 0.0_dp
        end if
        fsp = evalCouFsp(tax, kid, test, earn, wlf)
        y = wlf + fsp
    end function evalWelfareFoodStampsCouScalar

    pure function evalWelfareFoodStampsSinScalar(tax, kid, earn) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: kid
        real(dp), intent(in) :: earn
        real(dp) :: wlf, fsp, y
        logical, parameter :: test = .false.
        if (kid > 0) then
            wlf = evalSinKidsWelfare(tax, test, earn)
        else
            wlf = 0.0_dp
        end if
        fsp = evalSinFsp(tax, kid, test, earn, wlf)
        y = wlf + fsp
    end function evalWelfareFoodStampsSinScalar

    function evalSinKidsWelfare(tax, test, x) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        logical, intent(in) :: test
        real(dp), intent(in) :: x

        real(dp) :: y
        logical :: eligible
        real(dp) :: netx

        eligible = .true.

        ! apply gross and net test
        if ( test ) then
            if ( tax%sinKidsGrossTest ) then
                if ( x > tax%sinKidsGrossNeedScale * tax%sinKidsNeedStd ) then
                    eligible = .false.
                end if
            end if
            if ( tax%sinKidsNetTest .and. eligible ) then
                netx = (x - tax%sinKidsEligDisAmt) * (1.0_dp - tax%sinKidsEligDisPct)
                if ( netx > tax%sinKidsNetNeedScale * tax%sinKidsNeedStd ) then
                    eligible = .false.
                end if
            end if
        end if

        if ( eligible ) then
            netx = max(x - tax%sinKidsBenDisAmt, 0.0_dp) * (1.0_dp - tax%sinKidsBenDisPct)
            y = max( tax%sinKidsBenRatio * ( tax%sinKidsPaymentStd - netx ), 0.0_dp )
            if ( tax%sinKidsMaxBen ) then
                y = min( y, tax%sinKidsMaxBenVal )
            end if
        else
            y = 0.0_dp
        end if

    end function evalSinKidsWelfare

    pure function evalCouKidsWelfare(tax, test, x) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        logical, intent(in) :: test
        real(dp), intent(in) :: x

        real(dp) :: y
        logical :: eligible
        real(dp) :: netx

        eligible = .true.

        ! apply gross and net test
        if ( test ) then
            if ( tax%couKidsGrossTest ) then
                if ( x > tax%couKidsGrossNeedScale * tax%couKidsNeedStd ) then
                    eligible = .false.
                end if
            end if
            if ( tax%couKidsNetTest .and. eligible ) then
                netx = (x - tax%couKidsEligDisAmt) * (1.0_dp - tax%couKidsEligDisPct)
                if ( netx > tax%couKidsNetNeedScale * tax%couKidsNeedStd ) then
                    eligible = .false.
                end if
            end if
        end if

        if ( eligible ) then
            netx = max(x - tax%couKidsBenDisAmt, 0.0_dp) * (1.0_dp - tax%couKidsBenDisPct)
            y = max( tax%couKidsBenRatio * ( tax%couKidsPaymentStd - netx ), 0.0_dp )
            if ( tax%couKidsMaxBen ) then
                y = min( y, tax%couKidsMaxBenVal )
            end if
        else
            y = 0.0_dp
        end if

    end function evalCouKidsWelfare

    pure function evalSinFsp(tax, kid, test, x, wlf) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: kid
        logical, intent(in) :: test
        real(dp), intent(in) :: x
        real(dp), intent(in) :: wlf
        real(dp) :: netx, y
        real(dp) :: grossElig, netElig, maxFsp, stdDed

        if (kid > 0) then
            grossElig = tax%sinKidsGrossElig
            netElig = tax%sinKidsNetElig
            maxFsp = tax%sinKidsMaxFsp
            stdDed = tax%sinKidsStdDed
        else
            grossElig = tax%sinGrossElig
            netElig = tax%sinNetElig
            maxFsp = tax%sinMaxFsp
            stdDed = tax%sinStdDed
        end if

        ! apply the initial gross and net tests
        if ( test ) then
            ! gross test
            if ( x < grossElig ) then
                ! net test
                netx = max( 0.8_dp * x + wlf - stdDed, 0.0_dp )
                if ( netx < netElig ) then
                    y = max( maxFsp - 0.3_dp * netx, 0.0_dp )
                else
                    y = 0.0_dp
                end if
            else
                y = 0.0_dp
            end if
        else
            netx = max( 0.8_dp * x + wlf - stdDed, 0.0_dp )
            y = max( maxFsp - 0.3_dp * netx, 0.0_dp )
        end if

    end function evalSinFsp

    pure function evalCouFsp(tax, kid, test, x, wlf) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: kid
        logical, intent(in) :: test
        real(dp), intent(in) :: x
        real(dp), intent(in) :: wlf
        real(dp) :: netx, y
        real(dp) :: grossElig, netElig, maxFsp, stdDed

        if (kid > 0) then
            grossElig = tax%couKidsGrossElig
            netElig = tax%couKidsNetElig
            maxFsp = tax%couKidsMaxFsp
            stdDed = tax%couKidsStdDed
        else
            grossElig = tax%couGrossElig
            netElig = tax%couNetElig
            maxFsp = tax%couMaxFsp
            stdDed = tax%couStdDed
        end if

        ! apply the initial gross and net tests
        if ( test ) then
            ! gross test
            if ( x < grossElig ) then
                ! net test
                netx = max( 0.8_dp * x + wlf - stdDed, 0.0_dp )
                if ( netx < netElig ) then
                    y = max( maxFsp - 0.3_dp * netx, 0.0_dp )
                else
                    y = 0.0_dp
                end if
            else
                y = 0.0_dp
            end if
        else
            netx = max( 0.8_dp * x + wlf - stdDed, 0.0_dp )
            y = max( maxFsp - 0.3_dp * netx, 0.0_dp )
        end if

    end function evalCouFsp

    subroutine checkTaxSupport(pF, pM, maxtime, verbose)

        implicit none

        type(p_t), intent(in) :: pM(NTYPE1), pF(NTYPE2)
        real(dp), intent(in) :: maxtime
        logical, intent(in) :: verbose
        real(dp) :: maxW1, maxW2
        integer :: pX1, pX2
        integer :: ierr

        maxW1 = 0.0_dp
        maxW2 = 0.0_dp
        ierr = 0

        do pX1 = 1, NTYPE1
            maxW1 = max(maxW1, pM(pX1)%wageNode(NODEW) )
        end do

        do pX2 = 1, NTYPE2
            maxW2 = max(maxW2, pF(pX2)%wageNode(NODEW) )
        end do

        if ( maxtime * maxW1 > TAXBR(NTAXBR) ) then
            ierr = ierr + 1
            print *, 'maximum male earnings exceeds TAXBR(NTAXBR)'
            print *, maxtime * maxW1, TAXBR(NTAXBR)
        end if

        if ( maxtime * maxW2 > TAXBR(NTAXBR) ) then
            ierr = ierr + 1
            print *, 'maximum female earnings exceeds TAXBR(NTAXBR)'
            print *, maxtime * maxW2, TAXBR(NTAXBR)
        end if

        if (ierr>0) stop

        if ( verbose ) then
            print *, 'male:   ', maxtime * maxW1, TAXBR(NTAXBR)
            print *, 'female: ', maxtime * maxW2, TAXBR(NTAXBR)
        end if

    end subroutine checkTaxSupport

    pure subroutine calcTaxCoef(tax, taxins)
        implicit none
        type(tax_t), intent(inout) :: tax
        integer, intent(in) :: taxins
        integer :: iX1, iX2
        integer :: ixGender
        real(dp) :: P(3), Q(3), R(3)
        real(dp) :: u(3)
        real(dp) :: v(3)
        real(dp) :: c(3)
        real(dp) :: sc

        if ( taxins == TAX_JOINT .or. taxins == TAX_JOINTGBT .or. taxins == TAX_JOINTGBTALL ) then

            ! couples, 3d surface
            do iX2 = 1, NTAXBR-1
                do iX1 = 1, NTAXBR-1

                    ! lower triangle
                    P(1) = TAXBR(iX1)
                    P(2) = TAXBR(iX2)
                    P(3) = tax%tCou(iX1, iX2)
                    Q(1) = TAXBR(iX1)
                    Q(2) = TAXBR(iX2 + 1)
                    Q(3) = tax%tCou(iX1, iX2 + 1)
                    R(1) = TAXBR(iX1 + 1)
                    R(2) = TAXBR(iX2)
                    R(3) = tax%tCou(iX1 + 1, iX2)

                    u = Q - P
                    v = R - P

                    c(1) = u(2) * v(3) - u(3) * v(2)
                    c(2) = u(3) * v(1) - u(1) * v(3)
                    c(3) = u(1) * v(2) - u(2) * v(1)

                    tax%tCouL(1, iX1, iX2) = P(3) + (c(1) * P(1) + c(2) * P(2)) / c(3)
                    tax%tCouL(2, iX1, iX2) = -c(1) / c(3)
                    tax%tCouL(3, iX1, iX2) = -c(2) / c(3)

                    ! upper triangle
                    P(1) = TAXBR(iX1 + 1)
                    P(2) = TAXBR(iX2)
                    P(3) = tax%tCou(iX1 + 1, iX2)
                    Q(1) = TAXBR(iX1 + 1)
                    Q(2) = TAXBR(iX2 + 1)
                    Q(3) = tax%tCou(iX1 + 1, iX2 + 1)
                    R(1) = TAXBR(iX1)
                    R(2) = TAXBR(iX2 + 1)
                    R(3) = tax%tCou(iX1, iX2 + 1)

                    u = Q - P
                    v = R - P

                    c(1) = u(2) * v(3) - u(3) * v(2)
                    c(2) = u(3) * v(1) - u(1) * v(3)
                    c(3) = u(1) * v(2) - u(2) * v(1)

                    tax%tCouU(1, iX1, iX2) = P(3) + (c(1) * P(1) + c(2) * P(2)) / c(3)
                    tax%tCouU(2, iX1, iX2) = -c(1) / c(3)
                    tax%tCouU(3, iX1, iX2) = -c(2) / c(3)

                end do
            end do

        else

            do iX1 = 1, NTAXBR - 1
                tax%tCouL(1, iX1, 1) = tax%tCou(iX1, 1) - TAXBR(iX1) * ( tax%tCou(iX1 + 1, 1) - tax%tCou(iX1, 1) ) / TAXDBR(iX1)
                tax%tCouL(2, iX1, 1) = ( tax%tCou(iX1 + 1, 1) - tax%tCou(iX1, 1) ) / TAXDBR(iX1)
            end do

            if (taxins == TAX_INDEPENDENT3) then
                sc = 0.5_dp * tax%tCouL(1, 1, 1)
                do iX1 = 1, NTAXBR - 1
                    tax%tCouL(1, iX1, 1) = tax%tCouL(1, iX1, 1) - sc
                end do
            end if

        end if ! taxins == JOINT

        ! singles
        do ixGender = 1, 2
            do iX1 = 1, NTAXBR - 1
                tax%tSinL(1, iX1, ixGender) = tax%tSin(iX1, ixGender) - TAXBR(iX1) &
                    * ( tax%tSin(iX1 + 1, ixGender) - tax%tSin(iX1, ixGender) ) / TAXDBR(iX1)
                tax%tSinL(2, iX1, ixGender) = ( tax%tSin(iX1 + 1, ixGender) - tax%tSin(iX1, ixGender) ) / TAXDBR(iX1)
            end do
        end do

        tax%taxins = taxins

    end subroutine calcTaxCoef

    function evalTaxElasCouScalar(tax, taxmode, ad, kid, earn1, earn2, factor) result(y)
        use mathfn
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: taxmode
        integer, intent(in) :: ad
        integer, intent(in) :: kid
        real(dp), intent(in) :: earn1, earn2
        real(dp) :: y, tau
        integer :: iX, tX
        real(dp) :: earnFirst, earnSecond, earnTotal
        real(dp) :: factor

        ! ad refers to the adult whose net wage we are changing
        !
        ! earnFirst: earnings of the adult who is taxed first, and subject to the original tax schedule
        ! earnSecond: earnings of the adult who is taxed second, and whose net wage we increase
        ! earnTotal: sum of earnFirst and earnSecond

        if ( ad == 1 ) then
            ! net elasticity for men, so tax women first
            earnFirst = earn2
            earnSecond = earn1
        else
            ! net elasticity for women, so tax men first
            earnFirst = earn1
            earnSecond = earn2
        end if

        earnTotal = earnFirst + earnSecond

        ! original schedule applied to first earner
        if (kid == 0) then
            if (earnFirst <= tax%couBreaks(1)) then
                y = tax%couNet(1)
                tX = 1
            elseif ( earnFirst >= tax%couBreaks(tax%couNumKinks) ) then
                y = tax%couVirtual(tax%couNumKinks) + tax%couMtr(tax%couNumKinks) * earnFirst
                tX = tax%couNumKinks
            else
                do iX = 1, tax%couNumKinks - 1
                    if (earnFirst >= tax%couBreaks(iX) .and. earnFirst < tax%couBreaks(iX + 1)) then
                        y = tax%couVirtual(iX) + tax%couMtr(iX) * earnFirst
                        tX = iX
                        exit
                    end if
                end do
            end if

            ! childless couple medicaid (will evaluate to zero)
            if (tax%medicaid) then

                if (earn1 > 0.0_dp ) then ! male working
                    if (earnTotal < tax%coupleMedicaidWork) then
                        y = y - tax%adultMedicaidValue(SEX_MALE)
                    end if
                else ! not working
                    if (earnTotal < tax%coupleMedicaidNotWork) then
                        y = y - tax%adultMedicaidValue(SEX_MALE)
                    end if
                end if

                if (earn2 > 0.0_dp ) then ! female working
                    if (earnTotal < tax%coupleMedicaidWork) then
                        y = y - tax%adultMedicaidValue(SEX_FEMALE)
                    end if
                else ! not working
                    if (earnTotal < tax%coupleMedicaidNotWork) then
                        y = y - tax%adultMedicaidValue(SEX_FEMALE)
                    end if
                end if

            end if

        else
            if (earnFirst <= tax%couKidsBreaks(1)) then
                y = tax%couKidsNet(1)
                tX = 1
            elseif ( earnFirst >= tax%couKidsBreaks(tax%couKidsNumKinks) ) then
                y = tax%couKidsVirtual(tax%couKidsNumKinks) + tax%couKidsMtr(tax%couKidsNumKinks) * earnFirst
                tX = tax%couKidsNumKinks
            else
                do iX = 1, tax%couKidsNumKinks - 1
                    if (earnFirst >= tax%couKidsBreaks(iX) .and. earnFirst < tax%couKidsBreaks(iX + 1)) then
                        y = tax%couKidsVirtual(iX) + tax%couKidsMtr(iX) * earnFirst
                        tX = iX
                        exit
                    end if
                end do
            end if

            ! parent and child medicaid (we make medicaid independent of factor)
            if (tax%medicaid) then

                if (earn1 > 0.0_dp ) then ! male parent working
                    if (earnTotal < tax%coupleKidsMedicaidParentWork) then
                        y = y - tax%adultMedicaidValue(SEX_MALE)
                    end if
                else ! not working
                    if (earnTotal < tax%coupleKidsMedicaidParentNotWork) then
                        y = y - tax%adultMedicaidValue(SEX_MALE)
                    end if
                end if

                if (earn2 > 0.0_dp ) then ! female parent working
                    if (earnTotal < tax%coupleKidsMedicaidParentWork) then
                        y = y - tax%adultMedicaidValue(SEX_FEMALE)
                    end if
                else ! not working
                    if (earnTotal < tax%coupleKidsMedicaidParentNotWork) then
                        y = y - tax%adultMedicaidValue(SEX_FEMALE)
                    end if
                end if

                ! child medicaid
                if (earnTotal < tax%coupleKidsMedicaidChild) then
                    y = y - tax%kidsMedicaidValue
                end if

            end if
        end if

        ! residual (male earnings)
        if (kid == 0) then

            if (earnSecond <= 0.0_dp) then
                return
            else
                tau = tax%couMtr(tX) * factor + 1.0_dp - factor
                if (tX == tax%couNumKinks) then
                    y = y + tau * ( earnTotal - earnFirst )
                else
                    y = y + tau * ( min(earnTotal, tax%couBreaks(tX + 1)) - earnFirst )
                    do iX = tX + 1, tax%couNumKinks - 1
                        if (earnTotal >= tax%couBreaks(iX)) then
                            tau = tax%couMtr(iX) * factor + 1.0_dp - factor
                            y = y + tau * ( min(earnTotal, tax%couBreaks(iX + 1)) - tax%couBreaks(iX) )
                        else
                            exit
                        end if
                    end do
                    if (earnTotal >= tax%couBreaks(tax%couNumKinks)) then
                        tau = tax%couMtr(tax%couNumKinks) * factor + 1.0_dp - factor
                        y = y + tau * ( earnTotal - tax%couBreaks(tax%couNumKinks) )
                    end if
                end if
            end if

        else

            if (earnSecond <= 0.0_dp) then
                return
            else
                tau = tax%couKidsMtr(tX) * factor + 1.0_dp - factor
                if (tX == tax%couKidsNumKinks) then
                    y = y + tau * ( earnTotal - earnFirst )
                else
                    y = y + tau * ( min(earnTotal, tax%couKidsBreaks(tX + 1)) - earnFirst )
                    do iX = tX + 1, tax%couKidsNumKinks - 1
                        if (earnTotal >= tax%couKidsBreaks(iX)) then
                            tau = tax%couKidsMtr(iX) * factor + 1.0_dp - factor
                            y = y + tau * ( min(earnTotal, tax%couKidsBreaks(iX + 1)) - tax%couKidsBreaks(iX) )
                        else
                            exit
                        end if
                    end do
                    if (earnTotal >= tax%couKidsBreaks(tax%couKidsNumKinks)) then
                        tau = tax%couKidsMtr(tax%couKidsNumKinks) * factor + 1.0_dp - factor
                        y = y + tau * ( earnTotal - tax%couKidsBreaks(tax%couKidsNumKinks) )
                    end if
                end if
            end if

        end if

    end function evalTaxElasCouScalar

    pure subroutine getLocalTaxCouScalar(tax, kid, earn1, earn2, tv, t1, t2)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: kid
        real(dp), intent(in) :: earn1, earn2
        real(dp), intent(out) :: tv, t1, t2
        integer :: iX1, iX2

        if (earn1 <= TAXBR(1) .and. earn2 <= TAXBR(1)) then
            tv = tax%tCouL(1, 1, 1)
        else if (earn1 >= TAXBR(NTAXBR) .and. earn2 >= TAXBR(NTAXBR)) then
            ! y = tax%tCouU(1, NTAXBR - 1, NTAXBR - 1) + tax%tCouU(2, NTAXBR - 1, NTAXBR - 1) * earn1 &
            !   + tax%tCouU(3, NTAXBR - 1, NTAXBR - 1) * earn2
            tv = tax%tCouU(1, NTAXBR - 1, NTAXBR - 1)
            t1 = tax%tCouU(2, NTAXBR - 1, NTAXBR - 1)
            t2 = tax%tCouU(3, NTAXBR - 1, NTAXBR - 1)

        else
            do iX2 = 1, NTAXBR - 1
                if ( earn2 >= TAXBR(iX2) .and. ( earn2 < TAXBR(iX2 + 1) &
                        .or. (iX2 == NTAXBR - 1 .and. earn2 >= TAXBR(NTAXBR)) ) ) then
                    do iX1 = 1, NTAXBR - 1
                        if ( earn1 >= TAXBR(iX1) .and. ( earn1 < TAXBR(iX1 + 1) &
                                .or. (iX1 == NTAXBR - 1 .and. earn1 >= TAXBR(NTAXBR) ) ) ) then
                            if ( earn1 - TAXBR(iX1) + (earn2 - TAXBR(iX2)) &
                                * TAXDBR(iX1) / TAXDBR(iX2) >= (TAXDBR(iX1)) ) then
                                ! y = tax%tCouU(1, iX1, iX2) + tax%tCouU(2, iX1, iX2) * earn1 + tax%tCouU(3, iX1, iX2) * earn2
                                tv = tax%tCouU(1, iX1, iX2)
                                t1 = tax%tCouU(2, iX1, iX2)
                                t2 = tax%tCouU(3, iX1, iX2)
                            else
                                ! y = tax%tCouL(1, iX1, iX2) + tax%tCouL(2, iX1, iX2) * earn1 + tax%tCouL(3, iX1, iX2) * earn2
                                tv = tax%tCouL(1, iX1, iX2)
                                t1 = tax%tCouL(2, iX1, iX2)
                                t2 = tax%tCouL(3, iX1, iX2)
                            end if
                            exit
                        end if
                    end do
                    exit
                end if
            end do
        end if
    end subroutine getLocalTaxCouScalar


    pure function evalTaxCouScalar(tax, taxmode, kid, earn1, earn2) result(y)
        use mathfn
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: taxmode
        integer, intent(in) :: kid
        real(dp), intent(in) :: earn1, earn2
        real(dp) :: earn, w, y, y1, y2
        integer :: iX, iX1, iX2

!         y = -1
        if (taxmode == 0) then

            earn = earn1 + earn2

            ! calculate welfare and food stamps
            ! now using integrated schedule
            ! w = evalWelfareFoodStampsCouScalar(tax, kid, earn)
            w = 0.0_dp

            if (kid == 0) then

                if (earn <= tax%couBreaks(1)) then
                    y = tax%couNet(1)
                elseif ( earn >= tax%couBreaks(tax%couNumKinks) ) then
                    y = tax%couVirtual(tax%couNumKinks) + tax%couMtr(tax%couNumKinks) * earn
                else
                    do iX = 1, tax%couNumKinks - 1
                        if (earn >= tax%couBreaks(iX) .and. earn < tax%couBreaks(iX + 1)) then
                            y = tax%couVirtual(iX) + tax%couMtr(iX) * earn
                        end if
                    end do
                end if

                ! childless couple medicaid (will evaluate to zero)
                if (tax%medicaid) then

                    if (earn1 > 0.0_dp ) then ! male working
                        if (earn < tax%coupleMedicaidWork) then
                            y = y - tax%adultMedicaidValue(SEX_MALE)
                        end if
                    else ! not working
                        if (earn < tax%coupleMedicaidNotWork) then
                            y = y - tax%adultMedicaidValue(SEX_MALE)
                        end if
                    end if

                    if (earn2 > 0.0_dp ) then ! female working
                        if (earn < tax%coupleMedicaidWork) then
                            y = y - tax%adultMedicaidValue(SEX_FEMALE)
                        end if
                    else ! not working
                        if (earn < tax%coupleMedicaidNotWork) then
                            y = y - tax%adultMedicaidValue(SEX_FEMALE)
                        end if
                    end if

                end if

            else

                if (earn <= tax%couKidsBreaks(1)) then
                    y = tax%couKidsNet(1)
                elseif ( earn >= tax%couKidsBreaks(tax%couKidsNumKinks) ) then
                    y = tax%couKidsVirtual(tax%couKidsNumKinks) + tax%couKidsMtr(tax%couKidsNumKinks) * earn
                else
                    do iX = 1, tax%couKidsNumKinks - 1
                        if (earn >= tax%couKidsBreaks(iX) .and. earn < tax%couKidsBreaks(iX + 1)) then
                            y = tax%couKidsVirtual(iX) + tax%couKidsMtr(iX) * earn
                        end if
                    end do
                end if

                ! parent and child medicaid
                if (tax%medicaid) then

                    if (earn1 > 0.0_dp ) then ! male parent working
                        if (earn < tax%coupleKidsMedicaidParentWork) then
                            y = y - tax%adultMedicaidValue(SEX_MALE)
                        end if
                    else ! not working
                        if (earn < tax%coupleKidsMedicaidParentNotWork) then
                            y = y - tax%adultMedicaidValue(SEX_MALE)
                        end if
                    end if

                    if (earn2 > 0.0_dp ) then ! female parent working
                        if (earn < tax%coupleKidsMedicaidParentWork) then
                            y = y - tax%adultMedicaidValue(SEX_FEMALE)
                        end if
                    else ! not working
                        if (earn < tax%coupleKidsMedicaidParentNotWork) then
                            y = y - tax%adultMedicaidValue(SEX_FEMALE)
                        end if
                    end if

                    ! child medicaid
                    if (earn < tax%coupleKidsMedicaidChild) then
                        y = y - tax%kidsMedicaidValue
                    end if

                end if

            end if

            y = y - w

        else

            select case(tax%taxins)
            case(TAX_JOINT, TAX_JOINTGBT, TAX_JOINTGBTALL)
                if (earn1 <= TAXBR(1) .and. earn2 <= TAXBR(1)) then
                    y = tax%tCouL(1, 1, 1)
                else if (earn1 >= TAXBR(NTAXBR) .and. earn2 >= TAXBR(NTAXBR)) then
                    y = tax%tCouU(1, NTAXBR - 1, NTAXBR - 1) + tax%tCouU(2, NTAXBR - 1, NTAXBR - 1) * earn1 &
                      + tax%tCouU(3, NTAXBR - 1, NTAXBR - 1) * earn2
                else
                    do iX2 = 1, NTAXBR - 1
                        if ( earn2 >= TAXBR(iX2) .and. ( earn2 < TAXBR(iX2 + 1) &
                                .or. (iX2 == NTAXBR - 1 .and. earn2 >= TAXBR(NTAXBR)) ) ) then
                            do iX1 = 1, NTAXBR - 1
                                if ( earn1 >= TAXBR(iX1) .and. ( earn1 < TAXBR(iX1 + 1) &
                                        .or. (iX1 == NTAXBR - 1 .and. earn1 >= TAXBR(NTAXBR) ) ) ) then
                                    if ( earn1 - TAXBR(iX1) + (earn2 - TAXBR(iX2)) &
                                        * TAXDBR(iX1) / TAXDBR(iX2) >= (TAXDBR(iX1)) ) then
                                        y = tax%tCouU(1, iX1, iX2) + tax%tCouU(2, iX1, iX2) * earn1 + tax%tCouU(3, iX1, iX2) * earn2
                                    else
                                        y = tax%tCouL(1, iX1, iX2) + tax%tCouL(2, iX1, iX2) * earn1 + tax%tCouL(3, iX1, iX2) * earn2
                                    end if
                                    exit
                                end if
                            end do
                            exit
                        end if
                    end do
                end if
            case(TAX_INDEPENDENT)

                if (earn1 <= TAXBR(1)) then
                    y1 = tax%tCouL(1, 1, 1)
                elseif (earn1 >= TAXBR(NTAXBR)) then
                    y1 = tax%tCouL(1, NTAXBR - 1, 1) + tax%tCouL(2, NTAXBR - 1, 1) * earn1
                else
                    do iX = 1, NTAXBR - 1
                        if (earn1 >= TAXBR(iX) .and. earn1 < TAXBR(iX + 1)) then
                            y1 = tax%tCouL(1, iX, 1) + tax%tCouL(2, iX, 1) * earn1
                        end if
                    end do
                end if

                if (earn2 <= TAXBR(1)) then
                    y2 = tax%tCouL(1, 1, 1)
                elseif (earn2 >= TAXBR(NTAXBR)) then
                    y2 = tax%tCouL(1, NTAXBR - 1, 1) + tax%tCouL(2, NTAXBR - 1, 1) * earn2
                else
                    do iX = 1, NTAXBR - 1
                        if (earn2 >= TAXBR(iX) .and. earn2 < TAXBR(iX + 1)) then
                            y2 = tax%tCouL(1, iX, 1) + tax%tCouL(2, iX, 1) * earn2
                        end if
                    end do
                end if

                y = y1 + y2

            case(TAX_INDEPENDENT3)

                earn = 0.5_dp * (earn1 + earn2)

                if (earn <= TAXBR(1)) then
                    y = tax%tCouL(1, 1, 1)
                elseif (earn >= TAXBR(NTAXBR)) then
                    y = tax%tCouL(1, NTAXBR - 1, 1) + tax%tCouL(2, NTAXBR - 1, 1) * earn
                else
                    do iX = 1, NTAXBR - 1
                        if (earn >= TAXBR(iX) .and. earn < TAXBR(iX + 1)) then
                            y = tax%tCouL(1, iX, 1) + tax%tCouL(2, iX, 1) * earn
                        end if
                    end do
                end if

                y = y * 2.0_dp !- tax%tCouL(1,1,1)

            case(TAX_FULLJOINT2)

                ! for FULLJOINT the tax brackets are doubled
                earn = earn1 + earn2

                if (earn <= 1.0_dp * TAXBR(1)) then
                    y = tax%tCouL(1, 1, 1)
                elseif (earn >= 1.0_dp * TAXBR(NTAXBR)) then
                    y = tax%tCouL(1, NTAXBR - 1, 1) + tax%tCouL(2, NTAXBR - 1, 1) * earn
                else
                    do iX = 1, NTAXBR - 1
                        if (earn >= 1.0_dp * TAXBR(iX) .and. earn < 1.0_dp * TAXBR(iX + 1)) then
                            y = tax%tCouL(1, iX, 1) + tax%tCouL(2, iX, 1) * earn
                        end if
                    end do
                end if

            case default

            end select


        end if

    end function evalTaxCouScalar


    pure function evalTaxSinScalar(tax, taxmode, kid, sex, earn) result(y)
        use mathfn
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: taxmode
        integer, intent(in) :: kid
        integer, intent(in) :: sex
        real(dp), intent(in) :: earn
        real(dp) :: w
        real(dp) :: y
        integer :: iX

        if (taxmode == 0) then

            ! calculate welfare and foodstamps
            ! now using integrated schedule
            ! w = evalWelfareFoodStampsSinScalar(tax, kid, earn)
            w = 0.0_dp

            if (kid == 0) then

                if (earn <= tax%sinBreaks(1)) then
                    y = tax%sinNet(1)
                elseif ( earn >= tax%sinBreaks(tax%sinNumKinks) ) then
                    y = tax%sinVirtual(tax%sinNumKinks) + tax%sinMtr(tax%sinNumKinks) * earn
                else
                    do iX = 1, tax%sinNumKinks - 1
                        if (earn >= tax%sinBreaks(iX) .and. earn < tax%sinBreaks(iX + 1)) then
                            y = tax%sinVirtual(iX) + tax%sinMtr(iX) * earn
                        end if
                    end do
                end if

                ! childless medicaid (will be zero here)
                if (tax%medicaid) then
                    if (earn > 0.0_dp ) then ! working
                        if (earn < tax%singleMedicaidWork) then
                            y = y - tax%adultMedicaidValue(sex)
                        end if
                    else ! not working
                        if (earn < tax%singleMedicaidNotWork) then
                            y = y - tax%adultMedicaidValue(sex)
                        end if
                    end if
                end if

            else

                if (earn <= tax%sinKidsBreaks(1)) then
                    y = tax%sinKidsNet(1)
                elseif ( earn >= tax%sinKidsBreaks(tax%sinKidsNumKinks) ) then
                    y = tax%sinKidsVirtual(tax%sinKidsNumKinks) + tax%sinKidsMtr(tax%sinKidsNumKinks) * earn
                else
                    do iX = 1, tax%sinKidsNumKinks - 1
                        if (earn>=tax%sinKidsBreaks(iX) .and. earn<tax%sinKidsBreaks(iX + 1)) then
                            y = tax%sinKidsVirtual(iX) + tax%sinKidsMtr(iX) * earn
                        end if
                    end do
                end if

                ! parent and child medicaid
                if (tax%medicaid) then

                    if (earn > 0.0_dp ) then ! working
                        if (earn < tax%singleKidsMedicaidParentWork) then
                            y = y - tax%adultMedicaidValue(sex)
                        end if
                    else ! not working
                        if (earn < tax%singleKidsMedicaidParentNotWork) then
                            y = y - tax%adultMedicaidValue(sex)
                        end if
                    end if

                    ! child medicaid
                    if (earn < tax%singleKidsMedicaidChild) then
                        y = y - tax%kidsMedicaidValue
                    end if

                end if

            end if

            ! include welfare in tax measure
            y = y - w

        else

            if (earn <= TAXBR(1)) then
                y = tax%tSinL(1, 1, sex)
            elseif (earn >= TAXBR(NTAXBR)) then
                y = tax%tSinL(1, NTAXBR - 1, sex) + tax%tSinL(2, NTAXBR - 1, sex) * earn
            else
                do iX = 1, NTAXBR - 1
                    if (earn >= TAXBR(iX) .and. earn < TAXBR(iX + 1)) then
                        y = tax%tSinL(1, iX, sex) + tax%tSinL(2, iX, sex) * earn
                    end if
                end do
            end if

        end if

    end function evalTaxSinScalar

    pure function evalTaxElasSinScalar(tax, taxmode, kid, sex, earn, factor) result(y)
        use mathfn
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: taxmode
        integer, intent(in) :: kid
        integer, intent(in) :: sex ! does not do anything -> empirical tax schedule
        real(dp), intent(in) :: earn
        real(dp), intent(in) :: factor
        real(dp) :: y
        integer :: tX
        real(dp) :: tau

        if (kid == 0) then
            if ( earn <= tax%sinBreaks(1) ) then
                y = tax%sinNet(1)
            else
                y = tax%sinNet(1)
                do tX = 1, tax%sinNumKinks - 1
                    tau = tax%sinMtr(tX) * factor + 1.0_dp - factor
                    if ( earn >= tax%sinBreaks(tX) ) then
                        y = y + tau*( min(earn, tax%sinBreaks(tX + 1)) - tax%sinBreaks(tX) )
                    else
                        exit
                    end if
                end do
            end if

            ! childless medicaid (will be zero here)
            if (tax%medicaid) then
                if (earn > 0.0_dp ) then ! working
                    if (earn < tax%singleMedicaidWork) then
                        y = y - tax%adultMedicaidValue(sex)
                    end if
                else ! not working
                    if (earn < tax%singleMedicaidNotWork) then
                        y = y - tax%adultMedicaidValue(sex)
                    end if
                end if
            end if

        else

            if ( earn <= tax%sinKidsBreaks(1) ) then
                y = tax%sinKidsNet(1)
            else
                y = tax%sinKidsNet(1)
                do tX = 1, tax%sinKidsNumKinks - 1
                    tau = tax%sinKidsMtr(tX) * factor + 1.0_dp - factor
                    if ( earn >= tax%sinKidsBreaks(tX) ) then
                        y = y + tau*( min(earn, tax%sinKidsBreaks(tX + 1)) - tax%sinKidsBreaks(tX) )
                    else
                        exit
                    end if
                end do
            end if

            ! parent and child medicaid (we make medicaid independent of factor)
            if (tax%medicaid) then

                if (earn > 0.0_dp ) then ! working
                    if (earn < tax%singleKidsMedicaidParentWork) then
                        y = y - tax%adultMedicaidValue(sex)
                    end if
                else ! not working
                    if (earn < tax%singleKidsMedicaidParentNotWork) then
                        y = y - tax%adultMedicaidValue(sex)
                    end if
                end if

                ! child medicaid
                if (earn < tax%singleKidsMedicaidChild) then
                    y = y - tax%kidsMedicaidValue
                end if

            end if

        end if

    end function evalTaxElasSinScalar

    pure function evalTaxSinVector(tax, taxmode, kid, sex, earn) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: taxmode
        integer, intent(in) :: kid
        integer, intent(in) :: sex
        real(dp), intent(in) :: earn(:)
        real(dp) :: y(size(earn))
        integer :: hX
        do hX = 1, size(earn)
            y(hX) = evalTaxSinScalar( tax, taxmode, kid, sex, earn(hX) )
        end do
    end function evalTaxSinVector

    pure function evalTaxCouVector12(tax, taxmode, kid, earn1, earn2) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: taxmode
        integer, intent(in) :: kid
        real(dp), intent(in) :: earn1(:)
        real(dp), intent(in) :: earn2(size(earn1))
        real(dp) :: y(size(earn1))
        integer :: hX
        do hX = 1, size(earn1)
            y(hX) = evalTaxCouScalar( tax, taxmode, kid, earn1(hX), earn2(hX) )
        end do
    end function evalTaxCouVector12

    pure function evalTaxCouVector1(tax, taxmode, kid, earn1, earn2) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: taxmode
        integer, intent(in) :: kid
        real(dp), intent(in) :: earn1(:)
        real(dp), intent(in) :: earn2
        real(dp) :: y(size(earn1))
        integer :: hX
        do hX = 1, size(earn1)
            y(hX) = evalTaxCouScalar( tax, taxmode, kid, earn1(hX), earn2 )
        end do
    end function evalTaxCouVector1

    pure function evalTaxCouVector2(tax, taxmode, kid, earn1, earn2) result(y)
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: taxmode
        integer, intent(in) :: kid
        real(dp), intent(in) :: earn1
        real(dp), intent(in) :: earn2(:)
        real(dp) :: y(size(earn2))
        integer :: hX
        do hX = 1, size(earn2)
            y(hX) = evalTaxCouScalar( tax, taxmode, kid, earn1, earn2(hX) )
        end do
    end function evalTaxCouVector2

    subroutine monotonicTaxPenalty(x, opt, penalty)
        implicit none
        real(dp), intent(in) :: x(NTAXPARAM)
        type(opt_t), intent(in) :: opt
        real(dp), intent(out) :: penalty
        real(dp) :: xtax(NTAXPARAM)
        type(tax_t) :: tax
        real(dp) :: penalty1, penalty2
        real(dp) :: lastMtr, thisMtr
        integer :: ixL1, ixL0

        ! recover tax
        call rescaleUnitIntervalToTaxLevel(x, xtax,  opt%taxins, MINY)
        call setTax( xtax, tax, TAXMODE, opt%taxins )

        ! init
        penalty1 = 0.0_dp
        penalty2 = 0.0_dp

        ! singles
        lastMtr = (tax%tSin(2, 1) - tax%tSin(1, 1)) / TAXDBR(1)

        do ixL0 = 2, NTAXBR - 1
            thisMtr = (tax%tSin(ixL0 + 1, 1) - tax%tSin(ixL0, 1)) / TAXDBR(ixL0)
            if (thisMtr < lastMtr) then
                penalty1 = penalty1 + (thisMtr - lastMtr) ** 2
            end if
            lastMtr = thisMtr
        end do

        lastMtr = (tax%tSin(2, 2) - tax%tSin(1, 2)) / TAXDBR(1)

        do ixL0 = 2, NTAXBR - 1
            thisMtr = (tax%tSin(ixL0 + 1, 2) - tax%tSin(ixL0, 2)) / TAXDBR(ixL0)
            if (thisMtr < lastMtr) then
                penalty2 = penalty2 + (thisMtr - lastMtr) ** 2
            end if
            lastMtr = thisMtr
        end do

        ! couples
        do ixL1 = 1, NTAXBR
            lastMtr = (tax%tCou(2, ixL1) - tax%tCou(1, ixL1)) / TAXDBR(1)
            do ixL0 = 2, NTAXBR - 1
                thisMtr = (tax%tCou(ixL0 + 1, ixL1) - tax%tCou(ixL0, ixL1)) / TAXDBR(ixL0)
                if (thisMtr < lastMtr) then
                    penalty1 = penalty1 + (thisMtr - lastMtr) ** 2
                end if
                lastMtr = thisMtr
            end do
        end do

        do ixL0 = 1, NTAXBR
            lastMtr = (tax%tCou(ixL0, 2) - tax%tCou(ixL0, 1)) / TAXDBR(1)
            do ixL1 = 2, NTAXBR - 1
                thisMtr = (tax%tCou(ixL0, ixL1 + 1) - tax%tCou(ixL0, ixL1)) / TAXDBR(ixL1)
                if (thisMtr < lastMtr) then
                    penalty2 = penalty2 + (thisMtr - lastMtr) ** 2
                end if
                lastMtr = thisMtr
            end do
        end do

        penalty = 100.0_dp * (penalty1 + penalty2)

    end subroutine monotonicTaxPenalty

    pure subroutine setTax(x, tax, mode, taxins)
        implicit none
        real(dp), intent(in) :: x(NTAXPARAM)
        type(tax_t), intent(out) :: tax
        integer, intent(in) :: mode
        integer, intent(in) :: taxins
        integer :: ixL0, ixL1, ixParam

        select case(mode)
        case(TAXLEVEL)
            tax%tSin(:, 1) = x(1:NTAXBR) ! tax parameters for singles
            ixParam = NTAXBR
            select case(taxins)
            case(TAX_JOINT)
                tax%tSin(:, 2) = tax%tSin(:, 1)
                do ixL1 = 1, NTAXBR
                    do ixL0 = ixL1, NTAXBR
                        ixParam = ixParam + 1
                        tax%tCou(ixL0, ixL1) = x(ixParam)
                    end do
                end do
                ! symmetry
                do ixL1 = 1, NTAXBR
                    do ixL0 = 1, ixL1 - 1
                        tax%tCou(ixL0, ixL1) = tax%tCou(ixL1, ixL0)
                    end do
                end do
            case(TAX_INDEPENDENT)
                tax%tSin(:, 2) = tax%tSin(:, 1)
                tax%tCou(1:NTAXBR, 1) = tax%tSin(1:NTAXBR, 1)
            case(TAX_INDEPENDENT3)
                tax%tSin(:, 2) = tax%tSin(:, 1)
                tax%tCou(1:NTAXBR, 1) = tax%tSin(1:NTAXBR, 1)

            case(TAX_FULLJOINT2)
                tax%tSin(:, 2) = tax%tSin(:, 1)
                tax%tCou(1:NTAXBR, 1) = tax%tSin(1:NTAXBR, 1)
            case(TAX_JOINTGBT)
                tax%tSin(:, 2) = tax%tSin(:, 1)
                do ixL1 = 1, NTAXBR
                    do ixL0 = 1, NTAXBR
                        ixParam = ixParam + 1
                        tax%tCou(ixL0, ixL1) = x(ixParam)
                    end do
                end do
            case(TAX_JOINTGBTALL)
                tax%tSin(:, 2) = x(NTAXBR + 1 : 2 * NTAXBR) ! tax parameters for singles
                ixParam = 2 * NTAXBR
                do ixL1 = 1, NTAXBR
                    do ixL0 = 1, NTAXBR
                        ixParam = ixParam + 1
                        tax%tCou(ixL0, ixL1) = x(ixParam)
                    end do
                end do
            case default
                stop 'unknown taxins'
            end select

        case(TAXRATE)
            stop 'not supported'
        end select

        tax%mtrViolation = 0.0_dp
        do ixL0 = 1, NTAXBR
            do ixL1 = 2, NTAXBR
                tax%mtrViolation = tax%mtrViolation &
                    + max( tax%tCou(ixL0, ixL1) - tax%tCou(ixL0, ixL1 - 1) - TAXDBR(ixL1 - 1), 0.0_dp )
            end do
        end do

        call calcTaxCoef(tax, taxins)
        tax%medicaid = .false.

    end subroutine setTax

    subroutine taxJointToTaxJointGBT(xin, xout)
        implicit none
        real(dp), intent(in) :: xin(NTAXPARAM)
        real(dp), intent(out) :: xout(NTAXPARAM)
        integer :: ixParam
        integer :: ixL0, ixL1
        real(dp) :: tSin(NTAXBR)
        real(dp) :: tCou(NTAXBR, NTAXBR)

        ! input
        ixParam = 0
        do ixL0 = 1, NTAXBR
            ixParam = ixParam + 1
            tSin(ixL0) = xin(ixParam)
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = ixL1, NTAXBR
                ixParam = ixParam + 1
                tCou(ixL0, ixL1) = xin(ixParam)
            end do
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = 1, ixL1 - 1
                tCou(ixL0, ixL1) = tCou(ixL1, ixL0)
            end do
        end do

        ! output
        ixParam = 0
        do ixL0 = 1, NTAXBR
            ixParam = ixParam + 1
            xout(ixParam) = tSin(ixL0)
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = 1, NTAXBR
                ixParam = ixParam + 1
                xout(ixParam) = tCou(ixL0, ixL1)
            end do
        end do

    end subroutine taxJointToTaxJointGBT

    subroutine taxJointToTaxJointGBTAll(xin, xout)
        implicit none
        real(dp), intent(in) :: xin(NTAXPARAM)
        real(dp), intent(out) :: xout(NTAXPARAM)
        integer :: ixParam
        integer :: ixL0, ixL1
        real(dp) :: tSin(NTAXBR)
        real(dp) :: tCou(NTAXBR, NTAXBR)

        ! input
        ixParam = 0
        do ixL0 = 1, NTAXBR
            ixParam = ixParam + 1
            tSin(ixL0) = xin(ixParam)
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = ixL1, NTAXBR
                ixParam = ixParam + 1
                tCou(ixL0, ixL1) = xin(ixParam)
            end do
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = 1, ixL1 - 1
                tCou(ixL0, ixL1) = tCou(ixL1, ixL0)
            end do
        end do

        ! output
        ixParam = 0
        do ixL0 = 1, NTAXBR
            ixParam = ixParam + 1
            xout(ixParam) = tSin(ixL0)
        end do

        do ixL1 = 1, NTAXBR
            ixParam = ixParam + 1
            xout(ixParam) = tSin(ixL1)
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = 1, NTAXBR
                ixParam = ixParam + 1
                xout(ixParam) = tCou(ixL0, ixL1)
            end do
        end do

    end subroutine taxJointToTaxJointGBTAll

    subroutine taxJointGBTtoTaxJointGBTAll(xin, xout)
        implicit none
        real(dp), intent(in) :: xin(NTAXPARAM)
        real(dp), intent(out) :: xout(NTAXPARAM)
        integer :: ixParam
        integer :: ixL0, ixL1
        real(dp) :: tSin(NTAXBR)
        real(dp) :: tCou(NTAXBR, NTAXBR)

        ! input
        ixParam = 0
        do ixL0 = 1, NTAXBR
            ixParam = ixParam + 1
            tSin(ixL0) = xin(ixParam)
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = 1, NTAXBR
                ixParam = ixParam + 1
                tCou(ixL0, ixL1) = xin(ixParam)
            end do
        end do

        ! output
        ixParam = 0
        do ixL0 = 1, NTAXBR
            ixParam = ixParam + 1
            xout(ixParam) = tSin(ixL0)
        end do

        do ixL1 = 1, NTAXBR
            ixParam = ixParam + 1
            xout(ixParam) = tSin(ixL1)
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = 1, NTAXBR
                ixParam = ixParam + 1
                xout(ixParam) = tCou(ixL0, ixL1)
            end do
        end do

    end subroutine taxJointGBTtoTaxJointGBTAll

    subroutine getTaxParameterBounds(bl, bu, ixParam, thistax)
        implicit none
        real(dp), intent(inout) :: bl(:)
        real(dp), intent(inout) :: bu(:)
        integer, intent(inout) :: ixParam
        integer, intent(in) :: thistax
        integer :: ixL0, ixL1

        ! singles
        do ixL0 = 1, NTAXBR - 1
            ixParam = ixParam + 1
            bl(ixParam) = -TAXDBR(iXL0)
            bu(ixParam) =  TAXDBR(iXL0)
        end do

        select case(thistax)
        case(TAX_JOINT)

            ! couples, first directional deviation
            do ixL1 = 1, NTAXBR - 1
                do ixL0 = ixL1, NTAXBR - 1
                    ixParam = ixParam + 1
                    bl(ixParam) = -TAXDBR(iXL0)
                    bu(ixParam) =  TAXDBR(iXL0)
                end do
            end do

            ! couples, second directional deviation
            do ixL1 = 1, NTAXBR - 1
                do ixL0 = ixL1, NTAXBR - 1
                    ixParam = ixParam + 1
                    bl(ixParam) = -TAXDBR(iXL1)
                    bu(ixParam) =  TAXDBR(iXL1)
                end do
            end do

        case(TAX_INDEPENDENT)
            ! ...
        case(TAX_INDEPENDENT3)
            ! ...
        case(TAX_FULLJOINT2)
            ! ...
        case(TAX_JOINTGBT)

            ! couples, first directional deviation
            do ixL1 = 1, NTAXBR - 1
                do ixL0 = 1, NTAXBR - 1
                    ixParam = ixParam + 1
                    bl(ixParam) = -TAXDBR(iXL0)
                    bu(ixParam) =  TAXDBR(iXL0)
                end do
            end do

            ! couples, second directional deviation
            do ixL1 = 1, NTAXBR - 1
                do ixL0 = 1, NTAXBR - 1
                    ixParam = ixParam + 1
                    bl(ixParam) = -TAXDBR(iXL1)
                    bu(ixParam) =  TAXDBR(iXL1)
                end do
            end do
        case(TAX_JOINTGBTALL)

            do ixL0 = 1, NTAXBR - 1
                ixParam = ixParam + 1
                bl(ixParam) = -TAXDBR(iXL0)
                bu(ixParam) =  TAXDBR(iXL0)
            end do

            ! couples, first directional deviation
            do ixL1 = 1, NTAXBR - 1
                do ixL0 = 1, NTAXBR - 1
                    ixParam = ixParam + 1
                    bl(ixParam) = -TAXDBR(iXL0)
                    bu(ixParam) =  TAXDBR(iXL0)
                end do
            end do

            ! couples, second directional deviation
            do ixL1 = 1, NTAXBR - 1
                do ixL0 = 1, NTAXBR - 1
                    ixParam = ixParam + 1
                    bl(ixParam) = -TAXDBR(iXL1)
                    bu(ixParam) =  TAXDBR(iXL1)
                end do
            end do
        case default
            stop 'error with thistax parameter'
        end select

    end subroutine getTaxParameterBounds

    subroutine getTaxLinearRestrictionsLevelScaled(mode, thismin, thistax, a)
        implicit none
        integer, parameter :: lda  = MAXVALNTAXCON
        integer, parameter :: nmax = NTAXPARAM + NTYPEALL
        integer, intent(in) :: mode
        real(dp), intent(in) :: thismin
        integer, intent(in) :: thistax
        real(dp), intent(out) :: a(lda, nmax)
        integer :: tauLev(NTAXBR, NTAXBR)
        integer :: ixL0, ixL1, ixR, ixParam

        select case(mode)
        case(TAXLEVEL)

            a = 0.0_dp
            ixR = 0

            ! singles
            do ixL0 = 1, NTAXBR - 1
                ixR = ixR + 1
                a(ixR, ixL0 + 1) = TAXBR(ixL0 + 1) - thismin
                a(ixR, ixL0) = -(TAXBR(ixL0) - thismin)
            end do

            select case(thistax)
            case(TAX_JOINT)

                ixParam = NTAXBR
                tauLev = -9999
                do ixL1 = 1, NTAXBR
                    do ixL0 = ixL1, NTAXBR
                        ixParam = ixParam + 1
                        tauLev(ixL0, ixL1) = ixParam
                    end do
                end do

                ! couples, first directional deviation
                do ixL1 = 1, NTAXBR - 1
                    do ixL0 = ixL1, NTAXBR - 1
                        ixR = ixR + 1
                        a(ixR, tauLev(ixL0 + 1, ixL1)) = TAXBR(ixL0 + 1) + TAXBR(ixL1) - thismin
                        a(ixR, tauLev(ixL0, ixL1)) = -(TAXBR(ixL0) + TAXBR(ixL1) - thismin)
                    end do
                end do

                ! couples, second directional deviation
                do ixL1 = 1, NTAXBR - 1
                    do ixL0 = ixL1, NTAXBR - 1
                        ixR = ixR + 1
                        a(ixR, tauLev(ixL0 + 1, ixL1 + 1)) = TAXBR(ixL0 + 1) + TAXBR(ixL1 + 1) - thismin
                        a(ixR, tauLev(ixL0 + 1, ixL1)) = -(TAXBR(ixL0 + 1) + TAXBR(ixL1) - thismin)
                    end do
                end do
            case(TAX_INDEPENDENT)
                ! ...
            case(TAX_INDEPENDENT3)
                ! ...
            case(TAX_FULLJOINT2)
                ! ...
            case(TAX_JOINTGBT)

                ixParam = NTAXBR
                tauLev = -9999
                do ixL1 = 1, NTAXBR
                    do ixL0 = 1, NTAXBR
                        ixParam = ixParam + 1
                        tauLev(ixL0, ixL1) = ixParam
                    end do
                end do

                ! couples, first directional deviation
                do ixL1 = 1, NTAXBR - 1
                    do ixL0 = 1, NTAXBR - 1
                        ixR = ixR + 1
                        a(ixR, tauLev(ixL0 + 1, ixL1)) = TAXBR(ixL0 + 1) + TAXBR(ixL1) - thismin
                        a(ixR, tauLev(ixL0, ixL1)) = -(TAXBR(ixL0) + TAXBR(ixL1) - thismin)
                    end do
                end do

                ! couples, second directional deviation
                do ixL1 = 1, NTAXBR - 1
                    do ixL0 = 1, NTAXBR - 1
                        ixR = ixR + 1
                        a(ixR, tauLev(ixL0 + 1, ixL1 + 1)) = TAXBR(ixL0 + 1) + TAXBR(ixL1 + 1) - thismin
                        a(ixR, tauLev(ixL0 + 1, ixL1)) = -(TAXBR(ixL0 + 1) + TAXBR(ixL1) - thismin)
                    end do
                end do

            case(TAX_JOINTGBTALL)

                ixParam = NTAXBR
                do ixL0 = 1, NTAXBR - 1
                    ixR = ixR + 1
                    ixParam = ixParam + 1
                    a(ixR, ixParam + 1) = TAXBR(ixL0 + 1) - thismin
                    a(ixR, ixParam) = -(TAXBR(ixL0) - thismin)
                end do

                ixParam = NTAXBR * 2
                tauLev = -9999
                do ixL1 = 1, NTAXBR
                    do ixL0 = 1, NTAXBR
                        ixParam = ixParam + 1
                        tauLev(ixL0, ixL1) = ixParam
                    end do
                end do

                ! couples, first directional deviation
                do ixL1 = 1, NTAXBR - 1
                    do ixL0 = 1, NTAXBR - 1
                        ixR = ixR + 1
                        a(ixR, tauLev(ixL0 + 1, ixL1)) = TAXBR(ixL0 + 1) + TAXBR(ixL1) - thismin
                        a(ixR, tauLev(ixL0, ixL1)) = -(TAXBR(ixL0) + TAXBR(ixL1) - thismin)
                    end do
                end do

                ! couples, second directional deviation
                do ixL1 = 1, NTAXBR - 1
                    do ixL0 = 1, NTAXBR - 1
                        ixR = ixR + 1
                        a(ixR, tauLev(ixL0 + 1, ixL1 + 1)) = TAXBR(ixL0 + 1) + TAXBR(ixL1 + 1) - thismin
                        a(ixR, tauLev(ixL0 + 1, ixL1)) = -(TAXBR(ixL0 + 1) + TAXBR(ixL1) - thismin)
                    end do
                end do


            case default
                stop 'getTaxLinearRestrictionsLevel: unknown thistax setting'
            end select
        case default
            stop 'getTaxLinearRestrictionsLevel: mode must be TAXLEVEL'
        end select

    end subroutine getTaxLinearRestrictionsLevelScaled


    subroutine checkTaxLinearRestrictionsLevel(x, bl, bu, a)

        implicit none

        integer, parameter :: nmax = NTAXPARAM + NTYPEALL
        integer, parameter :: lda = MAXVALNTAXCON
        real(dp), intent(in) :: x(nmax)
        real(dp), intent(in) :: bl(lda)
        real(dp), intent(in) :: bu(lda)
        real(dp), intent(in) :: a(lda,nmax)
        integer :: ixR

        real(dp) :: aTr(nmax, lda)
        real(dp) :: mult(nmax)
        aTr = transpose(a)

        do ixR = 1, lda
            mult(ixR) = sum( aTr(:, ixR) * x )
            write(*, '(I4,3(",",2X,F12.6))') ixR, bl(ixR), mult(ixR), bu(ixR)
        end do

    end subroutine checkTaxLinearRestrictionsLevel

    subroutine rescaleTaxLevelToUnitInterval(x, y, miny)
        implicit none
        real(dp), intent(in) :: x(NTAXPARAM)
        real(dp), intent(out) :: y(NTAXPARAM)
        real(dp), intent(in) :: miny
        integer :: ixParam, ixL0, ixL1

        ixParam = 0

        do ixL0 = 1, NTAXBR
            ixParam = ixParam + 1
            y(ixParam) = ( x(ixParam) - miny ) / ( TAXBR(ixL0) - miny )
        end do

        do ixL1 = 1, NTAXBR
            do ixL0 = ixL1, NTAXBR
                ixParam = ixParam + 1
                y(ixParam) = ( x(ixParam) - miny ) / ( TAXBR(ixL0) + TAXBR(ixL1) - miny )
            end do
        end do

    end subroutine rescaleTaxLevelToUnitInterval

    subroutine rescaleUnitIntervalToTaxLevel(x, y, taxins, miny)
        implicit none
        real(dp), intent(in) :: x(NTAXPARAM)
        real(dp), intent(out) :: y(NTAXPARAM)
        integer, intent(in) :: taxins
        real(dp), intent(in) :: miny
        integer :: ixParam, ixL0, ixL1

        ixParam = 0

        ! singles
        do ixL0 = 1, NTAXBR
            ixParam = ixParam + 1
            y(ixParam) = miny + x(ixParam) * ( TAXBR(ixL0) - miny )
        end do

        select case(taxins)
        case(TAX_JOINT)
            do ixL1 = 1, NTAXBR
                do ixL0 = ixL1, NTAXBR
                    ixParam = ixParam + 1
                    y(ixParam) = miny + x(ixParam) * ( TAXBR(ixL0) + TAXBR(ixL1) - miny )
                end do
            end do
        case(TAX_INDEPENDENT)
            ! ...
        case(TAX_INDEPENDENT3)
            ! ...
        case(TAX_FULLJOINT2)
            ! ...
        case(TAX_JOINTGBT)
            do ixL1 = 1, NTAXBR
                do ixL0 = 1, NTAXBR
                    ixParam = ixParam + 1
                    y(ixParam) = miny + x(ixParam) * ( TAXBR(ixL0) + TAXBR(ixL1) - miny )
                end do
            end do
        case(TAX_JOINTGBTALL)
            do ixL1 = 1, NTAXBR
                ixParam = ixParam + 1
                y(ixParam) = miny + x(ixParam) * ( TAXBR(ixL1) - miny )
            end do
            do ixL1 = 1, NTAXBR
                do ixL0 = 1, NTAXBR
                    ixParam = ixParam + 1
                    y(ixParam) = miny + x(ixParam) * ( TAXBR(ixL0) + TAXBR(ixL1) - miny )
                end do
            end do
        case default
            stop 'rescaleUnitIntervalToTaxLevel: unknown taxins setting'
        end select

    end subroutine rescaleUnitIntervalToTaxLevel

    subroutine kinks(tax, cou, kid, sex, earn1, earn2, kinks_earn, kinks_net, kinks_mtr, kinks_dis, kinks_num)

        use globaldef

        implicit none

        type(tax_t), intent(in) :: tax
        logical, intent(in) :: cou
        integer, intent(in) :: kid
        integer, intent(in) :: sex
        real(dp), intent(in) :: earn1, earn2

        real(dp), intent(out), dimension(MAXKINKS) :: kinks_earn, kinks_net, kinks_mtr
        logical, intent(out), dimension(MAXKINKS) :: kinks_dis
        integer, intent(out) :: kinks_num

        real(dp) :: taxcomp0, taxcomp1
        real(dp) :: taxrate0, taxrate1
        real(dp) :: earn, earn0 !,ern0,ernb !,ern0b

        real(dp), parameter :: mtrtol = 1.0e-5_dp
        real(dp), parameter :: distol = 1.01_dp
        real(dp), parameter :: etol = 0.00001_dp

        ! maxstep is the main parameter that determines the number of evaluations,
        ! if too large then may miss some discontinuities
        real(dp), parameter :: maxstep = 5.00_dp
        real(dp), parameter :: minstep = maxstep / 10000.0_dp

        integer :: kinkidx

        real(dp) :: temp

        real(dp) :: earn_b, earn_a, earn_mid, temp_a, temp_b, dearn, rate_a, rate_b

        integer :: ev
        ev = 0

        if (earn2 - earn1 < maxstep) return
        if (earn1 < 0.0_dp) return

        ! calculate income at lower range
        if ( cou ) then
            taxcomp0 = evalTax( tax, 0, kid, earn1, 0.0_dp )
        else
            taxcomp0 = evalTax( tax, 0, kid, sex, earn1 )
        end if

!         taxcomp0 = evalWelfareFoodStampsCouScalar(tax,kid,earn1)
        ev = ev + 1

        !step to get marginal rate
        earn = earn1 + minstep
        if ( cou ) then
            taxcomp1 = evalTax( tax, 0, kid, earn, 0.0_dp )
        else
            taxcomp1 = evalTax( tax, 0, kid, sex, earn )
        end if
!         taxcomp1 = evalWelfareFoodStampsCouScalar(tax,kid,earn)
        ev = ev + 1

        taxrate1 = (taxcomp1 - taxcomp0) / minstep

        kinks_earn(1) = earn1
        kinks_net(1) = taxcomp0
        kinks_mtr(1) = taxrate1
        kinks_dis(1) = .false.

        kinkidx = 2

        earn0 = earn1 + minstep

        taxrate0 = taxrate1
        taxcomp0 = taxcomp1

loopmax : do

            if (kinkidx >= MAXKINKS) exit

            earn = earn + maxstep

            if (earn > earn2) exit

            if ( cou ) then
                taxcomp1 = evalTax( tax, 0, kid, earn, 0.0_dp )
            else
                taxcomp1 = evalTax( tax, 0, kid, sex, earn )
            end if

            ev = ev + 1

            taxrate1 = (taxcomp1 - taxcomp0) / maxstep

            !if a mtr change detected
            if (abs(taxrate1 - taxrate0) > mtrtol) then

                earn_b = earn
                earn_a = earn0
                rate_a = taxrate0
                temp_a = taxcomp0

                !use bisection to identify the change point more accurately
                do !while (abs(hrs_b-hrs_a)>0.00001_dp)

                    if (abs(earn_b - earn_a) > etol) then
                        !midpoint of domain
                        earn_mid = 0.5_dp * (earn_b + earn_a)
                        dearn    = 0.5_dp * (earn_b - earn_a)

                        if ( cou ) then
                            temp_b = evalTax( tax, 0, kid, earn_mid, 0.0_dp )
                        else
                            temp_b = evalTax( tax, 0, kid, sex, earn_mid )
                        end if
                        ev = ev + 1

                        rate_b = (temp_b - temp_a) / dearn

                        !which direction to move?
                        if (abs(rate_b - rate_a) > mtrtol) then
                            earn_b = earn_mid
                        else
                            earn_a = earn_mid
                            temp_a = temp_b
                        end if
                    else

                        if ( cou ) then
                            temp_a = evalTax( tax, 0, kid, earn_a, 0.0_dp )
                        else
                            temp_a = evalTax( tax, 0, kid, sex, earn_a )
                        end if
                        ev = ev + 1

                        if ( cou ) then
                            temp_b = evalTax( tax, 0, kid, earn_b, 0.0_dp )
                        else
                            temp_b = evalTax( tax, 0, kid, sex, earn_b )
                        end if
                        ev = ev + 1

                        dearn = earn_b - earn_a
                        taxrate1 = (temp_b - temp_a) / dearn

                        !is there a discontinuity?
                        if (abs(taxrate1) > distol) then
                            kinks_earn(kinkidx) = earn_a
                            kinks_net(kinkidx) = temp_a
                            kinks_dis(kinkidx) = .true.
                            if (taxcomp1 > temp_b) then
                                kinks_mtr(kinkidx) = 9.999_dp
                            else
                                kinks_mtr(kinkidx) = -9.999_dp
                            end if
                            kinkidx = kinkidx + 1
                        end if
                        exit
                    end if

                end do

                !step to get marginal rate
                earn = earn_b + minstep
                if ( cou ) then
                    taxcomp1 = evalTax( tax, 0, kid, earn, 0.0_dp )
                else
                    taxcomp1 = evalTax( tax, 0, kid, sex, earn )
                end if
                ev = ev + 1

                taxrate1 = (taxcomp1 - temp_b) / minstep

                kinks_earn(kinkidx) = earn_b
                kinks_net(kinkidx) = temp_b
                kinks_mtr(kinkidx) = taxrate1
                kinks_dis(kinkidx) = .false.

                kinkidx = kinkidx + 1

            end if

            taxrate0 = taxrate1
            taxcomp0 = taxcomp1
            earn0 = earn

        end do loopmax

        if (kinkidx < MAXKINKS) then
            !end point
            if ( cou ) then
                temp = evalTax( tax, 0, kid, earn2, 0.0_dp )
            else
                temp = evalTax( tax, 0, kid, sex, earn2 )
            end if
            ev = ev + 1

            taxcomp1 = temp !taxpoint

            kinks_earn(kinkidx) = earn2
            kinks_net(kinkidx) = taxcomp1
            kinks_mtr(kinkidx) = kinks_mtr(kinkidx - 1)
        else
            print *, 'MAXKINKS is exceeded'
        end if

        kinks_num = min(kinkidx, MAXKINKS)

    end subroutine kinks

end module taxes
