clear
set more off

cd ~/git/familytax/taxes

local all_mstat = "2 2 3 1"
local all_state = "0 0 0 0"
local all_year = "2006 2006 2006 2006"
local all_depchild = "2 0 2 0"
local all_tlabel = "coukid cou sinkid sin"

cap log close taxlog
log using "output/tax.log", replace text name(taxlog)

local minearn = 0
local maxearn = 1000000
local nearn = 50001
local stepearn = (`maxearn'-`minearn')/`=`nearn'-1'
di `stepearn'

local nobs = `nearn'

set obs `nobs'

gen double pwages = `minearn' + (_n-1)*`stepearn'
gen double swages = 0

local n : word count `all_mstat'

forval x = 1/`n' {
	
	preserve

	local mstat : word `x' of `all_mstat'
	local state : word `x' of `all_state'
	local year  : word `x' of `all_year'
	local depchild : word `x' of `all_depchild'
	local tlabel : word `x' of `all_tlabel'

	gen mstat = `mstat'
	gen state = `state'
	gen year = `year'
	gen depchild = `depchild'
	gen depx = depchild

	taxsim9, replace
	save "output/tax_`tlabel'", replace

	gen double tottax = fiitax+fica
	gen double mtr = (tottax[_n+1]-tottax)/`stepearn'
	gen double mtrts = (frate+ficar)/100

	gen byte dmtr = mtrts~=mtrts[_n-1]
	gen int bracket = sum(dmtr)
	su bracket

	local nbracket = r(max)
	egen mean_pwages = mean(pwages), by(bracket)
	egen mean_tottax = mean(tottax), by(bracket)

	gen double bc_mtr = mtrts
	gen double bc_lower_earn = 0 if bracket==1
	gen double bc_lower_tax = tottax[1] if bracket==1

	forval bracket = 2/`nbracket' {
		qui gen double tempearn = ( mean_tottax + bc_lower_earn[_n-1]*bc_mtr[_n-1]-bc_mtr*mean_pwages - bc_lower_tax[_n-1] )/(bc_mtr[_n-1]-bc_mtr) if bracket==`bracket' & dmtr==1
		qui gen double temptax  = bc_lower_tax[_n-1] + ( tempearn - bc_lower_earn[_n-1] )*bc_mtr[_n-1]

		qui egen double tempearn2 = min(tempearn), by(bracket)
		qui egen double temptax2 = min(temptax), by(bracket)

		qui replace bc_lower_earn = tempearn2 if bracket==`bracket'
		qui replace bc_lower_tax = temptax2 if bracket==`bracket'
		
		drop tempearn tempearn2
		drop temptax temptax2

		}

	collapse bc_* year, by(bracket)
	qui count
	local n = r(N)

	cap file close taxrecord1
	qui file open taxrecord1 using "output/tax_`tlabel'.dat", text write replace

	file write taxrecord1 "[TAXES]" _n _n
	file write taxrecord1 "TAX_LABEL `tlabel'" _n _n
	file write taxrecord1 "TAX_BANDS "
	forval ix = 1/`=`n'-1' {
		local val = bc_lower_earn[`ix']
		file write taxrecord1 "`val', "
		}
	local val = bc_lower_earn[`n']
	file write taxrecord1 "`val'" _n _n

	file write taxrecord1 "TAX_RATES "
	forval ix = 1/`=`n'-1' {
		local val = bc_mtr[`ix']
		file write taxrecord1 "`val', "
		}
	local val = bc_mtr[`n']
	file write taxrecord1 "`val'" _n _n

	file write taxrecord1 "TAX_LEVEL "
	forval ix = 1/`=`n'-1' {
		local val = bc_lower_tax[`ix']
		file write taxrecord1 "`val', "
		}
	local val = bc_lower_tax[`n']
	file write taxrecord1 "`val'" _n _n

	local val = year[1]
	file write taxrecord1 "TAX_YEAR `val'" _n _n
	file write taxrecord1 "TAX_PRICES `val'" _n _n
	file close taxrecord1

	restore
	}

log close taxlog

