program tools_variance_matrix
    use globaldef
    use test3kid
    use moments
    use mathfn
    use mpi
    implicit none

    real(dp) :: x(NPARAM)
    logical  :: transformParam
    real(dp) :: varParam(NPARAM, NPARAM)

    logical :: verbose

    type(opt_t) :: opt

    real(dp) :: DTrW(NPARAM, NMOM)
    real(dp) :: DTrWD(NPARAM, NPARAM)
    real(dp) :: DTrWDInv(NPARAM, NPARAM)
    real(dp) :: DTrWV(NPARAM, NMOM)
    real(dp) :: WD(NMOM, NPARAM)
    real(dp) :: DTrWVWD(NPARAM, NPARAM)
    real(dp) :: DTrWDInv_DTrWVWD(NPARAM, NPARAM)
    real(dp) :: sensitivity(NPARAM, NMOM)

    real(dp) :: D(NMOM, NPARAM)
    real(dp) :: DTr(NPARAM, NMOM)
    real(dp) :: W(NMOM, NMOM) ! weighting matrix
    real(dp) :: V(NMOM, NMOM) ! covariance matrix

    real(dp) :: WVec(NMOM)
    real(dp) :: x2(NPARAM)

    integer :: ixM, iX, jX
    integer :: funit

    type(mom_t) :: momData, vmomData, modemom, weightMom

    integer :: ierror, id, nproc

    integer, parameter :: nsamples = 200
    real(dp) :: x_r(NPARAM, nsamples)
    integer :: ixR

    integer :: narg
    integer :: istat
    integer :: estmode
    integer :: ixTrans
    character(len = 32) :: filetag, thisarg
    character(len = 128) :: fpath
    character(len = 128) :: fnamesol

    ! x vector

    narg = command_argument_count()
    if (narg==0) then
        filetag = ''
        fpath = 'output/estimation/variance/'
        fnamesol = 'output/estimation/xsol_market_mpec_mpi.asc'
        estmode = 0
    else if (narg==1) then
        call get_command_argument(1, value = filetag, status = istat)
        fpath = 'output/estimation/' // trim(adjustl(filetag)) // '/variance/'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        estmode = 0
    else if (narg == 2) then
        call get_command_argument(1, value = filetag, status = istat)
        fpath = 'output/estimation/' // trim(adjustl(filetag)) // '/variance/'
        fnamesol = 'output/estimation/' // trim(adjustl(filetag)) // '/xsol_market_mpec_mpi.asc'
        call get_command_argument(2, value = thisarg, status = istat)
        read(thisarg, '(i10)') estmode
    else
        call mpi_finalize(ierror)
        stop 'incorrect number of command arguments supplied'
    end if

    ! read x input
    open(newunit = funit, file = fnamesol, status = 'old', action = 'read')
    do iX = 1, NPARAM
        read(funit, *) x(iX)
    end do
    close(funit)

    verbose = .true.

    id = 0

     call mpi_init(ierror)

     !obtain id for each node
     call mpi_comm_rank(mpi_comm_world, id, ierror)

     !returns the number of nodes
     call mpi_comm_size(mpi_comm_world, nproc, ierror)

     if (nproc < NPARAM + 1) then
         write (IOUT, *) NPARAM + 1, 'processors required'
         call mpi_finalize(ierror)
         stop
     end if

    call setOptions(opt, OPT_ESTIMATION, estmode)

    do ixTrans = 1, 2

        if ( ixTrans == 1 ) then
            transformParam = .true.
        else
            transformParam = .false.
        end if

        if ( transformParam ) then
            if ( verbose .and. id == 0 ) then
                write(IOUT, *) 'Calculating standard errors for TRANSFORMED parameter vector...'
            end if
            call getTransformedX(x, x2)
            opt%xmode = X_NOTRANSFORM
        else
            if ( verbose .and. id == 0 ) then
                write(IOUT, *) 'Calculating standard errors for UNTRANSFORMED parameter...'
            end if
            x2 = x
            opt%xmode = X_TRANSFORM
        end if

        if (id == 0) then
            if ( transformParam ) then
                open(newunit = funit, file = trim(adjustl(fpath)) // '/x.csv', status = 'replace', action = 'write')
            else
                open(newunit = funit, file = trim(adjustl(fpath)) // '/x_raw.csv', status = 'replace', action = 'write')
            end if

            do iX = 1, NPARAM
                write(funit, '(F32.8)') x2(iX)
            end do
            close(funit)

            open(newunit = funit, file = trim(adjustl(fpath)) // '/label.csv', status = 'replace', action = 'write')
            call printxlab(x2, funit)
            close(funit)
        end if

        if ( id == 0 )  then

            if ( verbose .and. id == 0 ) then
                write(IOUT, *) 'reading empirical covariance matrix...'
            end if

            call readMomentVarianceMatrix(opt, V)

            if ( verbose .and. id == 0 ) then
                write(IOUT, *) 'reading weighting matrix...'
            end if

            ! read moment files
            call readMomentFiles('input', opt, momData, vmomData, modemom)
            call weightingMatrix(vmomData, modemom, weightMom, ATUSWT)

            Wvec = transfer( weightMom, (/ 1.0_dp /) )

            if ( verbose .and. id == 0 ) then
                write(IOUT, *) 'calculating moment derivative matrix...'
            end if

        end if

        call calculateMomentDerivative(x2, opt, D)

        if ( id == 0 ) then

            DTr = transpose(D)

            if ( verbose .and. id == 0 ) then
                write(IOUT, *) 'calculating parameter variance matrix...'
            end if

            W = 0.0_dp
            do ixM = 1, NMOM
                W(ixM, ixM) = WVec(iXM)
            end do

            DTrW = matmul(DTr, W)
            DTrWD = matmul(DTrW, D)

            call matrixInverse(DTrWD, DTrWDInv, NPARAM)

            DTrWV = matmul(DTrW, V)
            WD = matmul(W, D) ! transpose DTrW
            DTrWVWD = matmul(DTrWV, WD)


            DTrWDInv_DTrWVWD = matmul(DTrWDInv, DTrWVWD)
            sensitivity = matmul(DTrWDInv, DTrW)
            varParam = matmul(DTrWDInv_DTrWVWD, DTrWDInv)

            if ( verbose .and. id == 0 ) then
                write(IOUT, *) 'parameter standard errors...'
                write(IOUT, *)
                do iX = 1, NPARAM
                    print *, iX, x2(iX), sqrt(varParam(iX, iX))
                end do
                write(IOUT,*)
            end if

        end if

        if ( id == 0 ) then

            if ( transformParam ) then
                open(newunit = funit, file = trim(adjustl(fpath)) // '/var.csv', status = 'replace', action = 'write')
            else
                open(newunit = funit, file = trim(adjustl(fpath)) // '/var_raw.csv', status = 'replace', action = 'write')
            end if

            do jX = 1, NPARAM
                do iX = 1, NPARAM - 1
                    write(funit, '(F32.8,",")', advance = 'no') varParam(iX, jX)
                end do
                write(funit, '(F32.8)') varParam(NPARAM, jX)
            end do
            close(funit)

            if ( transformParam ) then
                open(newunit = funit, file = trim(adjustl(fpath)) // '/D.csv', status = 'replace', action = 'write')
                do jX = 1, NPARAM
                    do iX = 1, NMOM-1
                        write(funit, '(G0.16,",")', advance = 'no') D(iX, jX)
                    end do
                    write(funit, '(G0.16)') D(NMOM, jX)
                end do
                close(funit)

                open(newunit = funit, file = trim(adjustl(fpath)) // '/V.csv', status = 'replace', action = 'write')
                do jX = 1, NMOM
                    do iX = 1, NMOM-1
                        write(funit, '(G0.16,",")', advance = 'no') V(iX, jX)
                    end do
                    write(funit, '(G0.16)') V(NMOM, jX)
                end do
                close(funit)

                open(newunit = funit, file = trim(adjustl(fpath)) // '/W.csv', status = 'replace', action = 'write')
                do jX = 1, NMOM
                    do iX = 1, NMOM-1
                        write(funit, '(G0.16,",")', advance='no') W(iX, jX)
                    end do
                    write(funit,'(G0.16)') W(NMOM, jX)
                end do
                close(funit)

                open(newunit = funit, file = trim(adjustl(fpath)) // '/sensitivity.csv', status = 'replace', action = 'write')
                do jX = 1, NPARAM
                    do iX = 1, NMOM-1
                        write(funit, '(G0.16,",")', advance='no') sensitivity(jX, iX)
                    end do
                    write(funit,'(G0.16)') sensitivity(jX, NMOM)
                end do
                close(funit)

            end if

            call sampleDist(x2, varParam, x_r, nsamples)
            do ixR = 1, nsamples
                if ( transformParam ) then
                    open(newunit = funit, file = trim(adjustl(fpath)) // '/samples/x' // intToStr(ixR) // '.csv', &
                        status = 'replace', action = 'write')
                else
                    open(newunit = funit, file = trim(adjustl(fpath)) // '/samples/x_raw' // intToStr(ixR) // '.csv', &
                        status = 'replace', action = 'write')
                end if
                do iX = 1, NPARAM
                    write(funit, '(F32.8,",")') x_r(iX, ixR)
                end do
                close(funit)
            end do

        end if

    end do ! ixTrans

    call mpi_finalize(ierror)

end program tools_variance_matrix

subroutine sampleDist(x, vc, x_r, nsample)

    use globaldef, only : dp, NPARAM

    implicit none

    integer, intent(in) :: nsample
    real(dp), intent(in) :: x(NPARAM)             ! point estimates of x
    real(dp), intent(in) :: vc(NPARAM, NPARAM)    ! variance matrix
    real(dp), intent(out) :: x_r(NPARAM, nsample) ! sampled vector x

    !these variables/parameters are used by NAG
    integer, parameter :: n = NPARAM, ic = n, nr = (n + 1) * (n + 2) / 2
    real(dp) :: r(nr)
    integer :: ifail
    integer :: ixR

    ! call g05zaf('O')
    call g05cbf(0)
    ifail = 0
    call g05eaf(x, n, vc, ic, 0.0_dp, r, nr, ifail)

    do ixR = 1, nsample
        ifail = 0
        call g05ezf(x_r(:, ixR), n, r, nr, ifail)
    end do

end subroutine sampleDist
