module test3kid

    use globaldef

    interface wageIntegrateFast
        module procedure wageIntegrateFast_1d
        module procedure correlatedWageIntegrateFast_2d
    end interface wageIntegrateFast

    interface sde
        module procedure sde_scalar
        module procedure sde_vector
    end interface sde

    interface cove
        module procedure cove_scalar
        module procedure cove_vector
    end interface cove

    type xi_t
        integer :: i
        integer :: n
        integer :: count
        logical :: trans
        real(dp), allocatable :: x(:)
        real(dp), allocatable :: y(:)
        integer, allocatable :: xtype(:)
        character(len = 64), allocatable :: xlab(:)
        character(len = 128), allocatable :: xlablong(:)
    contains
        procedure :: val => xi_val
        procedure :: transform => xi_transform
        procedure :: init => xi_init
        procedure :: reset => xi_reset
        procedure :: final => xi_final
    end type

contains

    pure subroutine xi_init(xi, x)
        class(xi_t), intent(inout) :: xi
        real(dp), intent(in) :: x(:)
        xi%n = size(x)
        xi%i = 1
        allocate(xi%x(xi%n))
        allocate(xi%y(xi%n))
        allocate(xi%xtype(xi%n))
        allocate(xi%xlab(xi%n))
        allocate(xi%xlablong(xi%n))
        xi%x = x
        xi%trans = .true.
    end subroutine xi_init

    function xi_val(xi, xlab, xlablong, xtype) result(y)
        class(xi_t), intent(inout) :: xi
        character(len = *), intent(in) :: xlab
        character(len = *), intent(in) :: xlablong
        integer, intent(in) :: xtype
        real(dp) :: y
        y = xi%x(xi%i)
        xi%xlab(xi%i) = xlab
        xi%xlablong(xi%i) = xlablong
        xi%xtype(xi%i) = xtype
        xi%i = xi%i + 1
    end function xi_val

    function xi_transform(xi, transformation, xlab, xlablong, xtype) result(y)
        class(xi_t), intent(inout) :: xi
        integer, intent(in) :: transformation
        character(len = *), intent(in) :: xlab
        character(len = *), intent(in) :: xlablong
        integer, intent(in) :: xtype
        real(dp) :: y
        if (xi%trans) then
            select case(transformation)
            case(TRANSFORM_EXP)
                y = exp( xi%x(xi%i) )
                xi%xlab(xi%i) = xlab // ' [exp]'
            case(TRANSFORM_ONEEXP)
                y = 1.0_dp+exp( xi%x(xi%i) )
                xi%xlab(xi%i) = xlab // ' [1+exp]'
            case(TRANSFORM_LOGIT)
                y = 1.0_dp / ( 1.0_dp + exp(-xi%x(xi%i)) )
                xi%xlab(xi%i) = xlab // ' [logit]'
            case(TRANSFORM_CORR)
                y = ( 1.0_dp - exp(-xi%x(xi%i)) ) / ( 1.0_dp + exp(-xi%x(xi%i)) )
                xi%xlab(xi%i) = xlab // ' [corr]'
            case(TRANSFORM_LINEAR)
                y = xi%x(xi%i)
                xi%xlab(xi%i) = xlab
            case(TRANSFORM_ONE_EXP)
                y = 1.0_dp-exp( xi%x(xi%i) )
                xi%xlab(xi%i) = xlab // ' [1-exp]'
            case(TRANSFORM_INVEXP)
                y = 1.0_dp / exp( xi%x(xi%i) )
                xi%xlab(xi%i) = xlab // ' [1/exp]'
            case default
                y = xi%x(xi%i)
                xi%xlab(xi%i) = xlab // ' [???]'
            end select
            xi%y(xi%i) = y
            xi%xtype(xi%i) = xtype
            xi%xlablong(xi%i) = xlablong
        else
            y = xi%x(xi%i)
            xi%y(xi%i) = y
            xi%xlab(xi%i) = xlab
            xi%xlablong(xi%i) = xlablong
        end if
        xi%i = xi%i + 1
    end function xi_transform

    pure subroutine xi_reset(xi)
        class(xi_t), intent(inout) :: xi
        xi%i = 1
    end subroutine xi_reset

    pure subroutine xi_final(xi)
        class(xi_t), intent(inout) :: xi
        xi%count = xi%i - 1
    end subroutine xi_final

    subroutine getTransformedX(x, xout)

        implicit none

        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(out) :: xout(NPARAM)

        real(dp) :: xtax(NTAXPARAM)
        integer :: xtype(NPARAM)
        integer  :: market, estmode

        ! options
        type(opt_t) :: opt

        ! tax schedules
        type(tax_t) :: tax

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)

        ! marriage market
        type(m_t) :: m

        estmode = 0
        call setOptions(opt, OPT_ESTIMATION, estmode)

        opt%tmode = 0
        opt%xmode = X_TRANSFORM
        opt%verbose = .false.
        market = 0

        call setup( x, xout, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd )

    end subroutine getTransformedX

    subroutine swffun(x, xlam, xtax, opt, ret, theta)
!$      use omp_lib
        use taxes
        implicit none
        real(dp), intent(in)  :: x(NPARAM)
        real(dp), intent(out) :: xlam(NTYPEALL)
        real(dp), intent(in)  :: xtax(NTAXPARAM)
        type(opt_t), intent(in) :: opt
        type(ret_t), intent(out) :: ret
        real(dp), intent(in) :: theta

        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)

        ! tax schedules
        type(tax_t) :: tax

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)

        ! marriage market
        type(m_t) :: m

        ! indexing
        integer :: pX, pX1, pX2

        integer :: market

        real(dp) :: s1tot, s2tot

        type(eall_t) :: eallM(NTYPE1, 0:1)
        type(eall_t) :: eallF(NTYPE2, 0:1)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1)

        market = 0

        call setup( x, xt, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd )

        call checkTaxSupport( pF, pM, grid%gridval(MAX_N(TIME_HOURS)), .false. )

        call cpu_time(s1tot)
!$      s1tot = omp_get_wtime()

        m%theta = theta
        call new( tax, m, h, pM, pF, grid, rnd, opt, eallM, eallF, eall )

        if ( opt%mode == MODE_MOMENTS ) then
            call allocationToMoments( opt, ret, m, eallM, eallF, eall )
        end if

        call calculateWelfareAdjustment(m, .false.)
        ret%swf = aggregateWelfare( pM, pF, h, m, eallM, eallF, eall )
        ret%rev = aggregateTaxes( pM, pF, h, m, eallM, eallF, eall )

        call cpu_time(s2tot)
!$      s2tot = omp_get_wtime()
        ret%timer = s2tot - s1tot

        ! pareto weight vector
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                pX = pX1 + (pX2 - 1) * NTYPE1
                xlam(pX) = m%lam(pX1, pX2)
            end do
        end do

        ! marriage market state
        pX = 0
        do pX1 = 1, NTYPE1
            pX = pX + 1
            ret%marriage(pX) = m%numSingle1(pX1)
            ret%marriageMatrix(pX1, 0) = m%numSingle1(pX1)
        end do

        do pX2 = 1, NTYPE2
            pX = pX + 1
            ret%marriage(pX) = m%numSingle2(pX2)
            ret%marriageMatrix(0, pX2) = m%numSingle2(pX2)
        end do

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                pX = pX + 1
                ret%marriage(pX) = m%typeDemand(pX1, pX2)
                ret%marriageMatrix(pX1, pX2) = m%typeDemand(pX1, pX2)
            end do
        end do

        if (allocated(ret%EuLamGrid1)) deallocate(ret%EuLamGrid1)
        if (allocated(ret%EuLamGrid2)) deallocate(ret%EuLamGrid2)
        allocate(ret%EuLamGrid1(opt%nlam, NTYPE1, NTYPE2))
        allocate(ret%EuLamGrid2(opt%nlam, NTYPE1, NTYPE2))
        allocate(ret%lamgrid(opt%nlam))

        ret%Eu1 = m%Eu1
        ret%Eu2 = m%Eu2
        ret%EuLamGrid1 = m%EuLamGrid1
        ret%EuLamGrid2 = m%EuLamGrid2
        ret%lam = m%lam
        ret%lamgrid = m%lamgrid
        ret%demand = m%typeDemand
        ret%supply = m%typeSupply
        ret%demand = m%typeDemand
        ret%supply = m%typeSupply

    end subroutine swffun

    subroutine swffun_mpec(x, xlam, xtax, opt, ret, theta)
!$      use omp_lib
        use taxes
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(in) :: xlam(NTYPEALL)
        real(dp), intent(in) :: xtax(NTAXPARAM)
        type(opt_t), intent(in) :: opt
        type(ret_t), intent(out) :: ret
        real(dp), intent(in) :: theta

        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)

        ! tax schedules
        type(tax_t) :: tax

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)

        ! marriage market
        type(m_t) :: m

        ! indexing
        integer :: pX, pX1, pX2

        integer :: market

        real(dp) :: s1tot, s2tot

        type(eall_t)  :: eallM(NTYPE1, 0:1)
        type(eall_t)  :: eallF(NTYPE2, 0:1)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1)

        market = 0

        call setup( x, xt, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd )

        call checkTaxSupport( pF, pM, grid%gridval(MAX_N(TIME_HOURS)), verbose = .false. )
        call cpu_time(s1tot)
!$      s1tot = omp_get_wtime()

        m%theta = theta
        call new( tax, m, h, pM, pF, grid, rnd, opt, eallM, eallF, eall, lamfixed = xlam )

        if ( opt%mode == MODE_MOMENTS ) then
            call allocationToMoments( opt, ret, m, eallM, eallF, eall )
        end if

        call calculateWelfareAdjustment(m, .false.)
        ret%swf = aggregateWelfare( pM, pF, h, m, eallM, eallF, eall )
        ret%rev = aggregateTaxes( pM, pF, h, m, eallM, eallF, eall )

        call cpu_time(s2tot)
!$      s2tot = omp_get_wtime()
        ret%timer = s2tot - s1tot

        ! equilibrium constraint vector
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                pX = pX1 + (pX2 - 1) * NTYPE1
                ret%eqcon(pX) = m%excessDemand(pX1, pX2)
            end do
        end do

        ! marriage market state
        pX = 0
        do pX1 = 1, NTYPE1
            pX = pX + 1
            ret%marriage(pX) = m%numSingle1(pX1)
        end do

        do pX2 = 1, NTYPE2
            pX = pX + 1
            ret%marriage(pX) = m%numSingle2(pX2)
        end do

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                pX = pX + 1
                ret%marriage(pX) = m%typeDemand(pX1, pX2)
            end do
        end do

        allocate(ret%lamgrid(opt%nlam))

        ret%Eu1 = m%Eu1
        ret%Eu2 = m%Eu2
        ret%lam = m%lam
        ret%lamgrid = m%lamgrid
        ret%demand = m%typeDemand
        ret%supply = m%typeSupply

        if ( opt%saveMom ) then
            call writeWeight( opt%savePath // '/weight.csv', m%lam )
        end if

    end subroutine swffun_mpec

    subroutine swffun_weights(x, xlam, xtax, optIn, ret, theta, fname)
!$      use omp_lib
        use taxes
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(in) :: xlam(NTYPEALL)
        real(dp), intent(in) :: xtax(NTAXPARAM)
        type(opt_t), intent(in) :: optIn
        type(ret_t), intent(out) :: ret
        real(dp), intent(in) :: theta
        character(len = *), intent(in) :: fname
        type(opt_t) :: opt
        real(dp) :: swfGridD(NTAXBR - 1, NTAXBR - 1)
        real(dp) :: pmfGridD(NTAXBR - 1, NTAXBR - 1)
        integer :: ixT1, ixT2
        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)

        ! tax schedules
        type(tax_t) :: tax

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)

        ! marriage market
        type(m_t) :: m

        ! indexing
        integer :: pX, pX1, pX2, kX

        integer :: market

        type(eall_t) :: eallM(NTYPE1, 0:1)
        type(eall_t) :: eallF(NTYPE2, 0:1)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1)

        real(dp) :: swfWeight(NTAXBR - 1, NTAXBR - 1)
        real(dp) :: sumSwfWeight
        integer :: funit

        opt = optIn
        market = 0

        call setup( x, xt, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd )

        call checkTaxSupport( pF, pM, grid%gridval(MAX_N(TIME_HOURS)), verbose = .false. )

        m%theta = theta

        call new( tax, m, h, pM, pF, grid, rnd, opt, eallM, eallF, eall, lamfixed = xlam )
        call calculateWelfareAdjustment(m, .false.)
        ret%swf = aggregateWelfare( pM, pF, h, m, eallM, eallF, eall )
        ret%rev = aggregateTaxes( pM, pF, h, m, eallM, eallF, eall )
        print *, m%theta, ret%swf

        ! equilibrium constraint vector
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                pX = pX1 + (pX2 - 1) * NTYPE1
                ret%eqcon(pX) = m%excessDemand(pX1, pX2)
                print *, pX1, pX2, ret%eqcon(pX)
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    print *, 'measure', pM(pX1, kX)%frac * m%Pr1(pX1, pX2)
                end do
            end do
        end do

        opt%swfInc = .true.
        pmfGridD = 0.0_dp
        swfGridD = 0.0_dp
        do iXT2 = 1, NTAXBR - 1 ! highest income bin
            do ixT1 = 1, ixT2 !NTAXBR-1 ! no higer income bin
                opt%inc1 = ixT1
                opt%inc2 = ixT2

                call new( tax, m, h, pM, pF, grid, rnd, opt, eallM, eallF, eall, lamfixed = xlam )
                swfGridD(ixT1, ixT2) = aggregateWelfareInc( pM, pF, h, m, eallM, eallF, eall )
                pmfGridD(ixT1, ixT2) = aggregatePmfInc( pM, pF, h, m, eallM, eallF, eall )

                print *, swfGridD(ixT1, ixT2), ret%swf, pmfGridD(ixT1, ixT2)
            end do
        end do

        ! normalize
        pmfGridD = pmfGridD / sum(pmfGridD)
        swfWeight = (swfGridD - ret%swf) / WEIGHTINC2
        sumSwfWeight = 0.0_dp
        do iXT2 = 1, NTAXBR - 1 ! highest income bin
            do ixT1 = 1, ixT2
                sumSwfWeight = sumSwfWeight + swfWeight(ixT1, ixT2)
            end do
        end do
        swfWeight = swfWeight / pmfGridD / sumSwfWeight

        open(newunit = funit, file = fname, status = 'replace', action = 'write')
        do iXT2 = 1, NTAXBR - 1 ! highest income bin
            do ixT1 = 1, ixT2
                write(funit, '(I4,",",I4,",",F8.2,",",F8.2,",",F8.2,",",F8.2,",",F8.5,",",F8.5)') &
                      iXT1, ixT2, TAXBR(iXT1), TAXBR(iXT1 + 1), TAXBR(iXT2), TAXBR(iXT2 + 1), &
                      pmfGridD(ixT1, ixT2), swfWeight(ixT1, ixT2)
            end do
        end do
        close(funit)

    end subroutine swffun_weights



    subroutine swffun_mpec_fixed(x, xlam, xtax, xmarriage, opt, ret, theta)
!$      use omp_lib
        use taxes
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(in) :: xlam(NTYPEALL)
        real(dp), intent(in) :: xtax(NTAXPARAM)
        real(dp), intent(in) :: xmarriage(NTYPE1 + NTYPE2 + NTYPE1 * NTYPE2)
        type(opt_t), intent(in) :: opt
        type(ret_t), intent(out) :: ret
        real(dp), intent(in) :: theta
        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)

        type(tax_t) :: tax

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)

        ! marriage market
        type(m_t) :: m

        ! indexing
        integer :: pX, pX1, pX2

        integer :: market

        real(dp) :: s1tot, s2tot

        type(eall_t) :: eallM(NTYPE1, 0:1)
        type(eall_t) :: eallF(NTYPE2, 0:1)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1)

        market = 0

        call setup( x, xt, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd )

        call checkTaxSupport( pF, pM, grid%gridval(MAX_N(TIME_HOURS)), verbose = .false. )

        call cpu_time(s1tot)
!$      s1tot = omp_get_wtime()

        pX = 0
        do pX1 = 1, NTYPE1
            pX = pX + 1
            m%numSingle1(pX1) = xmarriage(pX)
        end do

        do pX2 = 1, NTYPE2
            pX = pX + 1
            m%numSingle2(pX2) = xmarriage(pX)
        end do

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                pX = pX + 1
                m%typeDemand(pX1, pX2) = xmarriage(pX)
            end do
        end do

        m%typeSupply = m%typeDemand

        m%PrSingle1 = m%numSingle1 / pM(:, 0)%frac
        m%PrSingle2 = m%numSingle2 / pF(:, 0)%frac
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                m%Pr1(pX1, pX2) = m%typeDemand(pX1, pX2) / pM(pX1, 0)%frac
            end do
        end do

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                m%Pr2(pX1, pX2) = m%typeSupply(pX1, pX2) / pF(pX2, 0)%frac
            end do
        end do

        m%theta = theta
        call new( tax, m, h, pM, pF, grid, rnd, opt, eallM, eallF, eall, lamfixed = xlam )

        if ( opt%mode == MODE_MOMENTS ) then
            call allocationToMoments( opt, ret, m, eallM, eallF, eall )
        end if

        call calculateWelfareAdjustment(m, .false.)
        ret%swf = aggregateWelfare( pM, pF, h, m, eallM, eallF, eall )
        ret%rev = aggregateTaxes( pM, pF, h, m, eallM, eallF, eall )

        call cpu_time(s2tot)
!$      s2tot = omp_get_wtime()
        ret%timer = s2tot - s1tot

        if ( opt%simMom ) then
            call allocationToMoments( opt, ret, m, eallM, eallF, eall )
        end if

        ! marriage market state
        ret%marriage = xmarriage

    end subroutine swffun_mpec_fixed

    subroutine objfun_mpec_market(x, xlam, xtax, opt, ret)
!$      use omp_lib
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(in) :: xlam(NTYPEALL * NMARKET)
        real(dp), intent(in) :: xtax(NTAXPARAM)
        type(opt_t), intent(in) :: opt
        type(ret_t), intent(out) :: ret

        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)

        real(dp) :: lam(NTYPE1, NTYPE2, NMARKET)

        ! indexing
        integer :: pX, pX1, pX2, mX

        ! tax schedules
        type(tax_t) :: tax(NMARKET)

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1, NMARKET)
        type(p_t) :: pF(NTYPE2, 0:1, NMARKET)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1, NMARKET)

        ! marriage market
        type(m_t) :: m(NMARKET)

        ! dummy arguments
        real(dp) :: s1tot, s2tot
        type(eall_t) :: eallM(NTYPE1, 0:1, NMARKET)
        type(eall_t) :: eallF(NTYPE2, 0:1, NMARKET)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1, NMARKET)

        ! pareto weights
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX1 + (pX2 - 1) * NTYPE1 + (mX - 1) * NTYPE1 * NTYPE2
                    lam(pX1, pX2, mX) = xlam(pX)
                end do
            end do
        end do

        do mX = 1, NMARKET
            call setup( x, xt, xtype, xtax, opt, mX, tax(mX), m(mX), h(:, :, :, mX), &
                pM(:, :, mX), pF(:, :, mX), grid, rnd )
        end do

        call cpu_time(s1tot)
!$      s1tot = omp_get_wtime()

        do mX = 1, NMARKET
            call new( tax(mX), m(mX), h(:, :, :, mX), pM(:, :, mX), pF(:, :, mX), grid, rnd, opt, &
                eallM(:, :, mX), eallF(:, :, mX), eall(:, :, :, mX), lamfixed = lam(:, :, mX) )
        end do

        call allocationToMomentsMarket( opt, ret, m, eallM, eallF, eall )
        ret%rev = aggregateTaxesMarket( pM, pF, h, m, eallM, eallF, eall )

        call cpu_time(s2tot)
!$      s2tot = omp_get_wtime()
        ret%timer = s2tot - s1tot

        ! equilibrium constraint vector
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX1 + (pX2 - 1) * NTYPE1 + (mX - 1) * NTYPE1 * NTYPE2
                    ret%eqconMarket(pX) = m(mX)%excessDemand(pX1, pX2)
                end do
            end do
        end do

        if ( opt%saveMom ) then
            call writeWeightMarket( opt%savePath // '/weight.csv', m)
            call writeMarriageMarket( opt%savePath // '/marriage.csv', m, ret )
            call writeTag( opt%savePath // '/tag.asc', opt%tag )
        end if

    end subroutine objfun_mpec_market

    subroutine objfun_market(x, xlam, xtax, opt, ret)
!$      use omp_lib
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(out) :: xlam(NTYPEALL * NMARKET)
        real(dp), intent(in) :: xtax(NTAXPARAM)
        type(opt_t), intent(in) :: opt
        type(ret_t), intent(out) :: ret

        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)

        ! indexing
        integer :: pX, pX1, pX2, mX

        ! tax schedules
        type(tax_t) :: tax(NMARKET)

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1, NMARKET)
        type(p_t) :: pF(NTYPE2, 0:1, NMARKET)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1, NMARKET)

        ! marriage market
        type(m_t) :: m(NMARKET)

        ! dummy arguments
        real(dp) :: s1tot, s2tot
        type(eall_t) :: eallM(NTYPE1, 0:1, NMARKET)
        type(eall_t) :: eallF(NTYPE2, 0:1, NMARKET)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1, NMARKET)

        do mX = 1, NMARKET
            call setup( x, xt, xtype, xtax, opt, mX, tax(mX), m(mX), h(:, :, :, mX), &
                pM(:, :, mX), pF(:, :, mX), grid, rnd )
        end do

        m%theta= 0.0_dp
        m%marketSize = m%marketSize / sum( m%marketSize )

        call cpu_time(s1tot)
!$      s1tot = omp_get_wtime()

        do mX = 1, NMARKET
            call new( tax(mX), m(mX), h(:, :,  :, mX), pM(:, :, mX), pF(:, :, mX), grid, rnd, opt, &
                      eallM(:, :, mX), eallF(:, :, mX), eall(:, :, :,mX) )
        end do


        call allocationToMomentsMarket( opt, ret, m, eallM, eallF, eall )

        call cpu_time(s2tot)
!$      s2tot = omp_get_wtime()
        ret%timer = s2tot - s1tot

        ! equilibrium constraint vector (zero)
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX1 + (pX2 - 1) * NTYPE1 + (mX - 1) * NTYPE1 * NTYPE2
                    ret%eqconMarket(pX) = m(mX)%excessDemand(pX1, pX2)
                end do
            end do
        end do

        ! pareto weight vector
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX1 + (pX2 - 1) * NTYPE1 + (mX - 1) * NTYPE1 * NTYPE2
                    xlam(pX) = m(mX)%lam(pX1, pX2)
                end do
            end do
        end do

        if ( opt%saveMom ) then
            call writeMarriageMarket( opt%savePath // '/marriage.csv', m, ret )
            call writeWeightMarket( opt%savePath // '/weight.asc', m )
            call writeTag( opt%savePath // '/tag.asc', opt%tag )
        end if

    end subroutine objfun_market

    subroutine printxlab(x, funit)
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        integer, intent(in), optional :: funit

        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)
        real(dp) :: xtax(NTAXPARAM)
        type(opt_t) :: opt
        integer :: market
        type(tax_t) :: tax
        type(grid_t) :: grid
        type(rnd_t) :: rnd
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t) :: m
        integer :: estmode
        estmode = 0
        call setOptions(opt, OPT_ESTIMATION, estmode)
        opt%verbose = .true.
        opt%verboseExtra = .true.
        market = -1

        if (present(funit)) then
            opt%vunit = funit
        end if

        call setup( x, xt, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd )

    end subroutine printxlab

    subroutine getxtype(xtype)
        implicit none
        integer, intent(out) :: xtype(NPARAM)
        real(dp) :: x(NPARAM)
        real(dp) :: xt(NPARAM)
        real(dp) :: xtax(NTAXPARAM)
        type(opt_t) :: opt
        integer :: market, estmode
        type(tax_t) :: tax
        type(grid_t) :: grid
        type(rnd_t) :: rnd
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t) :: m
        x = 0.5_dp
        market = -1
        estmode = 0
        call setOptions(opt, OPT_ESTIMATION, estmode)
        call setup( x, xt, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd )
    end subroutine getxtype


    subroutine printHomeProductivity(x)
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)
        real(dp) :: xtax(NTAXPARAM)
        type(opt_t) :: opt
        integer :: market, estmode
        type(tax_t) :: tax
        type(grid_t) :: grid
        type(rnd_t) :: rnd
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t) :: m
        integer :: pX1, pX2

        estmode = 0
        market = 1
        call setOptions(opt, OPT_ESTIMATION, estmode)

        call setup(x, xt, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd)

        print *
        print *, 'MALE SINGLE PRODUCTIVITY (KIDS/NOKIDS)'
        print *

        do pX1 = 1, NTYPE1
            print *, pX1, pM(pX1, 1)%homeSlope, pM(pX1, 0)%homeSlope
        end do

        print *
        print *, 'FEMALE SINGLE PRODUCTIVITY (KIDS/NOKIDS)'
        print *

        do pX2 = 1, NTYPE2
            print *, pX2, pF(pX2, 1)%homeSlope, pF(pX2, 0)%homeSlope
        end do

        print *
        print *, 'HOUSEHOLD PRODUCTIVITY (KIDS/NOKIDS)'
        print *

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                print *, pX1, pX2, h(pX1, pX2, 1)%A, h(pX1, pX2, 0)%A
            end do
        end do

        print *

    end subroutine printHomeProductivity

    subroutine printxfort(x)
        use mathfn
        implicit none
        real(dp), intent(in) :: x(:)
        character(len = *), parameter :: rfmt = '(G25.16)'
        integer :: iX
        integer :: nX
        nX = size(x)
        write(IOUT, '(A)')
        write(IOUT, '(A)') '    x = (/ ' // realToStr(x(1), rfmt) // '_dp, &'
        do iX = 2, nX-1
            write(IOUT, '(A)') '    ' // realToStr(x(iX),rfmt) // '_dp, &'
        end do
        write(IOUT, '(A)') '    ' // realToStr(x(nX),rfmt) // '_dp /)'
        write(IOUT, '(A)')
    end subroutine printxfort

    subroutine setup( x, xt, xtype, xtax, opt, market, tax, m, h, pM, pF, grid, rnd )
        use environment
        use mathfn, only : intToStr
        use taxes
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(out) :: xt(NPARAM)
        integer, intent(out) :: xtype(NPARAM)
        real(dp), intent(in) :: xtax(NTAXPARAM)

        ! options
        type(opt_t), intent(in) :: opt

        ! market identifier
        integer, intent(in) :: market

        ! tax schedules
        type(tax_t), intent(out) :: tax

        ! time allocation grid
        type(grid_t), intent(out) :: grid

        ! state specific errors
        type(rnd_t), intent(out) :: rnd

        ! person type attibutes
        type(p_t), intent(out) :: pM(NTYPE1, 0:1)
        type(p_t), intent(out) :: pF(NTYPE2, 0:1)

        ! household type attibutes
        type(h_t), intent(out) :: h(NTYPE1, NTYPE2, 0:1)

        ! marriage market
        type(m_t), intent(out) :: m

        ! integer :: gridind(n)
        type(xi_t) :: xi
        integer :: iX
        integer, parameter :: seed(1) = (/ 1452780529 /)

        real(dp) :: evsd

        ! tax parameters
        character(len = *), parameter :: taxdir = 'taxes/output/'

        ! indexing
        integer :: pX1, pX2, kX

        real(dp) :: nu
        character(len=32) :: typeStr

        real(dp) :: msigl0, fsigl0, msigl1, fsigl1, sigl
        real(dp) :: sigQ, betQ, betc, sigc
        real(dp) :: mfc1, psi

        real(dp) :: cesMcoef

        real(dp) :: betl

        real(dp) :: tempf(NTYPE2), tempsame(NTYPE1)
        logical, parameter :: commonpref = .true.
!         real(dp) :: Ascale, AscaleKids

        ! read distribution of types
        if (market >= 0) then

            if ( market > 0 ) then
                call readFracKids( 'input/type_' // opt%locname // '_market' // intToStr(market) // '.asc', opt%verboseExtra, pM, pF, h, m, opt%pMscale, opt%pFscale )
            else
                call readFracKids( 'input/type_' // opt%locname // '.asc', opt%verboseExtra, pM, pF, h, m, opt%pMscale, opt%pFscale )
                m%taxId = 0
            end if

            if ( opt%tmode == 0 ) then
                call readTaxes( 'taxes/output', m%taxId, opt%verboseExtra, tax )
                ! note: food stamps and welfare are integrated in the tax schedule
                if (opt%medicaid) then
                    call readMedicaid( 'taxes/output', 'input/medicaid.asc', m%taxId, opt%verboseExtra, tax )
                    tax%medicaid = .true.
                else
                    tax%medicaid = .false.
                end if
                ! call readFoodStamps( 'taxes/output/fsp.dat', opt%verboseExtra, tax )
                ! call readWelfare( 'taxes/output', m%taxId, opt%verboseExtra, tax )

            else
                call setTax( xtax, tax, TAXMODE, opt%taxins )
            end if

        end if

        call xi%init(x)

        select case(opt%xmode)
        case(X_TRANSFORM)
            xi%trans = .true.
        case(X_NOTRANSFORM)
            xi%trans = .false.
        case default
            stop 'opt%xmode not recognized'
        end select

        evsd = xi%transform( TRANSFORM_EXP, 'MF: sd state-specific errors', 'State specific error, s.d.', &
            XTYPE_RND )

        pM%sd = evsd
        pF%sd = evsd

        ! male wages: do not depend on kids

        do pX1 = 1, NTYPE1

            typeStr = 'M type ' // intToStr(pX1)

            pM(pX1, 0)%wage_mu = xi%transform( TRANSFORM_EXP, trim(typeStr) // ': wage_mu', &
                                               'Male, ' // trim(TYPENAME1(pX1)) // ': mean', XTYPE_WAGEM ) ! mean log wages
            pM(pX1, 0)%wage_sd = xi%transform( TRANSFORM_EXP, trim(typeStr) // ': wage_sd', &
                                               'Male, ' // trim(TYPENAME1(pX1)) // ': s.d.', XTYPE_WAGEM ) ! sd log wages

            pM(pX1, 1)%wage_mu = pM(pX1, 0)%wage_mu
            pM(pX1, 1)%wage_sd = pM(pX1, 0)%wage_sd

        end do

        ! female wages: do not depend on kids

        do pX2 = 1, NTYPE2

            typeStr = 'F type ' // intToStr(pX2)

            pF(pX2, 0)%wage_mu = xi%transform( TRANSFORM_EXP, trim(typeStr) // ': wage_mu', &
                                               'Female, ' // trim(TYPENAME2(pX2)) // ': mean', XTYPE_WAGEF ) ! mean log wages
            pF(pX2, 0)%wage_sd = xi%transform( TRANSFORM_EXP, trim(typeStr) // ': wage_sd', &
                                               'Female, ' // trim(TYPENAME2(pX2)) // ': s.d.', XTYPE_WAGEF ) ! sd log wages

            pF(pX2, 1)%wage_mu = pF(pX2, 0)%wage_mu
            pF(pX2, 1)%wage_sd = pF(pX2, 0)%wage_sd

        end do

        pF%wage_mu = pF%wage_mu + opt%logwf_mu
        pF%wage_sd = pF%wage_sd + opt%logwf_sd

        mfc1 = xi%transform( TRANSFORM_EXP, 'MF: fc', 'Fixed costs (kids)', XTYPE_FC )

        nu = 0.0_dp
        pM%comp = 1!fc2earn
        pF%comp = 1!fc2earn

        sigl = xi%transform( TRANSFORM_EXP, 'MF: sigl', 'Leisure curvature, $\sigma_{\ell}$', XTYPE_LEISURE )
        msigl0 = sigl
        msigl1 = sigl
        fsigl0 = sigl
        fsigl1 = sigl

        betc = 1.0_dp

        sigc = 1.0_dp

        ! public good preference (cobb douglas aggregation)
        sigQ = xi%transform( TRANSFORM_LINEAR, 'MF: sigQ', 'Home good curvature, $\sigma_{Q}$', XTYPE_HOMEPREF )
        pM%sigQ = sigQ
        pF%sigQ = sigQ

        ! male coefficient in CES production function for public good
        cesMcoef = xi%transform( TRANSFORM_LOGIT, 'HH: cesMcoef', 'Male production share', XTYPE_HOMEPROD )

        ! distribution of marital shocks
        m%muTheta0 = 0.0_dp
        m%sdTheta0 = xi%transform( TRANSFORM_INVEXP, 'HH: match shock sd', 'Marital shock, s.d.', XTYPE_MARRND )

        pM(:, 0)%fc = 0
        pF(:, 0)%fc = 0
        pM(:, 1)%fc = mfc1
        pF(:, 1)%fc = pM(:, 1)%fc

        pM%fckid = 0
        pF%fckid = 0
        h%fckid1 = 0
        h%fckid2 = 0

        if (commonpref) then

            betl = xi%transform( TRANSFORM_EXP, 'MF: betl', 'Leisure scale', XTYPE_LEISURE )
            pM%betl = betl
            pF%betl = betl

            betQ = xi%transform( TRANSFORM_EXP, 'MF: betQ', 'Home good scale', XTYPE_HOMEPREF )
            pM%betQ = betQ
            pF%betQ = betQ

        else

            betl = xi%transform( TRANSFORM_EXP, 'M: betl', 'Male leisure scale', XTYPE_LEISURE )
            pM%betl = betl

            betl = xi%transform( TRANSFORM_EXP,'F: betl', 'Female leisure scale', XTYPE_LEISURE )
            pF%betl = betl

            betQ = xi%transform( TRANSFORM_EXP,'M: betQ', 'Male home good scale', XTYPE_HOMEPREF )
            pM%betQ = betQ

            betQ = xi%transform( TRANSFORM_EXP,'F: betQ', 'Female home good scale', XTYPE_HOMEPREF )
            pF%betQ = betQ

        end if

        pM%homeSlope = 1.0_dp
        pF%homeSlope = 1.0_dp

        do pX1 = 1, NTYPE1
            pM(pX1 ,0)%homeSlope = xi%transform( TRANSFORM_EXP, 'MF: A prod(' // intToStr(pX1) // ') no kids', &
                                                 'Single productivity (no children), ' // trim(TYPENAME1(pX1)), XTYPE_HOMEPROD )
        end do

        pM(:, 1)%homeSlope = xi%transform( TRANSFORM_EXP, 'M: A prod(*) kids', 'Male productivity (children)', XTYPE_HOMEPROD )

        pF(:, 0)%homeSlope = pM(:, 0)%homeSlope

        do pX2 = 1, NTYPE2
            pF(pX2, 1)%homeSlope = xi%transform( TRANSFORM_EXP, 'F: A prod(' // intToStr(pX2) // ') kids', &
                                                 'Female productivity (children), ' // trim(TYPENAME2(pX2)), XTYPE_HOMEPROD )
        end do
        h(:, :,  0)%A = 1.0_dp
        do pX2 = 1, NTYPE2
            tempf(pX2) = xi%transform( TRANSFORM_EXP, 'MF: HH prod(' // intToStr(pX2) // ') kids', &
                                       'HH productivity (children) female, ' // trim(TYPENAME2(pX2)), XTYPE_HOMEPROD )
        end do

        do pX1 = 1, NTYPE1
            tempsame(pX1) = xi%transform( TRANSFORM_EXP, 'MF: HH prod same-ed ' // intToStr(pX1), &
                                          'HH productivity (children) educational homogamy, ' // trim(TYPENAME1(pX1)), XTYPE_HOMEPROD )
        end do

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                h(pX1, pX2, 1)%A = tempf(pX2)
                if (pX1 == pX2) then
                    h(pX1, pX2, 1)%A = h(pX1, pX2, 1)%A * tempsame(pX1)
                end if
            end do
        end do

        h%A = h%A * opt%scaleAh

        pF%homeSlope = pF%homeSlope * opt%scaleA
        pM%homeSlope = pM%homeSlope * opt%scaleA

        pM(:, 0)%sigl = msigl0
        pM(:, 1)%sigl = msigl1

        pF(:, 0)%sigl = fsigl0
        pF(:, 1)%sigl = fsigl1

        pM(:, :)%betc = betc
        pM(:, :)%sigc = sigc
        pF(:, :)%betc = betc
        pF(:, :)%sigc = sigc

        typeStr = 'couple'

        nu = 0
        psi = 0

        pM%betcl = 0!betclM
        pF%betcl = 0!betclF


        call xi%final()

        ! output transformed parameter vector
        xt = xi%y
        xtype = xi%xtype

        if ( opt%verboseExtra ) then
            do iX = 1, xi%count
                write(opt%vunit, '(I4,",",2X,F32.8,",",2X,(A),",",2X,(A))') iX, xi%x(iX), '"' // trim(adjustl(xi%xlab(iX))) // '"', &
                                                                            '"' // trim(adjustl(xi%xlablong(iX))) // '"'
            end do
        end if

        if ( market < 0 ) then
            return
        end if

        pM%wlf = 0.0_dp
        pF%wlf = 0.0_dp
        pM%psi = psi!junk !1!psi
        pF%psi = psi!junk !1!psi

        ! calulated derived objects from pM
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                pM(pX1, kX)%taxmode = opt%tmode
                call p_derived( pM(pX1, kX) )
                call p_wageGrid( pM(pX1, kX) )
            end do
        end do

        ! calulated derived objects from pF
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                pF(pX2, kX)%taxmode = opt%tmode
                call p_derived( pF(pX2, kX) )
                call p_wageGrid( pF(pX2, kX) )
            end do
        end do

        ! populate h
        if (opt%correlation < 0.0_dp) then
#           ifdef _TYPE3_
            h(1, 1, :)%rho = .29_dp
            h(1, 2, :)%rho = .23_dp
            h(1, 3, :)%rho = .26_dp
            h(2, 1, :)%rho = .20_dp
            h(2, 2, :)%rho = .23_dp
            h(2, 3, :)%rho = .19_dp
            h(3, 1, :)%rho = .15_dp
            h(3, 2, :)%rho = .15_dp
            h(3, 3, :)%rho = .25_dp
#           else
            stop 'not supported'
#           endif
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1

                    h(pX1, pX2, kX)%taxmode = opt%tmode
                    h(pX1, pX2, kX)%nu = nu

                    h(pX1, pX2, kX)%match = 0.0_dp
                    if (opt%correlation >= 0.0_dp) then
                        h(pX1, pX2, kX)%rho = opt%correlation
                    end if

                    if (kX == 0) then
                        h(pX1, pX2, kX)%coefM = cesMcoef
                    else
                        h(pX1, pX2, kX)%coefM = cesMcoef !Kids
                    end if

                    h(pX1, pX2, kX)%coefF = 1.0_dp - h(pX1, pX2, kX)%coefM
                    call h_jointWageDensity( h(pX1, pX2, kX), pM(pX1, kX), pF(pX2, kX), .false. )

                end do
            end do
        end do

        ! generate time allocation grids (used by all types and matches)
        call setgrid(opt, grid)

        ! generate state specific errors
        call generateStateErrors(rnd)

        m%theta = 0.0_dp
        m%marketId = market

    end subroutine setup

    subroutine moment(grid, eall)
        implicit none
        type(grid_t), intent(in) :: grid
        type(eall_t), intent(inout) :: eall

        integer :: iX, hX

        real(dp) :: hrvec(n)
        real(dp) :: fracHr(0:2), fracHrW(0:2), fracHrW2(0:2), totalHomeHr(0:2), totalHomeHrSq(0:2)
        real(dp) :: fracHome(NHH)
        real(dp) :: totalHourWork, totalHourWorkSq
        real(dp) :: totalHomeWork(0:1), totalHomeWorkSq(0:1), totalWork(0:1)
        real(dp) :: totalHome, totalHomeSq
        real(dp) :: totalHour, totalHourSq

        real(dp) :: totalWageWork, totalWageSqWork

        real(dp) :: meanW(0:2), meanW2(0:2), sdW(0:2)
        real(dp) :: sdHome
        real(dp) :: meanHomeHr(0:2), meanHomeHrSq(0:2), sdHomeHr(0:2)
        real(dp) :: meanHoursWork, meanHoursWorkSq, sdHoursWork
        real(dp) :: meanHomeWork(0:1), meanHomeWorkSq(0:1), sdHomeWork(0:1)
        real(dp) :: meanWageWork, meanWageSqWork, sdWageWork

        real(dp) :: sdHours

        real(dp) :: meanEarnWork, sdEarnWork
        real(dp) :: totalEarnWork, totalEarnSqWork, sdEarn
        real(dp) :: meanEarnWorkSq

        ! labour supply in aggregated categories
        fracHr = 0.0_dp
        fracHrW = 0.0_dp
        fracHrW2 = 0.0_dp
        totalHomeHr = 0.0_dp
        totalHomeHrSq = 0.0_dp
        fracHome = 0.0_dp
        totalHourWork = 0.0_dp
        totalHourWorkSq = 0.0_dp
        totalHomeWork = 0.0_dp
        totalHomeWorkSq = 0.0_dp
        totalWork = 0.0_dp
        totalHome = 0.0_dp
        totalHomeSq = 0.0_dp
        totalHour = 0.0_dp
        totalHourSq = 0.0_dp
        totalWageWork = 0.0_dp
        totalWageSqWork = 0.0_dp
        totalEarnWork = 0.0_dp
        totalEarnSqWork = 0.0_dp

        do iX = 1, ndim

            do hX = 0, 2
                if ( grid%gzInd(iX, TIME_HOURS) >= BAND_WORKL(hX) .and. grid%gzInd(iX, TIME_HOURS) <= BAND_WORKU(hX) ) then
                    fracHr(hX) = fracHr(hX) + eall%Eallocation(iX)
                    fracHrW(hX) = fracHrW(hX) + eall%EwageAllocation(iX)
                    fracHrW2(hX) = fracHrW2(hX) + eall%EwageSqAllocation(iX)
                    totalHomeHr(hX) = totalHomeHr(hX) + eall%Eallocation(iX) * grid%gz(iX, TIME_PUBLIC)
                    totalHomeHrSq(hX) = totalHomeHrSq(hX) + eall%Eallocation(iX) * grid%gzSq(iX, TIME_PUBLIC)
                end if
            end do

            do hX = 1, NHH
                if ( grid%gzInd(iX, TIME_PUBLIC) >= BAND_HOMEL(hX) .and. grid%gzInd(iX, TIME_PUBLIC) <= BAND_HOMEU(hX) ) then
                    fracHome(hX) = fracHome(hX) + eall%Eallocation(iX)
                end if
            end do

            ! conditional on employment status
            if ( grid%gzInd(iX, TIME_HOURS) > 1 ) then
                totalHourWork = totalHourWork + eall%Eallocation(iX) * grid%gz(iX, TIME_HOURS)
                totalHourWorkSq = totalHourWorkSq + eall%Eallocation(iX) * grid%gzSq(iX, TIME_HOURS)
                totalHomeWork(1) = totalHomeWork(1) + eall%Eallocation(iX) * grid%gz(iX, TIME_PUBLIC)
                totalHomeWorkSq(1) = totalHomeWorkSq(1) + eall%Eallocation(iX) * grid%gzSq(iX, TIME_PUBLIC)
                totalWork(1) = totalWork(1) + eall%Eallocation(iX)
                totalWageWork = totalWageWork + eall%EwageAllocation(iX)
                totalWageSqWork = totalWageSqWork + eall%EwageSqAllocation(iX)
                totalEarnWork = totalEarnWork + eall%EwageLevelAllocation(iX) * grid%gz(iX, TIME_HOURS)
                totalEarnSqWork = totalEarnSqWork + eall%EwageLevelSqAllocation(iX) * grid%gzSq(iX, TIME_HOURS)
            else
                totalHomeWork(0) = totalHomeWork(0) + eall%Eallocation(iX) * grid%gz(iX, TIME_PUBLIC)
                totalHomeWorkSq(0) = totalHomeWorkSq(0) + eall%Eallocation(iX) * grid%gzSq(iX, TIME_PUBLIC)
                totalWork(0) = totalWork(0) + eall%Eallocation(iX)
            end if

            ! unconditional home time
            totalHome = totalHome + eall%Eallocation(iX) * grid%gz(iX, TIME_PUBLIC)
            totalHomeSq = totalHomeSq + eall%Eallocation(iX) * grid%gzSq(iX, TIME_PUBLIC)

            ! unconditional hours time
            totalHour = totalHour + eall%Eallocation(iX) * grid%gz(iX, TIME_HOURS)
            totalHourSq = totalHourSq + eall%Eallocation(iX) * grid%gzSq(iX, TIME_HOURS)

        end do

        ! distribution of wages in aggregated hours cell
        meanW  = fracHrW / fracHr
        meanW2 = fracHrW2 / fracHr
        sdW = sqrt( max( meanW2 - meanW ** 2, 0.0_dp ) )

        meanWageWork = totalWageWork / totalWork(1)
        meanWageSqWork = totalWageSqWork / totalWork(1)
        sdWageWork = sqrt( max( meanWageSqWork - meanWageWork ** 2, 0.0_dp ) )

        sdHome = sqrt( max( totalHomeSq - totalHome ** 2, 0.0_dp ) )

        meanHomeHr = totalHomeHr / fracHr
        meanHomeHrSq = totalHomeHrSq / fracHr
        sdHomeHr = sqrt( max( meanHomeHrSq - meanHomeHr ** 2, 0.0_dp ) )

        ! conditional work hours
        meanHoursWork = totalHourWork / totalWork(1)
        meanHoursWorkSq = totalHourWorkSq / totalWork(1)
        sdHoursWork = sqrt( max( meanHoursWorkSq - meanHoursWork ** 2, 0.0_dp ) )

        ! unconditional work hours
        sdHours = sqrt( max( totalHourWorkSq - totalHourWork ** 2, 0.0_dp ) )

        ! conditional earnings hours
        meanEarnWork = totalEarnWork / totalWork(1)
        meanEarnWorkSq = totalEarnSqWork / totalWork(1)
        sdEarnWork = sqrt( max( meanEarnWorkSq - meanEarnWork ** 2, 0.0_dp ) )

        ! unconditional earning hours
        sdEarn = sqrt( max( totalEarnSqWork - totalEarnWork ** 2, 0.0_dp ) )


        meanHomeWork = totalHomeWork / totalWork
        meanHomeWorkSq = totalHomeWorkSq / totalWork
        sdHomeWork = sqrt( max( meanHomeWorkSq - meanHomeWork ** 2, 0.0_dp ) )

        ! copy to eall

        eall%hours = fracHr
        eall%hourshW = meanHoursWork ! conditional work hours (mean)
        eall%hourshWsd = sdHoursWork ! conditional work hours (s.d.)
        eall%hoursh = totalHourWork  ! unconditional work hours (mean)
        eall%hourshsd = sdHours      ! unconditional work hours (s.d.)
        eall%earnW = meanEarnWork ! conditional earnings (mean)
        eall%earnWsd = sdEarnWork ! conditional earnings (s.d.)
        eall%earn = totalEarnWork  ! unconditional earnings (mean)
        eall%earnsd = sdEarn      ! unconditional earnings (s.d.)

        eall%wage = meanW
        eall%wagesd = sdW

        eall%wageWork = meanWageWork
        eall%wagesdWork = sdWageWork

        ! proportion working
        eall%work = totalWork(1)

        eall%home = fracHome
        eall%homeh = totalHome
        eall%homehsd = sdHome

        ! home production hours by own employment status
        eall%homehW = meanHomeWork
        eall%homehWsd = sdHomeWork

        ! home production hours by own hours
        eall%homehH = meanHomeHr
        eall%homehHsd = sdHomeHr

        hrvec = 0.0_dp
        do iX = 1, ndim
            do hX = 1, n
                if ( grid%gzInd(iX, TIME_HOURS) == hX)  then
                    hrvec(hX) = hrvec(hX) + eall%Eallocation(iX)
                end if
            end do
        end do

    end subroutine moment

    subroutine momentHousehold(grid, eall)
        implicit none
        type(grid_t), intent(in) :: grid
        type(eall2_t), intent(inout) :: eall

        integer :: aX1, aX2
        integer :: hX1, hX2

        real(dp) :: fracHr(0:2, 0:2)
        real(dp) :: fracHr1(0:2)
        real(dp) :: fracHr2(0:2)
        real(dp) :: fracHome1(NHH)
        real(dp) :: fracHome2(NHH)

        real(dp) :: wage1Hr1(0:2)
        real(dp) :: wage2Hr2(0:2)
        real(dp) :: wageSq1Hr1(0:2)
        real(dp) :: wageSq2Hr2(0:2)

        real(dp) :: fracWork1(0:1)
        real(dp) :: fracWork2(0:1)

        real(dp) :: wage1Work1
        real(dp) :: wageSq1Work1
        real(dp) :: wage2Work2
        real(dp) :: wageSq2Work2

        real(dp) :: wage1Work2(0:1)
        real(dp) :: wageSq1Work2(0:1)
        real(dp) :: wage2Work1(0:1)
        real(dp) :: wageSq2Work1(0:1)

        real(dp) :: home1Hr1(0:2), homeSq1Hr1(0:2)
        real(dp) :: home1Hr2(0:2), homeSq1Hr2(0:2)
        real(dp) :: home2Hr2(0:2), homeSq2Hr2(0:2)
        real(dp) :: home2Hr1(0:2), homeSq2Hr1(0:2)

        real(dp) :: home1Work1(0:1)
        real(dp) :: homeSq1Work1(0:1)
        real(dp) :: home2Work2(0:1)
        real(dp) :: homeSq2Work2(0:1)

        real(dp) :: home1Work2(0:1)
        real(dp) :: homeSq1Work2(0:1)
        real(dp) :: home2Work1(0:1)
        real(dp) :: homeSq2Work1(0:1)

        real(dp) :: work1Work2(0:1)
        real(dp) :: work2Work1(0:1)

        real(dp) :: frac12
        real(dp) :: wage12
        real(dp) :: wageSq12
        real(dp) :: wage1Work12
        real(dp) :: wage2Work12

        real(dp) :: home1
        real(dp) :: homeSq1
        real(dp) :: home2
        real(dp) :: homeSq2


        ! male home hours
        real(dp) :: meanHome1, meanHomeSq1, sdHome1
        real(dp) :: meanHome1Hr1(0:2), meanHomeSq1Hr1(0:2), sdHome1Hr1(0:2)
        real(dp) :: meanHome1Work1(0:1), meanHomeSq1Work1(0:1), sdHome1Work1(0:1)
        real(dp) :: meanHome1Hr2(0:2), meanHomeSq1Hr2(0:2), sdHome1Hr2(0:2)
        real(dp) :: meanHome1Work2(0:1), meanHomeSq1Work2(0:1), sdHome1Work2(0:1)

        ! female home hours
        real(dp) :: meanHome2, meanHomeSq2, sdHome2
        real(dp) :: meanHome2Hr2(0:2), meanHomeSq2Hr2(0:2), sdHome2Hr2(0:2)
        real(dp) :: meanHome2Work2(0:1), meanHomeSq2Work2(0:1), sdHome2Work2(0:1)
        real(dp) :: meanHome2Hr1(0:2), meanHomeSq2Hr1(0:2), sdHome2Hr1(0:2)
        real(dp) :: meanHome2Work1(0:1), meanHomeSq2Work1(0:1), sdHome2Work1(0:1)

        real(dp) :: meanWage1Hr1(0:2), meanWageSq1Hr1(0:2), sdWage1Hr1(0:2)
        real(dp) :: meanWage1Work2(0:1), meanWageSq1Work2(0:1), sdWage1Work2(0:1)

        real(dp) :: meanWage2Hr2(0:2), meanWageSq2Hr2(0:2), sdWage2Hr2(0:2)
        real(dp) :: meanWage2Work1(0:1), meanWageSq2Work1(0:1), sdWage2Work1(0:1)

        real(dp) :: meanWage1Work12, meanWage2Work12, meanWage12, meanWageSq12, sdWage12, covWage12

        real(dp) :: hours1Work1, hoursSq1Work1
        real(dp) :: hours2Work2, hoursSq2Work2
        real(dp) :: meanHours1Work1, meanHoursSq1Work1, sdHours1Work1
        real(dp) :: meanHours2Work2, meanHoursSq2Work2, sdHours2Work2

        real(dp) :: meanWage1Work1, meanWageSq1Work1, sdWage1Work1
        real(dp) :: meanWage2Work2, meanWageSq2Work2, sdWage2Work2

        real(dp) :: sdHours1, sdHours2

        real(dp) :: earn1Work1, earn2Work2
        real(dp) :: meanEarn1Work1, meanEarn2Work2
        real(dp) :: meanEarnSq1Work1, meanEarnSq2Work2
        real(dp) :: earnSq1Work1, earnSq2Work2
        real(dp) :: sdEarn1Work1, sdEarn2Work2, sdEarn1, sdEarn2

        ! real(dp) :: meanWage1Work12, meanWage1Work12
        real(dp) :: meanWageSq1Work12, meanWageSq2Work12
        real(dp) :: wageSq1Work12, wageSq2Work12
        real(dp) :: sdWage1Work12, sdWage2Work12

        meanWage1Work1 = 0.0_dp
        meanWageSq1Work1 = 0.0_dp
        meanWage2Work2 = 0.0_dp
        meanWageSq2Work2 = 0.0_dp

        fracHr = 0.0_dp
        fracHr1 = 0.0_dp
        fracHr2 = 0.0_dp
        fracHome1 = 0.0_dp
        fracHome2 = 0.0_dp

        wage1Hr1 = 0.0_dp
        wage2Hr2 = 0.0_dp
        wageSq1Hr1 = 0.0_dp
        wageSq2Hr2 = 0.0_dp

        fracWork1 = 0.0_dp
        fracWork2 = 0.0_dp

        hours1Work1 = 0.0_dp
        hoursSq1Work1 = 0.0_dp
        hours2Work2 = 0.0_dp
        hoursSq2Work2 = 0.0_dp

        wage1Work1 = 0.0_dp
        wageSq1Work1 = 0.0_dp
        wage2Work2 = 0.0_dp
        wageSq2Work2 = 0.0_dp

        wage1Work2 = 0.0_dp
        wageSq1Work2 = 0.0_dp
        wage2Work1 = 0.0_dp
        wageSq2Work1 = 0.0_dp

        home1Hr1 = 0.0_dp
        homeSq1Hr1 = 0.0_dp
        home2Hr2 = 0.0_dp
        homeSq2Hr2 = 0.0_dp

        home1Work1 = 0.0_dp
        homeSq1Work1 = 0.0_dp
        home2Work2 = 0.0_dp
        homeSq2Work2 = 0.0_dp

        home1Work2 = 0.0_dp
        homeSq1Work2 = 0.0_dp
        home2Work1 = 0.0_dp
        homeSq2Work1 = 0.0_dp

        home1Hr2 = 0.0_dp
        homeSq1Hr2 = 0.0_dp
        home2Hr1 = 0.0_dp
        homeSq2Hr1 = 0.0_dp

        work1Work2 = 0.0_dp
        work2Work1 = 0.0_dp

        frac12 = 0.0_dp
        wage12 = 0.0_dp
        wageSq12 = 0.0_dp
        wage1Work12 = 0.0_dp
        wage2Work12 = 0.0_dp
        wageSq1Work12 = 0.0_dp
        wageSq2Work12 = 0.0_dp

        home1 = 0.0_dp
        homeSq1 = 0.0_dp
        home2 = 0.0_dp
        homeSq2 = 0.0_dp

        earn1Work1 = 0.0_dp
        earn2Work2 = 0.0_dp
        earnSq1Work1 = 0.0_dp
        earnSq2Work2 = 0.0_dp

        do aX2 = 1, NDIM
            do aX1 = 1, NDIM

                ! JOINT allocation
                do hX2 = 0, 2
                    if ( grid%gzInd(aX2, TIME_HOURS) >= BAND_WORKL(hX2) .and. grid%gzInd(aX2, TIME_HOURS) <= BAND_WORKU(hX2) ) then
                        do hX1 = 0, 2
                            if ( grid%gzInd(aX1, TIME_HOURS) >= BAND_WORKL(hX1) .and. grid%gzInd(aX1, TIME_HOURS) <= BAND_WORKU(hX1) ) then
                                fracHr(hX1, hX2) = fracHr(hX1, hX2) + eall%Eallocation(aX1, aX2)
                            end if
                        end do ! hX1
                    end if
                end do ! hX2

                ! MALE hours
                do hX1 = 0, 2
                    if ( grid%gzInd(aX1, TIME_HOURS) >= BAND_WORKL(hX1) .and. grid%gzInd(aX1, TIME_HOURS) <= BAND_WORKU(hX1) ) then

                        fracHr1(hX1) = fracHr1(hX1) + eall%Eallocation(aX1, aX2)

                        wage1Hr1(hX1)  = wage1Hr1(hX1)  + eall%Ewage1Allocation(aX1, aX2)
                        wageSq1Hr1(hX1) = wageSq1Hr1(hX1) + eall%EwageSq1Allocation(aX1, aX2)

                        home1Hr1(hX1) = home1Hr1(hX1) + eall%Eallocation(aX1, aX2) * grid%gz(aX1, TIME_PUBLIC)
                        homeSq1Hr1(hX1) = homeSq1Hr1(hX1) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX1, TIME_PUBLIC)

                        home2Hr1(hX1) = home2Hr1(hX1) + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                        homeSq2Hr1(hX1) = homeSq2Hr1(hX1) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                    end if
                end do ! hX1

                ! FEMALE hours
                do hX2 = 0, 2
                    if ( grid%gzInd(aX2, TIME_HOURS) >= BAND_WORKL(hX2) .and. grid%gzInd(aX2, TIME_HOURS) <= BAND_WORKU(hX2) ) then

                        fracHr2(hX2) = fracHr2(hX2) + eall%Eallocation(aX1, aX2)

                        wage2Hr2(hX2) = wage2Hr2(hX2) + eall%Ewage2Allocation(aX1, aX2)
                        wageSq2Hr2(hX2) = wageSq2Hr2(hX2) + eall%EwageSq2Allocation(aX1, aX2)

                        home2Hr2(hX2) = home2Hr2(hX2) + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                        homeSq2Hr2(hX2) = homeSq2Hr2(hX2) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                        home1Hr2(hX2) = home1Hr2(hX2) + eall%Eallocation(aX1, aX2) * grid%gz(aX1, TIME_PUBLIC)
                        homeSq1Hr2(hX2) = homeSq1Hr2(hX2) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX1, TIME_PUBLIC)

                    end if
                end do ! hX2

                ! MALE employment
                if ( grid%gzInd(aX1, TIME_HOURS) > 1 ) then

                    fracWork1(1) = fracWork1(1) + eall%Eallocation(aX1, aX2)
                    wage1Work1 = wage1Work1 + eall%Ewage1Allocation(aX1, aX2)
                    wageSq1Work1 = wageSq1Work1 + eall%EwageSq1Allocation(aX1, aX2)

                    hours1Work1 = hours1Work1 + eall%Eallocation(aX1, aX2) * grid%gz(aX1, TIME_HOURS)
                    hoursSq1Work1 = hoursSq1Work1 + eall%Eallocation(aX1, aX2) * grid%gzSq(aX1, TIME_HOURS)

                    earn1Work1 = earn1Work1 + eall%EwageLevel1Allocation(aX1, aX2) * grid%gz(aX1, TIME_HOURS)
                    earnSq1Work1 = earnSq1Work1 + eall%EwageLevelSq1Allocation(aX1, aX2) * grid%gzSq(aX1, TIME_HOURS)

                    home1Work1(1) = home1Work1(1) + eall%Eallocation(aX1, aX2) * grid%gz(aX1, TIME_PUBLIC)
                    homeSq1Work1(1) = homeSq1Work1(1) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX1, TIME_PUBLIC)

                    home2Work1(1) = home2Work1(1) + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                    homeSq2Work1(1) = homeSq2Work1(1) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                    ! spouse employment
                    if ( grid%gzInd(aX2, TIME_HOURS)>1 ) then

                        work1Work2(1) = work1Work2(1) + eall%Eallocation(aX1, aX2)
                        wage1Work2(1) = wage1Work2(1) + eall%Ewage1Allocation(aX1, aX2)
                        wageSq1Work2(1) = wageSq1Work2(1) + eall%EwageSq1Allocation(aX1, aX2)

                    else

                        work1Work2(0) = work1Work2(0) + eall%Eallocation(aX1, aX2)
                        wage1Work2(0) = wage1Work2(0) + eall%Ewage1Allocation(aX1, aX2)
                        wageSq1Work2(0) = wageSq1Work2(0) + eall%EwageSq1Allocation(aX1, aX2)

                    end if

                else

                    fracWork1(0) = fracWork1(0) + eall%Eallocation(aX1, aX2)
                    home1Work1(0) = home1Work1(0) + eall%Eallocation(aX1, aX2) * grid%gz(aX1, TIME_PUBLIC)
                    homeSq1Work1(0) = homeSq1Work1(0) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX1, TIME_PUBLIC)

                    home2Work1(0) = home2Work1(0) + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                    homeSq2Work1(0) = homeSq2Work1(0) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                end if

                ! female employment
                if ( grid%gzInd(aX2, TIME_HOURS) > 1 ) then

                    fracWork2(1) = fracWork2(1) + eall%Eallocation(aX1, aX2)
                    wage2Work2 = wage2Work2 + eall%Ewage2Allocation(aX1, aX2)
                    wageSq2Work2 = wageSq2Work2 + eall%EwageSq2Allocation(aX1, aX2)

                    hours2Work2 = hours2Work2 + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_HOURS)
                    hoursSq2Work2 = hoursSq2Work2 + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_HOURS)

                    earn2Work2 = earn2Work2 + eall%EwageLevel2Allocation(aX1, aX2) * grid%gz(aX2, TIME_HOURS)
                    earnSq2Work2 = earnSq2Work2 + eall%EwageLevelSq2Allocation(aX1, aX2) * grid%gzSq(aX2, TIME_HOURS)

                    home2Work2(1) = home2Work2(1) + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                    homeSq2Work2(1) = homeSq2Work2(1) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                    home1Work2(1) = home1Work2(1) + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                    homeSq1Work2(1) = homeSq1Work2(1) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                    ! spouse employment
                    if ( grid%gzInd(aX1, TIME_HOURS) > 1 ) then

                        work2Work1(1) = work2Work1(1) + eall%Eallocation(aX1, aX2)
                        wage2Work1(1) = wage2Work1(1) + eall%Ewage2Allocation(aX1, aX2)
                        wageSq2Work1(1) = wageSq2Work1(1) + eall%EwageSq2Allocation(aX1, aX2)

                    else

                        work2Work1(0) = work2Work1(0) + eall%Eallocation(aX1, aX2)
                        wage2Work1(0) = wage2Work1(0) + eall%Ewage2Allocation(aX1, aX2)
                        wageSq2Work1(0) = wageSq2Work1(0) + eall%EwageSq2Allocation(aX1, aX2)

                    end if

                else

                    fracWork2(0) = fracWork2(0) + eall%Eallocation(aX1, aX2)
                    home2Work2(0) = home2Work2(0) + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                    homeSq2Work2(0) = homeSq2Work2(0) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                    home1Work2(0) = home1Work2(0) + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                    homeSq1Work2(0) = homeSq1Work2(0) + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                end if


                ! male home hours
                do hX1 = 1, NHH
                    if ( grid%gzInd(aX1, TIME_PUBLIC) >= BAND_HOMEL(hX1) .and. grid%gzInd(aX1, TIME_PUBLIC) <= BAND_HOMEU(hX1) ) then
                        fracHome1(hX1) = fracHome1(hX1) + eall%Eallocation(aX1, aX2)
                    end if
                end do

                ! female home hours
                do hX2 = 1, NHH
                    if ( grid%gzInd(aX2, TIME_PUBLIC) >= BAND_HOMEL(hX2) .and. grid%gzInd(aX2, TIME_PUBLIC) <= BAND_HOMEU(hX2) ) then
                        fracHome2(hX2) = fracHome2(hX2) + eall%Eallocation(aX1, aX2)
                    end if
                end do

                home1 = home1 + eall%Eallocation(aX1, aX2) * grid%gz(aX1, TIME_PUBLIC)
                homeSq1 = homeSq1 + eall%Eallocation(aX1, aX2) * grid%gzSq(aX1, TIME_PUBLIC)

                home2 = home2 + eall%Eallocation(aX1, aX2) * grid%gz(aX2, TIME_PUBLIC)
                homeSq2 = homeSq2 + eall%Eallocation(aX1, aX2) * grid%gzSq(aX2, TIME_PUBLIC)

                ! both working
                if ( grid%gzInd(aX1, TIME_HOURS) > 1 .and. grid%gzInd(aX2, TIME_HOURS) > 1 ) then

                    frac12 = frac12 + eall%Eallocation(aX1, aX2)

                    wage1Work12 = wage1Work12 + eall%Ewage1Allocation(aX1, aX2)
                    wage2Work12 = wage2Work12 + eall%Ewage2Allocation(aX1, aX2)
                    wageSq1Work12 = wageSq1Work12 + eall%EwageSq1Allocation(aX1, aX2)
                    wageSq2Work12 = wageSq2Work12 + eall%EwageSq2Allocation(aX1, aX2)

                    wage12 = wage12 + eall%Ewage12Allocation(aX1, aX2)
                    wageSq12 = wageSq12 + eall%EwageSq12Allocation(aX1, aX2)

                end if

            end do ! aX1
        end do ! aX2

        ! distribution of male earnings conditional on employment
        meanEarn1Work1 = earn1Work1 / fracWork1(1)
        meanEarnSq1Work1 = earnSq1Work1 / fracWork1(1)
        call sde( meanEarn1Work1, meanEarnSq1Work1, sdEarn1Work1 )
        call sde( earn1Work1, earnSq1Work1, sdEarn1 )


        ! distribution of female earnings conditional on employment
        meanEarn2Work2 = earn2Work2 / fracWork2(1)
        meanEarnSq2Work2 = earnSq2Work2 / fracWork2(1)
        call sde( meanEarn2Work2, meanEarnSq2Work2, sdEarn2Work2 )
        call sde( earn2Work2, earnSq2Work2, sdEarn2 )

        ! distribution of male hours conditional on employment
        meanHours1Work1 = hours1Work1 / fracWork1(1)
        meanHoursSq1Work1 = hoursSq1Work1 / fracWork1(1)
        call sde( meanHours1Work1, meanHoursSq1Work1, sdHours1Work1 )
        call sde( hours1Work1, hoursSq1Work1, sdHours1 )

        ! distribution of female hours conditional on employment
        meanHours2Work2 = hours2Work2 / fracWork2(1)
        meanHoursSq2Work2 = hoursSq2Work2 / fracWork2(1)
        call sde( meanHours2Work2, meanHoursSq2Work2, sdHours2Work2 )
        call sde( hours2Work2, hoursSq2Work2, sdHours2 )

        ! distribution of male wages conditional on male hours
        meanWage1Hr1 = wage1Hr1 / fracHr1
        meanWageSq1Hr1 = wageSq1Hr1 / fracHr1
        call sde( meanWage1Hr1, meanWageSq1Hr1, sdWage1Hr1 )

        ! distribution of male wages conditional on male employment
        meanWage1Work1 = wage1Work1 / fracWork1(1)
        meanWageSq1Work1 = wageSq1Work1 / fracWork1(1)
        call sde( meanWage1Work1, meanWageSq1Work1, sdWage1Work1 )

        ! distribution of female wages conditional on female hours
        meanWage2Hr2 = wage2Hr2 / fracHr2
        meanWageSq2Hr2 = wageSq2Hr2 / fracHr2
        call sde( meanWage2Hr2, meanWageSq2Hr2, sdWage2Hr2 )

        ! distribution of male wages conditional on female employment status
        meanWage1Work2 = wage1Work2 / work1Work2
        meanWageSq1Work2 = wageSq1Work2 / work1Work2
        call sde( meanWage1Work2, meanWageSq1Work2, sdWage1Work2 )

        ! distribution of female wages conditional on male employment status
        meanWage2Work1 = wage2Work1 / work2Work1
        meanWageSq2Work1 = wageSq2Work1 / work2Work1
        call sde( meanWage2Work1, meanWageSq2Work1, sdWage2Work1 )

        ! distribution of female wages conditional on female employment
        meanWage2Work2 = wage2Work2 / fracWork2(1)
        meanWageSq2Work2 = wageSq2Work2 / fracWork2(1)
        call sde( meanWage2Work2, meanWageSq2Work2, sdWage2Work2 )

        meanWage1Work1 = wage1Work1 / fracWork1(1)
        meanWageSq1Work1 = wageSq1Work1 / fracWork1(1)

        ! distribution of joint wages
        meanWage1Work12 = wage1Work12 / frac12
        meanWage2Work12 = wage2Work12 / frac12
        meanWageSq1Work12 = wageSq1Work12 / frac12
        meanWageSq2Work12 = wageSq2Work12 / frac12
        meanWage12 = wage12 / frac12
        meanWageSq12 = wageSq12 / frac12
        call sde( meanWage12, meanWageSq12, sdWage12 )
        call cove( meanWage1Work12, meanWage2Work12, meanWage12, covWage12 )
        call sde( meanWage1Work12, meanWageSq1Work12, sdWage1Work12 )
        call sde( meanWage2Work12, meanWageSq2Work12, sdWage2Work12 )

        ! unconditional distribution of male home time
        meanHome1 = home1
        meanHomeSq1 = homeSq1
        call sde( meanHome1, meanHomeSq1, sdHome1 )

        ! unconditional distribution of female home time
        meanHome2 = home2
        meanHomeSq2 = homeSq2
        call sde( meanHome2, meanHomeSq2, sdHome2 )

        ! male home time conditional on male hours
        meanHome1Hr1 = home1Hr1 / fracHr1
        meanHomeSq1Hr1 = homeSq1Hr1 / fracHr1
        call sde( meanHome1Hr1, meanHomeSq1Hr1, sdHome1Hr1 )

        ! male home time conditional on male employment
        meanHome1Work1 = home1Work1 / fracWork1
        meanHomeSq1Work1 = homeSq1Work1 / fracWork1
        call sde( meanHome1Work1, meanHomeSq1Work1, sdHome1Work1 )

        ! male home time conditional on female hours
        meanHome1Hr2 = home1Hr2 / fracHr2
        meanHomeSq1Hr2 = homeSq1Hr2 / fracHr2
        call sde( meanHome1Hr2, meanHomeSq1Hr2, sdHome1Hr2 )

        ! male home time conditional on female employment
        meanHome1Work2 = home1Work2 / fracWork2
        meanHomeSq1Work2 = homeSq1Work2 / fracWork2
        call sde( meanHome1Work2, meanHomeSq1Work2, sdHome1Work2 )



        ! female home time conditional on female hours
        meanHome2Hr2 = home2Hr2 / fracHr2
        meanHomeSq2Hr2 = homeSq2Hr2 / fracHr2
        call sde( meanHome2Hr2, meanHomeSq2Hr2, sdHome2Hr2 )

        ! female home time conditional on female employment
        meanHome2Work2 = home2Work2 / fracWork2
        meanHomeSq2Work2 = homeSq2Work2 / fracWork2
        call sde( meanHome2Work2, meanHomeSq2Work2, sdHome2Work2 )

        ! female home time conditional on male hours
        meanHome2Hr1 = home2Hr1 / fracHr1
        meanHomeSq2Hr1 = homeSq2Hr1 / fracHr1
        call sde( meanHome2Hr1, meanHomeSq2Hr1, sdHome2Hr1 )

        ! female home time conditional on male employment
        meanHome2Work1 = home2Work1 / fracWork1
        meanHomeSq2Work1 = homeSq2Work1 / fracWork1
        call sde( meanHome2Work1, meanHomeSq2Work1, sdHome2Work1 )

        ! **** IPUMS ****

        ! joint labour supply allocations
        eall%jhours = fracHr
        ! joint work (=sum PT/PT, PT/FT, FT/PT, FT/FT)
        eall%jwork = eall%jhours(1, 1) + eall%jhours(1, 2) + eall%jhours(2, 1) + eall%jhours(2, 2)

        ! marginal hours distribution
        eall%mhours = fracHr1
        eall%fhours = fracHr2

        ! proportion working
        eall%mwork = fracWork1
        eall%fwork = fracWork2

        ! conditional distribution male labour supply
        eall%mhourshMw = meanHours1Work1
        eall%mhourshMwsd = sdHours1Work1

        ! conditional distribution female labour supply
        eall%fhourshFw = meanHours2Work2
        eall%fhourshFwsd = sdHours2Work2

        ! unconditional distribution male labour supply
        eall%mhoursh = hours1Work1
        eall%mhourshsd = sdHours1

        ! unconditional distribution female labour supply
        eall%fhoursh = hours2Work2
        eall%fhourshsd = sdHours2

        ! male wage distribution
        eall%mwage = meanWage1Hr1
        eall%mwagesd = sdWage1Hr1
        eall%mwageMw = meanWage1Work1
        eall%mwagesdMw = sdWage1Work1

        ! female wage distribution
        eall%fwage = meanWage2Hr2
        eall%fwagesd = sdWage2Hr2
        eall%fwageFw = meanWage2Work2
        eall%fwagesdFw = sdWage2Work2

        ! male wage distribution by spousal employment state
        eall%mwageFw = meanWage1Work2
        eall%mwageFwsd = sdWage1Work2

        ! female wage distribution by spousal employment state
        eall%fwageMw = meanWage2Work1
        eall%fwageMwsd = sdWage2Work1

        ! product of wages within a match
        eall%jwage = meanWage12
        eall%jwagesd = sdWage12
        eall%jwagecov = covWage12

        ! **** ATUS ****

        ! marginal distribution of home production time
        eall%mhome = fracHome1
        eall%fhome = fracHome2

        eall%mhomeh = meanHome1
        eall%mhomehsd = sdHome1
        eall%fhomeh = meanHome2
        eall%fhomehsd = sdHome2

        ! male home production hours by own employment status
        eall%mhomehMw = meanHome1Work1
        eall%mhomehMwsd = sdHome1Work1

        ! male home production hours by own hours
        eall%mhomehMh = meanHome1Hr1
        eall%mhomehMhsd = sdHome1Hr1

        ! male home production by female employment status
        eall%mhomehFw = meanHome1Work2
        eall%mhomehFwsd = sdHome1Work2

        ! male home production by female hours
        eall%mhomehFh = meanHome1Hr2
        eall%mhomehFhsd = sdHome1Hr2

        ! female home production hours by own employment status
        eall%fhomehFw = meanHome2Work2
        eall%fhomehFwsd = sdHome2Work2

        ! female home production hours by own hours
        eall%fhomehFh = meanHome2Hr2
        eall%fhomehFhsd = sdHome2Hr2

        ! female home production by male employment status
        eall%fhomehMw = meanHome2Work1
        eall%fhomehMwsd = sdHome2Work1

        ! female home production by male hours
        eall%fhomehMh = meanHome2Hr1
        eall%fhomehMhsd = sdHome2Hr1

        ! EARNINGS

        ! conditional distribution male earnings
        eall%mearnMw = meanEarn1Work1
        eall%mearnMwsd = sdEarn1Work1

        ! unconditional distribution male earnings
        eall%mearn = earn1Work1
        eall%mearnsd = sdEarn1

        ! conditional distribution female earnings
        eall%fearnFw = meanEarn2Work2
        eall%fearnFwsd = sdEarn2Work2

        ! unconditional distribution female labour supply
        eall%fearn = earn2Work2
        eall%fearnsd = sdEarn2

        eall%mwageJw = meanWage1Work12
        eall%mwagesdJw = sdWage1Work12
        eall%fwageJw = meanWage2Work12
        eall%fwagesdJw = sdWage2Work12

    end subroutine momentHousehold

    subroutine sde_scalar( x, x2, z )
        implicit none
        real(dp), intent(in) :: x
        real(dp), intent(in) :: x2
        real(dp), intent(out) :: z
        z = sqrt( max( x2 - x ** 2, 0.0_dp) )
    end subroutine sde_scalar

    subroutine sde_vector( x, x2, z )
        implicit none
        real(dp), intent(in) :: x(:)
        real(dp), intent(in) :: x2(size(x))
        real(dp), intent(out) :: z(size(x))
        z = sqrt( max( x2 - x ** 2, 0.0_dp) )
    end subroutine sde_vector

    subroutine cove_vector( x, y, xy, z )
        implicit none
        real(dp), intent(in) :: x(:)
        real(dp), intent(in) :: y(size(x))
        real(dp), intent(in) :: xy(size(x))
        real(dp), intent(out) :: z(size(x))
        z = xy - x * y
    end subroutine cove_vector

    subroutine cove_scalar( x, y, xy, z )
        implicit none
        real(dp), intent(in) :: x
        real(dp), intent(in) :: y
        real(dp), intent(in) :: xy
        real(dp), intent(out) :: z
        z = xy - x * y
    end subroutine cove_scalar

    subroutine elas(tax, m, h, pM, pF, grid, rnd, opt, estmode, lam, kid, verbose, texpath, outpath, ftag)
        use mathcons
        use mathfn
        use taxes
        implicit none

        type(tax_t), intent(in) :: tax(NMARKET)
        type(m_t), intent(inout) :: m(NMARKET)
        type(h_t), intent(in) :: h(NTYPE1, NTYPE2, 0:1, NMARKET)
        type(p_t), intent(in) :: pM(NTYPE1, 0:1, NMARKET)
        type(p_t), intent(in) :: pF(NTYPE2, 0:1, NMARKET)
        type(grid_t), intent(in) :: grid
        type(rnd_t), intent(in) :: rnd

        type(opt_t), intent(in)  :: opt
        integer, intent(in) :: estmode
        type(opt_t) :: opt2

        real(dp), intent(in) :: lam(NTYPE1, NTYPE2, NMARKET)
        integer, intent(in) :: kid
        logical, intent(in) :: verbose
        character(len = *), intent(in) :: texpath
        character(len = *), intent(in) :: outpath
        integer, intent(in) :: ftag

        integer, parameter :: seed(1) = (/ 498328231 /)

        real(dp) :: u_base(NDIM), u_reform(NDIM)
        real(dp) :: uCou_base(NDIM, NDIM), uCou_reform(NDIM, NDIM)
        real(dp) :: hours_baseR(NR), work_baseR(NR), home_baseR(NR)
        real(dp) :: hours_reformR(NR), work_reformR(NR), home_reformR(NR)

        real(dp) :: hoursM_baseR(NR), homeM_baseR(NR), workM_baseR(NR), workM_basereformR(NR)
        real(dp) :: hoursM_reformR(NR), homeM_reformR(NR), workM_reformR(NR)
        real(dp) :: hoursF_baseR(NR), homeF_baseR(NR), workF_baseR(NR), workF_basereformR(NR)
        real(dp) :: hoursF_reformR(NR), homeF_reformR(NR), workF_reformR(NR)

        real(dp) :: work_baseW(NODEW), work_reformW(NODEW)
        real(dp) :: home_baseW(NODEW), home_reformW(NODEW)
        real(dp) :: condHours_baseW(NODEW), condHours_reformW(NODEW)

        real(dp) :: condHours_base(NTYPE1, 0:1, NMARKET), condHours_reform(NTYPE1, 0:1, NMARKET), home_base(NTYPE1, 0:1, NMARKET)
        real(dp) :: home_reform(NTYPE1, 0:1, NMARKET), work_base(NTYPE1, 0:1, NMARKET), work_reform(NTYPE1, 0:1, NMARKET)

        real(dp) :: workM_baseW(NODEW, NODEW), workM_reformW(NODEW, NODEW), workM_basereformW(NODEW, NODEW)
        real(dp) :: workF_baseW(NODEW, NODEW), workF_reformW(NODEW, NODEW), workF_basereformW(NODEW, NODEW)
        real(dp) :: homeM_baseW(NODEW, NODEW), homeM_reformW(NODEW, NODEW)
        real(dp) :: homeF_baseW(NODEW, NODEW), homeF_reformW(NODEW, NODEW)
        real(dp) :: condHoursM_baseW(NODEW, NODEW), condHoursM_reformW(NODEW, NODEW)
        real(dp) :: condHoursF_baseW(NODEW, NODEW), condHoursF_reformW(NODEW, NODEW)
        real(dp) :: homeM_base(NTYPE1, NTYPE2, 0:1, NMARKET), condHoursM_base(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp) :: homeM_reform(NTYPE1, NTYPE2, 0:1, NMARKET), condHoursM_reform(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp) :: workM_base(NTYPE1, NTYPE2, 0:1, NMARKET), workM_reform(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp) :: workM_basereform(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp) :: homeF_base(NTYPE1, NTYPE2, 0:1, NMARKET), condHoursF_base(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp) :: homeF_reform(NTYPE1, NTYPE2, 0:1, NMARKET), condHoursF_reform(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp) :: workF_base(NTYPE1, NTYPE2, 0:1, NMARKET), workF_reform(NTYPE1, NTYPE2, 0:1, NMARKET)
        real(dp) :: workF_basereform(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: intWorkM_base, intWorkM_reform, intHomeM_base, intHomeM_reform, intCondHoursM_base, intCondHoursM_reform
        real(dp) :: intWorkM_basereform
        real(dp) :: intWorkF_base, intWorkF_reform, intHomeF_base, intHomeF_reform, intCondHoursF_base, intCondHoursF_reform
        real(dp) :: intWorkF_basereform
        real(dp) :: intWork_base, intWork_reform, intHome_base, intHome_reform, intCondHours_base, intCondHours_reform

        real(dp) :: elas_SingleMaleEmploymentOwnWage, elas_SingleMaleEmploymentOwnWage2, elas_SingleMaleCondHoursOwnWage
        real(dp) :: elas_SingleMaleHomeOwnWage
        real(dp) :: elas_SingleFemaleEmploymentOwnWage, elas_SingleFemaleEmploymentOwnWage2, elas_SingleFemaleCondHoursOwnWage
        real(dp) :: elas_SingleFemaleHomeOwnWage
        real(dp) :: elas_MarriedMaleEmploymentOwnWage, elas_MarriedMaleEmploymentOwnWage2, elas_MarriedMaleCondHoursOwnWage
        real(dp) :: elas_MarriedMaleHomeOwnWage
        real(dp) :: elas_MarriedMaleEmploymentCrossWage, elas_MarriedMaleEmploymentCrossWage2, elas_MarriedMaleCondHoursCrossWage
        real(dp) :: elas_MarriedMaleHomeCrossWage
        real(dp) :: elas_MarriedFemaleEmploymentOwnWage, elas_MarriedFemaleEmploymentOwnWage2, elas_MarriedFemaleCondHoursOwnWage
        real(dp) :: elas_MarriedFemaleHomeOwnWage
        real(dp) :: elas_MarriedFemaleEmploymentCrossWage, elas_MarriedFemaleEmploymentCrossWage2
        real(dp) :: elas_MarriedFemaleCondHoursCrossWage, elas_MarriedFemaleHomeCrossWage

        real(dp) :: netinc_base(N, N, NODEW, NODEW), netinc_reform(N, N, NODEW, NODEW)
        integer :: ind_base, ind_reform
        integer :: indCou_base(2), indCou_reform(2)
        type(v_t) :: v_base, v_reform
        type(v_t) :: v_baseNoKidTax, v_reformNoKidTax
        type(v2_t) :: vCou_base, vCou_reform
        integer :: mX, jX, kX, pX1, pX2, wX0, wX1, wX2, rX0, aX2
        integer :: funit, kid0, kid1
        integer :: countNoKidTax
        real(dp) :: intSize

        if (kid < 0) then
            kid0 = 0
            kid1 = 1
        else
            kid0 = kid
            kid1 = kid
        end if

        ! different opt for scaling wage
        call setOptions(opt2, OPT_WAGESCALE, estmode)

        ! SINGLE MEN

        do mX = 1, NMARKET
            do kX = kid0, kid1
                do pX1 = 1, NTYPE1

                    if (verbose) then
                        print *, 'SINMALE', pX1, kX, mX
                    end if

                    call utilityPreCalc( pM(pX1, kX, mX), rnd, tax(mX), grid, kX, kX, SEX_MALE, opt, v_base )
                    call utilityPreCalc( pM(pX1, kX, mX), rnd, tax(mX), grid, kX, kX, SEX_MALE, opt2, v_reform )

                    if (kX == 1) then
                        call utilityPreCalc( pM(pX1, kX, mX), rnd, tax(mX), grid, kX, 0, SEX_MALE, opt, v_baseNoKidTax )
                        call utilityPreCalc( pM(pX1, kX, mX), rnd, tax(mX), grid, kX, 0, SEX_MALE, opt2, v_reformNoKidTax )
                        countNoKidTax = (1.0_dp - pM(pX1, kX, mX)%probKidTax) * NR
                    end if

                    do wX0 = 1, NODEW

                        !$omp parallel do &
                        !$omp default(shared) private(rX0, u_base, u_reform, ind_base, ind_reform)
                        do rX0 = 1, NR

                            if (kX == 1) then
                                if (rX0 <= countNoKidTax ) then
                                    u_base = v_baseNoKidTax%util(:, wX0) + pM(pX1, kX, mX)%sd * rnd%ev(:, rX0)
                                    u_reform = v_reformNoKidTax%util(:, wX0) + pM(pX1, kX, mX)%sd * rnd%ev(:, rX0)
                                else
                                    u_base = v_base%util(:, wX0) + pM(pX1, kX, mX)%sd * rnd%ev(:, rX0)
                                    u_reform = v_reform%util(:, wX0) + pM(pX1, kX, mX)%sd * rnd%ev(:, rX0)
                                end if
                            else
                                u_base = v_base%util(:, wX0) + pM(pX1, kX, mX)%sd * rnd%ev(:, rX0)
                                u_reform = v_reform%util(:, wX0) + pM(pX1, kX, mX)%sd * rnd%ev(:, rX0)
                            end if

                            ind_base = maxloc(u_base, dim = 1)
                            ind_reform = maxloc(u_reform, dim = 1)

                            hours_baseR(rX0) = grid%gz(ind_base, TIME_HOURS)
                            hours_reformR(rX0) = grid%gz(ind_reform, TIME_HOURS)

                            home_baseR(rX0) = grid%gz(ind_base, TIME_PUBLIC)
                            home_reformR(rX0) = grid%gz(ind_reform, TIME_PUBLIC)

                            work_baseR(rX0) = merge( 1.0_dp, 0.0_dp, hours_baseR(rX0) > 0.0_dp )
                            work_reformR(rX0) = merge( 1.0_dp, 0.0_dp, hours_reformR(rX0) > 0.0_dp )

                        end do
                        !$omp end parallel do

                        work_baseW(wX0) = sum(work_baseR) / real(NR, dp)
                        work_reformW(wX0) = sum(work_reformR) / real(NR, dp)

                        home_baseW(wX0) = sum(home_baseR) / real(NR, dp)
                        home_reformW(wX0) = sum(home_reformR) / real(NR, dp)

                        if ( sum(work_baseR) > 0.0_dp ) then
                            condHours_baseW(wX0) = sum( hours_baseR, mask = work_baseR > 0.0_dp ) / sum(work_baseR)
                            condHours_reformW(wX0) = sum( hours_reformR, mask = work_baseR > 0.0_dp ) / sum(work_baseR)
                        else
                            condHours_baseW(wX0) = 0.0_dp
                            condHours_reformW(wX0) = 0.0_dp
                        end if

                    end do

                    ! integrate: employment
                    work_base(pX1, kX, mX) = wageIntegrateFast( pM(pX1, kX, mX), work_baseW )
                    work_reform(pX1, kX, mX) = wageIntegrateFast( pM(pX1, kX, mX), work_reformW )

                    ! integrate: home time
                    home_base(pX1, kX, mX) = wageIntegrateFast( pM(pX1, kX, mX), home_baseW )
                    home_reform(pX1, kX, mX) = wageIntegrateFast( pM(pX1, kX, mX), home_reformW )

                    ! integrate: hours
                    condHours_base(pX1, kX, mX) = wageIntegrateFast( pM(pX1, kX, mX), work_baseW * condHours_baseW )
                    condHours_reform(pX1, kX, mX) = wageIntegrateFast( pM(pX1, kX, mX), work_baseW * condHours_reformW )

                end do ! pX1
            end do ! kX
        end do ! mX

        intWork_base = 0.0_dp
        intWork_reform = 0.0_dp

        intHome_base = 0.0_dp
        intHome_reform = 0.0_dp

        intCondHours_base = 0.0_dp
        intCondHours_reform = 0.0_dp

        intSize = 0.0_dp

        do mX = 1, NMARKET

            intSize = intSize + sum( m(mX)%numSingleKid1(:, kid0:kid1) )

            do kX = kid0, kid1
                do pX1 = 1, NTYPE1
                    intWork_base = intWork_base + m(mX)%numSingleKid1(pX1, kX) * work_base(pX1, kX, mX)
                    intWork_reform = intWork_reform + m(mX)%numSingleKid1(pX1, kX) * work_reform(pX1, kX, mX)

                    intHome_base = intHome_base + m(mX)%numSingleKid1(pX1, kX) * home_base(pX1, kX, mX)
                    intHome_reform = intHome_reform + m(mX)%numSingleKid1(pX1, kX) * home_reform(pX1, kX, mX)

                    intCondHours_base = intCondHours_base + m(mX)%numSingleKid1(pX1, kX) &
                        * work_base(pX1, kX, mX) * condHours_base(pX1, kX, mX)
                    intCondHours_reform = intCondHours_reform + m(mX)%numSingleKid1(pX1, kX) &
                        * work_base(pX1, kX, mX) * condHours_reform(pX1, kX, mX)
                end do ! pX1
            end do ! kX
        end do ! mX

        intCondHours_base = intCondHours_base / intWork_base
        intCondHours_reform = intCondHours_reform / intWork_base

        intWork_base = intWork_base / intSize
        intWork_reform = intWork_reform / intSize
        intHome_base = intHome_base / intSize
        intHome_reform = intHome_reform / intSize

        elas_SingleMaleEmploymentOwnWage  = (intWork_reform / intWork_base - 1.0_dp) / ELAS_STEP
        elas_SingleMaleEmploymentOwnWage2 = (intWork_reform - intWork_base ) / ELAS_STEP
        elas_SingleMaleCondHoursOwnWage = (intCondHours_reform / intCondHours_base - 1.0_dp) / ELAS_STEP
        elas_SingleMaleHomeOwnWage = (intHome_reform / intHome_base - 1.0_dp ) / ELAS_STEP

        if ( verbose ) then
            write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'single male employment own wage',        &
                                                      elas_SingleMaleEmploymentOwnWage,         &
                                                      rfp(elas_SingleMaleEmploymentOwnWage)
            write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'single male employment own wage 2',      &
                                                      elas_SingleMaleEmploymentOwnWage2,        &
                                                      rfp(elas_SingleMaleEmploymentOwnWage2)
            write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'single male conditional hours own wage', &
                                                      elas_SingleMaleCondHoursOwnWage,          &
                                                      rfp(elas_SingleMaleCondHoursOwnWage)
            write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'single male home time own wage',         &
                                                      elas_SingleMaleHomeOwnWage,               &
                                                      rfp(elas_SingleMaleHomeOwnWage)
        end if

        ! SINGLE WOMEN

        do mX = 1, NMARKET
            do kX = kid0, kid1
                do pX2 = 1, NTYPE2

                    if (verbose) then
                        print *, 'SINFEMALE', pX2, kX, mX
                    end if

                    call utilityPreCalc( pF(pX2, kX, mX), rnd, tax(mX), grid, kX, kX, SEX_FEMALE, opt, v_base )
                    call utilityPreCalc( pF(pX2, kX, mX), rnd, tax(mX), grid, kX, kX, SEX_FEMALE, opt2, v_reform )

                    do wX0 = 1, NODEW

                        !$omp parallel do &
                        !$omp default(shared) private(rX0, u_base, u_reform, ind_base, ind_reform)
                        do rX0 = 1, NR

                            u_base = v_base%util(:, wX0) + pF(pX2, kX, mX)%sd * rnd%ev(:, rX0)
                            u_reform = v_reform%util(:, wX0) + pF(pX2, kX, mX)%sd * rnd%ev(:, rX0)

                            ind_base = maxloc(u_base, dim = 1)
                            ind_reform = maxloc(u_reform, dim = 1)

                            hours_baseR(rX0) = grid%gz(ind_base, TIME_HOURS)
                            hours_reformR(rX0) = grid%gz(ind_reform, TIME_HOURS)

                            home_baseR(rX0) = grid%gz(ind_base, TIME_PUBLIC)
                            home_reformR(rX0) = grid%gz(ind_reform, TIME_PUBLIC)

                            work_baseR(rX0) = merge( 1.0_dp, 0.0_dp, hours_baseR(rX0) > 0.0_dp )
                            work_reformR(rX0) = merge( 1.0_dp, 0.0_dp, hours_reformR(rX0) > 0.0_dp )

                        end do
                        !$omp end parallel do

                        work_baseW(wX0) = sum(work_baseR) / real(NR, dp)
                        work_reformW(wX0) = sum(work_reformR) / real(NR, dp)

                        home_baseW(wX0) = sum(home_baseR) / real(NR, dp)
                        home_reformW(wX0) = sum(home_reformR) / real(NR, dp)

                        if ( sum(work_baseR) > 0.0_dp ) then
                            condHours_baseW(wX0) = sum( hours_baseR, mask = work_baseR > 0.0_dp ) / sum(work_baseR)
                            condHours_reformW(wX0) = sum( hours_reformR, mask = work_baseR > 0.0_dp ) / sum(work_baseR)
                        else
                            condHours_baseW(wX0) = 0.0_dp
                            condHours_reformW(wX0) = 0.0_dp
                        end if

                    end do

                    ! integrate: employment
                    work_base(pX2, kX, mX) = wageIntegrateFast( pF(pX2, kX, mX), work_baseW )
                    work_reform(pX2, kX, mX) = wageIntegrateFast( pF(pX2, kX, mX), work_reformW )

                    ! integrate: home time
                    home_base(pX2, kX, mX) = wageIntegrateFast( pF(pX2, kX, mX), home_baseW )
                    home_reform(pX2, kX, mX) = wageIntegrateFast( pF(pX2, kX, mX), home_reformW )

                    ! integrate: hours
                    condHours_base(pX2, kX, mX) = wageIntegrateFast( pF(pX2, kX, mX), work_baseW * condHours_baseW )
                    condHours_reform(pX2, kX, mX) = wageIntegrateFast( pF(pX2, kX, mX), work_baseW * condHours_reformW )

                end do ! pX2
            end do ! kX
        end do ! mX

        intWork_base = 0.0_dp
        intWork_reform = 0.0_dp

        intHome_base = 0.0_dp
        intHome_reform = 0.0_dp

        intCondHours_base = 0.0_dp
        intCondHours_reform = 0.0_dp

        intSize = 0.0_dp

        do mX = 1, NMARKET

            intSize = intSize + sum( m(mX)%numSingleKid2(:, kid0:kid1) )

            do kX = kid0, kid1
                do pX2 = 1, NTYPE2
                    intWork_base = intWork_base + m(mX)%numSingleKid2(pX2, kX) * work_base(pX2, kX, mX)
                    intWork_reform = intWork_reform + m(mX)%numSingleKid2(pX2, kX) * work_reform(pX2, kX, mX)

                    intHome_base = intHome_base + m(mX)%numSingleKid2(pX2, kX) * home_base(pX2, kX, mX)
                    intHome_reform = intHome_reform + m(mX)%numSingleKid2(pX2, kX) * home_reform(pX2, kX, mX)

                    intCondHours_base = intCondHours_base + m(mX)%numSingleKid2(pX2, kX) &
                        * work_base(pX2, kX, mX) * condHours_base(pX2, kX, mX)
                    intCondHours_reform = intCondHours_reform + m(mX)%numSingleKid2(pX2, kX) &
                        * work_base(pX2, kX, mX) * condHours_reform(pX2, kX, mX)
                end do ! pX2
            end do ! kX
        end do ! mX

        intCondHours_base = intCondHours_base / intWork_base
        intCondHours_reform = intCondHours_reform / intWork_base

        intWork_base = intWork_base / intSize
        intWork_reform = intWork_reform / intSize
        intHome_base = intHome_base / intSize
        intHome_reform = intHome_reform / intSize

        elas_SingleFemaleEmploymentOwnWage = (intWork_reform / intWork_base - 1.0_dp) / ELAS_STEP
        elas_SingleFemaleEmploymentOwnWage2 = (intWork_reform - intWork_base) / ELAS_STEP
        elas_SingleFemaleCondHoursOwnWage = (intCondHours_reform / intCondHours_base - 1.0_dp) / ELAS_STEP
        elas_SingleFemaleHomeOwnWage = (intHome_reform / intHome_base - 1.0_dp) / ELAS_STEP

        if ( verbose ) then
            write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'single female employment own wage',        & 
                                                      elas_SingleFemaleEmploymentOwnWage,         &
                                                      rfp(elas_SingleFemaleEmploymentOwnWage)
            write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'single female employment own wage 2',      &
                                                      elas_SingleFemaleEmploymentOwnWage2,        &
                                                      rfp(elas_SingleFemaleEmploymentOwnWage2)
            write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'single female conditional hours own wage', &
                                                      elas_SingleFemaleCondHoursOwnWage,          &
                                                      rfp(elas_SingleFemaleCondHoursOwnWage)
            write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'single female home time own wage',         &
                                                      elas_SingleFemaleHomeOwnWage,               &
                                                      rfp(elas_SingleFemaleHomeOwnWage)
        end if

        ! loop over male / female net wage change
        do jX = 1, 2

            ! determine whose net wage to scale
            select case(jX)
            case(1)
                call setOptions(opt2, OPT_WAGESCALE1, estmode)
            case(2)
                call setOptions(opt2, OPT_WAGESCALE2, estmode)
            case default
                stop 'elas: jX must be 1 (MALE) or 2 (FEMALE)'
            end select

            ! calculate objects conditional on (pX1, pX2, kX, mX)
            do mX = 1, NMARKET
                do kX = kid0, kid1
                    do pX2 = 1, NTYPE2
                        do pX1 = 1, NTYPE1

                            if (verbose) then
                                print *, jX, 'COUPLE', pX1, pX2, kX, mX
                            end if

                            call householdUtilityPreCalc( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), pF(pX2, kX, mX), rnd, tax(mX), &
                                                          grid, kX, opt, vCou_base, netinc_base, jointTaxes = .true. )
                            call householdUtilityPreCalc( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), pF(pX2, kX, mX), rnd, tax(mX), &
                                                          grid, kX, opt2, vCou_reform, netinc_reform, jointTaxes = .true. )
                            call householdUtilityPreCalcLambda( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), pF(pX2, kX, mX), tax(mX), &
                                                                lam(pX1, pX2, mX), grid, netinc_base, opt, vCou_base )
                            call householdUtilityPreCalcLambda( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), pF(pX2, kX, mX), tax(mX), &
                                                                lam(pX1, pX2, mX), grid, netinc_reform, opt, vCou_reform )

                            do wX2 = 1, NODEW
                                do wX1 = 1, NODEW

                                    !$omp parallel do &
                                    !$omp default(shared) private(rX0, uCou_base, uCou_reform, indCou_base, indCou_reform)
                                    do rX0 = 1, NR

                                        do aX2 = 1, NDIM
                                            uCou_base(:, aX2) = ( 1.0_dp - lam(pX1, pX2, mX) ) &
                                                *( vCou_base%utilM(:, aX2, wX1, wX2) + pM(pX1, kX, mX)%sd * rnd%ev(:, rX0) ) &
                                                + lam(pX1, pX2, mX) * ( vCou_base%utilF(:, aX2, wX1, wX2) + pF(pX2, kX, mX)%sd * rnd%ev2(aX2, rX0) )
                                            uCou_reform(:, aX2) = ( 1.0_dp - lam(pX1, pX2, mX) ) &
                                                *( vCou_reform%utilM(:, aX2, wX1, wX2) + pM(pX1, kX, mX)%sd * rnd%ev(:, rX0) ) &
                                                + lam(pX1, pX2, mX) * ( vCou_reform%utilF(:, aX2, wX1, wX2) + pF(pX2, kX, mX)%sd * rnd%ev2(aX2, rX0) )
                                        end do

                                        indCou_base = maxloc(uCou_base)
                                        indCou_reform = maxloc(uCou_reform)

                                        hoursM_baseR(rX0) = grid%gz(indCou_base(1), TIME_HOURS)
                                        hoursM_reformR(rX0) = grid%gz(indCou_reform(1), TIME_HOURS)

                                        homeM_baseR(rX0) = grid%gz(indCou_base(1), TIME_PUBLIC)
                                        homeM_reformR(rX0) = grid%gz(indCou_reform(1), TIME_PUBLIC)

                                        hoursF_baseR(rX0) = grid%gz(indCou_base(2), TIME_HOURS)
                                        hoursF_reformR(rX0) = grid%gz(indCou_reform(2), TIME_HOURS)

                                        homeF_baseR(rX0) = grid%gz(indCou_base(2), TIME_PUBLIC)
                                        homeF_reformR(rX0) = grid%gz(indCou_reform(2), TIME_PUBLIC)

                                        ! male: working in base, reform, and base/reform
                                        workM_baseR(rX0) = merge( 1.0_dp, 0.0_dp, hoursM_baseR(rX0) > 0.0_dp )
                                        workM_reformR(rX0) = merge( 1.0_dp, 0.0_dp, hoursM_reformR(rX0) > 0.0_dp )
                                        workM_basereformR(rX0) = merge( 1.0_dp, 0.0_dp, hoursM_baseR(rX0) > 0.0_dp &
                                                                                        .and. hoursM_reformR(rX0) > 0.0_dp )

                                        ! female: working in base, reform, and base/reform
                                        workF_baseR(rX0) = merge( 1.0_dp, 0.0_dp, hoursF_baseR(rX0) > 0.0_dp )
                                        workF_reformR(rX0) = merge( 1.0_dp, 0.0_dp, hoursF_reformR(rX0) > 0.0_dp )
                                        workF_basereformR(rX0) = merge( 1.0_dp, 0.0_dp, hoursF_baseR(rX0) > 0.0_dp &
                                                                                        .and. hoursF_reformR(rX0) > 0.0_dp )

                                    end do
                                    !$omp end parallel do

                                    workM_baseW(wX1, wX2) = sum(workM_baseR) / real(NR, dp)
                                    workM_reformW(wX1, wX2) = sum(workM_reformR) / real(NR, dp)
                                    workM_basereformW(wX1, wX2) = count(workM_baseR > 0.0_dp .and. workM_reformR > 0.0_dp) / real(NR, dp)

                                    workF_baseW(wX1, wX2) = sum(workF_baseR) / real(NR, dp)
                                    workF_reformW(wX1, wX2) = sum(workF_reformR) / real(NR, dp)
                                    workF_basereformW(wX1, wX2) = count(workF_baseR > 0.0_dp .and. workF_reformR > 0.0_dp) / real(NR, dp)

                                    homeM_baseW(wX1, wX2) = sum(homeM_baseR) / real(NR, dp)
                                    homeM_reformW(wX1, wX2) = sum(homeM_reformR) / real(NR, dp)

                                    homeF_baseW(wX1, wX2) = sum(homeF_baseR) / real(NR, dp)
                                    homeF_reformW(wX1, wX2) = sum(homeF_reformR) / real(NR, dp)

                                    if ( jX == 1 ) then

                                        ! conditional hours for males: note that workM_reformR>=workM_baseR
                                        if ( sum(workM_baseR) > 0.0_dp ) then
                                            condhoursM_baseW(wX1, wX2) = sum( hoursM_baseR, mask = workM_baseR > 0.0_dp ) / sum(workM_baseR)
                                            condhoursM_reformW(wX1, wX2) = sum( hoursM_reformR, mask = workM_baseR > 0.0_dp ) / sum(workM_baseR)
                                        else
                                            condhoursM_baseW(wX1, wX2) = 0.0_dp
                                            condhoursM_reformW(wX1, wX2) = 0.0_dp
                                        end if

                                        ! conditional hours for female: as women may become non-participants, this is conditional on working in
                                        ! both the base and reform system

                                        if ( sum(workF_basereformR) > 0.0_dp ) then
                                            condhoursF_baseW(wX1, wX2) = sum( hoursF_baseR, mask = workF_basereformR > 0.0_dp ) / sum(workF_basereformR)
                                            condhoursF_reformW(wX1, wX2) = sum( hoursF_reformR, mask = workF_basereformR > 0.0_dp ) / sum(workF_basereformR)
                                        else
                                            condhoursF_baseW(wX1, wX2) = 0.0_dp
                                            condhoursF_reformW(wX1, wX2) = 0.0_dp
                                        end if

                                    else

                                        ! conditional hours for male: as men may become non-participants, this is conditional on working in
                                        ! both the base and reform system

                                        if ( sum(workM_basereformR) > 0.0_dp ) then
                                            condhoursM_baseW(wX1, wX2) = sum( hoursM_baseR, mask = workM_basereformR > 0.0_dp ) / sum(workM_basereformR)
                                            condhoursM_reformW(wX1, wX2) = sum( hoursM_reformR, mask = workM_basereformR > 0.0_dp ) / sum(workM_basereformR)
                                        else
                                            condhoursM_baseW(wX1, wX2) = 0.0_dp
                                            condhoursM_reformW(wX1, wX2) = 0.0_dp
                                        end if

                                        ! conditional hours for females: note that workF_reformR>=workF_baseR

                                        if ( sum(workF_baseR) > 0.0_dp ) then
                                            condhoursF_baseW(wX1, wX2) = sum( hoursF_baseR, mask = workF_baseR > 0.0_dp ) / sum(workF_baseR)
                                            condhoursF_reformW(wX1, wX2) = sum( hoursF_reformR, mask = workF_baseR > 0.0_dp ) / sum(workF_baseR)
                                        else
                                            condhoursF_baseW(wX1, wX2) = 0.0_dp
                                            condhoursF_reformW(wX1, wX2) = 0.0_dp
                                        end if

                                    end if

                                end do
                            end do

                            ! integrate: male employment
                            workM_base(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                              pF(pX2, kX, mX), workM_baseW )
                            workM_reform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                pF(pX2, kX, mX), workM_reformW )
                            workM_basereform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                    pF(pX2, kX, mX), workM_basereformW )

                            ! integrate: male home time
                            homeM_base(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                              pF(pX2, kX, mX), homeM_baseW )
                            homeM_reform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                pF(pX2, kX, mX), homeM_reformW )

                            ! integrate: female employment
                            workF_base(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                              pF(pX2, kX, mX), workF_baseW )
                            workF_reform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                pF(pX2, kX, mX), workF_reformW )
                            workF_basereform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                    pF(pX2, kX, mX), workF_basereformW )

                            ! integrate: female home time
                            homeF_base(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                              pF(pX2, kX, mX), homeF_baseW )
                            homeF_reform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                pF(pX2, kX, mX), homeF_reformW )

                            ! integrate: male/female conditional hours

                            if ( jX == 1 ) then ! increasing male net wage

                                ! integrate: male hours
                                condhoursM_base(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                       pF(pX2, kX, mX), workM_baseW * condhoursM_baseW )
                                condhoursM_reform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                         pF(pX2, kX, mX), workM_baseW * condhoursM_reformW )
                                condhoursM_base(pX1, pX2, kX, mX) = condhoursM_base(pX1, pX2, kX, mX) / workM_base(pX1, pX2, kX, mX)
                                condhoursM_reform(pX1, pX2, kX, mX) = condhoursM_reform(pX1, pX2, kX, mX) / workM_base(pX1, pX2, kX, mX)

                                ! integrate: female hours
                                condhoursF_base(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                       pF(pX2, kX, mX), workF_basereformW * condhoursF_baseW )
                                condhoursF_reform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                         pF(pX2, kX, mX), workF_basereformW * condhoursF_reformW )
                                condhoursF_base(pX1, pX2, kX, mX) = condhoursF_base(pX1, pX2, kX, mX) / workF_basereform(pX1, pX2, kX, mX)
                                condhoursF_reform(pX1, pX2, kX, mX) = condhoursF_reform(pX1, pX2, kX, mX) / workF_basereform(pX1, pX2, kX, mX)

                            else ! increasing female net wage

                                ! integrate: male hours
                                condhoursM_base(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                       pF(pX2, kX, mX), workM_basereformW * condhoursM_baseW )
                                condhoursM_reform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                         pF(pX2, kX, mX), workM_basereformW * condhoursM_reformW )
                                condhoursM_base(pX1, pX2, kX, mX) = condhoursM_base(pX1, pX2, kX, mX) / workM_basereform(pX1, pX2, kX, mX)
                                condhoursM_reform(pX1, pX2, kX, mX) = condhoursM_reform(pX1, pX2, kX, mX) / workM_basereform(pX1, pX2, kX, mX)

                                ! integrate: female hours
                                condhoursF_base(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                       pF(pX2, kX, mX), workF_baseW * condhoursF_baseW )
                                condhoursF_reform(pX1, pX2, kX, mX) = wageIntegrateFast( h(pX1, pX2, kX, mX), pM(pX1, kX, mX), &
                                                                                         pF(pX2, kX, mX), workF_baseW * condhoursF_reformW )
                                condhoursF_base(pX1, pX2, kX, mX) = condhoursF_base(pX1, pX2, kX, mX) / workF_base(pX1, pX2, kX, mX)
                                condhoursF_reform(pX1, pX2, kX, mX) = condhoursF_reform(pX1, pX2, kX, mX) / workF_base(pX1, pX2, kX, mX)

                            end if

                        end do
                    end do
                end do
            end do

            intWorkM_base = 0.0_dp
            intWorkM_reform = 0.0_dp
            intWorkM_basereform = 0.0_dp

            intWorkF_base = 0.0_dp
            intWorkF_reform = 0.0_dp
            intWorkF_basereform = 0.0_dp

            intHomeM_base = 0.0_dp
            intHomeM_reform = 0.0_dp
            intHomeF_base = 0.0_dp
            intHomeF_reform = 0.0_dp

            intCondHoursM_base = 0.0_dp
            intCondHoursM_reform = 0.0_dp
            intCondHoursF_base = 0.0_dp
            intCondHoursF_reform = 0.0_dp

            intSize = 0.0_dp

            do mX = 1, NMARKET
                intSize = intSize + sum( m(mX)%typeDemandKid(:, :, kid0:kid1) )
                do kX = kid0, kid1
                    do pX2 = 1, NTYPE2
                        do pX1 = 1, NTYPE1

                            intWorkM_base = intWorkM_base + m(mX)%typeDemandKid(pX1, pX2, kX) * workM_base(pX1, pX2, kX, mX)
                            intWorkM_reform = intWorkM_reform + m(mX)%typeDemandKid(pX1, pX2, kX) * workM_reform(pX1, pX2, kX, mX)
                            intWorkM_basereform = intWorkM_basereform + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                * workM_basereform(pX1, pX2, kX, mX)

                            intWorkF_base = intWorkF_base + m(mX)%typeDemandKid(pX1, pX2, kX) * workF_base(pX1, pX2, kX, mX)
                            intWorkF_reform = intWorkF_reform + m(mX)%typeDemandKid(pX1, pX2, kX) * workF_reform(pX1, pX2, kX, mX)
                            intWorkF_basereform = intWorkF_basereform + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                * workF_basereform(pX1, pX2, kX, mX)

                            intHomeM_base = intHomeM_base + m(mX)%typeDemandKid(pX1, pX2, kX) * homeM_base(pX1, pX2, kX, mX)
                            intHomeM_reform = intHomeM_reform + m(mX)%typeDemandKid(pX1, pX2, kX) * homeM_reform(pX1, pX2, kX, mX)
                            intHomeF_base = intHomeF_base + m(mX)%typeDemandKid(pX1, pX2, kX) * homeF_base(pX1, pX2, kX, mX)
                            intHomeF_reform = intHomeF_reform + m(mX)%typeDemandKid(pX1, pX2, kX) * homeF_reform(pX1, pX2, kX, mX)

                            if ( jX == 1 ) then
                                intCondHoursM_base = intCondHoursM_base + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                    * workM_base(pX1, pX2, kX, mX) * condHoursM_base(pX1, pX2, kX, mX)
                                intCondHoursM_reform = intCondHoursM_reform + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                    * workM_base(pX1, pX2, kX, mX) * condHoursM_reform(pX1, pX2, kX, mX)
                                intCondHoursF_base = intCondHoursF_base + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                    * workF_basereform(pX1, pX2, kX, mX) * condHoursF_base(pX1, pX2, kX, mX)
                                intCondHoursF_reform = intCondHoursF_reform + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                    * workF_basereform(pX1, pX2, kX, mX) * condHoursF_reform(pX1, pX2, kX, mX)
                            else
                                intCondHoursM_base = intCondHoursM_base + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                    * workM_basereform(pX1, pX2, kX, mX) * condHoursM_base(pX1, pX2, kX, mX)
                                intCondHoursM_reform = intCondHoursM_reform + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                    * workM_basereform(pX1, pX2, kX, mX) * condHoursM_reform(pX1, pX2, kX, mX)
                                intCondHoursF_base = intCondHoursF_base + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                    * workF_base(pX1, pX2, kX, mX) * condHoursF_base(pX1, pX2, kX, mX)
                                intCondHoursF_reform = intCondHoursF_reform + m(mX)%typeDemandKid(pX1, pX2, kX) &
                                    * workF_base(pX1, pX2, kX, mX) * condHoursF_reform(pX1, pX2, kX, mX)
                            end if

                        end do
                    end do
                end do
            end do

            if ( jX == 1 ) then
                intCondHoursM_base = intCondHoursM_base / intWorkM_base
                intCondHoursM_reform = intCondHoursM_reform / intWorkM_base
                intCondHoursF_base = intCondHoursF_base / intWorkF_basereform
                intCondHoursF_reform = intCondHoursF_reform / intWorkF_basereform
            else
                intCondHoursM_base = intCondHoursM_base / intWorkM_basereform
                intCondHoursM_reform = intCondHoursM_reform / intWorkM_basereform
                intCondHoursF_base = intCondHoursF_base / intWorkF_base
                intCondHoursF_reform = intCondHoursF_reform / intWorkF_base
            end if

            intWorkM_base = intWorkM_base / intSize
            intWorkM_reform = intWorkM_reform / intSize
            intWorkF_base = intWorkF_base / intSize
            intWorkF_reform = intWorkF_reform / intSize
            intWorkF_basereform = intWorkF_basereform / intSize
            intHomeM_base = intHomeM_base / intSize
            intHomeM_reform = intHomeM_reform / intSize
            intHomeF_base = intHomeF_base / intSize
            intHomeF_reform = intHomeF_reform / intSize

            if ( verbose ) then
                print *
            end if

            if ( jX == 1 ) then

                elas_MarriedMaleEmploymentOwnWage = ( intWorkM_reform / intWorkM_base - 1.0_dp ) / ELAS_STEP
                elas_MarriedMaleEmploymentOwnWage2 = ( intWorkM_reform - intWorkM_base ) / ELAS_STEP
                elas_MarriedMaleCondHoursOwnWage = ( intCondHoursM_reform / intCondHoursM_base - 1.0_dp ) / ELAS_STEP
                elas_MarriedMaleHomeOwnWage = ( intHomeM_reform / intHomeM_base - 1.0_dp ) / ELAS_STEP

                elas_MarriedFemaleEmploymentCrossWage = ( intWorkF_reform / intWorkF_base - 1.0_dp ) / ELAS_STEP
                elas_MarriedFemaleEmploymentCrossWage2 = ( intWorkF_reform - intWorkF_base ) / ELAS_STEP
                elas_MarriedFemaleCondHoursCrossWage = ( intCondHoursF_reform / intCondHoursF_base - 1.0_dp ) / ELAS_STEP
                elas_MarriedFemaleHomeCrossWage =  ( intHomeF_reform / intHomeF_base - 1.0_dp ) / ELAS_STEP

                if ( verbose ) then
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'male employment own wage', &
                        elas_MarriedMaleEmploymentOwnWage, rfp(elas_MarriedMaleEmploymentOwnWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'female employment cross wage', &
                        elas_MarriedFemaleEmploymentCrossWage, rfp(elas_MarriedFemaleEmploymentCrossWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'male employment own wage 2', &
                        elas_MarriedMaleEmploymentOwnWage2, rfp(elas_MarriedMaleEmploymentOwnWage2)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'female employment cross wage 2', &
                        elas_MarriedFemaleEmploymentCrossWage2, rfp(elas_MarriedFemaleEmploymentCrossWage2)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'male conditional hours own wage', &
                        elas_MarriedMaleCondHoursOwnWage, rfp(elas_MarriedMaleCondHoursOwnWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'female conditional hours cross wage', &
                        elas_MarriedFemaleCondHoursCrossWage, rfp(elas_MarriedFemaleCondHoursCrossWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'male home time own wage', &
                        elas_MarriedMaleHomeOwnWage, rfp(elas_MarriedMaleHomeOwnWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'female home time cross wage', &
                        elas_MarriedFemaleHomeCrossWage, rfp(elas_MarriedFemaleHomeCrossWage)
                end if

            else

                elas_MarriedMaleEmploymentCrossWage = ( intWorkM_reform / intWorkM_base - 1.0_dp ) / ELAS_STEP
                elas_MarriedMaleEmploymentCrossWage2 = ( intWorkM_reform - intWorkM_base ) / ELAS_STEP
                elas_MarriedMaleCondHoursCrossWage = ( intCondHoursM_reform / intCondHoursM_base - 1.0_dp ) / ELAS_STEP
                elas_MarriedMaleHomeCrossWage = ( intHomeM_reform / intHomeM_base - 1.0_dp ) / ELAS_STEP

                elas_MarriedFemaleEmploymentOwnWage = ( intWorkF_reform / intWorkF_base - 1.0_dp ) / ELAS_STEP
                elas_MarriedFemaleEmploymentOwnWage2 = ( intWorkF_reform - intWorkF_base ) / ELAS_STEP
                elas_MarriedFemaleCondHoursOwnWage = ( intCondHoursF_reform / intCondHoursF_base - 1.0_dp ) / ELAS_STEP
                elas_MarriedFemaleHomeOwnWage =  ( intHomeF_reform / intHomeF_base - 1.0_dp ) / ELAS_STEP

                if ( verbose ) then
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'male employment cross wage', &
                        elas_MarriedMaleEmploymentCrossWage, rfp(elas_MarriedMaleEmploymentCrossWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'female employment own wage', &
                        elas_MarriedFemaleEmploymentOwnWage, rfp(elas_MarriedFemaleEmploymentOwnWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'male employment cross wage 2', &
                        elas_MarriedMaleEmploymentCrossWage2, rfp(elas_MarriedMaleEmploymentCrossWage2)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'female employment own wage 2', &
                        elas_MarriedFemaleEmploymentOwnWage2, rfp(elas_MarriedFemaleEmploymentOwnWage2)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'male conditional hours cross wage', &
                        elas_MarriedMaleCondHoursCrossWage, rfp(elas_MarriedMaleCondHoursCrossWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'female conditional hours own wage', &
                        elas_MarriedFemaleCondHoursOwnWage, rfp(elas_MarriedFemaleCondHoursOwnWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'male home time cross wage', &
                        elas_MarriedMaleHomeCrossWage, rfp(elas_MarriedMaleHomeCrossWage)
                    write(*, '((A),",",2X,F8.5,",",2X,F8.5)') 'female home time own wage', &
                        elas_MarriedFemaleHomeOwnWage, rfp(elas_MarriedFemaleHomeOwnWage)
                end if

            end if

        end do ! jX (MALE/FEMALE)

        if ( ftag > 0 ) then

            ! work hours
            ! ----------

            open(newunit = funit, file = outpath // '/elas_MarriedMaleCondHoursOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedMaleCondHoursOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedFemaleCondHoursOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedFemaleCondHoursOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_SingleMaleCondHoursOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_SingleMaleCondHoursOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_SingleFemaleCondHoursOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_SingleFemaleCondHoursOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedMaleCondHoursCrossWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedMaleCondHoursCrossWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedFemaleCondHoursCrossWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedFemaleCondHoursCrossWage
            close(funit)

            ! participation
            ! -------------

            open(newunit = funit, file = outpath // '/elas_MarriedMaleEmploymentOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedMaleEmploymentOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedFemaleEmploymentOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedFemaleEmploymentOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_SingleMaleEmploymentOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_SingleMaleEmploymentOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_SingleFemaleEmploymentOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_SingleFemaleEmploymentOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedMaleHomeCrossWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedMaleHomeCrossWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedFemaleEmploymentCrossWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedFemaleEmploymentCrossWage
            close(funit)

            ! home hours
            ! ----------

            open(newunit = funit, file = outpath // '/elas_MarriedMaleHomeOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedMaleHomeOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedFemaleHomeOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedFemaleHomeOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_SingleMaleHomeOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_SingleMaleHomeOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_SingleFemaleHomeOwnWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_SingleFemaleHomeOwnWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedMaleEmploymentCrossWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedMaleEmploymentCrossWage
            close(funit)

            open(newunit = funit, file = outpath // '/elas_MarriedFemaleHomeCrossWage' // intToStr(ftag) // '.csv', &
                 status = 'replace', action = 'write')
            write(funit, *) elas_MarriedFemaleHomeCrossWage
            close(funit)

        else

            open(newunit = funit, file = texpath, status = 'replace', action = 'write')
            write(funit, '(A)') "\begin{tabular}{lcccc}"
            write(funit, '(A)') "\toprule"
            write(funit, '(A)') "&\multicolumn{2}{c}{Married}&\multicolumn{2}{c}{Single}\\"
            write(funit, '(A)') "\cmidrule(lr){2-3}\cmidrule(lr){4-5}"
            write(funit, '(A)') "& Men& Women& Men& Women\\"
            write(funit, '(A)') "\midrule"

            write(funit, '(A)') "\emph{Work hours} & & & & \\"
            write(funit, '(A)') "\quad Own-wage elasticity &" // toStrTexF(elas_MarriedMaleCondHoursOwnWage, '(F5.2)', .true.) // " & " // &
                toStrTexF(elas_MarriedFemaleCondHoursOwnWage, '(F5.2)', .true.) // " & " // toStrTexF(elas_SingleMaleCondHoursOwnWage, '(F5.2)', .true.) // " & " // &
                toStrTexF(elas_SingleFemaleCondHoursOwnWage, '(F5.2)', .true.) // "\\"
            write(funit, '(A)') "\quad Cross-wage elasticity &" // toStrTexF(elas_MarriedMaleCondHoursCrossWage, '(F5.2)', .true.) // " & " // &
                toStrTexF(elas_MarriedFemaleCondHoursCrossWage, '(F5.2)', .true.) // " & " // "--" // " & " // "--" // "\\"
            write(funit, '(A)') "\emph{Participation} & & & & \\"
            write(funit, '(A)') "\quad Own-wage elasticity &" // toStrTexF(elas_MarriedMaleEmploymentOwnWage, '(F5.2)', .true.) // " & " // &
                toStrTexF(elas_MarriedFemaleEmploymentOwnWage, '(F5.2)', .true.) // " & " // toStrTexF(elas_SingleMaleEmploymentOwnWage, '(F5.2)', .true.) // " & " // &
                toStrTexF(elas_SingleFemaleEmploymentOwnWage, '(F5.2)', .true.) // "\\"
            write(funit, '(A)') "\quad Cross-wage elasticity &" // toStrTexF(elas_MarriedMaleEmploymentCrossWage, '(F5.2)', .true.) // " & " // &
                toStrTexF(elas_MarriedFemaleEmploymentCrossWage, '(F5.2)', .true.) // " & " // "--" // " & " // "--" // "\\"

            write(funit, '(A)') "\emph{Home hours} & & & & \\"
            write(funit, '(A)') "\quad Own-wage elasticity &", toStrTexF(elas_MarriedMaleHomeOwnWage, '(F5.2)', .true.) // " & " // &
                toStrTexF(elas_MarriedFemaleHomeOwnWage, '(F5.2)', .true.) // " & " // toStrTexF(elas_SingleMaleHomeOwnWage, '(F5.2)', .true.) // " & " // toStrTexF(elas_SingleFemaleHomeOwnWage, '(F5.2)', .true.) // "\\"
            write(funit, '(A)') "\quad Cross-wage elasticity &" // toStrTexF(elas_MarriedMaleHomeCrossWage, '(F5.2)', .true.) // " & " // &
                toStrTexF(elas_MarriedFemaleHomeCrossWage, '(F5.2)', .true.) // " & " // "--" // " & " // "--" // "\\"


            write(funit, '(A)') "\bottomrule"
            write(funit, '(A)') "\end{tabular}"

            close(funit)

        end if

    end subroutine elas


    pure function toStrTexF(x, rfmt, pneg) result(res)
        character(:), allocatable :: res
        real(dp), intent(in) :: x
        character(len = *), intent(in) :: rfmt
        logical, intent(in) :: pneg
        character(64) :: tempStr64
        write (tempStr64, rfmt) x
        tempStr64 = adjustl(tempStr64)
        if ((.not. pneg) .or. tempStr64(1:1) == '-') then
            res = trim(adjustl(tempStr64))
        else
            res = '\phantom{-}' // trim(adjustl(tempStr64))
        end if
    end function toStrTexF

    pure function lenToStrTexF(x, rfmt, pneg)
        implicit none
        real(dp), intent(in) :: x
        character(len = *), intent(in) :: rfmt
        logical, intent(in) :: pneg
        integer :: lenToStrTexF
        character(64) :: tempStr64
        write (tempStr64, rfmt) x
        tempStr64 = adjustl(tempStr64)
        if ((.not. pneg) .or. tempStr64(1:1) == '-') then
            lenToStrTexF = len_trim(tempStr64)
        else
            lenToStrTexF = len_trim('\phantom{-}') + len_trim(tempStr64)
        end if
    end function lenToStrTexF

    subroutine new( tax, m, h, pM, pF, grid, rnd, opt, eallM, eallF, eall, lamfixed )

        use taxes
        use mathfn
        use moments, only : aggregateEallSingle
!$      use omp_lib
        implicit none

        type(tax_t), intent(in) :: tax
        type(m_t), intent(inout) :: m
        type(h_t), intent(in) :: h(NTYPE1, NTYPE2, 0:1)
        type(p_t), intent(in) :: pM(NTYPE1, 0:1)
        type(p_t), intent(in) :: pF(NTYPE2, 0:1)
        type(grid_t), intent(in) :: grid
        type(rnd_t), intent(in) :: rnd
        type(opt_t), intent(in)  :: opt

        ! allocations
        type(eall_t), intent(out) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(out) :: eallF(NTYPE2, 0:1)
        type(eall2_t), intent(out) :: eall(NTYPE1, NTYPE2, 0:1)

        type(eall_t) :: eallM2(NTYPE1, 1:2)
        type(v_t) :: vM2(NTYPE1, 1:2)
        real(dp) :: prob(2)
        type(eall_t) :: tempVec2(2)

        real(dp), intent(in), optional :: lamfixed(NTYPE1, NTYPE2)

        type(v_t) :: vM(NTYPE1, 0:1)
        type(v_t) :: vF(NTYPE2, 0:1)

        real(dp), parameter :: lam_low = 0.01_dp
        real(dp), parameter :: lam_high = 1.0_dp - lam_low

        integer :: nlam
        integer :: nlam_main
        real(dp) :: netinc(N, N, NODEW, NODEW)
        real(dp) :: lamNEW(NTYPE1, NTYPE2)
        real(dp) :: EuLam1(NTYPE1, NTYPE2)
        real(dp) :: EuLam2(NTYPE1, NTYPE2)
        type(v2_t) :: v
        integer :: pX1, pX2, lX, kX
        real(dp) :: s1, s2

        real(dp) :: cons
        real(dp) :: sumExpEuLam1(NTYPE1), sumExpEuLam2(NTYPE2)
        real(dp) :: Eu1(NTYPE1)
        real(dp) :: Eu2(NTYPE2)
        real(dp) :: expEuLam1(NTYPE1, NTYPE2)
        real(dp) :: expEuLam2(NTYPE1, NTYPE2)

        real(dp) :: pdiff, ptol
        integer  :: piter, niter
        real(dp) :: PrSingle1(NTYPE1)
        real(dp) :: PrSingle2(NTYPE2)
        real(dp) :: PrSingleOld1(NTYPE1)
        real(dp) :: PrSingleOld2(NTYPE2)

        real(dp) :: expEu1(NTYPE1)
        real(dp) :: expEu2(NTYPE2)
        real(dp) :: consEu1(NTYPE1)
        real(dp) :: consEu2(NTYPE2)
        real(dp) :: tempMax

        real(dp) :: tempEu2(NTYPE2, 0:1)
        logical  :: solveEquilibrium
        real(dp) :: tempEuLamEq1(NTYPE1, NTYPE2, 0:1)
        real(dp) :: tempEuLamEq2(NTYPE1, NTYPE2, 0:1)
        real(dp) :: earn(NEARN)
        real(dp) :: forceAssortativeScale
        integer :: upf_unit

        real(dp), allocatable :: dEuLam12(:)
        real(dp), allocatable :: lamgrid(:)
        real(dp), allocatable :: EuLamGrid1(:, :, :)
        real(dp), allocatable :: EuLamGrid2(:, :, :)
        real(dp), allocatable :: tempEuLamGrid1(:, :, :, :)
        real(dp), allocatable :: tempEuLamGrid2(:, :, :, :)

        nlam_main = opt%nlam
        nlam = opt%nlam

        allocate(dEuLam12(nlam))
        allocate(lamgrid(nlam))
        allocate(EuLamGrid1(nlam, NTYPE1, NTYPE2))
        allocate(EuLamGrid2(nlam, NTYPE1, NTYPE2))
        allocate(tempEuLamGrid1(nlam, NTYPE1, NTYPE2, 0:1))
        allocate(tempEuLamGrid2(nlam, NTYPE1, NTYPE2, 0:1))

        ! forceAssortativeScale = opt%forceAssortativeScale
        forceAssortativeScale = opt%utilityShift

        call linspace(0.0_dp, 4000.0_dp, NEARN, earn)

        ! check for optional arguments
        if ( present(lamfixed) ) then
            solveEquilibrium = .false.
        else
            solveEquilibrium = .true.
        end if

        do kX = 0, 2
            do pX1 = 1, NTYPE1

                select case(kX)
                case(0)
                    call utilityPreCalc( pM(pX1, kX), rnd, tax, grid, kX, kX, SEX_MALE, opt, vM(pX1, kX) )
                    call expectedUtility( pM(pX1, kX), vM(pX1, kX), grid, rnd, opt, m%theta, .true., eallM(pX1, kX) )
                    if ( opt%mode == MODE_MOMENTS ) then
                        call moment( grid, eallM(pX1, kX) )
                    end if
                case(1)
                    call utilityPreCalc( pM(pX1, kX), rnd, tax, grid, kX, kX, SEX_MALE, opt, vM2(pX1, kX) )
                    call expectedUtility( pM(pX1, kX), vM2(pX1, kX), grid, rnd, opt, m%theta, .true., eallM2(pX1, kX) )
                    if ( opt%mode == MODE_MOMENTS ) then
                        call moment( grid, eallM2(pX1, kX) )
                    end if
                case(2)
                    ! kids, but not in tax schedule
                    call utilityPreCalc( pM(pX1, 1), rnd, tax, grid, 1, 0, SEX_MALE, opt, vM2(pX1, kX) )
                    call expectedUtility( pM(pX1, 1), vM2(pX1, kX), grid, rnd, opt, m%theta, .true., eallM2(pX1, kX) )
                    if ( opt%mode == MODE_MOMENTS ) then
                        call moment( grid, eallM2(pX1, kX) )
                    end if
                end select
            end do
        end do

        do pX1 = 1, NTYPE1
            prob(1) = pM(pX1, 1)%probKidTax
            prob(2) = 1.0_dp - prob(1)
            tempVec2(1) = eallM2(pX1, 1)
            tempVec2(2) = eallM2(pX1, 2)
            call aggregateEallSingle(2, prob, tempVec2, eallM(pX1, 1))
        end do

        Eu1 = 0.0_dp
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                Eu1(pX1) = Eu1(pX1) + pM(pX1, kX)%probTypeK * eallM(pX1, kX)%Eu
            end do
        end do


        ! unmatched (female)
        do kX = 0, 1
            do pX2 = 1, NTYPE2

                call utilityPreCalc( pF(pX2, kX), rnd, tax, grid, kX, kX, SEX_FEMALE, opt, vF(pX2, kX) )
                call expectedUtility( pF(pX2, kX), vF(pX2, kX), grid, rnd, opt, m%theta, .true., eallF(pX2, kX) )
                if ( opt%mode == MODE_MOMENTS ) then
                    call moment( grid, eallF(pX2, kX) )
                end if
                tempEu2(pX2, kX) = pF(pX2, kX)%probTypeK * eallF(pX2, kX)%Eu

            end do
        end do

        Eu2 = 0.0_dp
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                Eu2(pX2) = Eu2(pX2) + tempEu2(pX2, kX)
            end do
        end do

        if ( solveEquilibrium ) then

            call linSpace(lam_low, lam_high, nlam_main, lamgrid)

            !$omp parallel do collapse(3) &
            !$omp default(shared) private(pX1, pX2, kX, lX, v, netinc)
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1

                        ! calculate everything that does not depend upon the pareto weights
                        call householdUtilityPreCalc( h(pX1, pX2, kX), pM(pX1, kX), pF(pX2, kX), rnd, tax, grid, kX, &
                            opt, v, netinc, jointTaxes = .true. )

                        do lX = 1, nlam

                            call cpu_time(s1)
                            call householdUtilityPreCalcLambda( h(pX1, pX2, kX), pM(pX1, kX), pF(pX2, kX), tax, lamgrid(lX), &
                                                                grid, netinc, opt, v )

                            call expectedUtilityHouseholdWelfareFast( h(pX1, pX2, kX), pM(pX1, kX), pF(pX2, kX), v, &
                                    lamgrid(lX), rnd, opt, grid, eall(pX1, pX2, kX), m%theta )

                            tempEuLamGrid1(lX, pX1, pX2, kX) = h(pX1, pX2, kX)%probTypeK * eall(pX1, pX2, kX)%Eu1
                            tempEuLamGrid2(lX, pX1, pX2, kX) = h(pX1, pX2, kX)%probTypeK * eall(pX1, pX2, kX)%Eu2
                            call cpu_time(s2)

                        end do

                    end do !pX1
                end do !pX2
            end do !kX
            !$omp end parallel do

            EuLamGrid1 = 0.0_dp
            EuLamGrid2 = 0.0_dp
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        EuLamGrid1(:, pX1, pX2) = EuLamGrid1(:, pX1, pX2) + tempEuLamGrid1(:, pX1, pX2, kX)
                        EuLamGrid2(:, pX1, pX2) = EuLamGrid2(:, pX1, pX2) + tempEuLamGrid2(:, pX1, pX2, kX)
                    end do
                end do
            end do

            ! initialize counters and distance
            pdiff = huge(0.0_dp)
            ptol = epsilon(0.0_dp)
            piter = 0
            niter = 200

            ! guess of single probabilities
            PrSingle1 = 0.1_dp
            PrSingle2 = 0.1_dp

            ! check value functions are monotonic in lX conditional on (pX1, pX2)
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do lX = 2, nlam
                        if ( EuLamGrid1(lX, pX1, pX2) >= EuLamGrid1(lX - 1, pX1, pX2) ) then
                            print *, lX, pX1, pX2
                            print *, 'male value not monotonically decreasing in female weight'
                        end if

                        if ( EuLamGrid2(lX, pX1, pX2) <= EuLamGrid2(lX - 1, pX1, pX2) ) then
                            print *, lX, pX1, pX2
                            print *, 'female value not monotonically increasing in female weight'
                        end if

                    end do
                end do
            end do

            do while ( pdiff > ptol .and. piter < niter )

                ! update iteration counter
                piter = piter + 1

                ! keep track of previous single probabilities
                PrSingleOld1 = PrSingle1
                PrSingleOld2 = PrSingle2

                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1

                        cons = -( ( Eu1(pX1) - Eu2(pX2) ) / m%sdTheta0 + log( pF(pX2, 0)%frac * PrSingle2(pX2) ) &
                            - log( pM(pX1, 0)%frac * PrSingle1(pX1) ) )
                        dEuLam12 = ( EuLamGrid2(:, pX1, pX2) - EuLamGrid1(:, pX1, pX2) ) / m%sdTheta0
                        lamNEW(pX1, pX2) = lininter(dEuLam12, lamgrid, nlam, cons)
                        lamNEW(pX1, pX2) = min( max(lamgrid(1), lamNEW(pX1, pX2)), lamgrid(nlam) )

                        EuLam1(pX1, pX2) = lininter( lamgrid, EuLamGrid1(:, pX1, pX2), nlam, lamNEW(pX1, pX2) )
                        EuLam2(pX1, pX2) = lininter( lamgrid, EuLamGrid2(:, pX1, pX2), nlam, lamNEW(pX1, pX2) )

                    end do
                end do

                if ( forceAssortativeScale /= 0.0_dp ) then
                    do pX1 = 1, NTYPE1
                        EuLam1(pX1, pX1) = EuLam1(pX1, pX1) + forceAssortativeScale * m%sdTheta0
                        EuLam2(pX1, pX1) = EuLam2(pX1, pX1) + forceAssortativeScale * m%sdTheta0
                    end do
                end if

                do pX1 = 1, NTYPE1
                    tempMax = maxval( EuLam1(pX1, :), dim=1 )
                    consEu1(pX1) = max( Eu1(pX1), tempMax )
                end do

                do pX2 = 1, NTYPE2
                    tempMax = maxval( EuLam2(:, pX2), dim=1 )
                    consEu2(pX2) = max( Eu2(pX2), tempMax )
                end do

                expEu1 = exp( ( Eu1 - consEu1 ) / m%sdTheta0 )
                expEu2 = exp( ( Eu2 - consEu2 ) / m%sdTheta0 )

                do pX2 = 1, NTYPE2
                    expEuLam1(:, pX2) = exp( (EuLam1(:, pX2) - consEu1) / m%sdTheta0 )
                end do

                do pX1 = 1, NTYPE1
                    sumExpEuLam1(pX1) = expEu1(pX1) + sum( expEuLam1(pX1, :) )
                    PrSingle1(pX1) = expEu1(pX1) / sumExpEuLam1(pX1)
                end do

                do pX2 =1 , NTYPE2
                    expEuLam2(:, pX2) = exp( (EuLam2(:, pX2) - consEu2(pX2)) / m%sdTheta0 )
                end do

                do pX2 = 1, NTYPE2
                    sumExpEuLam2(pX2) = expEu2(pX2) + sum( expEuLam2(:, pX2) )
                    PrSingle2(pX2) = expEu2(pX2) / sumExpEuLam2(pX2)
                end do

                pdiff = sum( abs(PrSingle1 - PrSingleOld1) ) + sum( abs(PrSingle2 - PrSingleOld2) )

                if ( opt%verbose ) then
                    print *, piter, pdiff
                end if

            end do

            if ( opt%saveUpf ) then

                open(newunit = upf_unit, file = opt%savePath // '/upf_market' // intToStr(m%marketId) // '.csv', &
                    status = 'replace', action = 'write')

                do pX1 = 1, NTYPE1
                    write(upf_unit, '(I4,",",I4,",",I4,",",F12.8,",",F12.8)') 0, pX1, 0, Eu1(pX1), 0.0_dp
                end do
                do pX2 = 1, NTYPE2
                    write(upf_unit, '(I4,",",I4,",",I4,",",F12.8,",",F12.8)') 0, 0, pX2, 0.0_dp, Eu2(pX2)
                end do

                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        write(upf_unit, '(I4,",",I4,",",I4,",",F12.8,",",F12.8)') -9,  pX1, pX2, EuLam1(pX1, pX2), &
                            EuLam2(pX1, pX2)
                        write(upf_unit, '(I4,",",I4,",",I4,",",F12.8,",",F12.8)') -8,  pX1, pX2, 1.0_dp - lamNEW(pX1, pX2), &
                            lamNEW(pX1, pX2)
                        do lX = 1, nlam
                            write(upf_unit, '(I4,",",I4,",",I4,",",F12.8,",",F12.8)') lX, pX1, pX2, &
                                EuLamGrid1(lX, pX1, pX2), EuLamGrid2(lX, pX1, pX2)
                        end do
                    end do
                end do

                close(upf_unit)

            end if

            m%lam = lamNEW
            m%PrSingle1 = PrSingle1
            m%PrSingle2 = PrSingle2
            m%numSingle1 = PrSingle1 * pM(:, 0)%frac
            m%numSingle2 = PrSingle2 * pF(:, 0)%frac

            if ( opt%verbose ) then
                print *, 'begin weights'
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        print *, pX1, pX2, m%lam(pX1, pX2)
                    end do
                end do
                print *, 'end weights'
            end if

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    m%Pr1(pX1, pX2) = expEuLam1(pX1, pX2) / sumExpEuLam1(pX1) ! prob type 1 male chooses type 2 female
                    m%Pr2(pX1, pX2) = expEuLam2(pX1, pX2) / sumExpEuLam2(pX2) ! prob type 2 female chooses type 1 male
                end do
            end do

            ! demand of type pX1 males for type pX2 females
            ! supply of type pX2 females for type pX1 males
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    m%typeDemand(pX1, pX2) = pM(pX1, 0)%frac * m%Pr1(pX1, pX2)
                    m%typeSupply(pX1, pX2) = pF(pX2, 0)%frac * m%Pr2(pX1, pX2)
                    m%excessDemand(pX1, pX2) = m%typeDemand(pX1, pX2) - m%typeSupply(pX1, pX2)
                end do
            end do

            if ( opt%verbose ) then
                print *, 'approximate: demand/supply/excess demand'
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        print *, m%typeDemand(pX1, pX2), m%typeSupply(pX1, pX2), m%excessDemand(pX1, pX2)
                    end do
                end do
            end if

        else

            m%lam = lamfixed

        end if ! solveEquilibrium

        ! calculate allocations at equilibrium weights

        !$omp parallel do collapse(3) &
        !$omp default(shared) private(pX1, pX2, kX, v, netinc)
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                do kX = 0, 1

                    ! calculate everything that does not depend upon the pareto weights
                    call householdUtilityPreCalc( h(pX1, pX2, kX), pM(pX1, kX), pF(pX2, kX), rnd, tax, grid, kX, &
                        opt, v, netinc, jointTaxes = .true. )

                    if (opt%swfInc) then
                        v%swfInc = .true.
                    end if

                    call householdUtilityPreCalcLambda( h(pX1, pX2, kX), pM(pX1, kX), pF(pX2, kX), tax, m%lam(pX1, pX2), &
                        grid, netinc, opt, v )

                        if ( opt%mode == MODE_MOMENTS ) then

                            call expectedUtilityHouseholdFast( h(pX1, pX2, kX), pM(pX1, kX), pF(pX2, kX), v, m%lam(pX1, pX2), &
                                rnd, opt, grid, eall(pX1, pX2, kX) )

                            call momentHousehold( grid, eall(pX1, pX2, kX) )
                    else
                        call expectedUtilityHouseholdWelfareFast( h(pX1, pX2, kX), pM(pX1, kX), pF(pX2, kX), v, &
                            m%lam(pX1, pX2), rnd, opt, grid, eall(pX1, pX2, kX), m%theta )
                        end if

                    tempEuLamEq1(pX1, pX2, kX) = h(pX1, pX2, kX)%probTypeK * eall(pX1, pX2, kX)%Eu1
                    tempEuLamEq2(pX1, pX2, kX) = h(pX1, pX2, kX)%probTypeK * eall(pX1, pX2, kX)%Eu2

                end do !kX
            end do !pX1
        end do !pX2
        !$omp end parallel do

        ! expected demands at equilibrium weights
        if ( ( .not. solveEquilibrium ) .and. ( .not. opt%fixMarriageMarket ) ) then

            EuLam1 = 0.0_dp
            EuLam2 = 0.0_dp
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        EuLam1(pX1, pX2) = EuLam1(pX1, pX2) + tempEuLamEq1(pX1, pX2, kX)
                        EuLam2(pX1, pX2) = EuLam2(pX1, pX2) + tempEuLamEq2(pX1, pX2, kX)
                    end do
                end do
            end do

            if ( forceAssortativeScale /= 0.0_dp ) then
                do pX1 = 1, NTYPE1
                    EuLam1(pX1, pX1) = EuLam1(pX1, pX1) + forceAssortativeScale * m%sdTheta0
                    EuLam2(pX1, pX1) = EuLam2(pX1, pX1) + forceAssortativeScale * m%sdTheta0
                end do
            end if

            do pX1 = 1, NTYPE1
                tempMax = maxval( EuLam1(pX1, :), dim = 1 )
                consEu1(pX1) = max( Eu1(pX1), tempMax )
            end do

            do pX2 = 1, NTYPE2
                tempMax = maxval( EuLam2(:, pX2), dim = 1 )
                consEu2(pX2) = max( Eu2(pX2), tempMax )
            end do

            expEu1 = exp( ( Eu1 - consEu1 ) / m%sdTheta0 )
            expEu2 = exp( ( Eu2 - consEu2 ) / m%sdTheta0 )

            do pX2 = 1, NTYPE2
                expEuLam1(:, pX2) = exp( (EuLam1(:, pX2) - consEu1) / m%sdTheta0 )
            end do

            do pX1 = 1, NTYPE1
                sumExpEuLam1(pX1) = expEu1(pX1) + sum( expEuLam1(pX1, :) )
                PrSingle1(pX1) = expEu1(pX1) / sumExpEuLam1(pX1)
            end do

            do pX2 =1 , NTYPE2
                expEuLam2(:, pX2) = exp( (EuLam2(:, pX2) - consEu2(pX2)) / m%sdTheta0 )
            end do

            do pX2 = 1, NTYPE2
                sumExpEuLam2(pX2) = expEu2(pX2) + sum( expEuLam2(:, pX2) )
                PrSingle2(pX2) = expEu2(pX2) / sumExpEuLam2(pX2)
            end do

            m%PrSingle1 = PrSingle1
            m%PrSingle2 = PrSingle2
            m%numSingle1 = PrSingle1 * pM(:, 0)%frac
            m%numSingle2 = PrSingle2 * pF(:, 0)%frac

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    m%Pr1(pX1, pX2) = expEuLam1(pX1, pX2) / sumExpEuLam1(pX1) ! prob type 1 male chooses type 2 female
                    m%Pr2(pX1, pX2) = expEuLam2(pX1, pX2) / sumExpEuLam2(pX2) ! prob type 2 female chooses type 1 male
                end do
            end do

            ! demand of type pX1 males for type pX2 females
            ! supply of type pX2 females for type pX1 males
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    m%typeDemand(pX1, pX2) = pM(pX1, 0)%frac * m%Pr1(pX1, pX2)
                    m%typeSupply(pX1, pX2) = pF(pX2, 0)%frac * m%Pr2(pX1, pX2)
                    m%excessDemand(pX1, pX2) = m%typeDemand(pX1, pX2)-m%typeSupply(pX1, pX2)
                end do
            end do

            if ( opt%verbose ) then
                print *, 'evaluation: demand/supply/excess demand'
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        print *, m%typeDemand(pX1, pX2), m%typeSupply(pX1, pX2), m%excessDemand(pX1, pX2)
                    end do
                end do
            end if

        end if

        ! calculate measures with children
        ! --------------------------------

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    m%typeDemandKid(pX1, pX2, kX) = m%typeDemand(pX1, pX2) * h(pX1, pX2, kX)%probTypeK
                    m%typeSupplyKid(pX1, pX2, kX) = m%typeSupply(pX1, pX2) * h(pX1, pX2, kX)%probTypeK
                end do
            end do
        end do

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                m%numSingleKid1(pX1, kX) = m%numSingle1(pX1) * pM(pX1, kX)%probTypeK
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                m%numSingleKid2(pX2, kX) = m%numSingle2(pX2) * pF(pX2, kX)%probTypeK
            end do
        end do

        if (solveEquilibrium) then
            if (allocated(m%EuLamGrid1)) deallocate(m%EuLamGrid1)
            if (allocated(m%EuLamGrid2)) deallocate(m%EuLamGrid2)
            if (allocated(m%lamgrid))    deallocate(m%lamgrid)
            allocate(m%EuLamGrid1(opt%nlam, NTYPE1, NTYPE2))
            allocate(m%EuLamGrid2(opt%nlam, NTYPE1, NTYPE2))
            allocate(m%lamgrid(opt%nlam))
            m%lamgrid = lamgrid
        end if

        ! save male expected utility in m
        do pX1 = 1, NTYPE1
            m%Eu1(pX1, 0) = Eu1(pX1)
        end do
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                m%Eu1(pX1, pX2) = EuLam1(pX1, pX2)
                if (solveEquilibrium) then
                    m%EuLamGrid1(:, pX1, pX2) = EuLamGrid1(:, pX1, pX2)
                end if
            end do
        end do

        ! save female expected utility in m
        do pX2 = 1, NTYPE2
            m%Eu2(0, pX2) = Eu2(pX2)
        end do
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                m%Eu2(pX1, pX2) = EuLam2(pX1, pX2)
                if (solveEquilibrium) then
                    m%EuLamGrid2(:, pX1, pX2) = EuLamGrid2(:, pX1, pX2)
                end if
            end do
        end do

    end subroutine new

    subroutine calculateMarriagePenalty2(tax, tmode, kids, nearngrid, famearn, frac, bonus_penalty)
        use taxes
        use mathfn
        implicit none
        type(tax_t), intent(in) :: tax
        integer, intent(in) :: tmode
        real(dp), intent(in) :: famearn
        integer, intent(in) :: kids
        integer, intent(in) :: nearngrid
        real(dp), intent(out) :: frac(nearngrid)
        real(dp), intent(out) :: bonus_penalty(nearngrid)

        integer :: eX
        real(dp) :: earn1, earn2, tottax(nearngrid), tottaxSin(nearngrid)
        real(dp) :: tottaxSin1(nearngrid), tottaxSin2(nearngrid)
        real(dp) :: tottaxSinNoKids1(nearngrid), tottaxSinNoKids2(nearngrid)

        call linSpace(0.0_dp, 1.0_dp, nearngrid, frac)

        do eX = 1, nearngrid
            earn1 = frac(eX) * famearn
            earn2 = (1.0_dp - frac(eX)) * famearn
            tottax(eX) = evalTax( tax, tmode, kids, earn1, earn2 )

            tottaxSin1(eX) = evalTax( tax, tmode, kids, SEX_MALE, earn1 )
            tottaxSin2(eX) = evalTax( tax, tmode, kids, SEX_FEMALE, earn2 )

            if (kids > 0) then
                tottaxSinNoKids1(eX) = evalTax( tax, tmode, 0, SEX_MALE, earn1 )
                tottaxSinNoKids2(eX) = evalTax( tax, tmode, 0, SEX_FEMALE, earn2 )
            end if

        end do

        if (kids > 0) then
            tottaxSin = 0.5_dp * ( tottaxSin1 + tottaxSin2 + tottaxSinNoKids1 + tottaxSinNoKids2 )
        else
            tottaxSin = tottaxSin1 + tottaxSin2
        end if

        bonus_penalty = ( tottaxSin - tottax  )

    end subroutine calculateMarriagePenalty2

    subroutine marriagePenalty(opt, nearngrid, npen, famearn, kids, frac, bonus_penalty)
        use environment
        use mathfn
        implicit none
        type(opt_t), intent(in) :: opt
        integer, intent(in) :: nearngrid, npen
        real(dp), intent(in) :: famearn(npen)
        integer, intent(in) :: kids
        real(dp), intent(out) :: frac(nearngrid,npen)
        real(dp), intent(out) :: bonus_penalty(nearngrid, npen)
        real(dp) :: bonus_penalty_market(nearngrid, npen, NMARKET)

        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t) :: m
        type(tax_t) :: tax
        integer :: mX, fX

        bonus_penalty = 0.0_dp

        do mX = 1, NMARKET
            call readFracKids( 'input/type_' // opt%locname // '_market' // intToStr(mX) // '.asc', .false., pM, pF, h, m )
            call readTaxes( 'taxes/output', m%taxId, .false., tax )
            if (opt%medicaid) then
                call readMedicaid( 'taxes/output', 'input/medicaid.asc', m%taxId, .false., tax )
                tax%medicaid = .true.
            else
                tax%medicaid = .false.
            end if
            do fX = 1, npen
                call calculateMarriagePenalty2( tax, 0, kids, nearngrid, famearn(fX), frac(:, fX), bonus_penalty_market(:, fX, mX) )
            end do
            bonus_penalty = bonus_penalty + m%marketSize * bonus_penalty_market(:, :, mX)
        end do

    end subroutine marriagePenalty

    subroutine marriagePenaltySim(opt, xtax, nearngrid, npen, famearn, kids, frac, bonus_penalty)
        use taxes
        use environment
        implicit none
        type(opt_t), intent(in) :: opt
        real(dp), intent(in) :: xtax(NTAXPARAM)
        integer, intent(in) :: nearngrid, npen
        real(dp), intent(in) :: famearn(npen)
        integer, intent(in) :: kids
        real(dp), intent(out) :: frac(nearngrid, npen)
        real(dp), intent(out) :: bonus_penalty(nearngrid, npen)

        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t) :: m
        type(tax_t) :: tax
        integer :: eX
        call readFracKids( 'input/type_' // opt%locname // '.asc', .false., pM, pF, h, m )

        call setTax( xtax, tax, TAXMODE, TAX_JOINT )

        do eX = 1, npen
            call calculateMarriagePenalty2( tax, 1, kids, nearngrid, famearn(eX), frac(:, eX), bonus_penalty(:, eX) )
        end do

    end subroutine marriagePenaltySim

    subroutine allocationToMoments(opt, ret, m, eallM, eallF, eall)
        use moments
        implicit none
        type(opt_t), intent(in) :: opt
        type(ret_t), intent(out) :: ret
        type(m_t), intent(in) :: m
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1)

        type(mom_t) :: momModel, momData, vmomData, modemom, weightMom
        type(momExtra_t) :: momModelExtra, momDataExtra

        if ( opt%extraMom ) then
            call aggregateMoments2(m, eallM, eallF, eall, momModel, momModelExtra )
        else
            call aggregateMoments2(m, eallM, eallF, eall, momModel)
        end if

        ! read moment files
        call readMomentFiles('input', opt, momData, vmomData, modemom)
        call weightingMatrix(vmomData, modemom, weightMom, ATUSWT)

        if ( opt%extraMom) then
            call readMomentFilesExtra2('input/moments/' // opt%locname, 'point', momDataExtra)
        end if

        if ( opt%verbose ) then
            call printMomentsCompare(momModel, momData, weightMom)
            if (opt%extraMom ) then
                call printMomentsCompareExtra(momModelExtra, momDataExtra)
            end if
        end if

        if ( opt%savemom ) then
            call printMomentsCompare(momModel, momData, weightMom, fdir = opt%savepath)
            if ( opt%extraMom ) then
                call printMomentsCompareExtra(momModelExtra, momDataExtra, fdir = opt%savepath)
            end if
        end if

        ret%dist = momentDistance(momData, momModel, weightMom)
        ret%momVec = transfer( momModel, (/1.0_dp/) )
        if (opt%extraMom ) then
            ret%momExtraVec = transfer( momModelExtra, (/1.0_dp/) )
        else
            ret%momExtraVec = 0.0_dp
        end if

    end subroutine allocationToMoments

    subroutine allocationToMomentsMarket(opt, ret, m, eallM, eallF, eall)
        use moments
        implicit none
        type(opt_t), intent(in) :: opt
        type(ret_t), intent(out) :: ret
        type(m_t), intent(in) :: m(NMARKET)
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1, NMARKET)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1, NMARKET)
        type(eall2_t), intent(in)  :: eall(NTYPE1, NTYPE2, 0:1, NMARKET)

        type(mom_t) :: momModel, momData, vmomData, modemom, weightMom
        type(momExtra_t) :: momModelExtra, momDataExtra
        logical, parameter :: reweight = .false.

        if ( opt%extraMom ) then
            call aggregateMomentsMarket2(m, eallM, eallF, eall, momModel, momModelExtra )
        else
            call aggregateMomentsMarket2(m, eallM, eallF, eall, momModel)
        end if

        ! read moment files
        call readMomentFiles('input', opt, momData, vmomData, modemom)
        call weightingMatrix(vmomData, modemom, weightMom, ATUSWT)

        if ( reweight ) then

            weightMom%fhours_0F(:, 1, :) = weightMom%fhours_0F(:, 1, :) * 3.0_dp
            weightMom%mhours_M0(:, 1, :) = weightMom%mhours_M0(:, 1, :) * 3.0_dp

            weightMom%mhours_MF(:, 1, 1:NTYPE2, :) = weightMom%mhours_MF(:, 1, 1:NTYPE2, :) * 3.0_dp
            weightMom%mhours_MF(:, 2:NTYPE1, 1, :) = weightMom%mhours_MF(:, 2:NTYPE1, 1, :) * 3.0_dp

            weightMom%fhours_MF(:, 1, 1:NTYPE2, :) = weightMom%fhours_MF(:, 1, 1:NTYPE2, :) * 3.0_dp
            weightMom%fhours_MF(:, 2:NTYPE1, 1, :) = weightMom%fhours_MF(:, 2:NTYPE1, 1, :) * 3.0_dp

            weightMom%mhome_M0(:, 1, :) = weightMom%mhome_M0(:, 1, :) * 3.0_dp
            weightMom%fhome_0F(:, 1, :) = weightMom%fhome_0F(:, 1, :) * 3.0_dp
            weightMom%mhome_Mf(:, 1, :) = weightMom%mhome_Mf(:, 1, :) * 3.0_dp
            weightMom%fhome_mF(:, 1, :) = weightMom%fhome_mF(:, 1, :) * 3.0_dp

            weightMom%fhomeh_mF(:, 0) = weightMom%fhomeh_mF(:, 0) * 3.0_dp
            weightMom%mhomeh_Mf(:, 0) = weightMom%mhomeh_Mf(:, 0) * 3.0_dp

            weightMom%mhomeh_M_F = 30.0_dp
            weightMom%fhomeh_M_F = 30.0_dp
            weightMom%mhomeh_M0 = 30.0_dp
            weightMom%fhomeh_0F = 30.0_dp

        end if

        if ( opt%extraMom ) then
            call readMomentFilesExtra2('input/moments/' // opt%locname, 'point', momDataExtra)
        end if

        if ( opt%verbose ) then
            call printMomentsCompare(momModel, momData, weightMom)
            if ( opt%extraMom ) then
                call printMomentsCompareExtra(momModelExtra, momDataExtra)
            end if
        end if

        if ( opt%savemom ) then
            call printMomentsCompare(momModel, momData, weightMom, fdir = opt%savepath)
            if ( opt%extraMom ) then
                call printMomentsCompareExtra(momModelExtra, momDataExtra, fdir = opt%savepath)
            end if
        end if

        ret%dist = momentDistance(momData, momModel, weightMom)
        ret%momVec = transfer( momModel, (/1.0_dp/) )

        if ( opt%extraMom ) then
            ret%momExtraVec = transfer( momModelExtra, (/1.0_dp/) )
        else
            ret%momExtraVec = 0.0_dp
        end if

    end subroutine allocationToMomentsMarket


    subroutine calculateMomentDerivative(x, opt, D)
#       ifdef _MPI_
        use mpi
#       endif
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        type(opt_t), intent(in) :: opt
        real(dp), intent(out) :: D(NMOM,NPARAM)

        type(ret_t) :: ret
        real(dp) :: momModelVec(NMOM)
        real(dp) :: momModelVecPrime(NMOM)
        real(dp) :: momModelVecAll((NPARAM + 1) * NMOM)
! n
        real(dp) :: z(NPARAM)
        real(dp) :: xlam(NTYPEALL*NMARKET)
        real(dp) :: xtax(NTAXPARAM)

        real(dp), parameter :: eps = 1.0e-5_dp
        real(dp) :: epsrel

        integer :: iX
        integer :: id

#       ifdef _MPI_
        integer :: ierror, nproc
#       endif

#       ifdef _MPI_

        !obtain id for each node
        call mpi_comm_rank(mpi_comm_world, id, ierror)

#       endif

#       ifndef _MPI_
        do id = 0, NPARAM
#       endif

        z = x
        if (id > 0) then
            epsrel = eps*max( 1.0_dp, abs( x(id) ) )
            z(id) = x(id) + epsrel
        end if

        call objfun_market(z, xlam, xtax, opt, ret)
        momModelVecPrime = ret%momVec
        print *, id
#       ifdef _MPI_
            call MPI_GATHER(momModelVecPrime, NMOM, MPI_DOUBLE_PRECISION, &
                            momModelVecAll, NMOM, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierror)
#       else
            momModelVecAll(1 + id * NMOM : NMOM + id * NMOM) = momModelVecPrime
#       endif

#       ifndef _MPI_
        end do
#       endif

        momModelVec = momModelVecAll(1:NMOM)
        do iX = 1, NPARAM
            momModelVecPrime = momModelVecAll(1 + iX * NMOM : NMOM + iX * NMOM)
            epsrel = eps * max( 1.0_dp, abs( x(iX) ) )
            D(:, iX) = ( momModelVecPrime - momModelVec ) / epsrel
        end do

    end subroutine calculateMomentDerivative

    ! THIS IS NOT USED
    subroutine readWeightingMatrix(Wvec, opt, scale)
        use moments
        implicit none
        real(dp), intent(out) :: WVec(NMOM)
        type(opt_t), intent(in) :: opt
        real(dp), intent(in) :: scale
        type(mom_t) :: momData, vmomData, modemom, weightMom
        call readMomentFiles('input', opt, momData, vmomData, modemom)
        call weightingMatrix(vmomData, modemom, weightMom, scale)

        Wvec = transfer(weightMom, 1.0_dp)

    end subroutine readWeightingMatrix


    function aggregateTaxes( pM, pF, h, m, eallM, eallF, eall ) result(totalTaxes)
        implicit none
        type(p_t), intent(in) :: pM(NTYPE1, 0:1)
        type(p_t), intent(in) :: pF(NTYPE2, 0:1)
        type(h_t), intent(in) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t), intent(in) :: m
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1)

        real(dp) :: totalTaxes

        integer :: pX1, pX2
        integer :: kX

        logical, parameter :: doSingleMen = .true.
        logical, parameter :: doSingleWomen = .true.
        logical, parameter :: doMarriedMen = .true.
        logical, parameter :: doMarriedWomen = .true.

        totalTaxes = 0.0_dp

        ! 1. contribution of SINGLE men to taxes
        if ( doSingleMen ) then
            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    totalTaxes = totalTaxes + pM(pX1, kX)%frac * m%PrSingle1(pX1) * pM(pX1, kX)%probTypeK * eallM(pX1, kX)%Etottax
                end do
            end do
        end if

        ! 2. contribution of MARRIED men/women to taxes
        if ( doMarriedMen .or. doMarriedWomen ) then

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        totalTaxes = totalTaxes + m%typeDemand(pX1, pX2) * h(pX1, pX2, kX)%probTypeK * eall(pX1, pX2, kX)%Etottax
                    end do
                end do
            end do

        end if

        ! 3. contribution of SINGLE women to taxes
        if ( doSingleWomen ) then
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    totalTaxes = totalTaxes + pF(pX2, kX)%frac * m%PrSingle2(pX2) * pF(pX2, kX)%probTypeK * eallF(pX2, kX)%Etottax
                end do
            end do
        end if

    end function aggregateTaxes

    function aggregateTaxesMarket( pM, pF, h, m, eallM, eallF, eall ) result(totalTaxes)
        implicit none
        type(p_t), intent(in) :: pM(NTYPE1, 0:1, NMARKET)
        type(p_t), intent(in) :: pF(NTYPE2, 0:1, NMARKET)
        type(h_t), intent(in) :: h(NTYPE1, NTYPE2, 0:1, NMARKET)
        type(m_t), intent(in) :: m(NMARKET)
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1, NMARKET)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1, NMARKET)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1, NMARKET)

        real(dp) :: totalTaxes

        integer :: pX1, pX2
        integer :: mX, kX

        logical, parameter :: doSingleMen = .true.
        logical, parameter :: doSingleWomen = .true.
        logical, parameter :: doMarriedMen = .true.
        logical, parameter :: doMarriedWomen = .true.

        totalTaxes = 0.0_dp

        ! 1. contribution of SINGLE men to taxes
        if ( doSingleMen ) then
            do mX = 1, NMARKET
                do kX = 0, 1
                    do pX1 = 1, NTYPE1
                        totalTaxes = totalTaxes + m(mX)%marketSize * m(mX)%numSingleKid1(pX1, kX) * eallM(pX1, kX, mX)%Etottax
                    end do
                end do
            end do
        end if

        ! 2. contribution of MARRIED men/women to taxes
        if ( doMarriedMen .or. doMarriedWomen ) then

            do mX = 1, NMARKET
                do kX = 0, 1
                    do pX2 = 1, NTYPE2
                        do pX1 = 1, NTYPE1
                            totalTaxes = totalTaxes + m(mX)%marketSize * m(mX)%typeDemandKid(pX1, pX2, kX) &
                                * eall(pX1, pX2, kX, mX)%Etottax
                        end do
                    end do
                end do
            end do

        end if

        ! 3. contribution of SINGLE women to taxes
        if ( doSingleWomen ) then
            do mX = 1, NMARKET
                do kX = 0, 1
                    do pX2 = 1, NTYPE2
                        totalTaxes = totalTaxes + m(mX)%marketSize * m(mX)%numSingleKid2(pX2, kX) * eallF(pX2, kX, mX)%Etottax
                    end do
                end do
            end do
        end if

    end function aggregateTaxesMarket


    function aggregateWelfare2( pM, pF, h, m, eallM, eallF, eall ) result(totalWelfare)
        implicit none
        type(p_t), intent(in) :: pM(NTYPE1, 0:1)
        type(p_t), intent(in) :: pF(NTYPE2, 0:1)
        type(h_t), intent(in) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t), intent(in) :: m
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1)

        real(dp) :: totalWelfare
        real(dp) :: conditionalWelfare

        integer :: pX1, pX2, kX

        logical, parameter :: doSingleMen = .true.
        logical, parameter :: doSingleWomen = .true.
        logical, parameter :: doMarriedMen = .true.
        logical, parameter :: doMarriedWomen = .true.

        totalWelfare = 0.0_dp

        ! 1. contribution of SINGLE men to social welfare
        if ( doSingleMen ) then
            do kX = 0, 1
                do pX1 = 1, NTYPE1

                    conditionalWelfare = merge( eallM(pX1, kX)%Eswf + m%thetaAdjSingle1(pX1), &
                        ( eallM(pX1, kX)%Eswf * m%thetaAdjSingle1(pX1) - 1.0_dp ) / m%theta, m%theta >= 0.0_dp )

                    totalWelfare = totalWelfare &
                        + pM(pX1, kX)%frac * m%PrSingle1(pX1) * pM(pX1, kX)%probTypeK * conditionalWelfare

                end do
            end do
        end if

        ! 2. contribution of MARRIED men to social welfare
        if ( doMarriedMen ) then

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1

                        conditionalWelfare = merge( eall(pX1, pX2, kX)%Eswf1 + m%thetaAdj1(pX1, pX2), &
                            ( eall(pX1, pX2, kX)%Eswf1 * m%thetaAdj1(pX1, pX2) - 1.0_dp ) / m%theta, m%theta >= 0.0_dp )

                        totalWelfare = totalWelfare &
                            + pM(pX1, kX)%frac * m%Pr1(pX1, pX2) * h(pX1, pX2, kX)%probTypeK * conditionalWelfare

                    end do
                end do
            end do

        end if

        ! 3. contribution of SINGLE women to social welfare
        if ( doSingleWomen ) then
            do kX = 0, 1
                do pX2 = 1, NTYPE2

                    conditionalWelfare = merge( eallF(pX2, kX)%Eswf + m%thetaAdjSingle2(pX2), &
                        ( eallF(pX2, kX)%Eswf * m%thetaAdjSingle2(pX2) - 1.0_dp ) / m%theta, m%theta >= 0.0_dp )

                    totalWelfare = totalWelfare &
                        + pF(pX2, kX)%frac * m%PrSingle2(pX2) * pF(pX2, kX)%probTypeK * conditionalWelfare

                end do
            end do
        end if

        ! 4. contribution of MARRIED women to social welfare
        if ( doMarriedWomen ) then
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1

                        conditionalWelfare = merge( eall(pX1, pX2, kX)%Eswf2 + m%thetaAdj2(pX1, pX2), &
                            ( eall(pX1, pX2, kX)%Eswf2 * m%thetaAdj2(pX1, pX2) - 1.0_dp ) / m%theta, m%theta >= 0.0_dp )

                        totalWelfare = totalWelfare &
                            + pF(pX2, kX)%frac * m%Pr2(pX1, pX2) * h(pX1, pX2, kX)%probTypeK * conditionalWelfare

                    end do
                end do
            end do

        end if

    end function aggregateWelfare2


    function aggregateWelfare( pM, pF, h, m, eallM, eallF, eall ) result(totalWelfare)
        implicit none
        type(p_t), intent(in) :: pM(NTYPE1, 0:1)
        type(p_t), intent(in) :: pF(NTYPE2, 0:1)
        type(h_t), intent(in) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t), intent(in) :: m
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1)

        real(dp) :: totalWelfare
        real(dp) :: conditionalWelfare

        integer :: pX1, pX2, kX

        logical, parameter :: doSingleMen = .true.
        logical, parameter :: doSingleWomen = .true.
        logical, parameter :: doMarriedMen = .true.
        logical, parameter :: doMarriedWomen = .true.

        totalWelfare = 0.0_dp

        ! 1. contribution of SINGLE men to social welfare
        if ( doSingleMen ) then
            do kX = 0, 1
                do pX1 = 1, NTYPE1

                    conditionalWelfare = merge( eallM(pX1, kX)%Eswf + m%thetaAdjSingle1(pX1), &
                        - eallM(pX1, kX)%Eswf * m%thetaAdjSingle1(pX1), m%theta >= 0.0_dp )

                    totalWelfare = totalWelfare &
                        + pM(pX1, kX)%frac * m%PrSingle1(pX1) * pM(pX1, kX)%probTypeK * conditionalWelfare

                end do
            end do
        end if

        ! 2. contribution of MARRIED men to social welfare
        if ( doMarriedMen ) then

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1

                        conditionalWelfare = merge( eall(pX1, pX2, kX)%Eswf1 + m%thetaAdj1(pX1, pX2), &
                            - eall(pX1, pX2, kX)%Eswf1 * m%thetaAdj1(pX1, pX2), m%theta >= 0.0_dp )

                        totalWelfare = totalWelfare &
                            + pM(pX1, kX)%frac * m%Pr1(pX1, pX2) * h(pX1, pX2, kX)%probTypeK * conditionalWelfare

                    end do
                end do
            end do

        end if

        ! 3. contribution of SINGLE women to social welfare
        if ( doSingleWomen ) then
            do kX = 0, 1
                do pX2 = 1, NTYPE2

                    conditionalWelfare = merge( eallF(pX2, kX)%Eswf + m%thetaAdjSingle2(pX2), &
                        - eallF(pX2, kX)%Eswf * m%thetaAdjSingle2(pX2), m%theta >= 0.0_dp )

                    totalWelfare = totalWelfare &
                        + pF(pX2, kX)%frac * m%PrSingle2(pX2) * pF(pX2, kX)%probTypeK * conditionalWelfare

                end do
            end do
        end if

        ! 4. contribution of MARRIED women to social welfare
        if ( doMarriedWomen ) then
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1

                        conditionalWelfare = merge( eall(pX1, pX2, kX)%Eswf2 + m%thetaAdj2(pX1, pX2), &
                            - eall(pX1, pX2, kX)%Eswf2 * m%thetaAdj2(pX1, pX2), m%theta >= 0.0_dp )

                        totalWelfare = totalWelfare &
                            + pF(pX2, kX)%frac * m%Pr2(pX1, pX2) * h(pX1, pX2, kX)%probTypeK * conditionalWelfare

                    end do
                end do
            end do

        end if

    end function aggregateWelfare

    function aggregateWelfareInc( pM, pF, h, m, eallM, eallF, eall ) result(totalWelfare)
        implicit none
        type(p_t), intent(in) :: pM(NTYPE1, 0:1)
        type(p_t), intent(in) :: pF(NTYPE2, 0:1)
        type(h_t), intent(in) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t), intent(in) :: m
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1)

        real(dp) :: totalWelfare
        real(dp) :: conditionalWelfare

        integer :: pX1, pX2, kX

        logical, parameter :: doSingleMen = .true.
        logical, parameter :: doSingleWomen = .true.
        logical, parameter :: doMarriedMen = .true.
        logical, parameter :: doMarriedWomen = .true.

        totalWelfare = 0.0_dp

        ! 1. contribution of SINGLE men to social welfare
        if ( doSingleMen ) then
            do kX = 0, 1
                do pX1 = 1, NTYPE1

                    conditionalWelfare = merge( eallM(pX1, kX)%Eswf + m%thetaAdjSingle1(pX1), &
                        - eallM(pX1, kX)%Eswf * m%thetaAdjSingle1(pX1), m%theta >= 0.0_dp )

                    totalWelfare = totalWelfare &
                        + pM(pX1, kX)%frac * m%PrSingle1(pX1) * pM(pX1, kX)%probTypeK * conditionalWelfare

                end do
            end do
        end if

        ! 2. contribution of MARRIED men to social welfare
        if ( doMarriedMen ) then

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1

                        conditionalWelfare = merge( eall(pX1, pX2, kX)%EswfInc1 + m%thetaAdj1(pX1, pX2), &
                            - eall(pX1, pX2, kX)%EswfInc1 * m%thetaAdj1(pX1, pX2), m%theta >= 0.0_dp )

                        totalWelfare = totalWelfare &
                            + pM(pX1, kX)%frac * m%Pr1(pX1, pX2) * h(pX1, pX2, kX)%probTypeK * conditionalWelfare

                    end do
                end do
            end do

        end if

        ! 3. contribution of SINGLE women to social welfare
        if ( doSingleWomen ) then
            do kX = 0, 1
                do pX2 = 1, NTYPE2

                    conditionalWelfare = merge( eallF(pX2, kX)%Eswf + m%thetaAdjSingle2(pX2), &
                        - eallF(pX2, kX)%Eswf * m%thetaAdjSingle2(pX2), m%theta >= 0.0_dp )

                    totalWelfare = totalWelfare &
                        + pF(pX2, kX)%frac * m%PrSingle2(pX2) * pF(pX2, kX)%probTypeK * conditionalWelfare

                end do
            end do
        end if

        ! 4. contribution of MARRIED women to social welfare
        if ( doMarriedWomen ) then
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1

                        conditionalWelfare = merge( eall(pX1, pX2, kX)%EswfInc2 + m%thetaAdj2(pX1, pX2), &
                            - eall(pX1, pX2, kX)%EswfInc2 * m%thetaAdj2(pX1, pX2), m%theta >= 0.0_dp )

                        totalWelfare = totalWelfare &
                            + pF(pX2, kX)%frac * m%Pr2(pX1, pX2) * h(pX1, pX2, kX)%probTypeK * conditionalWelfare

                    end do
                end do
            end do

        end if

    end function aggregateWelfareInc

    function aggregatePmfInc( pM, pF, h, m, eallM, eallF, eall ) result(pmf)
        implicit none
        type(p_t), intent(in) :: pM(NTYPE1, 0:1)
        type(p_t), intent(in) :: pF(NTYPE2, 0:1)
        type(h_t), intent(in) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t), intent(in) :: m
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1)

        real(dp) :: pmf

        integer :: pX1, pX2, kX

        pmf = 0.0_dp

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                pmf = pmf + pM(pX1, kX)%frac * m%PrSingle1(pX1) * pM(pX1, kX)%probTypeK * 0.0_dp
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pmf = pmf + pM(pX1, kX)%frac * m%Pr1(pX1, pX2) * h(pX1, pX2, kX)%probTypeK * eall(pX1, pX2, kX)%dInc
                end do
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                pmf = pmf + pF(pX2, kX)%frac * m%PrSingle2(pX2) * pF(pX2, kX)%probTypeK * 0.0_dp
            end do
        end do

    end function aggregatePmfInc

    pure subroutine calculateWelfareAdjustment(m, null)
        use mathcons
        implicit none
        type(m_t), intent(inout) :: m
        logical, intent(in) :: null
        real(dp) :: GamAdj
        integer :: pX1, pX2

        if ( null ) then

            if ( m%theta >= 0.0_dp ) then
                m%thetaAdjSingle1 = 0.0_dp
                m%thetaAdjSingle2 = 0.0_dp
                m%thetaAdj1 = 0.0_dp
                m%thetaAdj2 = 0.0_dp
            else
                m%thetaAdjSingle1 = 1.0_dp
                m%thetaAdjSingle2 = 1.0_dp
                m%thetaAdj1 = 1.0_dp
                m%thetaAdj2 = 1.0_dp
            end if

        else

            if (m%theta >= 0.0_dp) then

                ! additive adjustment for social welfare function
                do pX1 = 1, NTYPE1
                    m%thetaAdjSingle1(pX1) = m%sdTheta0 * ( EULERMASCHERONI - log(m%PrSingle1(pX1)) )
                end do

                do pX2 = 1, NTYPE2
                    m%thetaAdjSingle2(pX2) = m%sdTheta0 * ( EULERMASCHERONI - log(m%PrSingle2(pX2)) )
                end do

                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        m%thetaAdj1(pX1, pX2) = m%sdTheta0 * ( EULERMASCHERONI - log(m%Pr1(pX1, pX2)) )
                        m%thetaAdj2(pX1, pX2) = m%sdTheta0 * ( EULERMASCHERONI - log(m%Pr2(pX1, pX2)) )
                    end do
                end do

            else

                GamAdj = gamma( 1.0_dp - m%theta * m%sdTheta0 )
                GamAdj = 1.0_dp !FIXME CHECKTHIS
                ! multiplicative adjustment for social welfare function
                do pX1 = 1, NTYPE1
                    m%thetaAdjSingle1(pX1) = GamAdj * m%PrSingle1(pX1) ** ( -m%sdTheta0 * m%theta )
                end do

                do pX2 = 1, NTYPE2
                    m%thetaAdjSingle2(pX2) = GamAdj * m%PrSingle2(pX2) ** ( -m%sdTheta0 * m%theta )
                end do

                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        m%thetaAdj1(pX1, pX2) = GamAdj * m%Pr1(pX1, pX2) ** ( -m%sdTheta0 * m%theta )
                        m%thetaAdj2(pX1, pX2) = GamAdj * m%Pr2(pX1, pX2) ** ( -m%sdTheta0 * m%theta )
                    end do
                end do

            end if

        end if ! ( null )

    end subroutine calculateWelfareAdjustment


    subroutine utilityPreCalc(p, rnd, tax, grid, kid, kidtax, sex, opt, v)
        use taxes
        implicit none
        type(p_t), intent(in) :: p
        type(rnd_t), intent(in) :: rnd
        type(tax_t), intent(in) :: tax
        type(grid_t), intent(in) :: grid
        integer, intent(in) :: kid
        integer, intent(in) :: kidtax
        integer, intent(in) :: sex
        type(opt_t), intent(in) :: opt
        type(v_t), intent(out) :: v

        real(dp) :: earn(N, NODEW)
        real(dp) :: taxable(N, NODEW)
        real(dp) :: tottax(N, NODEW)
        real(dp) :: netinc(N, NODEW)

        real(dp) :: util_l(N)
        real(dp) :: util_Q(N)
        real(dp) :: util_c(N,NODEW)

        real(dp) :: utilTr_c(NODEW, N)
        real(dp) :: earnTr(NODEW, N)
        real(dp) :: taxableTr(NODEW, N)
        real(dp) :: tottaxTr(NODEW, N)
        real(dp) :: netincTr(NODEW, N)

        real(dp) :: vcTr(NODEW, NDIM)
        real(dp) :: vearnTr(NODEW, NDIM)
        real(dp) :: vtaxableTr(NODEW, NDIM)
        real(dp) :: vtottaxTr(NODEW, NDIM)
        real(dp) :: vnetincTr(NODEW, NDIM)

        real(dp) :: timeLeisure(N)
        real(dp) :: timePublic(N)

        real(dp) :: fc(NDIM)
        real(dp) :: fcn(N)
        real(dp) :: jointQ(N)
        integer :: iX, jX, aX0
        integer :: wX0
        ! real(dp) :: elVec(NDIM)
        ! real(dp) :: QVec(NDIM)

        fcn = p%fc
        fcn(1) = 0.0_dp

        ! fixed utility costs
        fc = 0.0_dp

        if (opt%wageElasticity) then
            do wX0 = 1, NODEW
                earn(:, wX0) = p%wageNode(wX0) * grid%gridval
                taxable(:, wX0) = merge(p%nly, 0.0_dp, TAXNONLABOUR) + earn(:, wX0)
                do aX0 = 1, N
                    tottax(aX0, wX0) = evalTaxElasSinScalar(tax, p%taxmode, kidtax, sex, taxable(aX0, wX0), opt%szWageElasticity)
                end do
                netinc(:, wX0) = p%nly + earn(:, wX0) - tottax(:, wX0) - fcn
            end do
        else
            do wX0 = 1, NODEW
                earn(:, wX0) = p%wageNode(wX0) * grid%gridval
                taxable(:, wX0) = merge(p%nly, 0.0_dp, TAXNONLABOUR) + earn(:, wX0)
                tottax(:, wX0) = evalTax(tax, p%taxmode, kidtax, sex, taxable(:, wX0))
                netinc(:, wX0) = p%nly + earn(:, wX0) - tottax(:, wX0) - fcn
            end do
        end if

        netinc = CSCALE * max(netinc, CFLOOR)

        timeLeisure = max((MIN_TIME_LEISURE + grid%gridval) / TIME_TOTAL_FULL, 1.0e-12_dp)

        if (p%sigl == 1.0_dp) then
            util_l = log(timeLeisure)
        else
            util_l = (timeLeisure**(1.0_dp - p%sigl) - UCONS) / (1.0_dp - p%sigl)
        end if

        timePublic = (MIN_TIME_PUBLIC + grid%gridval) / TIME_TOTAL_FULL

        jointQ = QCONS + p%homeSlope * timePublic

        if (p%sigQ == 1.0_dp) then
            util_Q = log(jointQ)
        else
            util_Q = (jointQ ** (1.0_dp - p%sigQ) - UCONS) / (1.0_dp - p%sigQ)
        end if

        if (p%sigc == 1.0_dp) then
            do wX0 = 1, NODEW
                util_c(:, wX0) = log(netinc(:, wX0))
            end do
        else
            do wX0 = 1, NODEW
                util_c(:, wX0) = (netinc(:, wX0) ** (1.0_dp - p%sigc) - UCONS) / (1.0_dp - p%sigc)
            end do
        end if

        utilTr_c = transpose(util_c)
        earnTr = transpose(earn)
        taxableTr = transpose(taxable)
        tottaxTr = transpose(tottax)
        netincTr = transpose(netinc)

        do iX = 1, NDIM
            do jX = 1, N
                if (grid%gzInd(iX, TIME_LEISURE) == jX) then
                    v%l(iX) = util_l(jX)
                    exit
                end if
            end do
        end do

        do iX = 1, NDIM
            do jX = 1, N
                if (grid%gzInd(iX, TIME_PUBLIC) == jX) then
                    v%Qw(iX) = util_Q(jX)
                    exit
                end if
            end do
        end do

        do iX = 1, NDIM
            do jX = 1, N
                if (grid%gzInd(iX, TIME_HOURS) == jX) then
                    vcTr(:, iX) = utilTr_c(:, jX)
                    vearnTr(:, iX) = earnTr(:, jX)
                    vtaxableTr(:, iX) = taxableTr(:, jX)
                    vtottaxTr(:, iX) = tottaxTr(:, jX)
                    vnetincTr(:, iX) = netincTr(:, jX)
                    exit
                end if
            end do
        end do

        v%c = transpose(vcTr)
        v%earn = transpose(vearnTr)
        v%taxable = transpose(vtaxableTr)
        v%tottax = transpose(vtottaxTr)
        v%netinc = transpose(vnetincTr)

        ! separable preferences
        do wX0 = 1, NODEW
            v%util(:, wX0) = p%betl * v%l + p%betQ * v%Qw + p%betc * v%c(:, wX0) - fc
        end do

    end subroutine utilityPreCalc


    subroutine householdUtilityPreCalc(h, pM, pF, rnd, tax, grid, kid, opt, v, netinc, jointTaxes)
        use taxes
        implicit none
        type(h_t), intent(in) :: h
        type(p_t), intent(in) :: pM
        type(p_t), intent(in) :: pF
        type(rnd_t), intent(in) :: rnd
        type(tax_t), intent(in) :: tax
        type(grid_t), intent(in) :: grid
        integer, intent(in) :: kid
        type(opt_t), intent(in) :: opt
        type(v2_t), intent(out) :: v
        real(dp), intent(out) :: netinc(N, N, NODEW, NODEW)
        logical, intent(in) :: jointTaxes

        ! output from consumptionShare
        real(dp) :: earnM(N, NODEW)
        real(dp) :: earnF(N, NODEW)
        real(dp) :: taxableM(N, NODEW)
        real(dp) :: taxableF(N, NODEW)
        real(dp) :: tottax(N, N, NODEW, NODEW)
        real(dp) :: taxsin(N)
        real(dp) :: penalty(N)

        real(dp) :: jointQw(N, N)

        real(dp) :: utilM_l(N)
        real(dp) :: utilM_Qw(N, N)

        real(dp) :: utilF_l(N)
        real(dp) :: utilF_Qw(N, N)

        real(dp) :: vearnTrM(NODEW, NDIM)
        real(dp) :: vtaxableTrM(NODEW, NDIM)
        real(dp) :: vearnTrF(NODEW, NDIM)
        real(dp) :: vtaxableTrF(NODEW, NDIM)
        real(dp) :: vtottaxTr(NODEW, NODEW, NDIM, NDIM)
        real(dp) :: vnetincTr(NODEW, NODEW, NDIM, NDIM)

        real(dp) :: earnTrM(NODEW, N)
        real(dp) :: taxableTrM(NODEW, N)
        real(dp) :: earnTrF(NODEW, N)
        real(dp) :: taxableTrF(NODEW, N)
        real(dp) :: tottaxTr(NODEW, NODEW, N, N)
        real(dp) :: netincTr(NODEW, NODEW, N, N)

        real(dp) :: tottax1(N, NODEW), tottax2(N, NODEW)

        real(dp) :: timeLeisure(N), timePublic(N)
        real(dp) :: fcnM(N), fcnF(N)
        real(dp) :: fcnKid(N, N)
        real(dp) :: thisEarnF, thisEarnM, minEarn, maxEarn
        integer :: hX0, lX0, aX0, hX1, hX2, qX1, qX2
        integer :: wX1, wX2
        integer :: aX1, aX2

        ! male earnings and taxable income
        do wX1 = 1, NODEW
            earnM(:, wX1) = pM%wageNode(wX1) * grid%gridval
            taxableM(:, wX1) = merge(pM%nly, 0.0_dp, TAXNONLABOUR) + earnM(:, wX1)
        end do

        ! female earnings and taxable income
        do wX2 = 1, NODEW
            earnF(:, wX2) = pF%wageNode(wX2) * grid%gridval
            taxableF(:, wX2) = merge(pF%nly, 0.0_dp, TAXNONLABOUR) + earnF(:, wX2)
        end do

        ! fc for each adult
        fcnM = pM%fc
        fcnF = pF%fc
        fcnM(1) = 0.0_dp
        fcnF(1) = 0.0_dp
        fcnKid = 0

        if (opt%wageElasticity) then

            if (opt%wageElasticity1) then
                do wX2 = 1, NODEW
                    do wX1 = 1, NODEW
                        do hX2 = 1, N
                            do hX1 = 1, N
                                tottax(hX1, hX2, wX1, wX2) = evalTaxElasCouScalar(tax, h%taxmode, 1, kid, taxableM(hX1, wX1), &
                                    taxableF(hX2, wX2), opt%szWageElasticity1)
                                netinc(hX1, hX2, wX1, wX2) = pM%nly + pF%nly + earnM(hX1, wX1) + earnF(hX2, wX2) &
                                    - tottax(hX1, hX2, wX1, wX2) - fcnM(hX1) - fcnF(hX2) - fcnKid(hX1, hX2)
                            end do
                        end do
                    end do
                end do
            end if

            if (opt%wageElasticity2) then
                do wX2 = 1, NODEW
                    do wX1 = 1, NODEW
                        do hX2 = 1, N
                            do hX1 = 1, N
                                tottax(hX1, hX2, wX1, wX2) = evalTaxElasCouScalar(tax, h%taxmode, 2, kid, taxableM(hX1, wX1), &
                                    taxableF(hX2, wX2), opt%szWageElasticity2)
                                netinc(hX1, hX2, wX1, wX2) = pM%nly + pF%nly + earnM(hX1, wX1) + earnF(hX2, wX2) &
                                    - tottax(hX1, hX2, wX1, wX2) - fcnM(hX1) - fcnF(hX2) - fcnKid(hX1, hX2)
                            end do
                        end do
                    end do
                end do
            end if

        else

            ! total taxes and net family income for all wages and joint allocations
            if (jointTaxes) then

                do wX2 = 1, NODEW
                    do wX1 = 1, NODEW
                        do hX2 = 1, N
                            tottax(:, hX2, wX1, wX2) = evalTax(tax, h%taxmode, kid, taxableM(:, wX1), taxableF(hX2, wX2))
                            if (opt%marriageMarketElasticity) then
                                ! total tax liabtility if single
                                taxsin = evalTax(tax, h%taxmode, 0, SEX_MALE, taxableM(:, wX1)) + evalTax(tax, h%taxmode, kid, &
                                    SEX_FEMALE, taxableF(hX2, wX2))
                                ! tax penalty
                                penalty = tottax(:, hX2, wX1, wX2) - taxsin
                                ! increase penalty / decrease bonus by 1%
                                tottax(:, hX2, wX1, wX2) = tottax(:, hX2, wX1, wX2) + 0.01_dp * abs(penalty)
                            end if

                            netinc(:, hX2, wX1, wX2) = pM%nly + pF%nly + earnM(:, wX1) + earnF(hX2, wX2) &
                                - tottax(:, hX2, wX1, wX2) - fcnM - fcnF(hX2) - fcnKid(:, hX2)

                            if (opt%earnElasticity) then
                                thisEarnF = earnF(hX2, wX2)
                                do hX1 = 1, N
                                    thisEarnM = earnM(hX1, wX1)
                                    minEarn = min(thisEarnF, thisEarnM)
                                    maxEarn = max(thisEarnF, thisEarnM)

                                    if ( minearn >= TAXBR(opt%inc1) .and. minearn < TAXBR(opt%inc1 + 1) &
                                        .and. maxearn >= TAXBR(opt%inc2) .and. maxearn < TAXBR(opt%inc2 + 1) ) then

                                        netinc(hX1, hX2, wX1, wX2) = 1.1_dp * netinc(hX1, hX2, wX1, wX2)

                                    end if
                                end do
                            end if

                        end do
                    end do
                end do

            else

                do wX1 = 1, NODEW
                    tottax1(:, wX1) = evalTax(tax, pM%taxmode, kid, SEX_MALE, taxableM(:, wX1))
                end do

                do wX2 = 1, NODEW
                    tottax2(:, wX2) = evalTax(tax, pF%taxmode, kid, SEX_FEMALE, taxableF(:, wX2))
                end do

                do wX2 = 1, NODEW
                    do wX1 = 1, NODEW
                        do hX2 = 1, N
                            tottax(:, hX2, wX1, wX2) = tottax1(:, wX1) + tottax2(hX2, wX2)
                            netinc(:, hX2, wX1, wX2) = pM%nly + pF%nly + earnM(:, wX1) + earnF(hX2, wX2) &
                                - tottax(:, hX2, wX1, wX2) - fcnM - fcnF(hX2) - fcnKid(:, hX2)
                        end do
                    end do
                end do

            end if

        end if

        netinc = CSCALE * max(netinc, CFLOOR)
        timeLeisure = max((MIN_TIME_LEISURE + grid%gridval) / TIME_TOTAL_FULL, 1.0e-12_dp)

        timePublic = (MIN_TIME_PUBLIC + grid%gridval) / TIME_TOTAL_FULL
        do wX2 = 1, NODEW
            do wX1 = 1, NODEW
                do qX2 = 1, N
                    if (h%nu == 0.0_dp) then
                        jointQw(:, qX2) = QCONS + h%A * (timePublic ** h%coefM * timePublic(qX2) ** h%coefF)
                    else
                        jointQw(:, qX2) = h%A * (timePublic) * ( &
                            h%coefM + h%coefF * ( timePublic(qX2) / ( timePublic) )**h%nu ) ** (1.0_dp / h%nu)
                    end if
                end do
            end do
        end do

        ! male leisure utility
        if (pM%sigl == 1.0_dp) then
            utilM_l = log(timeLeisure)
        else
            utilM_l = (timeLeisure ** (1.0_dp - pM%sigl) - UCONS) / (1.0_dp - pM%sigl)
        end if

        ! female leisure utility
        if (pF%sigl == 1.0_dp) then
            utilF_l = log(timeLeisure)
        else
            utilF_l = (timeLeisure ** (1.0_dp - pF%sigl) - UCONS) / (1.0_dp - pF%sigl)
        end if

        ! male public good utility
        if (pM%sigQ == 1.0_dp) then
            utilM_Qw = log(jointQw)
        else
            utilM_Qw = (jointQw ** (1.0_dp - pM%sigQ) - UCONS) / (1.0_dp - pM%sigQ)
        end if

        if (pF%sigQ == 1.0_dp) then
            utilF_Qw = log(jointQw)
        else
            utilF_Qw = (jointQw ** (1.0_dp - pF%sigQ) - UCONS) / (1.0_dp - pF%sigQ)
        end if

        ! transpose objects
        earnTrM = transpose(earnM)
        taxableTrM = transpose(taxableM)

        earnTrF = transpose(earnF)
        taxableTrF = transpose(taxableF)

        do hX2 = 1, N
            do hX1 = 1, N
                tottaxTr(:, :, hX1, hX2) = tottax(hX1, hX2, :, :)
                netincTr(:, :, hX1, hX2) = netinc(hX1, hX2, :, :)
            end do
        end do

        ! leisure
        do aX0 = 1, NDIM
            do lX0 = 1, N
                if (grid%gzInd(aX0, TIME_LEISURE) == lX0) then
                    v%lM(aX0) = utilM_l(lX0)
                    v%lF(aX0) = utilF_l(lX0)
                    exit
                end if
            end do
        end do

        ! public good production (varies with time allocation of both adults)

        do aX2 = 1, NDIM
            do qX2 = 1, N
                if (grid%gzInd(aX2, TIME_PUBLIC) == qX2) then
                    do aX1 = 1, NDIM
                        do qX1 = 1, N
                            if (grid%gzInd(aX1, TIME_PUBLIC) == qX1) then
                                v%QM(aX1, aX2) = utilM_Qw(qX1, qX2)
                                v%QF(aX1, aX2) = utilF_Qw(qX1, qX2)
                                exit
                            end if
                        end do
                    end do
                    exit
                end if
            end do
        end do


        ! consumption (varies with time allocation of both adults)
        do aX2 = 1, NDIM
            do hX2 = 1, N
                if (grid%gzInd(aX2, TIME_HOURS) == hX2) then
                    do aX1 = 1, NDIM
                        do hX1 = 1, N
                            if (grid%gzInd(aX1, TIME_HOURS) == hX1) then
                                vtottaxTr(:, :, aX1, aX2) = tottaxTr(:, :, hX1, hX2)
                                vnetincTr(:, :, aX1, aX2) = netincTr(:, :, hX1, hX2)
                                exit
                            end if
                        end do
                    end do
                    exit
                end if
            end do
        end do

        ! earnings and taxable income
        do aX0 = 1, NDIM
            do hX0 = 1, N
                if (grid%gzInd(aX0, TIME_HOURS) == hX0) then
                    vearnTrM(:, aX0) = earnTrM(:, hX0)
                    vtaxableTrM(:, aX0) = taxableTrM(:, hX0)
                    vearnTrF(:, aX0) = earnTrF(:, hX0)
                    vtaxableTrF(:, aX0) = taxableTrF(:, hX0)
                    exit
                end if
            end do
        end do

        do aX2 = 1, NDIM
            do aX1 = 1, NDIM
                v%tottax(aX1, aX2, :, :) = vtottaxTr(:, :, aX1, aX2)
                v%netinc(aX1, aX2, :, :) = vnetincTr(:, :, aX1, aX2)
            end do
        end do

        v%earnM = transpose(vearnTrM)
        v%taxableM = transpose(vtaxableTrM)
        v%earnF = transpose(vearnTrF)
        v%taxableF = transpose(vtaxableTrF)

        ! separable preferences (utility excluding consumption)
        do aX2 = 1, NDIM
            v%utilncM(:, aX2) = pM%betl * v%lM + pM%betQ * v%QM(:, aX2) + h%match
            v%utilncF(:, aX2) = pF%betl * v%lF(aX2) + pF%betQ * v%QF(:, aX2) + h%match
        end do

        v%swfInc = .false.

    end subroutine householdUtilityPreCalc

    subroutine householdUtilityPreCalcLambda(h, pM, pF, tax, lam, grid, netinc, opt, v)
        use taxes
        implicit none
        type(h_t), intent(in) :: h
        type(p_t), intent(in) :: pM
        type(p_t), intent(in) :: pF
        type(tax_t), intent(in) :: tax
        real(dp), intent(in) :: lam
        type(grid_t), intent(in) :: grid
        real(dp), intent(in) :: netinc(N, N, NODEW, NODEW)
        type(opt_t), intent(in) :: opt
        type(v2_t), intent(inout) :: v

        real(dp) :: cM(N, N, NODEW, NODEW)
        real(dp) :: cF(N, N, NODEW, NODEW)

        real(dp) :: utilM_c(N, N, NODEW, NODEW)
        real(dp) :: utilF_c(N, N, NODEW, NODEW)

        real(dp) :: utilTrM_c(NODEW, NODEW, N, N)
        real(dp) :: utilTrF_c(NODEW, NODEW, N, N)

        real(dp) :: share(N, N, NODEW, NODEW)

        real(dp) :: vcTrM(NODEW, NODEW, NDIM, NDIM)
        real(dp) :: vcTrF(NODEW, NODEW, NDIM, NDIM)

        ! this is for the peturbation for calulation welfare weights
        real(dp) :: dInc(N, N, NODEW, NODEW)
        real(dp) :: cIncM2(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: cIncF2(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: cIncM(N, N, NODEW, NODEW)
        real(dp) :: cIncF(N, N, NODEW, NODEW)
        real(dp) :: vcTrIncM(NODEW, NODEW, NDIM, NDIM)
        real(dp) :: vcTrIncF(NODEW, NODEW, NDIM, NDIM)
        real(dp) :: utilTrIncM_c(NODEW, NODEW, N, N)
        real(dp) :: utilTrIncF_c(NODEW, NODEW, N, N)
        real(dp) :: utilIncM_c(N, N, NODEW, NODEW)
        real(dp) :: utilIncF_c(N, N, NODEW, NODEW)
        real(dp) :: earn1, earn2, maxearn, minearn

        real(dp) :: dTrInc(NODEW, NODEW, N, N)
        real(dp) :: vdTrInc(NODEW, NODEW, NDIM, NDIM)
        integer :: hX1, hX2, aX1, aX2, wX1, wX2

        ! consumption
        call consumptionShare(pM, pF, lam, netinc, cM, cF, share)

        if ( v%swfInc ) then
            dInc = 0.0_dp
            cIncM = cM
            cIncF = cF
            do wX2 = 1, NODEW
                do wX1 = 1, NODEW
                    do hX2 = 1, N
                        earn2 = pF%wageNode(wX2) * grid%gridval(hX2)
                        do hX1 = 1, N
                            earn1 = pM%wageNode(wX1) * grid%gridval(hX1)
                            maxearn = max(earn1, earn2)
                            minearn = min(earn1, earn2)
                            if ( minearn >= TAXBR(opt%inc1) .and. minearn < TAXBR(opt%inc1 + 1) &
                                .and. maxearn >= TAXBR(opt%inc2) .and. maxearn < TAXBR(opt%inc2 + 1) ) then
                                dInc(hX1, hX2, wX1, wX2) = 1.0_dp
                                cIncM(hX1, hX2, wX1, wX2) = cM(hX1, hX2, wX1, wX2) + (1.0_dp - share(hX1, hX2, wX1, wX2)) * WEIGHTINC2
                                cIncF(hX1, hX2, wX1, wX2) = cF(hX1, hX2, wX1, wX2) + (share(hX1, hX2, wX1, wX2)) * WEIGHTINC2
                            end if
                        end do
                    end do
                end do
            end do
        end if

        if (pM%sigc == 1.0_dp) then
            utilM_c = log(cM)
        else
            utilM_c = (cM ** (1.0_dp - pM%sigc) - UCONS) / (1.0_dp - pM%sigc)
        end if

        if (pF%sigc == 1.0_dp) then
            utilF_c = log(cF)
        else
            utilF_c = (cF ** (1.0_dp - pF%sigc) - UCONS) / (1.0_dp - pF%sigc)
        end if

        if ( v%swfInc ) then

            if (pM%sigc == 1.0_dp) then
                utilIncM_c = log(cIncM)
            else
                utilIncM_c = (cIncM ** (1.0_dp - pM%sigc) - UCONS) / (1.0_dp - pM%sigc)
            end if

            if (pF%sigc == 1.0_dp) then
                utilIncF_c = log(cIncF)
            else
                utilIncF_c = (cIncF ** (1.0_dp - pF%sigc) - UCONS) / (1.0_dp - pF%sigc)
            end if

        end if

        v%utilM_c = utilM_c
        v%utilF_c = utilF_c

        do hX2 = 1, N
            do hX1 = 1, N
                utilTrM_c(:, :, hX1, hX2) = utilM_c(hX1, hX2, :, :)
                utilTrF_c(:, :, hX1, hX2) = utilF_c(hX1, hX2, :, :)
            end do
        end do

        do aX2 = 1, NDIM
            do hX2 = 1, N
                if (grid%gzInd(aX2, TIME_HOURS) == hX2) then
                    do aX1 = 1, NDIM
                        do hX1 = 1, N
                            if (grid%gzInd(aX1, TIME_HOURS) == hX1) then
                                vcTrM(:, :, aX1, aX2) = utilTrM_c(:, :, hX1, hX2)
                                vcTrF(:, :, aX1, aX2) = utilTrF_c(:, :, hX1, hX2)
                                exit
                            end if
                        end do
                    end do
                    exit
                end if
            end do
        end do

        do aX2 = 1, NDIM
            do aX1 = 1, NDIM
                v%cM(aX1, aX2, :, :) = vcTrM(:, :, aX1, aX2)
                v%cF(aX1, aX2, :, :) = vcTrF(:, :, aX1, aX2)
            end do
        end do

        do wX2 = 1, NODEW
            do wX1 = 1, NODEW
                do aX2 = 1, NDIM
                    v%utilM(:, aX2, wX1, wX2) = v%utilncM(:, aX2) + pM%betc * v%cM(:, aX2, wX1, wX2)
                    v%utilF(:, aX2, wX1, wX2) = v%utilncF(:, aX2) + pF%betc * v%cF(:, aX2, wX1, wX2)
                end do
            end do
        end do

        if (v%swfInc) then

            do hX2 = 1, N
                do hX1 = 1, N
                    utilTrIncM_c(:, :, hX1, hX2) = utilIncM_c(hX1, hX2, :, :)
                    utilTrIncF_c(:, :, hX1, hX2) = utilIncF_c(hX1, hX2, :, :)
                    dTrInc(:, :, hX1, hX2) = dInc(hX1, hX2, :, :)
                end do
            end do

            do aX2 = 1, NDIM
                do hX2 = 1, N
                    if (grid%gzInd(aX2, TIME_HOURS) == hX2) then
                        do aX1 = 1, NDIM
                            do hX1 = 1, N
                                if (grid%gzInd(aX1, TIME_HOURS) == hX1) then
                                    vcTrIncM(:, :, aX1, aX2) = utilTrIncM_c(:, :, hX1, hX2)
                                    vcTrIncF(:, :, aX1, aX2) = utilTrIncF_c(:, :, hX1, hX2)
                                    vdTrInc(:, :, aX1, aX2) = dTrInc(:, :, hX1, hX2)
                                    exit
                                end if
                            end do
                        end do
                        exit
                    end if
                end do
            end do

            do aX2 = 1, NDIM
                do aX1 = 1, NDIM
                    cIncM2(aX1, aX2, :, :) = vcTrIncM(:, :, aX1, aX2)
                    cIncF2(aX1, aX2, :, :) = vcTrIncF(:, :, aX1, aX2)
                    v%dInc(aX1, aX2, :, :) = vdTrInc(:, :, aX1, aX2)
                end do
            end do

            do wX2 = 1, NODEW
                do wX1 = 1, NODEW
                    do aX2 = 1, NDIM
                        v%utilIncM(:, aX2, wX1, wX2) = v%utilncM(:, aX2) + pM%betc * cIncM2(:, aX2, wX1, wX2)
                        v%utilIncF(:, aX2, wX1, wX2) = v%utilncF(:, aX2) + pF%betc * cIncF2(:, aX2, wX1, wX2)
                    end do
                end do
            end do
        end if

    end subroutine householdUtilityPreCalcLambda

    subroutine expectedUtility(p, v, grid, rnd, opt, theta, doalloc, eall)
        use mathfn
        implicit none
        type(p_t), intent(in) :: p
        type(v_t), intent(in) :: v
        type(grid_t), intent(in) :: grid
        ! real(dp), intent(in) :: gz(NDIM, GZDIM)
        type(rnd_t), intent(in) :: rnd
        type(opt_t), intent(in) :: opt
        real(dp), intent(in) :: theta
        type(eall_t), intent(out) :: eall
        logical, intent(in) :: doalloc
        integer :: wX0, rX0
        real(dp), dimension(NDIM) :: EuVec, EswfVec, EuVecShift, expEuVecShift, Pr
        real(dp) :: maxEuVec, sumExpEuVecShift

        real(dp), dimension(NR) :: EuNode, EswfNode
        real(dp), dimension(NR) :: EearnNode, EtaxableNode, EtottaxNode
        real(dp), dimension(NODEW) :: EuW, EswfW, EearnW, EtaxableW, EtottaxW, EnetincW

        integer :: axY, aX0
        real(dp) :: expThetaEuVec(NDIM)
        real(dp) :: tempWageVector(NODEW)

        real(dp) :: PrSum(NDIM)
        real(dp) :: Eallocation(NDIM, NODEW), EallocationTr(NODEW, NDIM)

        integer, parameter :: neall = storage_size( eall ) / storage_size( 1.0_dp )

        do wX0 = 1, NODEW

            PrSum = 0.0_dp

            do rX0 = 1, NR

                EuVec = v%util(:, wX0) + p%sd * rnd%ev(:, rX0)

                if (theta == 0.0_dp) then
                    expThetaEuVec = 1.0_dp
                    EswfVec = EuVec
                else
                    expThetaEuVec = exp(min(theta * EuVec, 600.0_dp))
                    EswfVec = expThetaEuVec
                end if

                if ( SMOOTH ) then
                    maxEuVec = maxval(EuVec)
                    EuVecShift = SMSC * (EuVec - maxEuVec)
                    expEuVecShift = exp(EuVecShift)
                    sumExpEuVecShift = sum(expEuVecShift)
                    Pr = expEuVecShift / sumExpEuVecShift
                    if (BOUND_PROB) then
                        Pr = max(Pr, BOUND_PROB_VAL)
                    end if
                    EuNode(rX0) = sum(Pr * EuVec) !expected individual utility of each allocation for each node
                    EswfNode(rX0) = sum(Pr * EswfVec) !expected individual utility of each allocation for each node
                    EtottaxNode(rX0) = sum(Pr * v%tottax(:, wX0))    ! total tax
                else
                    axY = maxloc(EuVec, dim = 1)
                    Pr = 0.0_dp
                    Pr(axY) = 1.0_dp
                    EuNode(rX0) = EuVec(axY)
                    EswfNode(rX0) = EswfVec(axY)
                    EtottaxNode(rX0) = v%tottax(axY, wX0)
                end if


                ! expected allocations
                if (doalloc) then
                    if ( SMOOTH ) then
                        EearnNode(rX0) = sum(Pr * v%earn(:, wX0))
                        EtaxableNode(rX0) = sum(Pr * v%taxable(:, wX0))
                        PrSum = PrSum + Pr
                    else
                        EearnNode(rX0) = v%earn(axY, wX0)
                        EtaxableNode(rX0) = v%taxable(axY, wX0)
                        PrSum = PrSum + Pr
                    end if
                end if

            end do ! rX0

            EuW(wX0) = sum(EuNode) / real(NR, dp)
            EswfW(wX0) = sum(EswfNode) / real(NR, dp)
            EtottaxW(wX0) = sum(EtottaxNode) / real(NR, dp)

            if ( doalloc ) then
                EearnW(wX0) = sum(EearnNode) / real(NR, dp)
                EtaxableW(wX0) = sum(EtaxableNode) / real(NR, dp)
                Eallocation(:, wX0) = PrSum / real(NR, dp)
            end if

        end do

        eall%Eu = wageIntegrateFast(p, EuW)
        eall%Eswf = wageIntegrateFast(p, EswfW)
        eall%Etottax = wageIntegrateFast(p, EtottaxW)

        if ( doalloc ) then

            eall%Eearn = wageIntegrateFast(p, EearnW)
            eall%Etaxable = wageIntegrateFast(p, EtaxableW)
            eall%Enetinc = wageIntegrateFast(p, EnetincW)
            EallocationTr = transpose(Eallocation)
            do aX0 = 1, NDIM
                eall%Eallocation(aX0) = wageIntegrateFast(p, EallocationTr(:, aX0))
                tempWageVector = p%logWageNode * EallocationTr(:, aX0)
                eall%EwageAllocation(aX0) = wageIntegrateFast(p, tempWageVector )
                tempWageVector = tempWageVector * p%logWageNode
                eall%EwageSqAllocation(aX0) = wageIntegrateFast(p, tempWageVector )

                tempWageVector = p%wageNode * EallocationTr(:, aX0)
                eall%EwageLevelAllocation(aX0) = wageIntegrateFast(p, tempWageVector )
                tempWageVector = tempWageVector * p%wageNode
                eall%EwageLevelSqAllocation(aX0) = wageIntegrateFast(p, tempWageVector )

            end do

        end if

    end subroutine expectedUtility


    subroutine expectedUtilityHouseholdFast(h, pM, pF, v, lam, rnd, opt, grid, eall)
        use mathfn
        implicit none
        type(h_t), intent(in) :: h
        type(p_t), intent(in) :: pM
        type(p_t), intent(in) :: pF
        type(v2_t), intent(in) :: v
        real(dp), intent(in) :: lam
        type(rnd_t), intent(in) :: rnd
        type(opt_t), intent(in) :: opt
        type(grid_t), intent(in) :: grid
        real(dp), dimension(NDIM, NDIM) :: EuVec, expEuVecShift, Pr
        real(dp) :: sumExpEuVecShift
        type(eall2_t), intent(out) :: eall
        real(dp), dimension(NR) :: EuNode1, EuNode2
        real(dp), dimension(NODEW, NODEW) :: EuW1, EuW2
        integer :: aX1, aX2
        integer :: wX1, wX2
        integer :: rX0
        integer :: axY(2)
        real(dp) :: expev1(NDIM, NR), expev2(NDIM, NR), expevAll(NDIM, NDIM, NR), evAll(NDIM, NDIM, NR)
        real(dp) :: EuVecAll(NDIM, NDIM)
        real(dp) :: EuVecAllShift(NDIM, NDIM)
        real(dp) :: expEuVecAllShift(NDIM, NDIM)
        real(dp) :: maxEuVecAll
        real(dp) :: Eallocation(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: tempWageMatrix(NODEW, NODEW)
        real(dp) :: tempWageMatrixProd(NODEW, NODEW)

        real(dp) :: PrSum(NDIM, NDIM)
        real(dp) :: Pr1(NDIM)
        real(dp) :: Pr2(NDIM)
        real(dp) :: Pr2temp
        real(dp) :: y

        real(dp) :: EtottaxW12(NODEW, NODEW)

        real(dp) :: EearnNode1(NR)
        real(dp) :: EearnNode2(NR)
        real(dp) :: EtaxableNode1(NR)
        real(dp) :: EtaxableNode2(NR)
        real(dp) :: EearnW1(NODEW, NODEW)
        real(dp) :: EearnW2(NODEW, NODEW)
        real(dp) :: EtaxableW1(NODEW, NODEW)
        real(dp) :: EtaxableW2(NODEW, NODEW)
        logical :: doalloc

        doalloc = .true.
        if ( SMOOTH ) then
            expev1 = exp( min(SMSC * (1.0_dp - lam) * pM%sd * rnd%ev, 700.0_dp) )
            expev2 = exp( min(SMSC * lam * pF%sd * rnd%ev2, 700.0_dp ) )
            do rX0 = 1, NR
                do aX2 = 1, NDIM
                    expevAll(:, aX2, rX0) = expev1(:, rX0) * expev2(aX2, rX0)
                end do
            end do
        else

            do rX0 = 1, NR
                do aX2 = 1, NDIM
                    evAll(:, aX2, rX0) = (1.0_dp - lam) * pM%sd * rnd%ev(:, rX0) + lam * pF%sd * rnd%ev2(aX2, rX0)
                end do
            end do

        end if

        do wX2 = 1, NODEW ! female wages
            do wX1 = 1, NODEW ! male wages

! =============== NO PREF HETRO ===============
                    !household welfare excluding state specific errors
                    do aX2 = 1, NDIM
                        EuVecAll(:, aX2) = (1.0_dp - lam) * v%utilM(:, aX2, wX1, wX2) + lam * v%utilF(:, aX2, wX1, wX2)
                    end do

                    if ( SMOOTH ) then
                        maxEuVecAll = maxval(EuVecAll)
                        EuVecAllShift = SMSC * (EuVecAll - maxEuVecAll)
                        expEuVecAllShift = exp(EuVecAllShift)
                    end if

! =============== NO PREF HETRO ===============


                PrSum = 0.0_dp

                do rX0 = 1, NR ! state specific errors
                    if ( SMOOTH ) then


! =============== NO PREF HETRO ===============
                        expEuVecShift = expEuVecAllShift*expevAll(:, :, rX0)
! =============== NO PREF HETRO ===============

                        sumExpEuVecShift = sum(expEuVecShift)
                        Pr = expEuVecShift / sumExpEuVecShift
                        if (BOUND_PROB) then
                            Pr = max(Pr, BOUND_PROB_VAL)
                        end if

                        Pr1 = 0.0_dp
                        do aX2 = 1, NDIM
                            Pr2temp = 0.0_dp
                            do aX1 = 1, NDIM
                                y = Pr(aX1, aX2)
                                Pr1(aX1) = Pr1(aX1) + y
                                PrSum(aX1, aX2) = PrSum(aX1, aX2) + y
                                Pr2temp = Pr2temp + y
                            end do
                            Pr2(aX2) = Pr2temp
                        end do

                        ! =============== NO PREF HETRO ===============
                        ! contribution of state specific error to utility
                        ! expected individual utility of each allocation for each node
                        EuNode1(rX0) = pM%sd * sum(Pr1 * rnd%ev(:, rX0))
                        EuNode2(rX0) = pF%sd * sum(Pr2 * rnd%ev2(:, rX0))
                        ! =============== NO PREF HETRO ===============

                        if (doalloc) then
                            EearnNode1(rX0) = sum(Pr1 * v%earnM(:, wX1))       ! earnings (ad1)
                            EtaxableNode1(rX0) = sum(Pr1 * v%taxableM(:, wX1)) ! taxable income (ad1)
                            EearnNode2(rX0) = sum(Pr2 * v%earnF(:, wX2))       ! earnings (ad2)
                            EtaxableNode2(rX0) = sum(Pr2 * v%taxableF(:, wX2)) ! taxable income (ad2)
                        end if

                    else

                        ! household welfare including individual state specific error
                        EuVec = EuVecAll + evAll(:, :,  rX0)
                        axY = maxloc(EuVec)

                        EuNode1(rX0) = pM%sd * rnd%ev(axY(1), rX0)
                        EuNode2(rX0) = pF%sd * rnd%ev2(axY(2), rX0)

                        PrSum(axY(1), axY(2)) = PrSum(axY(1), axY(2)) + 1.0_dp

                        if (doalloc) then
                            EearnNode1(rX0) = v%earnM(axY(1), wX1)       ! earnings (ad1)
                            EtaxableNode1(rX0) = v%taxableM(axY(1), wX1) ! taxable income (ad1)
                            EearnNode2(rX0) = v%earnF(axY(2), wX2)       ! earnings (ad2)
                            EtaxableNode2(rX0) = v%taxableF(axY(2), wX2) ! taxable income (ad2)
                        end if

                    end if

                end do

                ! for taxes, only the joint wage / allocation distribution matters
                EtottaxW12(wX1, wX2) = sum( PrSum * v%tottax(:, :, wX1, wX2) ) / real(NR, dp)

! =============== NO PREF HETRO ===============
                EuW1(wX1, wX2) = ( sum(PrSum * v%utilM(:, :, wX1, wX2)) + sum(EuNode1) ) / real(NR, dp)
                EuW2(wX1, wX2) = ( sum(PrSum * v%utilF(:, :, wX1, wX2)) + sum(EuNode2) ) / real(NR, dp)
! =============== NO PREF HETRO ===============

                Eallocation(:, :, wX1, wX2) = PrSum / real(NR, dp)

                if (doalloc) then
                    EearnW1(wX1, wX2) = sum(EearnNode1) / real(NR, dp)
                    EtaxableW1(wX1, wX2) = sum(EtaxableNode1) / real(NR, dp)
                    EearnW2(wX1, wX2) = sum(EearnNode2) / real(NR, dp)
                    EtaxableW2(wX1, wX2) = sum(EtaxableNode2) / real(NR, dp)
                end if

            end do
        end do

        do aX2 = 1, NDIM
            do aX1 = 1, NDIM
                tempWageMatrix = Eallocation(aX1, aX2, :, :)
                eall%Eallocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrix)

                ! multiply by male wage
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%logWageNode * tempWageMatrix(:, wX2)
                end do
                eall%Ewage1Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%logWageNode * tempWageMatrixProd(:, wX2)
                end do
                eall%EwageSq1Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)

                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%wageNode * tempWageMatrix(:, wX2)
                end do
                eall%EwageLevel1Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%wageNode * tempWageMatrixProd(:, wX2)
                end do
                eall%EwageLevelSq1Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)


                ! eall%Eearn1Allocation(aX1, aX2) = eall%Ewage1Allocation(aX1, aX2) * hrsatax

                ! multiply by female wage
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pF%logWageNode(wX2) * tempWageMatrix(:, wX2)
                end do
                eall%Ewage2Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pF%logWageNode(wX2) * tempWageMatrixProd(:, wX2)
                end do
                eall%EwageSq2Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)

                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pF%wageNode(wX2) * tempWageMatrix(:, wX2)
                end do
                eall%EwageLevel2Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pF%wageNode(wX2) * tempWageMatrixProd(:, wX2)
                end do
                eall%EwageLevelSq2Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)


                ! multiply by male and female wage
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%logWageNode * pF%logWageNode(wX2) * tempWageMatrix(:, wX2)
                end do
                eall%Ewage12Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%logWageNode * pF%logWageNode(wX2) * tempWageMatrixProd(:, wX2)
                end do
                eall%EwageSq12Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)

            end do
        end do

        eall%Eu1 = wageIntegrateFast(h, pM, pF, EuW1)
        eall%Eu2 = wageIntegrateFast(h, pM, pF, EuW2)
        eall%Etottax = wageIntegrateFast(h, pM, pF, EtottaxW12)

        if ( doalloc ) then
            eall%Eearn1 = wageIntegrateFast(h, pM, pF, EearnW1)
            eall%Etaxable1= wageIntegrateFast(h, pM, pF, EtaxableW1)
            eall%Eearn2 = wageIntegrateFast(h, pM, pF, EearnW2)
            eall%Etaxable2 = wageIntegrateFast(h, pM, pF, EtaxableW2)
        end if

    end subroutine expectedUtilityHouseholdFast



    subroutine expectedUtilityHouseholdWelfareFast(h, pM, pF, v, lam, rnd, opt, grid, eall, theta)
        use mathfn
        implicit none
        type(h_t), intent(in) :: h
        type(p_t), intent(in) :: pM
        type(p_t), intent(in) :: pF
        type(v2_t), intent(in) :: v
        real(dp), intent(in) :: lam
        real(dp), intent(in) :: theta
        type(rnd_t), intent(in) :: rnd
        type(opt_t), intent(in) :: opt
        type(grid_t), intent(in) :: grid
        real(dp), dimension(NDIM, NDIM) :: EuVec, expEuVecShift, Pr
        real(dp) :: sumExpEuVecShift
        type(eall2_t), intent(out) :: eall
        real(dp), dimension(NR) :: EuNode1, EuNode2
        real(dp), dimension(NODEW, NODEW) :: EuW1, EuW2
        integer :: aX1, aX2
        integer :: wX1, wX2
        integer :: rX0
        integer :: axY(2)
        real(dp) :: expev1(NDIM, NR), expev2(NDIM, NR), expevAll(NDIM, NDIM, NR), evAll(NDIM, NDIM, NR)
        real(dp) :: EuVecAll(NDIM, NDIM)
        real(dp) :: EuVecAllShift(NDIM, NDIM)
        real(dp) :: expEuVecAllShift(NDIM, NDIM)
        real(dp) :: maxEuVecAll
        real(dp) :: Eallocation(NDIM, NDIM, NODEW, NODEW)
        real(dp) :: tempWageMatrix(NODEW, NODEW)
        real(dp) :: tempWageMatrixProd(NODEW, NODEW)

        real(dp) :: PrSum(NDIM, NDIM)
        real(dp) :: Pr1(NDIM)
        real(dp) :: Pr2(NDIM)
        real(dp) :: Pr2temp
        real(dp) :: y

        real(dp) :: expSwfEv1(NDIM, NR)
        real(dp) :: expSwfEv2(NDIM, NR)
        real(dp) :: expSwfEv1All(NDIM, NDIM, NR)
        real(dp) :: expSwfEv2All(NDIM, NDIM, NR)
        real(dp) :: expSwfAll1(NDIM, NDIM)
        real(dp) :: expSwfAll2(NDIM, NDIM)

        real(dp) :: EtottaxW12(NODEW, NODEW)
        real(dp) :: EswfVec1(NDIM, NDIM, NR)
        real(dp) :: EswfVec2(NDIM, NDIM, NR)
        real(dp) :: EswfNode1(NR)
        real(dp) :: EswfNode2(NR)
        real(dp) :: EswfW1(NODEW, NODEW)
        real(dp) :: EswfW2(NODEW, NODEW)

        ! used for calculating welfare weights
        real(dp) :: EuIncW1(NODEW, NODEW)
        real(dp) :: EuIncW2(NODEW, NODEW)
        real(dp) :: EswfIncW1(NODEW, NODEW)
        real(dp) :: EswfIncW2(NODEW, NODEW)
        real(dp) :: EswfNodeInc1(NR)
        real(dp) :: EswfNodeInc2(NR)
        real(dp) :: expSwfAllInc1(NDIM, NDIM)
        real(dp) :: expSwfAllInc2(NDIM, NDIM)
        real(dp) :: EswfVecInc1(NDIM, NDIM, NR)
        real(dp) :: EswfVecInc2(NDIM, NDIM, NR)
        real(dp) :: dIncW(NODEW, NODEW)

        if ( SMOOTH ) then
            expev1 = exp(SMSC * (1.0_dp - lam) * pM%sd * rnd%ev)
            expev2 = exp(SMSC * lam * pF%sd * rnd%ev2)
            do rX0 = 1, NR
                do aX2 = 1, NDIM
                    expevAll(:, aX2, rX0) = expev1(:, rX0) * expev2(aX2, rX0)
                end do
            end do

        else

            do rX0 = 1, NR
                do aX2 = 1, NDIM
                    evAll(:, aX2, rX0) = (1.0_dp - lam) * pM%sd * rnd%ev(:, rX0) + lam * pF%sd * rnd%ev2(aX2, rX0)
                end do
            end do

        end if


        ! for social welfare function
        if (theta <= 0.0_dp) then
            expSwfEv1 = exp( theta * pM%sd * rnd%ev  )
            expSwfEv2 = exp( theta * pF%sd * rnd%ev2 )

            do rX0 = 1, NR
                do aX2 = 1, NDIM
                    expSwfEv1All(:, aX2, rX0) = expSwfEv1(:, rX0)
                end do
            end do
            do rX0 = 1, NR
                do aX2 = 1, NDIM
                    expSwfEv2All(:, aX2, rX0) = expSwfEv2(aX2, rX0)
                end do
            end do
        end if

        do wX2 = 1, NODEW ! female wages
            do wX1 = 1, NODEW ! male wages

                ! household welfare excluding state specific errors
                do aX2 = 1, NDIM
                    EuVecAll(:, aX2) = (1.0_dp - lam) * v%utilM(:, aX2, wX1, wX2) + lam * v%utilF(:, aX2, wX1, wX2)
                end do

                if ( SMOOTH ) then
                    maxEuVecAll = maxval(EuVecAll)
                    EuVecAllShift = SMSC * (EuVecAll - maxEuVecAll)
                    expEuVecAllShift = exp(EuVecAllShift)
                end if


                ! for social welfare function
                if (theta < 0.0_dp) then

                    expSwfAll1 = exp( min( theta * v%utilM(:, :, wX1, wX2), 700.0_dp ) )
                    expSwfAll2 = exp( min( theta * v%utilF(:, :, wX1, wX2), 700.0_dp ) )
                    do rX0 = 1, NR
                        EswfVec1(:, :, rX0) = expSwfAll1 * expSwfEv1All(:, :, rX0)
                        EswfVec2(:, :, rX0) = expSwfAll2 * expSwfEv2All(:, :, rX0)
                    end do

                    if ( v%swfInc ) then
                        expSwfAllInc1 = exp( min( theta * v%utilIncM(:, :, wX1, wX2), 700.0_dp ) )
                        expSwfAllInc2 = exp( min( theta * v%utilIncF(:, :, wX1, wX2), 700.0_dp ) )
                        do rX0 = 1, NR
                            EswfVecInc1(:, :, rX0) = expSwfAllInc1 * expSwfEv1All(:, :, rX0)
                            EswfVecInc2(:, :, rX0) = expSwfAllInc2 * expSwfEv2All(:, :, rX0)
                        end do
                    end if

                end if

                PrSum = 0.0_dp

                do rX0 = 1, NR ! state specific errors
                    if ( SMOOTH ) then

                        expEuVecShift = expEuVecAllShift * expevAll(:, :, rX0)

                        sumExpEuVecShift = sum(expEuVecShift)
                        Pr = expEuVecShift / sumExpEuVecShift
                        if (BOUND_PROB) then
                            Pr = max(Pr, BOUND_PROB_VAL)
                        end if

                        Pr1 = 0.0_dp
                        do aX2 = 1, NDIM
                            Pr2temp = 0.0_dp
                            do aX1 = 1, NDIM
                                y = Pr(aX1, aX2)
                                Pr1(aX1) = Pr1(aX1) + y
                                PrSum(aX1, aX2) = PrSum(aX1, aX2) + y
                                Pr2temp = Pr2temp + y
                            end do
                            Pr2(aX2) = Pr2temp
                        end do

                        ! expected individual utility of each allocation for each node
                        EuNode1(rX0) = pM%sd * sum(Pr1 * rnd%ev(:, rX0))
                        EuNode2(rX0) = pF%sd * sum(Pr2 * rnd%ev2(:, rX0))

                        if (theta < 0.0_dp) then
                            EswfNode1(rX0) = sum(Pr * EswfVec1(:, :, rX0)) !expected individual welfare at each node
                            EswfNode2(rX0) = sum(Pr * EswfVec2(:, :, rX0)) !expected individual welfare at each node
                            if ( v%swfInc ) then
                                EswfNodeInc1(rX0) = sum(Pr * EswfVecInc1(:, :, rX0))
                                EswfNodeInc2(rX0) = sum(Pr * EswfVecInc2(:, :, rX0))
                            end if
                        end if

                    else

                        ! household welfare including individual state specific error
                        EuVec = EuVecAll + evAll(:, :, rX0)
                        axY = maxloc(EuVec)

                        EuNode1(rX0) = pM%sd * rnd%ev(axY(1), rX0)
                        EuNode2(rX0) = pF%sd * rnd%ev2(axY(2), rX0)

                        PrSum(axY(1), axY(2)) = PrSum(axY(1), axY(2)) + 1.0_dp

                    end if

                end do

                ! for taxes, only the joint wage / allocation distribution matters
                EtottaxW12(wX1, wX2) = sum( PrSum * v%tottax(:, :, wX1, wX2) ) / real(NR, dp)

                EuW1(wX1, wX2) = ( sum(PrSum * v%utilM(:, :, wX1, wX2)) + sum(EuNode1) ) / real(NR, dp)
                EuW2(wX1, wX2) = ( sum(PrSum * v%utilF(:, :, wX1, wX2)) + sum(EuNode2) ) / real(NR, dp)
                if ( v%swfInc ) then
                    EuIncW1(wX1, wX2) = ( sum(PrSum * v%utilIncM(:, :, wX1, wX2)) + sum(EuNode1) ) / real(NR, dp)
                    EuIncW2(wX1, wX2) = ( sum(PrSum * v%utilIncF(:, :, wX1, wX2)) + sum(EuNode2) ) / real(NR, dp)
                end if

                if (theta < 0.0_dp) then
                    EswfW1(wX1, wX2) = sum( EswfNode1 ) / real(NR, dp)
                    EswfW2(wX1, wX2) = sum( EswfNode2 ) / real(NR, dp)
                    if (v%swfInc) then
                        EswfIncW1(wX1, wX2) = sum( EswfNodeInc1 ) / real(NR, dp)
                        EswfIncW2(wX1, wX2) = sum( EswfNodeInc2 ) / real(NR, dp)
                        dIncW(wX1, wX2) = sum(PrSum * v%dInc(:, :, wX1, wX2)) / real(NR, dp)
                    end if
                else
                    EswfW1(wX1, wX2) = EuW1(wX1, wX2)
                    EswfW2(wX1, wX2) = EuW2(wX1, wX2)
                    if ( v%swfInc ) then
                        EswfIncW1(wX1, wX2) = EuIncW1(wX1, wX2)
                        EswfIncW2(wX1, wX2) = EuIncW2(wX1, wX2)
                        dIncW(wX1, wX2) = sum(PrSum * v%dInc(:, :, wX1, wX2)) / real(NR, dp)
                    end if
                end if

                Eallocation(:, :, wX1, wX2) = PrSum / real(NR, dp)

            end do
        end do


        do aX2 = 1, NDIM
            do aX1 = 1, NDIM
                tempWageMatrix = Eallocation(aX1, aX2, :, :)
                eall%Eallocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrix)

                ! multiply by male wage
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%logWageNode * tempWageMatrix(:, wX2)
                end do
                eall%Ewage1Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%logWageNode * tempWageMatrixProd(:, wX2)
                end do
                eall%EwageSq1Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)

                ! multiply by female wage
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pF%logWageNode(wX2) * tempWageMatrix(:, wX2)
                end do
                eall%Ewage2Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pF%logWageNode(wX2) * tempWageMatrixProd(:, wX2)
                end do
                eall%EwageSq2Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)

                ! multiply by male and female wage
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%logWageNode * pF%logWageNode(wX2) * tempWageMatrix(:, wX2)
                end do
                eall%Ewage12Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)
                do wX2 = 1, NODEW
                    tempWageMatrixProd(:, wX2) = pM%logWageNode * pF%logWageNode(wX2) * tempWageMatrixProd(:, wX2)
                end do
                eall%EwageSq12Allocation(aX1, aX2) = wageIntegrateFast(h, pM, pF, tempWageMatrixProd)

                ! tax revenue

            end do
        end do

        eall%Eu1 = wageIntegrateFast(h, pM, pF, EuW1)
        eall%Eu2 = wageIntegrateFast(h, pM, pF, EuW2)
        eall%Eswf1 = wageIntegrateFast(h, pM, pF, EswfW1)
        eall%Eswf2 = wageIntegrateFast(h, pM, pF, EswfW2)
        eall%Etottax = wageIntegrateFast(h, pM, pF, EtottaxW12)

        if ( v%swfInc ) then
            eall%EswfInc1 = wageIntegrateFast(h, pM, pF, EswfIncW1)
            eall%EswfInc2 = wageIntegrateFast(h, pM, pF, EswfIncW2)
            eall%dInc = wageIntegrateFast(h, pM, pF, dIncW)
        end if

    end subroutine expectedUtilityHouseholdWelfareFast


    pure function wageIntegrate_2d(pM, pF, x) result(y)
        use mathfn
        implicit none
        type(p_t), intent(in) :: pM, pF
        real(dp), intent(in) :: x(NODEW, NODEW)
        real(dp) :: y
        real(dp) :: tempVector1(NODEW)
        real(dp) :: tempVector2(NODEW)
        integer :: wX2

        if (NODEW == 1) then
            y = x(1, 1)
        else
            ! conditional on female wage wX2, integrate over male wages
            do wX2 = 1, NODEW
                tempVector1 = x(:, wX2) * pM%wageDensity
                tempVector2(wX2) = trapz( pM%wageNode, tempVector1, NODEW )
            end do

            ! now integrate over female wages
            tempVector1 = tempVector2 * pF%wageDensity
            y = trapz( pF%wageNode, tempVector1, NODEW )
        end if

    end function wageIntegrate_2d



    pure function correlatedWageIntegrateFast_2d(h, pM, pF, x) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: correlatedWageIntegrateFast_2d
        use mathfn
        implicit none
        type(h_t), intent(in) :: h
        type(p_t), intent(in) :: pM, pF
        real(dp),  intent(in) :: x(NODEW, NODEW)
        real(dp) :: y
        real(dp) :: tempVector2(NODEW)
        integer :: wX2

        if (NODEW == 1) then
            y = x(1, 1)
        else
            ! conditional on female wage wX2, integrate over male wages
            do wX2 = 1, NODEW
                tempVector2(wX2) = sum( h%wageTrapzWeightM(:, wX2) * x(:, wX2) )
            end do

            ! now integrate over female wages
            y = sum( pF%dwageNode * tempVector2 )
        end if

    end function correlatedWageIntegrateFast_2d


    pure function correlatedWageIntegrate_2d(h, pM, pF, x) result(y)
        use mathfn
        implicit none
        type(h_t), intent(in) :: h
        type(p_t), intent(in) :: pM, pF
        real(dp),  intent(in) :: x(NODEW, NODEW)
        real(dp) :: y
        real(dp) :: tempVector1(NODEW)
        real(dp) :: tempVector2(NODEW)
        integer :: wX2

        if (NODEW == 1) then
            y = x(1,1)
        else
            ! conditional on female wage wX2, integrate over male wages
            do wX2 = 1, NODEW
                tempVector1 = x(:, wX2) * h%jointWageDensity(:, wX2)
                tempVector2(wX2) = trapz( pM%wageNode, tempVector1, NODEW )
            end do

            ! now integrate over female wages
            y = trapz( pF%wageNode, tempVector2, NODEW )
        end if

    end function correlatedWageIntegrate_2d


    pure function wageIntegrate_1d(p, x) result(y)
        use mathfn
        implicit none
        type(p_t), intent(in) :: p
        real(dp), intent(in) :: x(NODEW)
        real(dp) :: y
        real(dp) :: tempVector(NODEW)

        if (NODEW == 1) then
            y = x(1)
        else
            ! now integrate over female wages
            tempVector = x * p%wageDensity
            y = trapz( p%wageNode, tempVector, NODEW )
        end if

    end function wageIntegrate_1d

    pure function wageIntegrateFast_1d(p, x) result(y)
        use mathfn
        implicit none
        type(p_t), intent(in) :: p
        real(dp), intent(in) :: x(NODEW)
        real(dp) :: y

        if (NODEW == 1) then
            y = x(1)
        else
            ! now integrate over female wages
            y = sum( p%wageTrapzWeight * x )
        end if

    end function wageIntegrateFast_1d

    subroutine consumptionShare(pM, pF, lam, netinc, cM, cF, share)
        use mathfn
        implicit none
        type(p_t), intent(in) :: pF
        type(p_t), intent(in) :: pM
        real(dp), intent(in) :: lam
        real(dp), intent(in) :: netinc(N, N, NODEW, NODEW)
        real(dp), intent(out) :: cM(N, N, NODEW, NODEW)
        real(dp), intent(out) :: cF(N, N, NODEW, NODEW)
        real(dp), intent(out) :: share(N, N, NODEW, NODEW)
        integer :: wX1, wX2
        integer :: hX1, hX2
        real(dp), parameter :: mins = 1.0e-12_dp
        real(dp), parameter :: maxs = 1.0_dp - mins
        real(dp), parameter :: tol = 1.0e-12_dp
        logical,  parameter :: loglinear = .false.
        integer :: iuser(1)
        real(dp) :: ruser(4)
        integer :: ifail
        real(dp) :: logJointc(N, N, NODEW, NODEW)
        real(dp) :: jointcPow1(N, N, NODEW, NODEW)
        real(dp) :: jointcPow2(N, N, NODEW, NODEW)
        real(dp) :: loglam, log1_lam
        real(dp) :: rootMins, rootMaxs
        real(dp) :: shareScalar
        real(dp) :: lamRatio
        logical, parameter :: numerical = .false.

        if ( loglinear ) then
            iuser(1) = 1
            loglam = log(lam)
            log1_lam = log(1.0_dp-lam)
        else
            iuser(1) = 0
        end if

        if ( pM%sigc == pF%sigc ) then

            if (lam < mins) then
                share = mins
            elseif (lam>maxs) then
                share = maxs
            else

                if ( numerical ) then ! not needed, for testing
                    if ( loglinear ) then
                        ruser(1) = log1_lam + pM%logBetc
                        ruser(2) = loglam + pF%logBetc
                        ruser(3) = pM%sigc
                        ruser(4) = pF%sigc
                    else
                        ruser(1) = (1.0_dp - lam) * pM%betc
                        ruser(2) = lam * pF%betc
                        ruser(3) = pM%sigc
                        ruser(4) = pF%sigc
                    end if

                    call zeroinDouble( mins, maxs, shareScalar, consumptionShareEquation, &
                        iuser, ruser, tol, ifail )

                    if (ifail /= 0) then

                        rootMins = consumptionShareEquation(mins, iuser, ruser)
                        rootMaxs = consumptionShareEquation(maxs, iuser, ruser)

                        if ( abs(rootMins) < abs(rootMaxs) ) then
                            shareScalar = mins
                        else
                            shareScalar = maxs
                        end if

                    end if

                else

                    lamRatio = (pF%betc / pM%betc) * lam / (1.0_dp - lam)
                    shareScalar = 1.0_dp / ( 1.0_dp + lamRatio**(-1.0_dp / pM%sigc) )

                end if

                share = shareScalar

            end if

        else

            if ( loglinear ) then
                logJointc = log(netinc)
            else
                jointcPow1 = netinc ** (1.0_dp - pM%sigc)
                jointcPow2 = netinc ** (1.0_dp - pF%sigc)
            end if

            if (lam < mins) then
                share = mins
            elseif (lam>maxs) then
                share = maxs
            else

                ! these are not changing in the loop
                ruser(3) = pM%sigc
                ruser(4) = pF%sigc

                do wX2 = 1, NODEW
                    do wX1 = 1, NODEW
                        do hX2 = 1, N
                            do hX1 = 1, N

                                if ( loglinear ) then
                                    ruser(1) = log1_lam + pM%logBetc + (1.0_dp - pM%sigc)*logJointc(hX1, hX2, wX1, wX2)
                                    ruser(2) = loglam + pF%logBetc + (1.0_dp - pF%sigc)*logJointc(hX1, hX2, wX1, wX2)
                                else
                                    ruser(1) = (1.0_dp - lam) * pM%betc * jointcPow1(hX1, hX2, wX1, wX2) !**(1.0_dp-pM%sigc)
                                    ruser(2) = lam*pF%betc * jointcPow2(hX1, hX2, wX1, wX2) !**(1.0_dp-pF%sigc)
                                end if

                                call zeroinDouble( mins, maxs, share(hX1, hX2, wX1, wX2), consumptionShareEquation, &
                                    iuser, ruser, tol, ifail )

                                if (ifail /= 0) then

                                    rootMins = consumptionShareEquation(mins, iuser, ruser)
                                    rootMaxs = consumptionShareEquation(maxs, iuser, ruser)

                                    if ( abs(rootMins) < abs(rootMaxs) ) then
                                        share(hX1, hX2, wX1, wX2) = mins
                                    else
                                        share(hX1, hX2, wX1, wX2) = maxs
                                    end if

                                end if

                            end do
                        end do
                    end do
                end do
            end if

        end if

        cM = (1.0_dp - share) * netinc
        cF = netinc - cM

    end subroutine consumptionShare


    pure function consumptionShareEquation(x, iuser, ruser) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: consumptionShareEquation
        implicit none
        real(dp), intent(in) :: x
        integer,  intent(in) :: iuser(:)
        real(dp), intent(in) :: ruser(:)
        real(dp) :: y
        real(dp) :: base(2)
        real(dp) :: power(2)
        real(dp) :: basepow(2)
        if (iuser(1) > 0) then
            base(1) = 1.0_dp - x
            base(2) = x
            basepow = log(base)
            y = ruser(1) - ruser(2) - ruser(3) * basepow(1) + ruser(4) * basepow(2)
        else
            base(1) = 1.0_dp - x
            base(2) = x
            power(1) = -ruser(3)
            power(2) = -ruser(4)
            basepow = base ** power
            y = ruser(1) * basepow(1) - ruser(2) * basepow(2)
        end if

    end function consumptionShareEquation

    subroutine elasticityMarket(x, xtax, estmode, texpath, outpath, ftag)
        use moments
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(in) :: xtax(NTAXPARAM)
        integer, intent(in) :: estmode
        character(len = *), intent(in) :: texpath
        character(len = *), intent(in) :: outpath
        integer, intent(in) :: ftag
        type(opt_t) :: opt

        integer :: ftag2
        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)
        real(dp) :: lam(NTYPE1, NTYPE2, NMARKET)

        ! tax schedules
        type(tax_t) :: tax(NMARKET)

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1, NMARKET)
        type(p_t) :: pF(NTYPE2, 0:1, NMARKET)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1, NMARKET)

        ! marriage market
        type(m_t) :: m(NMARKET)

        ! dummy arguments
        type(eall_t) :: eallM(NTYPE1, 0:1, NMARKET)
        type(eall_t) :: eallF(NTYPE2, 0:1, NMARKET)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1, NMARKET)

        integer :: mX

        integer, parameter :: indBase  = 1
        integer, parameter :: indWageM = 2
        integer, parameter :: indWageF = 3

        call setOptions(opt, OPT_ESTIMATION, estmode)
        opt%extraMom = .true.

        do mX = 1, NMARKET
            call setup( x, xt, xtype, xtax, opt, mX, tax(mX), m(mX), h(:, :, :,mX), &
                        pM(:, :, mX), pF(:, :, mX), grid, rnd )
        end do

        m%marketSize = m%marketSize / sum( m%marketSize )

        do mX = 1, NMARKET
            print *, 'new', mX
            call new( tax(mX), m(mX), h(:, :, :, mX), pM(:, :, mX), pF(:, :, mX), grid, rnd, opt, &
                eallM(:, :, mX), eallF(:, :, mX), eall(:, :, :, mX) )

            lam(:, :, mX) = m(mX)%lam

        end do

        ! if ftag < 0 it creates the tex table file, otherwise, the output file
        call elas( tax, m, h, pM, pF, grid, rnd, opt, estmode, lam, -1, .true., &
                   texpath // '/elasticity.tex', outpath, ftag )

    end subroutine elasticityMarket

    subroutine elasticityMarriage(x, xtax, estmode, verbose, fpath, ftag)
        use mathfn
        use moments
        implicit none
        real(dp), intent(in) :: x(NPARAM)
        real(dp), intent(in) :: xtax(NTAXPARAM)
        integer, intent(in) :: estmode
        logical, intent(in) :: verbose
        character(len=*), intent(in), optional :: fpath
        integer, intent(in), optional :: ftag

        type(momExtra_t) :: momExtra
        type(opt_t) :: opt
        type(ret_t) :: ret

        real(dp) :: xt(NPARAM)
        integer :: xtype(NPARAM)
        real(dp) :: lam(NTYPE1, NTYPE2, NMARKET, 2)

        ! tax schedules
        type(tax_t) :: tax(NMARKET)

        ! time allocation grid
        type(grid_t) :: grid

        ! state specific errors
        type(rnd_t) :: rnd

        ! person type attibutes
        type(p_t) :: pM(NTYPE1, 0:1, NMARKET)
        type(p_t) :: pF(NTYPE2, 0:1, NMARKET)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1, NMARKET)

        ! marriage market
        type(m_t) :: m(NMARKET)

        ! dummy arguments
        type(eall_t) :: eallM(NTYPE1, 0:1, NMARKET)
        type(eall_t) :: eallF(NTYPE2, 0:1, NMARKET)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1, NMARKET)

        integer :: cX, mX, pX1, pX2

        integer, parameter :: indBase  = 1
        integer, parameter :: indWageM = 2
        integer, parameter :: indWageF = 3
        integer :: funit

        real(dp) :: jtype_M0(NTYPE1, 2)
        real(dp) :: jtype_0F(NTYPE2, 2)
        real(dp) :: jtype_MF(NTYPE1, NTYPE2, 2)
        real(dp) :: lamAvg(NTYPE1, NTYPE2, 2)

        character(len=*), parameter :: fmt = '(3(I2,",",2X),F28.16)'

        call setOptions(opt, OPT_ESTIMATION, estmode)
        opt%extraMom = .true.

        do mX = 1, NMARKET
            call setup(x, xt, xtype, xtax, opt, mX, tax(mX), m(mX), h(:, :, :, mX), &
                       pM(:, : ,mX), pF(:, : ,mX), grid, rnd)
        end do

        m%marketSize = m%marketSize / sum(m%marketSize)
        lamAvg = 0.0_dp

        do cX = 1, 2

            if (cX == 2) then
                opt%marriageMarketElasticity = .true.
            else
                opt%marriageMarketElasticity = .false.
            end if

            do mX = 1, NMARKET

                if (verbose) then
                    print *, cX, mX
                end if

                call new(tax(mX), m(mX), h(:, :, :, mX), pM(:, :, mX), pF(:, :, mX), grid, rnd, opt, &
                         eallM(:, :, mX), eallF(:, :, mX), eall(:, :, :, mX))

                lam(:, :, mX,cX) = m(mX)%lam

            end do

            call allocationToMomentsMarket(opt, ret, m, eallM, eallF, eall)

            momExtra = transfer( ret%momExtraVec, momExtra )

            jtype_M0(:, cX) = momExtra%jtype_M0
            jtype_0F(:, cX) = momExtra%jtype_0F
            jtype_MF(:, :, cX) = momExtra%jtype_MF

            do mX = 1, NMARKET
                do pX2 = 1, NTYPE2
                    do pX1 = 1, NTYPE1
                        lamAvg(pX1, pX2, cX) = lamAvg(pX1, pX2, cX) + m(mX)%marketSize * lam(pX1, pX2, mX, cX)
                    end do
                end do
            end do

        end do

        if (verbose) then
            write(IOUT,'(A)')
            write(IOUT,'(A)') 'Marriage market matching function'
            write(IOUT,'(A)') '---------------------------------'
            write(IOUT,'(A)')
        end if

        if ( present(ftag) ) then
            if ( present(fpath) ) then
                open(newunit = funit, file = fpath // '/mmf' // intToStr(ftag) // '.csv', status = "replace")
            else
                open(newunit = funit, file = 'output/post/mmf' // intToStr(ftag) // '.csv', status = "replace")
            end if
        else
            if ( present(fpath) ) then
                open(newunit = funit, file = fpath // '/mmf.csv', status = "replace")
            else
                open(newunit = funit, file = 'output/post/mmf.csv', status = "replace")
            end if
        end if

        do cX = 1, 2
            do pX1 = 1, NTYPE1
                write(funit, fmt) cX, pX1, 0, jtype_M0(pX1, cX)
                if (verbose) then
                    write(IOUT, '(3(I4,",",2X),F10.7)') cX, pX1, 0, jtype_M0(pX1, cX)
                end if
            end do

            do pX2 = 1, NTYPE2
                write(funit, fmt) cX, 0, pX2, jtype_0F(pX2, cX)
                if (verbose) then
                    write(IOUT, '(3(I4,",",2X),F10.7)') cX, 0, pX2, jtype_0F(pX2, cX)
                end if
            end do

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    write(funit, fmt) cX, pX1, pX2, jtype_MF(pX1, pX2, cX)
                    if (verbose) then
                        write(IOUT, '(3(I4,",",2X),F10.7)') cX, pX1, pX2, jtype_MF(pX1, pX2, cX)
                    end if
                end do
            end do
        end do
        close(funit)

        if (verbose) then
            write(IOUT,'(A)')
            write(IOUT,'(A)') 'Average Pareto weights'
            write(IOUT,'(A)') '---------------------------------'
            write(IOUT,'(A)')
        end if

        if ( present(ftag) ) then
            if ( present(fpath) ) then
                open( newunit = funit, file = fpath // '/lam' // intToStr(ftag) // '.csv', status = "replace" )
            else
                open( newunit = funit, file = 'output/marriage_elas/lam' // intToStr(ftag) // '.csv', status = "replace" )
            end if
        else
            if ( present(fpath) ) then
                open( newunit = funit, file = fpath // '/lam.csv', status = "replace" )
            else
                open( newunit = funit, file = 'output/marriage_elas/lam.csv', status = "replace" )
            end if
        end if

        do cX = 1, 2
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    write(funit, fmt) cX, pX1, pX2, lamAvg(pX1, pX2, cX)
                    if (verbose) then
                        write(IOUT, '(3(I4,",",2X),F10.7)') cX, pX1, pX2, lamAvg(pX1, pX2, cX)
                    end if
                end do
            end do
        end do
        close(funit)

    end subroutine elasticityMarriage

    subroutine xwrite(x, fname, tag)
        use mathfn, only : intToStr
        implicit none
        real(dp), intent(in) :: x(:)
        character(len = *), intent(in) :: fname
        integer, intent(in) :: tag
        integer :: funit, iX

        open(newunit = funit, file = fname // '_' // intToStr(tag), status = 'replace', action = 'write')
        do iX = 1, size(x)
            write(funit, *) x(iX)
        end do
        close(funit)

    end subroutine xwrite

    subroutine xread(x, fname)
        implicit none
        real(dp), intent(out) :: x(:)
        character(len = *), intent(in) :: fname
        integer :: funit, iX

        open(newunit = funit, file = fname, status = 'old', action = 'read')
        do iX = 1, size(x)
            read(funit,*) x(iX)
        end do
        close(funit)

    end subroutine xread

    subroutine setgrid(opt, grid)
        use mathfn
        implicit none
        type(opt_t), intent(in) :: opt
        type(grid_t), intent(out) :: grid
        integer :: iX, ixD, ixG, ixG2
        integer :: i1, i2, i3

        real(dp), parameter :: cliptol = 0.5_dp * real(TIME_TOTAL, dp) / real(N - 1, dp)

        call linspace( 0.0_dp, real(TIME_TOTAL, dp), N, grid%gridval )

        do iX = 1, N
            grid%gridind(iX) = iX
        end do

        grid%gz = 0.0_dp
        grid%gzind = 0

        ! dummy run
        ixD = 0
        do ixG = 1, N
            do ixG2 = ixG, n
                i1 = iXG
                i2 = N + 1 - ixG2
                i3 = N + 2 - i1 - i2
                if ( i1 >= MIN_N(1) .and. i2 >= MIN_N(2) .and. i3 >= MIN_N(3) .and. i1 <= MAX_N(1) &
                    .and. i2 <= MAX_N(2) .and. i3 <= MAX_N(3) ) then
                    ixD = ixD + 1
                end if
            end do
        end do

        if (ixD /= NDIM) then
            print *, 'globaldef NDIM', NDIM
            print *, 'required  NDIM', ixD
            stop
        end if

        ixD = 0
        do ixG = 1, N
            do ixG2 = ixG, N
                i1 = iXG
                i2 = N + 1 - ixG2
                i3 = N + 2 - i1 - i2
                if ( i1 >= MIN_N(1) .and. i2 >= MIN_N(2) .and. i3 >= MIN_N(3) .and. i1 <= MAX_N(1) .and. i2 <= MAX_N(2) &
                    .and. i3 <= MAX_N(3) ) then
                    ixD = ixD + 1
                    grid%gzind(ixD, TIME_LEISURE) = i1
                    grid%gzind(ixD, TIME_PUBLIC) = i2
                    grid%gzind(ixD, TIME_HOURS) = i3

                    grid%gz(ixD, TIME_LEISURE) = grid%gridval(i1)
                    grid%gz(ixD, TIME_PUBLIC) = grid%gridval(i2)
                    grid%gz(ixD, TIME_HOURS) = grid%gridval(i3)
                end if
            end do
        end do

        do iX = 1, NDIM
            grid%gz(iX, TIME_EMPLOYMENT) = merge(1.0_dp, 0.0_dp, grid%gz(iX, TIME_HOURS) > cliptol)
        end do

        grid%gz(:, TIME_LEISURE) = MIN_TIME_LEISURE + grid%gz(:, TIME_LEISURE)
        grid%gz(:, TIME_PUBLIC) = MIN_TIME_PUBLIC + grid%gz(:, TIME_PUBLIC)

        grid%gzind(:, TIME_EMPLOYMENT) = -1

        grid%gzSq = grid%gz ** 2

    end subroutine setgrid

    subroutine generateStateErrors(rnd)
        use mathfn
        use mathcons
        implicit none
        type(rnd_t), intent(out) :: rnd
        real(dp) :: unif(NDIM, NR), unif1(NR), unif2(NR)
        integer :: id(NR)
        integer :: rX0

        integer, parameter :: seed2(1) = (/ 1452780529 /)

        ! draw uniform random numbers
        call random_seed(put = seed2)
        call random_number(unif)

        rnd%ev = -log(-log(unif)) - EULERMASCHERONI

        do rX0 = 1, NR
            id(rX0) = rX0
        end do

        ! ev2 is a random perturbation of ev1
        call shuffle(id, NR)

        do rX0 = 1, NR
            rnd%ev2(:, rX0) = rnd%ev(:, id(rX0))
        end do

        ! transpose
        rnd%evTr = transpose(rnd%ev)
        rnd%ev2Tr = transpose(rnd%ev2)

    end subroutine generateStateErrors

    subroutine saveStateErrors(fname, rnd)
        implicit none
        character(len = *), intent(in) :: fname
        type(rnd_t), intent(in) :: rnd
        integer :: aX0, rX0
        integer :: funit
        open(newunit = funit, file = fname, status = 'replace', action = 'write')
        do rX0 = 1, NR
            do aX0 = 1, NDIM
                write(funit, '(I4,",",I4,",",G25.16,",",G25.16)') rX0, aX0, rnd%ev(aX0, rX0), rnd%ev2(aX0, rX0)
            end do
        end do
        close(funit)
    end subroutine saveStateErrors

    pure subroutine boxMullerTransform(u1, u2, z1, z2, n)
        use mathcons
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: u1(n)
        real(dp), intent(in) :: u2(n)
        real(dp), intent(out) :: z1(n)
        real(dp), intent(out) :: z2(n)

        real(dp) :: q1(n), q2(n)

        q1 = sqrt( -2.0_dp * log(u1) )
        q2 = TWOPI * u2

        z1 = q1 * cos(q2)
        z2 = q1 * sin(q2)

    end subroutine boxMullerTransform

    subroutine p_wageGrid(p)
        use mathfn
        implicit none
        type(p_t), intent(inout) :: p
        real(dp), parameter :: tailprob = 0.02_dp
        call generateWageGrid( p%wage_mu, p%wage_sd, tailprob, p%wageNode, p%wageNodeStd, &
                               p%wageDensity, NODEW, .false. )

        p%logWageNode = log(p%wageNode)

        p%dwageNode(1) = 0.5_dp * ( p%wageNode(2) - p%wageNode(1) )
        p%dwageNode(2:NODEW-1) = 0.5_dp * ( p%wageNode(3:NODEW) - p%wageNode(1:NODEW-2) )
        p%dwageNode(NODEW) = 0.5_dp * ( p%wageNode(NODEW) - p%wageNode(NODEW-1) )

        p%wageTrapzWeight = p%dwageNode * p%wageDensity

        call generateWageGrid( p%wage_mu, p%wage_sd, tailprob, p%wageNodeFine, p%wageNodeFineStd, &
                               p%wageDensityFine, NODEWFINE, .false. )

    end subroutine p_wageGrid

    subroutine h_jointWageDensity(h, pM, pF, verbose)
        use mathcons
        use mathfn
        implicit none
        type(h_t), intent(inout) :: h
        type(p_t), intent(in) :: pM
        type(p_t), intent(in) :: pF
        logical, intent(in) :: verbose

        real(dp) :: z1(NODEW), z1_2(NODEW)
        real(dp) :: z2(NODEW), z2_2(NODEW)

        real(dp) :: tempJointWageDensity(NODEW, NODEW)
        real(dp) :: tempInt(NODEW)
        real(dp) :: cons1, cons2, y

        integer :: wX2

        cons1 = 0.5_dp / ( PI * pM%wage_sd * pF%wage_sd * sqrt(1.0_dp - h%rho * h%rho) )
        cons2 = 0.5_dp / (1.0_dp - h%rho ** 2)

        z1 = ( log(pM%wageNode) - pM%wage_mu ) / pM%wage_sd
        z2 = ( log(pF%wageNode) - pF%wage_mu ) / pF%wage_sd

        z1_2 = z1 ** 2
        z2_2 = z2 ** 2

        do wX2 = 1, NODEW
            tempJointWageDensity(:, wX2) = cons1 * exp( -cons2 * (z1_2 + z2_2(wX2) - 2.0_dp * h%rho * z1 * z2(wX2) ) ) &
                / pM%wageNode / pF%wageNode(wX2)
        end do ! wX2

        ! integrate conditional on female wage wX2
        do wX2 = 1, NODEW
            tempInt(wX2) = trapz( pM%wageNode, tempJointWageDensity(:, wX2), NODEW )
        end do ! wX2

        ! now integrate over wX2
        y = trapz( pF%wageNode, tempInt, NODEW )

        if ( verbose ) then
            print *, 'joint wage integration', y
        end if

        if (y > 0.0_dp) then
            h%jointWageDensity = tempJointWageDensity / y
        else
            h%jointWageDensity = 0.0_dp
        end if

        do wX2 = 1, NODEW
            h%wageTrapzWeightM(:, wX2) = pM%dwageNode * h%jointWageDensity(:, wX2)
        end do

    end subroutine h_jointWageDensity

    subroutine generateWageGrid(mu, sd, tailprob, w, wstd, dy, nw, verbose)
        use mathfn
        implicit none
        real(dp), intent(in) :: mu, sd
        real(dp), intent(in) :: tailprob
        integer, intent(in) :: nw
        real(dp), intent(out) :: w(nw)
        real(dp), intent(out) :: wstd(nw)
        real(dp), intent(out) :: dy(nw)
        logical,  intent(in)  :: verbose
        real(dp) :: wmin, wmax
        real(dp) :: logwmin, logwmax
        real(dp) :: logw(nw)
        real(dp) :: cons

        if (nw == 1) then

            w = mu
            dy = 0.0_dp

        else

            ! common support for all types
            wmin = WMINA
            wmax = WMAXA

            logwmin = log(wmin)
            logwmax = log(wmax)
            call linspace( logwmin, logwmax, nw, logw )
            wstd = ( logw - mu ) / sd

            w = exp(logw)

            w(1) = wmin
            w(nw) = wmax
            dy= lognpdfVector(w, nw, mu, sd)
            cons = trapz(w, dy, nw)
            dy = dy / cons

            if (verbose) then
                print *, 'wage grid integration', cons
            end if

        end if

    end subroutine generateWageGrid

    subroutine writeWelfare(opt, fdir, fsave)
        use mathfn, only : intToStr
        use taxes
        use environment
        implicit none
        type(opt_t), intent(in) :: opt
        character(len = *), intent(in) :: fdir
        character(len = *), intent(in) :: fsave
        integer :: mX, iX
        type(tax_t) :: tax(NMARKET)
        real(dp) :: wlf0, fsp0
        real(dp) :: wlf1, fsp1
        integer :: funit
        real(dp) :: earn
        type(p_t) :: pM(NTYPE1, 0:1)
        type(p_t) :: pF(NTYPE2, 0:1)
        type(h_t) :: h(NTYPE1, NTYPE2, 0:1)
        type(m_t) :: m

        do mX = 1, NMARKET
            ! read taxes only to get market tax id
            call readFracKids( 'input/type_' // opt%locname // '_market' // intToStr(mX) // '.asc', .false., pM, pF, h, m )
            call readFoodStamps( fdir // '/fsp.dat', .true., tax(mX) )
            call readWelfare( fdir, m%taxId, .true., tax(mX) )
        end do

        do mX = 1, NMARKET

            ! single no kids
            open(newunit = funit, file = fsave // '/welfare_sin_k0_m' // intToStr(mX) // '.asc', status = 'replace', action = 'write')
            do iX = 0, 2600

                earn = real(iX,dp) / 4.33_dp

                ! apply initial eligibility test
                wlf1 = 0.0_dp
                fsp1 = evalSinFsp( tax(mX), 0, .true., earn, wlf1 )

                ! ignore initial eligibility test
                wlf0 = 0.0_dp
                fsp0 = evalSinFsp( tax(mX), 0, .false., earn, wlf0 )

                write(funit,'(F12.8,4(",",F12.8))') earn, wlf1, fsp1, wlf0, fsp0

            end do
            close(funit)

            ! single kids
            open(newunit = funit, file = fsave // '/welfare_sin_k1_m' // intToStr(mX) // '.asc', status = 'replace', action = 'write')
            do iX = 0, 2600

                earn = real(iX, dp) / 4.33_dp

                ! apply initial eligibility test
                wlf1 = evalSinKidsWelfare( tax(mX), .true., earn )
                fsp1 = evalSinFsp( tax(mX), 1, .true., earn, wlf1 )

                ! ignore initial eligibility test
                wlf0 = evalSinKidsWelfare( tax(mX), .false., earn )
                fsp0 = evalSinFsp( tax(mX), 1, .false., earn, wlf0 )

                write(funit,'(F12.8,4(",",F12.8))') earn, wlf1, fsp1, wlf0, fsp0

            end do
            close(funit)

            ! couple no kids
            open(newunit = funit, file = fsave // '/welfare_cou_k0_m' // intToStr(mX) // '.asc', status = 'replace', action = 'write')
            do iX = 0, 2600

                earn = real(iX, dp) / 4.33_dp

                ! apply initial eligibility test
                wlf1 = 0.0_dp
                fsp1 = evalCouFsp( tax(mX), 0, .true., earn, wlf1 )

                ! ignore initial eligibility test
                wlf0 = 0.0_dp
                fsp0 = evalCouFsp( tax(mX), 0, .false., earn, wlf0 )

                write(funit,'(F12.8,4(",",F12.8))') earn, wlf1, fsp1, wlf0, fsp0

            end do
            close(funit)

            ! couple kids
            open(newunit = funit, file = fsave // '/welfare_cou_k1_m' // intToStr(mX) // '.asc', status = 'replace', action = 'write')
            do iX = 0, 2600

                earn = real(iX, dp) / 4.33_dp

                ! apply initial eligibility test
                wlf1 = evalCouKidsWelfare( tax(mX), .true., earn )
                fsp1 = evalCouFsp( tax(mX), 1, .true., earn, wlf1 )

                ! ignore initial eligibility test
                wlf0 = evalCouKidsWelfare( tax(mX), .false., earn )
                fsp0 = evalCouFsp( tax(mX), 1, .false., earn, wlf0 )

                write(funit,'(F12.8,4(",",F12.8))') earn, wlf1, fsp1, wlf0, fsp0

            end do
            close(funit)

        end do

    end subroutine writeWelfare

    subroutine writeMarriageMarket(fname, m, ret)
        implicit none
        character(len = *), intent(in) :: fname
        type(m_t), intent(in) :: m(NMARKET)
        type(ret_t), intent(inout) :: ret

        real(dp) :: numSingle1Avg(NTYPE1)
        real(dp) :: numSingle2Avg(NTYPE2)
        real(dp) :: typeDemandAvg(NTYPE1, NTYPE2)

        integer :: funit
        integer :: pX1, pX2, mX, pX

        ! equilibrium marriage market

        numSingle1Avg = 0.0_dp
        numSingle2Avg = 0.0_dp
        typeDemandAvg = 0.0_dp

        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                numSingle1Avg(pX1) = numSingle1Avg(pX1) + m(mX)%marketSize * m(mX)%numSingle1(pX1)
            end do
            do pX2 = 1, NTYPE2
                numSingle2Avg(pX2) = numSingle2Avg(pX2) + m(mX)%marketSize * m(mX)%numSingle2(pX2)
            end do
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    typeDemandAvg(pX1, pX2) = typeDemandAvg(pX1, pX2) + m(mX)%marketSize * m(mX)%typeDemand(pX1, pX2)
                end do
            end do
        end do

        pX = 0
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                pX = pX + 1
                ret%marriageMarket(pX) = m(mX)%numSingle1(pX1)
            end do
            do pX2 = 1, NTYPE2
                pX = pX + 1
                ret%marriageMarket(pX) = m(mX)%numSingle2(pX2)
            end do
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    pX = pX + 1
                    ret%marriageMarket(pX) = m(mX)%typeDemand(pX1, pX2)
                end do
            end do
        end do

        open(newunit = funit, file = fname, status = 'replace', action = 'write')

        do mX = 0, NMARKET

            do pX1 = 1, NTYPE1
                if (mX == 0) then
                    write(funit, '(3(I4,", "),F15.8)') mX, pX1, 0, numSingle1Avg(pX1)
                else
                    write(funit, '(3(I4,", "),F15.8)') mX, pX1, 0, m(mX)%numSingle1(pX1)
                end if
            end do

            do pX2 = 1, NTYPE2
                if (mX == 0) then
                    write(funit, '(3(I4,", "),F15.8)') mX, 0, pX2, numSingle2Avg(pX2)
                else
                    write(funit, '(3(I4,", "),F15.8)') mX, 0, pX2, m(mX)%numSingle2(pX2)
                end if
            end do

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1

                    if (mX == 0) then
                        write(funit, '(3(I4,", "),F15.8)') mX, pX1, pX2, typeDemandAvg(pX1, pX2)
                    else
                        write(funit, '(3(I4,", "),F15.8)') mX, pX1, pX2, m(mX)%typeDemand(pX1, pX2)
                    end if

                end do
            end do
        end do

        close(funit)

    end subroutine writeMarriageMarket


    subroutine writeWeightMarket(fname, m)
        implicit none
        character(len=*), intent(in) :: fname
        type(m_t), intent(in) :: m(NMARKET)
        real(dp) :: lamAvg(NTYPE1, NTYPE2)
        integer :: funit
        integer :: pX1, pX2, mX

        lamAvg = 0.0_dp
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    lamAvg(pX1, pX2) = lamAvg(pX1, pX2) + m(mX)%marketSize * m(mX)%lam(pX1, pX2)
                end do
            end do
        end do

        open(newunit = funit, file = fname, status = 'replace', action = 'write')

        do mX = 0, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1

                    if (mX == 0) then
                        write(funit, '(3(I4,", "),F15.8)') mX, pX1, pX2, lamAvg(pX1, pX2)
                    else
                        write(funit, '(3(I4,", "),F15.8)') mX, pX1, pX2, m(mX)%lam(pX1, pX2)
                    end if

                end do
            end do
        end do

        close(funit)

    end subroutine writeWeightMarket

    subroutine writeTag(fname, tag)
        implicit none
        character(len = *), intent(in) :: fname
        character(len = *), intent(in) :: tag
        integer :: funit

        open( newunit = funit, file = fname, status = 'replace', action = 'write' )
        write(funit,'(A)') trim(adjustl(tag))
        close(funit)

    end subroutine writeTag

    subroutine writeWeight(fname, lam)
        implicit none
        character(len = *), intent(in) :: fname
        real(dp), intent(in) :: lam(NTYPE1, NTYPE2)
        integer :: funit
        integer :: pX1, pX2

        open(newunit = funit, file = fname, status = 'replace', action = 'write')

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                write(funit,'(3(I4,", "),F15.8)') 0, pX1, pX2, lam(pX1, pX2)
            end do
        end do

        close(funit)

    end subroutine writeWeight

    subroutine saveRevenue(rev, fpath)
        implicit none
        real(dp), intent(in) :: rev
        character(len = *), intent(in) :: fpath
        integer :: funit
        open(newunit = funit, file = fpath // '/revenue.asc', status = 'replace', action = 'write')
        write(funit, *) rev
        close(funit)
    end subroutine saveRevenue

end module test3kid
